/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.cache.internal.MultipleWorkItemOperation");
dojo.require("com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.cache.internal.Cache");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

(function() {

var Cache = com.ibm.team.workitem.web.cache.internal.Cache;
var TYPE_ERROR = 4;
var TYPE_INFO = 0;
var TYPE_WARNING = 2;
dojo.declare("com.ibm.team.workitem.web.cache.internal.MultipleWorkItemOperation", null, {
	/*
	 * Construct the proxy passing the object
	 */
	constructor: function(parms /* attributeList, idList, log */){
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		this.attributeList = parms.attributeList || [];
		this.action = parms.action || false;
		this.idList = parms.idList || [];
		this.itemIdList = parms.itemIdList || [];
		this.log = parms.log;
		this.onSuccess = parms.onSuccess;
		this.onError = parms.onError;
		this.isExecuting = false;
		this._stopHandle = dojo.subscribe(com.ibm.team.workitem.web.ui.internal.page.EventTopics.STOP_BACKGROUND_OPERATIONS, dojo.hitch(this, this.stop));
	},
	destroy: function(){
		dojo.unsubscribe(this._stopHandle);
	},
	stop: function(showTerminatingMessage) {
		if(	this.isExecuting) {
			this.log.logSetStopButtonEnabled(false);
			if (showTerminatingMessage != false) {
				this.logTerminatingMessage({
					message: this.messages.msgBackgroundOperationStopped,
					newLine: true,
					showPrefix: false,
					severity: TYPE_ERROR
				});
				this._operationComplete();
			}
		}
		dojo.publish(com.ibm.team.workitem.web.ui.internal.page.EventTopics.WORKITEM_VIEW_CHANGED);
	},
	/* 
	 * Execute the operation
	 * 1) Save all open work items.
	 * 2) Construct the update
	 * 3) Execute the update
	 * 4) Print the report
	 */
	execute: function(){
		this.isExecuting = true;
		this._lastTerminatingMessage = false;
		if(this.attributeList.length == 0 || this.idList.length == 0) return;
		this.log.logSetStopButtonEnabled(true);
		this.log.logClearMessages();
		this._failedItemCount = 0;
		this._successfulItemCount = 0;
		this.logStep( {
			message: dojo.string.substitute(this.messages.msgLogProcessingItems, [ this.idList.length + ""]),
			newLine: true,
			showPrefix: true,
			severity: TYPE_INFO
		});
		this.operationIndex = 0;
		this.saveOpenWorkItems({ onSaveAllSuccess: dojo.hitch(this, this._processBulkUpdate), onSaveAllError: dojo.hitch(this, this.stop) });
		
	},
	saveOpenWorkItems: function(parms) {
		// Save all open work items
		this.openWorkItems = Cache.getAllItems({filterSelectors: [ "isChanged" ], filterAttributes: [ "isWorkItem"] });
		this.openWorkItemsIndex = 0;
		if (this.openWorkItems.length > 0) {
			var message = dojo.string.substitute(this.messages.msgConfirmingSaveAll, [this.openWorkItems.length + ""])
			this.logStep({
				message: message,
				newLine: true,
				showPrefix: false,
				severity:  TYPE_INFO
			});
			message = dojo.string.substitute(this.messages.promptConfirmSaveAll, [this.openWorkItems.length + ""])
			if (!this.log.logConfirm(message)) {
				if(parms.onError)	parms.onError({message: this.messages.msgLogOperationCancelled});
				this.log.logClearMessages();
				return;
			}
			var message = dojo.string.substitute(this.messages.msgStartingSaveAllOperation, [this.openWorkItems.length + ""]);
			this.logStep({
				message: message,
				newLine: false,
				showPrefix: false,
				severity:  TYPE_INFO
			});
			this._processNextSave(parms);
		}
		else {
			parms.onSaveAllSuccess();
		}
	},
	_saveAllWorkItemsComplete: function(parms) {	
		this.logMessage({
				message: this.messages.msgSaveAllComplete,
				newLine: true,
				showPrefix: false,
				severity:  TYPE_INFO
			});
		parms.onSaveAllSuccess(parms);
	},
	// Called after save all is complete or in process
	_processBulkUpdate: function() {
		var message = dojo.string.substitute(this.messages.msgStartingBulkUpdateOperation, [this.idList.length + ""]);
			this.logStep({
				message: message,
				newLine: true,
				showPrefix: false,
				severity:  TYPE_INFO
			});
		var attributeIdentifiers = [];
		var attributeValues = [];
		for(var key in this.attributeList) {
			attributeIdentifiers.push(key);
			attributeValues.push(this.attributeList[key]);
		}
		var srhArgs =  {
			onSuccess: dojo.hitch(this, this._bulkUpdateComplete),			
			onError: dojo.hitch(this, this._bulkUpdateError)
		};
		if(this.action) {
			attributeIdentifiers.push("action");
			attributeValues.push(this.action);
		}
		var appArgs =  {
			itemIds: this.itemIdList,
			attributeIdentifiers: attributeIdentifiers,
			attributeValues: attributeValues
		};
		
        var srh = new com.ibm.team.repository.web.transport.ServiceResponseHandler(srhArgs, "onSuccess", "onError");
		
		com.ibm.team.workitem.web.client.internal.WorkItemClient.postBulkChangeWorkItems(srh, appArgs);	
	},
	_getErrors: function(resultDTOs) {
// DTO:
//	severity: OK=0, WARNING=2, ERROR=4
//  operationName: the name
//  message: error
//  infosArray: [] array of details
//  workItem
//     workItem.itemId: _uuid
//     workItem.stateId: _uuid
//     workItem.attributes
//
		var errorList = [];
		for(var i=0; i<resultDTOs.length; i++ ) {
			var dto = resultDTOs[i];
			if (dto.severity > 0) {
				var labelTypeId = dojo.string.substitute(this.messages.labelTypeId, [this.messages.labelWorkItem, this.idList[i]]);
				for (var j = 0; j < dto.infos.length; j++) {
					errorList.push({
						dto: dto,
						message: dto.message,
						summary: dto.infos[j].summary,
						details: dto.infos[j].description,
						severity: dto.infos[j].severity,
						index: i,
						labelTypeId: labelTypeId,
						id: this.idList[i]
					});
				}
			}
		}
		return errorList;
	},
	_bulkUpdateComplete: function(resultDTOs){
		var errorList = this._getErrors(resultDTOs);
		if(errorList.length == 0) {
			var message = this.messages.msgLogAttributeUpdateCompleteSuccess;
			var args = {
				message: message,
				newLine: true,
				showPrefix: false,
				severity:  TYPE_INFO
			}
			this.logTerminatingMessage(args);
			this.logMessage(args);
		} else {
//			var message = dojo.string.substitute(this.messages.msgLogAttributeUpdateCompleteWithFailures, [ errorList.length + "" ]);
			var message = this.messages.msgLogAttributeUpdateCompleteWithFailures;
			var terminatingError =errorList[0];
			if(terminatingError.severity ==4) {
				var details = dojo.string.substitute(this.messages.msgLogAttributeUpdateFailureWarning, [terminatingError.labelTypeId, terminatingError.message, terminatingError.details]);
			} else {
				var details =dojo.string.substitute(this.messages.msgLogAttributeUpdateFailureError, [terminatingError.labelTypeId, terminatingError.message, terminatingError.details]);
			}

			this.logTerminatingMessage({
				message: message,
				details: details,
				logMessage: false,
				newLine: true,
				showPrefix: false,
				severity:  terminatingError.severity
			});
			for(var i=0;i<errorList.length;i++) {
				var errorInfo = errorList[i];
			// 'msgLogAttributeUpdateFailureItem': "${0}: ${1}. Details: ${2}",  // error or warning, work item label, message, details
				if (errorInfo.severity == 4) {
					var message = dojo.string.substitute(this.messages.msgLogAttributeUpdateFailureWarning, [errorInfo.labelTypeId, errorInfo.message, errorInfo.details]);
					this.logMessage({
						message: message,
						newLine: true,
						showPrefix: false,
						severity:  TYPE_WARNING
					});
				}
				else {
					var message =dojo.string.substitute(this.messages.msgLogAttributeUpdateFailureError, [errorInfo.labelTypeId, errorInfo.message, errorInfo.details]);
					this.logMessage({
						message: message,
						newLine: true,
						showPrefix: false,
						severity:  TYPE_ERROR
					});
				}			    
				
			}
		}
		// Now we need to merge the resultDTOs into all the working copies.
		// so the values are correct and the save state is correct
		// Do this by loading them from the server
		var allWorkItems = Cache.getAllItems({filterAttributes: [ "isWorkItem" ]});
		this._refreshIndex = -1; // incremented first
		this._refreshItems = [];
		for(var i=0;i< allWorkItems.length;i++) {
			for(var j=0; j< resultDTOs.length;j++) {
				if(this.itemIdList[j] == allWorkItems[i].object.itemId) {
					this._refreshItems.push(allWorkItems[i]);		
				}
			}
		}
		if(this._refreshItems.length > 0) {
			this.logStep({
						message: this.messages.msgLogUpdatingRefreshingStart,
						newLine: true,
						showPrefix: false,
						severity:  TYPE_INFO
					});
			this._refreshNextItem();			
		} else {
			this._operationComplete();
		}
	},
	_operationComplete: function() {
		this.isExecuting = false;
		this.logEndOperation();
		this.logMessage({
						message: this.messages.msgLogOperationEnded,
						newLine: true,
						showPrefix: false,
						severity:  TYPE_INFO
					});
		this.log.logSetStopButtonEnabled(false);
		dojo.publish(com.ibm.team.workitem.web.ui.internal.page.EventTopics.WORKITEM_VIEW_CHANGED);
		if(this._lastTerminatingMessage && this._lastTerminatingMessage.severity > 0) {
			if(this.onError) this.onError();			
		} else {
			if(this.onSuccess) this.onSuccess();			
		}

	},
	_refreshNextItem: function() {
		if(this.workingCopy) {
			this.workingCopy.incrementUseCount();
			this.workingCopy = null;
		}
		if(!this.isExecuting) return;
		this._refreshIndex++;
		if(this._refreshIndex >= this._refreshItems.length ) {
			this._refreshAfterBulkUpdateComplete();
			return;
		}
	
		this.workingCopy = this._refreshItems[this._refreshIndex];
		if(!this.workingCopy) {
			logAlert("Error: Missing cache item.") // should never happen
			this.stop();
			return;
		}

		this.workingCopy.incrementUseCount();
		var message = dojo.string.substitute(this.messages.msgLogUpdatingRefreshing, [ this.workingCopy.getLinkLabel() ]);
		this.logMessage({
						message: message,
						newLine: true,
						showPrefix: false,
						severity:  TYPE_INFO
					});
		this._refreshFetch(this.workingCopy, dojo.hitch(this, this._refreshNextItem));
	},
	_refreshAfterBulkUpdateComplete: function() {
		this.logMessage({
						message: this.messages.msgLogRefreshComplete,
						newLine: true,
						showPrefix: false,
						severity:  TYPE_INFO
					});
	    this._operationComplete();
	},
	_bulkUpdateError: function(error){
		this.isExecuting = false;
		var message = dojo.string.substitute(this.messages.msgLogOperationFailed, [error.message]);
		var parms ={
						message: message,
						newLine: true,
						showPrefix: false,
						severity:  TYPE_ERROR
					};
		this.logMessage(parms);
		this.logTerminatingMessage(parms);
		this.logSetStopButtonEnabled(false);
		dojo.publish(com.ibm.team.workitem.web.ui.internal.page.EventTopics.WORKITEM_VIEW_CHANGED);
		this._operationComplete();
	},
	_processNextSave: function(parms) {
		if(!parms.ignoreIsExecuting && !this.isExecuting) return;
		if(this.openWorkItemsIndex == this.openWorkItems.length ) {
			this._saveAllWorkItemsComplete(parms);
			return;
		}
	
		this.workingCopy = this.openWorkItems[this.openWorkItemsIndex];
		if(!this.workingCopy) {
			this.logAlert("Error: Missing cache item:" + this.openWorkItemsIndex) // $NON-NLS-1 should never happen
			parms.onSaveAllError();
			return;
		}

		this.workingCopy.incrementUseCount();
		this._storeWorkingCopy(parms);
		},
	/* 
	 * store the working copy
	 */
	_storeWorkingCopy: function(parms) {
		var message = dojo.string.substitute(this.messages.msgLogUpdatingStoring, [ this.workingCopy.getId()+ ""]);
		this.logMessage({
						message: message,
						newLine: true,
						showPrefix: false,
						severity:  TYPE_INFO
					});
//		this.workingCopy.setIsInBackgroundOperation(true);
		this.workingCopy.storeWorkItem({
			instigator: this,
			ignoreIsExecuting: parms.ignoreIsExecuting == true,
			onSaveAllError: parms.onSaveAllError,
			onSaveAllSuccess: parms.onSaveAllSuccess,
			skipErrorMessage: true,
			onSuccess: dojo.hitch(this, this._storeComplete),
			onError: dojo.hitch(this, this._storeError),
			operationMsg: this.messages.busySaving
			});		
	},
	/*
	 * After saving reload to update all the values
	 */
	_storeComplete: function(parms) {
		var workItem = parms.workItem || this.workingCopy.object;
		if (workItem && workItem.attributes && workItem.attributes.id && workItem.attributes.id.id) {
			var proxy = Cache.getItem(workItem.attributes.id.id);
			this._refreshFetch(proxy, dojo.hitch(this, this._processNextSave), parms, this.messages.msgLogSaveComplete);
		}
	},
	_storeError: function(error, parms) {
		var idLabel = this.workingCopy ? this.workingCopy.getLinkLabel() : this.messages.labelUnknownItem;
		var errorMessage= error ? error.message : "";
		var message = dojo.string.substitute(this.messages.errorStoringInOperationSoCancel, [idLabel,  errorMessage]);
		this._showStoreError(message);
		if(parms.onSaveAllError) {
			parms.onSaveAllError(parms);
		}
	},

	_showStoreError: function(message) {
		if(this.workingCopy) {
			this.workingCopy.setIsInBackgroundOperation(false);
			this.workingCopy.decrementUseCount();
			this.workingCopy = null;	
		}
		var args = {
						message: message,
						newLine: true,
						showPrefix: false,
						severity:  TYPE_ERROR
					};
		this.logMessage(args);
		this.logTerminatingMessage(args);
		this.stop(/* show message */ false);
		this._operationComplete();
// Do the following to continue.
//		this.saveOperationIndex++;
//		this._failedItemCount++;
//		this._processNextSave();
	},

	// If another editor was editing the work item, then we need to reload it to get the full values stored
	// For example, setting priority sets the id to "2" but not the label to "Medium" until it is fetched again
	_refreshFetch: function(proxy, callback, callbackParms, message) {
		var id = proxy.getId();
		var parms = {
			    message: message,
				onSuccess: dojo.hitch(this, this._refreshFetchComplete),
				onError: dojo.hitch(this, this._refreshFetchError),
				processNext: callback,
				processNextParms: callbackParms,
				operationMsg: this.messages.busyLoading,
				id: id,
				skipErrorMessage: true,
				isForced: true,
				isInBackgroundOperation: true,
				isInit: false,
				fetchEditorPresentation: false,
				fetchInlineEditorPresentation: false
			}
		var message = dojo.string.substitute(this.messages.msgLogUpdatingRefreshing, [ id + ""]);
		this.logMessage({
						message: message,
						newLine: false,
						showPrefix: false,
						severity:  TYPE_INFO
					});
		proxy.fetchWorkItem(parms);
	},
	_refreshFetchComplete: function(parms) {
		parms.message = parms.message || this.messages.msgLogUpdateComplete;
		this._successfulItemCount++;
		this.openWorkItemsIndex++;
		var workItem = parms.workItem;
		if(!this.workingCopy && this.workingCopy && this.workingCopy.object) {
			workItem = this.workingCopy.object;
		}
		if(!workItem) return 	parms.processNext(parms.processNextParms);	
		if (workItem && workItem.attributes && workItem.attributes.id && workItem.attributes.id.id) {
			var proxy = Cache.getItem(workItem.attributes.id.id);
			if(!proxy) return 	parms.processNext(parms.processNextParms);		
		}
		var message = dojo.string.substitute(parms.message, [ proxy.getLinkLabel()]);
		this.logMessage({
						message: message,
						newLine: false,
						showPrefix: false,
						severity:  TYPE_INFO
					});
		proxy.setIsInBackgroundOperation(false);
		if(this.workingCopy) {
			this.workingCopy.decrementUseCount(true);
		}
		this.workingCopy = null;
	
		if(parms.processNext) {
			parms.processNext(parms.processNextParms);			
		}

	},
	_refreshFetchError: function(error) {
		var idLabel = this.workingCopy ? this.workingCopy.getLinkLabel() : this.messages.labelUnknownItem;
		var message = dojo.string.substitute(this.messages.errorFetchingInOperationSoCancel, [idLabel,  error.message]);
		this._showStoreError(message);
	},
	logEndOperation: function() {
		if(this.log) {
			this.log.logEndOperation();
		}
	},
	logClearMessages: function() {
		if (this.log) {
			this.log.logClearMessages(message);
		}
	},
	logAlert: function(message) {
		if (this.log) {
			this.log.logAlert(message);
		}
	},
	logConfirm: function(message) {
		if (this.log) {
			return this.log.logConfirm(message);
		}
		return confirm(message); // should never happen
	},	
	logStep: function(parms /* message */ ) {
		if(this.log) {
			this.log.logStep(parms);
		}
	},
	logEndOperation: function(parms /* message */ ) {
		if(this.log) {
			this.log.logEndOperation(parms);
		}
	},
	logTerminatingMessage: function(parms /* message, details, showPrefix, severity, newLine */ ) {
		this._lastTerminatingMessage = parms;
		if(this.log) {
			this.logEndOperation();
			this.log.logTerminatingMessage(parms);
			this.logMessage(parms);
		}
	},
	logMessage: function(parms /* message, showPrefix, severity, newLine */ ) {
		var prefix = !parms.showPrefix ? "" : (this.operationIndex!=null ? (this.operationIndex + 1) + ": " : "");
		parms.message = prefix + parms.message;
		if(this.log) {
			this.log.logMessage(parms);
		}
	}
}
);

})();
