/******************************************************************************* 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.cache.internal.WorkItemProxy");
dojo.require("com.ibm.team.workitem.web.cache.internal.ItemListProxy");
dojo.require("com.ibm.team.workitem.web.cache.internal.ItemProxy");
dojo.require("com.ibm.team.workitem.web.cache.internal.Cache");
dojo.require("com.ibm.team.workitem.web.cache.internal.WorkItemProxyClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("jazz.app.ResourceLink");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil"); // TODO: remove

(function() {

// These variables must match LinkCommand.java
var ADD_LINK= "addLink";
var REMOVE_LINK= "removeLink";

var SUBSCRIBERS_ID= "internalSubscriptions";
// The following must match WorkItemRestService.java
var UPDATE_LINKS = "updateLinks";
var ATTACHMENT_TYPE= "com.ibm.team.workitem.linktype.attachment";   // $NON-NLS-1$
 
var getAuthenticatedContributor	= com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var isGuest 					= com.ibm.team.repository.web.client.session.isGuest;
var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;

dojo.declare("com.ibm.team.workitem.web.cache.internal.WorkItemProxy", [com.ibm.team.workitem.web.cache.internal.ItemListProxy, com.ibm.team.workitem.web.cache.internal.WorkItemProxyClient], {
	/*
	 * Construct the proxy passing the object
	 */
	isWorkItem: true,
	constructor: function(object){
			this.changedAttributePaths  = {}
	},
	/*
	 * Set an attribute value, notifiying listeners
	 * The path is an array of attribute names, e.g., ["attributes", "internalPriority"]
	 */
	getValue: function(parms /* path */){
		var path = this._getPath(parms.path);
		if (path[0] == "linkTypes") {
			return this._getLinkType(parms)
		};
		// Special case for contributors/owners
		// If the request is for an id
		//            and the id is null
		//            and the itemId is not null
		//            set the id to the itemId
		if(parms.path && parms.path.length > 2 && path[0] == "attributes" && path[parms.path.length-1] == "id") {
			var value = this.inherited(arguments);
			if(value) return value;
			path[parms.path.length-1] = "itemId";
			parms.path = path;
			return this.inherited(arguments);
		}
		
		return this.inherited(arguments);
	},
	/*
	 * Called to fix anomolies that may exist after move/copy transmorgrification.
	 */
	prepareToBeEdited: function() {
		if(this._lastId && this._lastId != this.object.id) {
			this.object.id = this._lastId;
		}
	},
	handleObjectRefresh: function() {
		this.clearRetainedState();
	},
	getId: function(skipLabel) {
		if(this.isDestroyed) return this._cacheId;
		var id = -1;
		if(this.object && this.object.id) {
			return this.object.id;
		}
		if(this.object && this.object.attributes && this.object.attributes.id) {
			id = this.object.attributes.id.id || -1
		}
		if(!id && this.object && this.object.attributes && this.object.attributes.id && this.object.attributes.id.label) {
				id = this.object.attributes.id.label;
			} 
		if(!skipLabel && this._lastId != id) {
			this.getIdLabel(id);
		}
		this._lastId = id;
		return id;
	},
	getIdLabel: function(id){
		if((id==null) && this.idLabel != null) {
			return this.idLabel;
		}
		if(this._isInIdLabel) return this.idLabel;
		this._isInIdLabel = true;
		id = id || this.getId(true);
		var idLabel = id;
		if(id < 0) {
				idLabel = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil.getNewWorkItemTimestampLabel(this.object);
			} 
		this.idLabel = idLabel + "";
		this._isInIdLabel = false;
		return idLabel;
	},
	getItemId: function(){
		return this.object.itemId;
	},
	getLinkLabel: function() {
		var typeLabel = "Work Item"; // TODO:NLS
		if(this.object && this.object.attributes && this.object.attributes.workItemType && this.object.attributes.workItemType.label) {
			typeLabel =  this.object.attributes.workItemType.label;
		}
		return typeLabel + " " + this.getIdLabel(); // TODO:NLS
	}, 
	getTypeIconUrl: function() {
		if (this.object && this.object.attributes && this.object.attributes.workItemType && this.object.attributes.workItemType.iconUrl) {
			return this.object.attributes.workItemType.iconUrl;
		}
		return null;
	},
	getSummaryContent: function() {
		if (this.object && this.object.attributes && this.object.attributes.summary && this.object.attributes.summary.content) {
			return com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil.restoreTags(this.object.attributes.summary.content);
		}
		return ""
	}, 
	getSummaryLabel: function () {
		if(!this.object){
			return "";
		}
		
//		var id = this.getId();
//		if(id < 0)
//			return "An unsaved new work item";  // TODO:NLS
		
		var label = "";
		var summaryText =  this.getSummaryContent();
		
		try {
			label = this.getLinkLabel();
			if(summaryText) {
				label = dojo.string.substitute(this.messages.labelNameSummary, [label, summaryText]);
			}  
		} catch(e) {} // ignore
		return label
	}, 
	/* 
	 * Answer true if the value has changed
	 * TODO: Optimize this
	 */
	isChanged: function(){
		if(this.isDestroyed) return false;
		var isChanged = false;
		var cmds = {};
		var saveCommands = this.getSaveCommands(cmds);
		for (var x in saveCommands) {
			isChanged = true;
			break;
		}
		if(isChanged) return true;
		for (var x in this.changedAttributes) {
			isChanged = true;
			break;
		}
		if(isChanged) return true;
		return false;
	},
	/*
	 * Return an array of link types that match the parameters
	 */
	_filterLinkTypes: function(parms /* any properties of the link types */) {
		var result = parms.result || [];
		var linkTypes = this.object.linkTypes;
		if(!linkTypes) return result;

		for(var i=0;i<linkTypes.length;i++){
			var found;
			for(var key in parms) {
				found = false;
				if(linkTypes[i][key] == null) break; // no match
				if(parms[key] != linkTypes[i][key]) break; // no match
				found = true; // used when dropping out of the loop
			}
			if(found) {
				result.push(linkTypes[i]);
			}
		}
		return result;
	},
	_getLinkType: function(parms){
		if (!parms || !parms.path) 
			return null;
		if (!this.object.linkTypes) {
			this.object.linkTypes = [];
		}
		var path = this._getPath(parms.path);
		if (path.length == 1) {
			return this.object.linkTypes;
		}
	
		var result = null;
		/* find the referenced linktype */
		for (var i = 0; i < this.object.linkTypes.length; i++) {
			if (this.object.linkTypes[i].id == path[1]) {
				result = this.object.linkTypes[i].linkDTOs;
				if(!result && parms.defaultValue) {
					this.object.linkTypes[i].linkDTOs = parms.defaultValue;
					return this.object.linkTypes[i].linkDTOs;
				}
				break;
			}
		}
		if(!result && parms.defaultValue) {
			var type = {
				id: path[1],
				linkDTOs: parms.defaultValue
			}
			this.object.linkTypes.push(type);
			return type.linkDTOs;
		}

		return result;
	},
	setValue: function(parms /* caller, attributeId, path, savedAttributePath,  value */){
		var path = this._getPath(parms.path);
		var isChanged;
		if (path[0] == "linkTypes") {
			isChanged = this._setLinkTypeValue(parms)
		}
		else {
			// approval is a special case. Find the value in the approvals.approvals array
			if (path[0] == "approvals") {
				isChanged = this._setApprovalsValue(parms);
			}
		 else {
			isChanged = this.inherited(arguments) || isChanged; 
		}}
		
		if (isChanged || parms.forceNotify) {
			this.notifyOfItemChanged();
		} 
		
		return isChanged;
	},
	notifyOfItemChanged: function() {
		dojo.publish(EventTopics.WORKITEM_CHANGED, [this])
	},
	_setLinkTypeValue: function(parms){ /* returns isChanged */
		var path = this._getPath(parms.path);
		if (path.length == 1) {
			this.object.linkTypes = parms.value;
			this._checkForListeners(parms)
			return true;
		}
		if (!this.object.linkTypes) {
			this.object.linkTypes = [];
		}
		
		/* find the referenced linktype */
		var isFound = false;
		for (var i = 0; i < this.object.linkTypes.length; i++) {
			if (this.object.linkTypes[i].id == path[1]) {
				isFound = true;
				this.object.linkTypes[i].linkDTOs = parms.value || [];
				break;
			}
		}
		
		if (!isFound) {
			var linkType = {
				id: path[1],
				linkDTOs: parms.value ||
				[]
			}
			this.object.linkTypes.push(linkType);
		}
		this._checkForListeners(parms);
		return !isFound;
	},
	// An approval value is a single approval within an approval's approvals array
	_setApprovalsValue: function(parms){
		if (!this.object.approvals) {
			this.object.approvals = [];
		}

		if (!parms || !parms.path) 
			return false;
			
		var path = this._getPath(parms.path);
		if (path.length < 2) 
			return false;
			
		var isChanged;	
		if (this.object.approvals.length==0) {
			this.object.approvals = [parms.value];
			parms.value._approvalId = 0;
			isChanged = true;
		}
		else {
			var id = path[1];
			if (id < this.object.approvals.length) {
				this.object.approvals[id] = parms.value;
				isChanged = true;
			}
			else {
				// not found
				parms.value._approvalId = id;
				this.object.approvals.push(parms.value);
				isChanged = true;
			}
		}
		this._checkForListeners(parms);
		return isChanged;
	},
	// A work item was refreshed in a multi save so merge in the changes
	mergeResultDTO: function(resultDTO) {
		
	},
	getSaveCommands: function(allCommands){
		this._getApprovalSaveCommands(allCommands);
		this._getAttachmentSaveCommands(allCommands);
		this._getLinkTypeSaveCommands(allCommands);
//		this._getSubscriberSaveCommands(allCommands);
		return allCommands;
	},
	/*
	 * Add commands to the updateLinks commands
	 * Each command needs an attachment descriptor:
	 *      itemId: linkDTO.itemId
	 *      name: linkDTO.target.name
	 */
	_getAttachmentSaveCommands: function(allCommands){
	
		var attachmentDTOs = this.getValue({
			path: ["linkTypes", /* id */ "com.ibm.team.workitem.linktype.attachment", "linkDTOs"]
		});
		
		if (!attachmentDTOs) 
			return allCommands;
		
		var commands = [];
		for (var i = 0; i < attachmentDTOs.length; i++) {
			var attachmentDTO = attachmentDTOs[i];
			if (attachmentDTO._isNew) {
				var command = {
					cmd: ADD_LINK,
					type: ATTACHMENT_TYPE,
					itemId: attachmentDTO.itemId,
					name: attachmentDTO.target.name // Used in history pending changes
				}
				commands.push(command);
			}
			else { // Not new
				if (attachmentDTO.isRemoved) {
					var command = {
						cmd: REMOVE_LINK,
						type: ATTACHMENT_TYPE,
						itemId: attachmentDTO.itemId,
						name: attachmentDTO.target.name // Used in history pending changes
					}
					commands.push(command);
				}
			}
		}
		
		var updateLinks = allCommands[UPDATE_LINKS] ? allCommands[UPDATE_LINKS] : [];
		
		for (var i = 0; i < commands.length; i++) {
			updateLinks.push(commands[i]);
		}
		
		if (updateLinks.length > 0) {
			allCommands[UPDATE_LINKS] = updateLinks;
		}
		
		return allCommands;
	},
	/*
	 * Add values to the updateLinks commands
	 *
	 */
	_getLinkTypeSaveCommands: function(allCommands){
	
		var linkTypes = this.getValue({
			path: ["linkTypes"]
		});
		if (!linkTypes || linkTypes.length == 0) 
			return allCommands;
		
		var commands = [];
		for (var i = 0; i < linkTypes.length; i++) {
			this._getSaveCommandsForType(commands, linkTypes[i]);
		}
		
		var updateLinks = allCommands[UPDATE_LINKS] ? allCommands[UPDATE_LINKS] : [];
		
		for (var i = 0; i < commands.length; i++) {
			updateLinks.push(commands[i]);
		}
		
		if (commands.length > 0) {
			allCommands[UPDATE_LINKS] = updateLinks;
		}
		return allCommands;
	},
	_getSaveCommandsForType: function(commands, linkType){
		var linkDTOs = linkType.linkDTOs;
		if (!linkDTOs || linkDTOs.length == 0) 
			return;
		var linkDTO;
		for (var i = 0; i < linkDTOs.length; i++) {
			linkDTO = linkDTOs[i]
			if (linkDTO.id == "com.ibm.team.workitem.linktype.attachment") 
				continue;
			if (linkDTO._isNew || this.getId() < 0) {
				if (!linkDTO._isDeleted) {
					// For a new link, consolidate all commands into one command
					var command = {
						cmd: ADD_LINK,
						type: linkType.id,
						end: linkType.isSource ? "source" : "target",
						name: linkType.displayName || "link"// Used in history pending commands
					}
					if (linkDTO.itemId) 
						command.itemId = linkDTO.itemId;
					if (linkDTO.url) 
						command.uri = linkDTO.url;
					if (linkDTO.comment) 
						command.comment = linkDTO.comment;
					
					commands.push(command);
				}
			}
			else { // Not new
				if (linkDTO._isDeleted) {
					var command = {
						cmd: REMOVE_LINK,
						type: linkType.id,
						end: linkType.isSource ? "source" : "target",
						name: linkType.displayName || "link"// Used in history pending commands
					}
					if (linkDTO.itemId) 
						command.itemId = linkDTO.itemId;
					if (linkDTO.url) 
						command.uri = linkDTO.url;
					
					commands.push(command);
				}
			}
		}
	},
	/*
	 * Return an array  of commands. See work item 40464.
	 *
	 * Command Format is:
	 *
	 *      Create a new approval request for a specific user
	 *		var approvalType= ... // e.g. getApprovalTypes()[0].id
	 *		var contributorItemId= ...
	 *		var command= { cmd: "createApproval", param: { type: approvalType, name: "M5 Approval", approvers: [ { user: contributorItemId } ] }};
	 *
	 *		Update an approval descriptor with another name and dueDate
	 *		var approvalId= ...// The id of an approval descriptor, e.g. workItem.approvals[0].id
	 *		var date= ...
	 *		var command= { id: approvalId, cmd: "updateApproval", param: { name: "New name", dueDate: date.getTime() } };
	 *
	 *		Add a user to an existing approval request
	 *		var contributorItemId= ...
	 *		var approvalId= ...// The id of an approval descriptor, e.g. workItem.approvals[0].id
	 *		var command= { id: approvalId, cmd: "updateApprover", param: { user: contributorItemId } };
	 *
	 *		Change an approval state
	 *		var contributorItemId= ...
	 *		var state= ... // e.g. getApprovalStates()[0].id
	 *		var approvalId= ... // The id of an approval descriptor, e.g. workItem.approvals[0].id
	 *		var command= { id: approvalId, cmd: "updateApprover", param: { user: contributorItemId, state: state } };
	 *
	 *		Remove an approver
	 *		var contributorItemId= ...
	 *		var approvalId= ... // The id of an approval descriptor, e.g. workItem.approvals[0].id
	 *		var command= { id: approvalId, cmd: "removeApprover", param: { user: contributorItemId } };
	 *
	 *		Remove an approval descriptor
	 *		var approvalId= ... // The id of an approval descriptor, e.g. workItem.approvals[0].id
	 *		var command= { id: approvalId, cmd: "removeApproval" };
	 *
	 *    A removed approval stays on the approvals list but is not shown, it returns its own
	 *    removal command
	 *
	 */
	_getApprovalSaveCommands: function(allCommands){
		if (!this.object.approvals) 
			return allCommands;
		var commands = [];
		for (var i = 0; i < this.object.approvals.length; i++) {
			var approval = this.object.approvals[i];
			if (!approval) 
				continue;
			if (approval._isNew) {
				if (!approval._isDeleted) {
					// For a new approval, consolidate all commands into one command
					var command = {
						cmd: "createApproval",
						param: {
							type: approval.approvalType,
							name: approval.name
						}
					}
					if (approval.dueDate) {
						command.param.dueDate = approval.dueDate;
					}
					if (approval.approvals) {
						command.param.approvers = [];
						for (var j = 0; j < approval.approvals.length; j++) {
							subapproval = approval.approvals[j];
							if (subapproval && subapproval.approver) {
								var approverCommand = {
									user: subapproval.approver.itemId,
									state: subapproval.state
								}
								command.param.approvers.push(approverCommand)
							}
						}
					}
					commands.push(command);
				}
			}
			else {
				// For approval updates, send all the commands normally
				if (approval._commands) {
					for (var j = 0; j < approval._commands.length; j++) {
						if (approval._commands[j]) {
							commands.push(approval._commands[j]);
						}
					}
				}
				if (approval.approvals) {
					for (var j = 0; j < approval.approvals.length; j++) {
						var subapproval = approval.approvals[j];
						if (subapproval && subapproval.approver && subapproval.approver._command) {
							commands.push(subapproval.approver._command);
						}
					}
				}
				
			}
		}
		if (commands.length > 0) {
			allCommands.updateApprovals = commands;
		}
		return allCommands;
	},
	getFormElementValues: function(formElements, skipSubscribers){
		var self = this;
		for (var key in this.changedAttributes) {
			var savedAttributePath =  this.changedAttributes[key].savedAttributePath || this.changedAttributes[key].path || this.changedAttributePaths[key];
			self._addFormElementValues({
				path: savedAttributePath,
				attributeId: key,
				formElements: formElements
			})
		}
		
		// If current user made a comment, subscribe the current user
		if (!skipSubscribers && !isGuest() && formElements.internalComments && formElements.internalComments.length > 0) {
			var currentUser = getAuthenticatedContributor();
			if (currentUser.itemId != this.object.attributes.owner.itemId &&
			currentUser.itemId != this.object.attributes.creator.itemId) {
				this.addItems({ items: [currentUser], attributeId: SUBSCRIBERS_ID } );
			}
		}
	},
	/*
	 * Get the form element association for an attribute path
	 */
	_addFormElementValues: function(parms /* path, formElements, attributeId */){
		if (this.changedAttributes[parms.attributeId]) {
			if(parms.path[parms.path.length-1] == "items") {
				parms.formElements[parms.attributeId] = this._getItemsFormElements(this._getAttribute(this._getPath(parms.path)));
			} else {
				parms.formElements[parms.attributeId] = this._getAttribute(this._getPath(parms.path));
			}
		}
	},
	clearRetainedState: function(ignoreComment) {
		if(ignoreComment && this.object && this.object.attributes && this.object.attributes.comment) {
			delete this.object.attributes.comment;
		}
		var oldComment;
		if(!ignoreComment && this.changedAttributes && this.changedAttributes.comment) {
			oldComment = this.changedAttributes.comment;
		}
		this.changedAttributes = {};
		this.changedAttributePaths = {};
		if(oldComment) {
			this.changedAttributes.comment = oldComment;
			var listenerParms = {
				path: ["attributes", "comment"],
				event: "onchange",
				attributeId: "comment"
			};
			this.notifyListeners(listenerParms);
		}
	},
	getRetainedState: function(retainedState){
		if (this.changedAttributes && this.changedAttributes["comment"]) {
			retainedState["comment"] = this.changedAttributes["comment"].value;
		}
	},
	setRetainedState: function(retainedState){
		if (retainedState && retainedState.internalComments != null) {
			this.setValue({
				path: ["attributes", "comment", "content"],
				attributeId: "comment",
				isInitialization: false,
				value: retainedState.internalComments
			})
		}
	},
	/*
	 * This method is copied from WorkItemProxyFactory to avoid circular dependencies
	 */
	_getNewWorkItemProxy: function(parms /* id, createIfNeeded */) {
		var id = parms.id || lastId--;
		var workItemProxy = com.ibm.team.workitem.web.cache.internal.Cache.getItem(id);
		if(workItemProxy || !parms.createIfNeeded) return workItemProxy;
		
		/* if ID is -1 then it is a new item and should be converted to a unique id */
		if(id == -1) {
			id = -1 * Date.parse(new  Date().toString())
		}
		
		var object= {
			attributes: {
				id: {
					id: id
				}
			}
		}
		workItemProxy =  new com.ibm.team.workitem.web.cache.internal.WorkItemProxy({ object: object, isFetched: false}); // empty object initially
		com.ibm.team.workitem.web.cache.internal.Cache.addItem(id, workItemProxy);
		return workItemProxy;
	},
	applyResourceLinkToAnchor: function(parms, anchor) {
// Parms:
//			uri: resource's href for the hover,
//			presentationUri: anchor's href for the click,
//			label: label
//			retainLinkText: true/false, Keep the anchor's link text (false), or replace it with the label
//
			if(parms.presentationUri) {
					var m = /com.ibm.team.scm.ChangeSet\/([A-Za-z0-9_\-]+)\?.*workspace=([A-Za-z0-9_\-]+)/.exec(parms.presentationUri);
					if (m) {
					   	parms.presentationUri = "#action=com.ibm.team.scm.viewChangeSet&changeSetItemId=" + m[1] + "&workspaceItemId=" + m[2];
					}				
			}
			var resourceLinkArgs= parms;
			new jazz.app.ResourceLink(resourceLinkArgs, anchor);
		}	
}
);

})();
