/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.Dashboard");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardError");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardContainer");
dojo.require("com.ibm.team.dashboard.web.ui.internal.LeftNav");
dojo.require("com.ibm.team.dashboard.web.ui.internal.ViewletContainer");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.Observable");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker");
dojo.require("jazz.util.ResizeNotifier");
dojo.require("jazz.ui.StyledBox");
dojo.require("jazz.ui.Dialog");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServiceTracker = com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker;
var LeftNav = com.ibm.team.dashboard.web.ui.internal.LeftNav;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var DashboardContainer = com.ibm.team.dashboard.web.ui.internal.DashboardContainer;
var DashboardError = com.ibm.team.dashboard.web.ui.internal.DashboardError;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var Observable = com.ibm.team.dashboard.web.ui.internal.util.Observable;

var messages;

/**
 * An embeddable dashboard widget which allows any page to contain a dashboard. The
 * widget is customizable via constructor arguments, will fetch and load a dashboard
 * from the repository, and will persist changes made by the user.
 * 
 * The dashboard must be associated with a scope item of a specific scope type,
 * specifically a project area or team area. This restriction may be changed or removed
 * in the future.
 * 
 * @since 0.7
 * 
 * NOTE: This is provisional API. Clients using this API do so with the awareness that
 * this class may be removed or substantially changed at future times without warning.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.Dashboard",
	[Observable, dijit._Widget, dijit._Templated], {
	
	templateString: "<div class=\"com-ibm-team-dashboard-web-ui-internal-Dashboard\"></div>",
	
	/**
	 * (Required for SCOPE_PROJECT_AREA, SCOPE_TEAM_AREA, Optional for SCOPE_CONTRIBUTOR)
	 * The name of the project area containing the dashboard, e.g. "Jazz Foundation".
	 */
	projectArea: null,
	
	/**
	 * (Required) The scope of the dashboard; one of the following properties from
	 * DashboardConstants: SCOPE_PROJECT_AREA, SCOPE_TEAM_AREA.
	 */
	scope: null,
	
	/**
	 * (Required for SCOPE_TEAM_AREA) This is required if scope is SCOPE_TEAM_AREA,
	 * where it is an array of team names; a path from a root team to the target team,
	 * e.g. ["Jazz Foundation Development", "Web UI", "Dashboard"]. The use of this
	 * param may be	expanded for other scopes in the future.
	 */
	scopeItem: null,
	
	/**
	 * Constructs a new dashboard widget. The required parameters listed above must be
	 * passed in as properties to the first argument, as well as any optional arguments.
	 * 
	 * e.g.
	 * var myDashboard = new Dashboard({
	 *    scope: DashboardConstants.SCOPE_TEAM_AREA,
	 *    scopeItem: "Jazz Foundation Development/Web UI/Dashboard"
	 * });
	 * myDomNode.appendChild(myDashboard.domNode);
	 */
	constructor: function(args) {
		if (!messages) {
			messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		}
	},
	
	/**
	 * Destroys the dashboard widget. Should be called when the dashboard is removed
	 * from a page and no longer needed.
	 */
	/* void */ destroy: function() {
		if (this._serviceTracker) {
			var requests = this._serviceTracker.getActiveRequests();
			for (var i=0;i<requests.length;i++) {
				requests[i].abort();
			}
		}
		if (this.navigateAwayHandle) {
			PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
		}
		if (this.onBeforeUnloadHandle){
			PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);	
		}
		if (this._contentWidget) {
			this._contentWidget.destroy();
			delete this._contentWidget;
		}
		this.inherited(arguments);
	},
	
	
	/*------------------------------------------------------------------------
	 * INTERNAL CODE BELOW - DO NOT CALL THIS CODE.
	 * 
	 * If you need additional functionality, please request API via work item.
	 *-----------------------------------------------------------------------*/
	
	/**
	 * Only applicable to contributor scope dashboards. If false, the "Sharing Options"
	 * button will be hidden.
	 */
	allowSharing: true,

	/* void */ showTab: function(/* string */ tab) {
		if (this._contentWidget instanceof DashboardContainer) {
			if (tab != this.tab) {
				this.tab = tab;
				this._contentWidget.showViewletPageById(tab);
				this.propertyChanged("tab", tab);			
			}
			this.setActive();
		}
	},
	
	/* void */ setActive: function() {
		this._contentWidget.setActive();
		this._checkNavigateAwayHandle();
	},
	
	/* void */ postCreate: function() {
		var args = {tab: this.tab};
		switch (this.scope) {
			case DashboardConstants.SCOPE_TEAM_AREA:
				args.team = this.scopeItem.join("/");
				break;
			case DashboardConstants.SCOPE_CONTRIBUTOR:
				// internal usage only
				for (var i in this.scopeItem) {
					args[i] = this.scopeItem[i];
				}
				break;
		}
		this.viewDashboard(args, this.projectArea);
	},

	/* object */ getDashboardItem: function() {
		if (this._contentWidget instanceof DashboardContainer) {
			return this._contentWidget.dashboardItem;
		}
		return null;
	},
	
	/* ServiceTracker */ getServiceTracker: function() {
		if (!this._serviceTracker) {
			this._serviceTracker = new ServiceTracker();
		}
		return this._serviceTracker;
	},

	/* string */ getTitle: function() {
		if (this._contentWidget && this._contentWidget.dashboardItem) {
			var title = this._contentWidget.dashboardItem.title;
			if (this._contentWidget.currentPage && this._contentWidget.currentPage.viewletPageHelper) {
				var tabTitle = this._contentWidget.currentPage.viewletPageHelper.title;
				return dojo.string.substitute(messages.dashboardTitleWithTab, [title, tabTitle]);
			} else {
				return dojo.string.substitute(messages.dashboardTitle, [title]);
			}
		}
		return messages.dashboardsGenericTitle;
	},
	
	/* boolean */ isVisible: function() {
		return this._isVisible;
	},
	
	/* boolean */ navigateAwayEvent: function(event,workbenchNavigation) {
		if (event) {
			//if this function was called by an onbeforeunload event
			if (this._contentWidget && this._contentWidget.dashboardItem) {
				//return a message to show in the onBeforeUnload dialog
				return this._contentWidget.navigateAwayEvent(false, true);
			}
		} else {
			var navigateAwayParams = new Object();
			if (this._contentWidget && this._contentWidget instanceof DashboardContainer && this._contentWidget.dashboardItem) {
				navigateAwayParams = this._contentWidget.navigateAwayEvent(workbenchNavigation);
			} else {
				navigateAwayParams.doNavigate = true;
			}
			if (navigateAwayParams.doNavigate) {
				this._isVisible = false;
				if (navigateAwayParams.destroyWidget) {
					this._destroyContentWidget();
				}
				if (workbenchNavigation) {
					PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
					this.navigateAwayHandle = null;
					dojo.removeClass(document.body,"com-ibm-team-dashboard-body");
				}
			}
			return navigateAwayParams.doNavigate;	
		}
	},
	
	viewDashboard: function(args, projectArea) {
		this._isVisible = true;
		this._checkNavigateAwayHandle();
		
		var tab = args.tab;
		delete args.tab;
		delete args.action;
		
		var serviceObj = {
			dashboard: this,
			args: args,
			tab: tab,
			success: function(/* ResponseDTO */ responseDTO) {
				switch(responseDTO.status) {
					case "ERROR":
						// invalid request or server error
						this.dashboard._createError.call(this.dashboard,responseDTO.msg);
						break;
					case "OK":
					default:
						var contentWidget = this.dashboard._contentWidget;
						if (this.dashboard._isRequestedDashboardOpen.call(this.dashboard,responseDTO.data.dashboard.itemId)) {
							if (contentWidget.tabs)
								contentWidget.showViewletPageById(this.tab);
						} else {
							if (contentWidget && contentWidget.dashboardItem) {
								this.dashboard._createContentWidget(this.tab);
							} 
							this.dashboard._initializeContentWidget.call(this.dashboard,responseDTO.data);
						}
						dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
						this.dashboard.propertyChanged("onload");
						this.dashboard._isLoaded = true;
						break;
				}
			},
			failure: function(/* Error */ error) {
				if (error) {
					switch (error.status) {
						case 0:
							error.message = messages.connectionFailure;
							break;
						case 403:
							error.message = messages.errorNoPermission;
							break;
					}
				}
				this.dashboard._createError(messages.errorRetrieving+"<br/><br/>" + error.message);
			}
		};
		this._createContentWidget(tab);
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.getDashboard(args.id, args.team, projectArea, srh, this.getServiceTracker());
		setTimeout(dojo.hitch(this,this._preloadImages),0);
	},
	
	setCurrentDeleted: function () {
		this._contentWidget.deleted = true;
	},
	// ensure onload callbacks are fired if they are added after load completes
	// see work item 73132
	addListener: function(callback, property) {
		this.inherited(arguments);
		if (property == "onload" && this._isLoaded)
			callback("onload");
	},
	_isRequestedDashboardOpen: function(args) {
		if (this._contentWidget instanceof DashboardContainer && this._contentWidget.dashboardItem) {
			var dashboardItem = this._contentWidget.dashboardItem;
			var dashboardBundle = this._contentWidget.dashboardBundle;
			if (args.id) {
				return dashboardItem.itemId == args.id;
			} else if (args.team) {
				return dashboardItem.scope == DashboardConstants.SCOPE_TEAM_AREA && unescape(dashboardBundle.defaults.team.path) == args.team;
			} else {
				return dashboardItem.scope == DashboardConstants.SCOPE_PROJECT_AREA;
			}
		}
		return false;
	},
	_createContentWidget: function(/* string */ dashboardTabId) {
		if (this._contentWidget){
			this._destroyContentWidget();
		}
		if (!dashboardTabId)
			dashboardTabId = "tab1";
		this._contentWidget = new DashboardContainer({dashboardWidget: this, selectedTabId: dashboardTabId, allowSharing: this.allowSharing});
		this._contentWidget.addListener(dojo.hitch(this, this._onTabChange), "tab");
		this._contentWidget.addListener(dojo.hitch(this, this._bubblePropertyChange), "title");
		this._contentWidget.addListener(dojo.hitch(this, this._bubblePropertyChange), "shared");
		this._contentWidget.addListener(dojo.hitch(this, this.propertyChanged), "ondelete");
		this._setContent(this._contentWidget.domNode);
	},
	_onTabChange: function(/* string */ property, /* string */ tab) {
		this.tab = tab;
		this.propertyChanged(property, tab);
	},
	_bubblePropertyChange: function(/* string */ property, /* string */ arg) {
		this.propertyChanged(property, arg);
	},
	_initializeContentWidget: function(/* DashboardBundleDTO */ bundle) {
		if (this._contentWidget && this._contentWidget.setContent) {
			this._contentWidget.setContent(bundle);
		}
	},
	_createError: function(/* string */ message) {
		// clear old content first
		if (this._contentWidget) {
			this._destroyContentWidget();	
		}
		this._contentWidget = new DashboardError({message: message});
		this._contentWidget.populate();
		this._setContent(this._contentWidget.domNode);
	},
	_setContent: function(/* Node */ node) {
		this.domNode.innerHTML = "";
		this.domNode.appendChild(node);
	},
	_destroyContentWidget: function() {
		if (!this._contentWidget) return;
		this._contentWidget.domNode.parentNode.removeChild(this._contentWidget.domNode);
		this._contentWidget.destroy();
		this._contentWidget = null;
	},
	_preloadImages: function() {
		if (this.pictures) return;
		this.pictures = new Array();
		var versionId = (dojo.dependency !== undefined) ? "/_ver"+dojo.dependency.getWebBundleVersion("com.ibm.team.dashboard.web") : "/_ver";
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/content/icons/zoom-in.gif"+versionId));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/content/images/edit-contents.gif"+versionId));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/content/icons/add-tab.png"+versionId));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/content/icons/add-viewlet-tab.png"+versionId));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/metanav/images/tab-area-bg.gif"+versionId));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/metanav/images/tab-down-arrow.gif"+versionId));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/metanav/images/tab-left.gif"+versionId)); 
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/metanav/images/tab-middle.gif"+versionId)); 
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/metanav/images/tab-right.gif"+versionId));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/metanav/images/tab-selected-left.gif"+versionId)); 
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/metanav/images/tab-selected-middle.gif"+versionId)); 
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/metanav/images/tab-selected-right.gif"+versionId)); 
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/metanav/images/viewlet-page-top-left.gif"+versionId));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/metanav/images/viewlet-page-top-right.gif"+versionId));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/metanav/icons/delete.gif"+versionId));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/metanav/icons/refresh.gif"+versionId));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/metanav/icons/restore-default.gif"+versionId));
		setTimeout(dojo.hitch(this,this._preloadSecondaryImages),4000);
	},
	_preloadSecondaryImages: function() {
		var versionId = (dojo.dependency !== undefined) ? "/_ver"+dojo.dependency.getWebBundleVersion("com.ibm.team.dashboard.web") : "/_ver";
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/content/icons/add-tab-hov.png"+versionId));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/content/icons/add-viewlet-tab-hov-sel.png"+versionId));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/content/icons/zoom-in-hov.gif"+versionId));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/content/icons/zoom-out.gif"+versionId));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/content/icons/zoom-out-hov.gif"+versionId));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/chooser/images/gray-corner-fill-BL.gif"+versionId));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/chooser/images/gray-corner-fill-BR.gif"+versionId));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/chooser/images/gray-corner-fill-TL.gif"+versionId));
		this.pictures.push((new Image()).src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/chooser/images/gray-corner-fill-TR.gif"+versionId));
	},
	_checkNavigateAwayHandle: function() {
		if (!this.navigateAwayHandle) {
			this.navigateAwayHandle = PlatformUI.getWorkbench().registerBodyClickHandler(dojo.hitch(this, this._handleNavigate), /*priority*/ 10000);
			dojo.addClass(document.body,"com-ibm-team-dashboard-body");
		}
		if (!this.onBeforeUnloadHandle){
			this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this.navigateAwayEvent), /*priority*/ 10000);	
		}
	},
	_handleNavigate: function(urlObj) {
		var winLoc = new dojo._Url(window.location.href);
		var workbenchNavigation = false;
		if(urlObj.host && urlObj.scheme && !(urlObj.host === winLoc.host && urlObj.scheme === winLoc.scheme)) {
			// different domain - allow onBeforeUnload to prompt
			var navigateAway = true;
		} else if (urlObj.fragment && ((!urlObj.host && !urlObj.path) || (urlObj.path && urlObj.path === winLoc.path))) {
			// link within web UI
			if(urlObj.fragment.indexOf("com.ibm.team.dashboard") < 0) {
				// we're navigating to another area of the web UI
				workbenchNavigation = true;
			}
			if (this._stripTabParam(urlObj.fragment) == this._stripTabParam(winLoc.fragment)) {
				// the link is to the current url (not including the tab param)
				// ensure left nav highlighting is correct
				//this._leftNav.updateExplicitHighlighting(/* clear explicit flag */true);
				var navigateAway = true;
			} else
				var navigateAway = this.navigateAwayEvent(false, workbenchNavigation);
			// same exact base URI - prompt for save/cancel
		} else {
			var navigateAway = true;
		}
		return !navigateAway;
	},
	_stripTabParam: function(string) {
		if (!string)
			return string;
		var index = string.indexOf("&tab=");
		if (index == -1)
			return string;
		var result = string.substring(0,index);
		var end = string.substring(index+1);
		index = end.indexOf("&");
		if (index == -1)
			return result;
		return result + end.substring(index);
	}
});

})();
