/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.AllShared");

dojo.require("dojo.i18n");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.ui.FilterBox");
dojo.require("jazz.util.html");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.app.ResourceLink");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.process.web.client.projectarea");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var HTML = jazz.util.html;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var ServiceTracker = com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var FilterBox = jazz.ui.FilterBox;
var StyledTable = jazz.ui.StyledTable;
var ResourceLink = jazz.app.ResourceLink;

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

var PAGE_SIZE = 50;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.AllShared",
	[dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/AllShared.html"),
	
	constructor: function() {
		this.allDashboardsPrev = messages.allDashboardsPrev;
		this.allDashboardsNext = messages.allDashboardsNext;
		this.allDashboardsOf = messages.allDashboardsOf;
	},
	postCreate: function() {
		this._filterBox = new FilterBox({onChange: dojo.hitch(this,this._doFilter)});
		this._filterArea.appendChild(this._filterBox.domNode);
		this.connect(this._previous, "onclick", this._handlePrevious);
		this.connect(this._next, "onclick", this._handleNext);
		this._table = new StyledTable({columns: [{label: "Title", width: "50%", sortable: true}, {label: "Author", width: "50%", sortable: true}],
								trim: jazz.ui.styledTable.TrimStyles.GOLD,
								handleSort: dojo.hitch(this,this._runQuery,0,true),
								sortBy: 0});
		this._resultsArea.appendChild(this._table.domNode);
		this._runQuery();
	},
	
	_handleNext: function(event) {
		event.preventDefault();
		if (dojo.hasClass(this._next, "disabled"))
			return;
		this._runQuery(this._startIndex + PAGE_SIZE);
	},
	
	_handlePrevious: function(event) {
		event.preventDefault();
		if (dojo.hasClass(this._previous, "disabled"))
			return;
		this._runQuery(this._startIndex - PAGE_SIZE);
	},
	
	_runQuery: function(startIndex, clearToken) {
		PlatformUI.getWorkbench().showMessage(messages.allSharedSearching);
		if (clearToken)
			delete this._token;
		if (this._runQueryTimeout) {
			clearTimeout(this._runQueryTimeout);
			delete this._runQueryTimeout;
		}
		if (!this._nextQueryId)
			this._nextQueryId = 1;
		if (!startIndex)
			startIndex = 0;
		var orderBy = this._table.sortBy ? "author" : "title";
		var orderByAscending = this._table.sortByAscending;
		var serviceObject = {
			self: this,
			id: this._nextQueryId,
			success: function(results) {
				if (this.self._lastQueryId && this.self._lastQueryId > this.id)
					return;
				this.self._lastQueryId = this.id;
				this.self._token = results.token;
				this.self._renderResults(results);
				this.self._updatePageCount(results);
				PlatformUI.getWorkbench().hideMessageArea();
			},
			failure: function(error) {
				dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
			}
		};
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		DashboardClient.getSharedDashboards(getCurrentProjectArea().itemId, this._currentFilter || null, startIndex, PAGE_SIZE, this._token || null, orderBy, orderByAscending, srh, this._getServiceTracker());
		this._nextQueryId++;
	},
	
	_renderResults: function(results) {
		this._table.clearBody();
		if (results && results.dashboards && results.dashboards.length > 0) {
			for (var i = 0; i < results.dashboards.length; i++) {
				this._table.tbody.appendChild(this._createRow(results.dashboards[i]));
			}
		} else {
			this._table.tbody.appendChild(this._createNoneMessage());
		}
	},
	
	_createRow: function(dashboard) {
		var row = document.createElement("tr");
		var td = document.createElement("td");
		td.innerHTML = "<td><a href='#action=" + DashboardConstants.ACTION_VIEW_DASHBOARD + "&" + dashboard.urlParams + "'>" + HTML.escape(dashboard.title) + "</a>";
		row.appendChild(td);
		td = document.createElement("td");
		var link = new ResourceLink({
			uri: (net.jazz.ajax._dataServerRoot || Util.getContextPath(true)) + "/users/" + encodeURIComponent(dashboard.user.userId),
			presentationUri: "#action=com.ibm.team.repository.viewUserProfile&itemId=" + dashboard.user.itemId,
			label: HTML.escape(dashboard.user.name),
			retainLinkText: true
		});
		td.appendChild(link.domNode);
		row.appendChild(td);
		return row;
	},
	
	_createNoneMessage: function() {
		var row = document.createElement("tr");
		var td = document.createElement("td");
		td.colSpan = 2;
		td.innerHTML = HTML.escape(messages.allSharedNone);
		row.appendChild(td);
		return row;
	},
	
	_doFilter: function(filter) {
		this._currentFilter = filter;
		this._isRelatedFilterStale = true;
		// set a timeout to query with the new filter
		// need to ensure 0 is passed to _runQuery or setTimeout will pass some other value
		if (!this._runQueryTimeout) {
			this._runQueryTimeout = setTimeout(dojo.hitch(this,this._runQuery,0,true),500);
		}
	},
	
	_updatePageCount: function(results) {
		if (!results || !results.dashboards || !results.dashboards.length) {
			dojo.addClass(this._previous, "disabled");
			dojo.addClass(this._next, "disabled");
			this._currentStartCount.innerHTML = 0;
			this._currentEndCount.innerHTML = 0;
			this._totalCount.innerHTML = 0;
		} else {
			if (results.startIndex != 0)
				dojo.removeClass(this._previous, "disabled");
			else
				dojo.addClass(this._previous, "disabled");
			if (results.totalResults - results.startIndex > PAGE_SIZE)
				dojo.removeClass(this._next, "disabled");
			else
				dojo.addClass(this._next, "disabled");
			this._startIndex = results.startIndex;
			this._currentStartCount.innerHTML = this._startIndex + 1;
			this._currentEndCount.innerHTML = Math.min(this._startIndex + PAGE_SIZE, results.totalResults);
			this._totalCount.innerHTML = results.totalResults;
		}
	},
	_getServiceTracker: function() {
		if (!this._serviceTracker)
			this._serviceTracker = new ServiceTracker();
		return this._serviceTracker;
	},
	destroy: function() {
		// abort any remaining requests
		if (this._serviceTracker) {
			var requests = this._serviceTracker.getActiveRequests();
			for (var i=0;i<requests.length;i++) {
				requests[i].abort();
			}
		}
		this.inherited(arguments);
	}
	
});

})();
