/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.BookmarksViewlet");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.ui.HoverPopup");
dojo.require("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.OrderableList");
dojo.require("jazz.util.html");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.Form");

dojo.require("dojo.i18n");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "BookmarksMessages");

(function() {

var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var HoverPopup = jazz.ui.HoverPopup;
var HTML = jazz.util.html;
var OrderableList = com.ibm.team.dashboard.viewlets.web.ui.internal.utils.OrderableList;

var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;
var messages;

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.BookmarksViewlet",
	Viewlet, {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/BookmarksViewlet.html"),
	
	constructor: function(args) {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "BookmarksMessages");
		this.noBookmarks = messages.noBookmarks;
	},
	
	init: function() {
		this.draggableContainers = new Array();
		var nodes = I18nUtil.addMessage(this._noQueriesEditContentsLink, messages.noBookmarksEdit);
		this.connect(nodes[0], "onclick", this._doEdit);
	},
	refresh: function() {
		this.update();
		this.getSite().doneRefresh();
	},
	
	update: function() {
		if (this.editWidget && this.editWidget.destroy){
			this.editWidget.destroy();
		}
		this.editWidget = null;
		while(this.draggableContainers.length > 0) {
			var draggable = this.draggableContainers.pop();
			var div = draggable.node;
			if (div && div.parentNode) {
				div.parentNode.removeChild(div);	
			}
			draggable.destroy();
		}
		this._site.getActionArea().innerHTML = "";
		this.currentList = null;
		while(this.nodes && this.nodes.length > 0) {
			var node = this.nodes.pop();
			if (node && node.parentNode) {
				node.parentNode.removeChild(node);	
			}
		}
		while(this.categoryDivs && this.categoryDivs.length > 0) {
			var div = this.categoryDivs.pop();
			div.parentNode.removeChild(div);
		}
		this.categoryDivs = new Array();
		this.nodes = new Array();
		var bookmarks = this.getViewletState().get("bookmarks");
		bookmarks.sort(dojo.hitch(this,this._compareBookmarks));
		for (var i = 0; i < bookmarks.length; i++) {
			this.nodes.push(this.createBookmarkDomNode(bookmarks[i]));
		}
		this.bookmarks = bookmarks;
		this.populateBookmarks();
		if (this.isEditing()) {
			var doc = this._site.getActionArea().ownerDocument;
			var addLi = doc.createElement("div");
			addLi.className = "action add-bookmark";
			var addLink = doc.createElement("a");
			var img = doc.createElement("img");
			img.src = dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "graphics/bookmarks/icons/add_bkmrk.gif") + "";
			addLink.appendChild(img);
			addLink.appendChild(doc.createTextNode(messages.add));
			addLink.href = "";
			addLi.appendChild(addLink);
			this._site.getActionArea().appendChild(addLi);
			this.connect(addLink, "onclick", this._addBookmark);
			this._addLi = addLi;
		}
		if (this.nodes && this.nodes.length > 0) {
			dojo.addClass(this._noBookmarksDiv, "hidden");
		} else {
			dojo.removeClass(this._noBookmarksDiv, "hidden");
			if (this.isEditing() || !this._site.isDashboardEditing()) {
				dojo.addClass(this._noQueriesEditContentsLink, "hidden");
			} else {
				dojo.removeClass(this._noQueriesEditContentsLink, "hidden");
			}
		}
	},
	setDashboardEditing: function(editing) {
		if (this.isEditing() || !editing) {
			dojo.addClass(this._noQueriesEditContentsLink, "hidden");
		} else {
			dojo.removeClass(this._noQueriesEditContentsLink, "hidden");
		}
	},
	createBookmarkDomNode: function(/* object */ bookmark) {
		var div = document.createElement("div");
		div.className = "bookmark";
		var link = document.createElement("a");
		if (!this.isEditing()) {
			link.href = bookmark.href;	
		}
		if (this.getPreference("newWindow")) {
			link.target="_blank";
		}
		var textSpan = document.createElement("span");
		// IE shows the text select cursor on the span instead of the pointer unless we tell it otherwise (see work item 43665)
		textSpan.style.cursor = "pointer";
		var textNode = document.createTextNode(bookmark.title);
		textSpan.appendChild(textNode);
		link.appendChild(textSpan);
		if (this.isEditing()) {
			var editAction = new com.ibm.team.dashboard.viewlets.web.ui.internal.EditBookmarkAction({
				bookmark: bookmark,
				parent: div,
				buttonClass: "edit",
				callback: dojo.hitch(this,this._editBookmark)});
			editAction.domNode.title = messages.edit;
			link.appendChild(editAction.domNode);
			var deleteAction = new com.ibm.team.dashboard.viewlets.web.ui.internal.EditBookmarkAction({
				bookmark: bookmark,
				parent: div,
				buttonClass: "delete",
				callback: dojo.hitch(this,this._deleteBookmark)});
			deleteAction.domNode.title = messages.del;
			link.appendChild(deleteAction.domNode);
		}
		div.appendChild(link);
		if (bookmark.description && !this.isEditing() && !this.isZoomed()) {
			var html = "<div class='com-ibm-team-dashboard-viewlets-web-ui-internal-BookmarksViewlet-Tooltip'>" + 
				HTML.escape(bookmark.description) + "</div>";
			var tooltip = new HoverPopup({around: textSpan, html: html});
		}
		if (bookmark.description && this.isZoomed()) {
			var descDiv = document.createElement("div");
			if (dojo.hasClass(div, "singleIndent"))
				dojo.addClass(descDiv, "doubleIndent")
			else
				dojo.addClass(descDiv, "singleIndent")
			descDiv.innerHTML = HTML.escape(bookmark.description);
			div.appendChild(descDiv);
		}
		return div;
	},
	createSection: function(bookmark){
		var groupCategory = this.getPreference("groupCategory");
		groupCategory = groupCategory && groupCategory != "false";
		
		var alphaSort = this.getPreference("alphaSort");
		alphaSort = alphaSort && alphaSort != "false";
		
		if(groupCategory && bookmark.category){
			var catDiv = document.createElement("div");
			dojo.addClass(catDiv, "tagHeading");
			if (this.isZoomed()) {
				var heading = document.createElement("h2");
				heading.innerHTML = HTML.escape(bookmark.category);
				catDiv.appendChild(heading);
			} else
				catDiv.innerHTML = HTML.escape(bookmark.category);
			this._bookmarksWrapper.appendChild(catDiv);
			this.categoryDivs.push(catDiv);
		}
		if (this.isEditing() && !alphaSort) {
			var orderableList = document.createElement("div");
			this._bookmarksWrapper.appendChild(orderableList);
			if (groupCategory){
				this.currentList = new OrderableList(orderableList, {accept: ["bookmark"], bookmarks: this.bookmarks, nodes: this.nodes, category: bookmark.category});	
			} else {
				this.currentList = new OrderableList(orderableList, {accept: ["bookmark"], bookmarks: this.bookmarks, nodes: this.nodes});
			}
			this.draggableContainers.push(this.currentList);
		}
		this._currentCategory = bookmark.category;
	},
	populateBookmarks: function() {
		if (this.bookmarks.length == 0) {
			return;
		}
		
		var groupCategory = this.getPreference("groupCategory");
		groupCategory = groupCategory && groupCategory != "false";
		
		var alphaSort = this.getPreference("alphaSort");
		alphaSort = alphaSort && alphaSort != "false";

		if (groupCategory || !alphaSort) {
			this.createSection(this.bookmarks[0]);
		}
	
		for (var i = 0; i < this.bookmarks.length; i++) {
			if (groupCategory && this.bookmarks[i].category) {
				if (this._currentCategory != this.bookmarks[i].category) {
					if (this.bookmarks[i].category){
						this.createSection(this.bookmarks[i]);
					}
				}
				dojo.addClass(this.nodes[i], "singleIndent");
			}
			if (this._modifiedBookmark == this.bookmarks[i]) {
				dojo.addClass(this.nodes[i], "modified");
				delete this._modifiedBookmark;
			}
			if (this.isEditing() && !alphaSort){
				this.currentList.add({domNode: this.nodes[i]}, {type: "bookmark"});
			} else {
				this._bookmarksWrapper.appendChild(this.nodes[i]);	
			}
		}
	},
	
	_compareBookmarks: function(a, b) {
		var groupCategory = this.getPreference("groupCategory");
		if (groupCategory) {
			if (a.category || b.category) {
				if (!a.category || (b.category && a.category.toUpperCase() < b.category.toUpperCase())) return -1;
				if (!b.category || a.category.toUpperCase() > b.category.toUpperCase()) return 1;
			}
		}
		var alphaSort = this.getPreference("alphaSort");
		if (alphaSort) {
			if (a.title.toUpperCase() < b.title.toUpperCase()) return -1;
			if (a.title.toUpperCase() > b.title.toUpperCase()) return 1;
			return 0;
		}
		return a.id - b.id;
	},
	_doEdit: function(event) {
		dojo.stopEvent(event);
		this.getSite().editContents();
	},
	_addBookmark: function(event) {
		event.preventDefault();
		event.cancelBubble = true;
		if (this.editWidget && this.editWidget.domNode) {
			return;
		}
		if (this.nodes && this.nodes.length > 0) {
			var lastChild = this.nodes[this.nodes.length-1];	
		}
		if (lastChild && !dojo.hasClass(lastChild,"bookmark") && !dojo.hasClass(lastChild,"orderableList")) {
			var div = lastChild;
		} else {
			var div = this._bookmarksWrapper.appendChild(document.createElement("div"));	
		}
		this.editWidget = new com.ibm.team.dashboard.viewlets.web.ui.internal.EditBookmarkWidget(
			{li: div,
			callback: dojo.hitch(this,this._saveChanges)});
		this.editWidget.init();
	},
	
	_editBookmark: function(/* object */ bookmark, /* <li> */ li) {
		if (this.editWidget) {
			this.editWidget.destroy();
			this.editWidget = null;
		}
		this.editWidget = new com.ibm.team.dashboard.viewlets.web.ui.internal.EditBookmarkWidget(
			{bookmark: bookmark,
			li: li,
			callback: dojo.hitch(this,this._saveChanges)});
		this.editWidget.init();
	},
	
	_deleteBookmark: function(/* object */ bookmark) {
		var bookmarks = this.bookmarks;
		for (var i = 0; i < bookmarks.length; i++) {
			if (bookmarks[i].id == bookmark.id) {
				bookmarks.splice(i,1);
				break;
			}
	
		}
		this.getViewletState().set("bookmarks", bookmarks);
		this.update();
	},
	
	/* object */ getDefaultState: function() {
		var state = new Object();
		state.bookmarks = new Array();
		state.nextId = 1;
		return state;
	},
	
	_saveChanges: function(/* object */ bookmark) {
		// if this is a new bookmark, assign an id
		var bookmarks = this.bookmarks;
		if (!bookmark.id && bookmark.id != 0) {
			bookmark.id = this.getViewletState().get("nextId");
			this.getViewletState().set("nextId", bookmark.id+1);
			bookmarks.push(bookmark);
		} else {
			for (var i in bookmarks) {
				if (bookmarks[i] && bookmarks[i].id == bookmark.id) {
					dojo.mixin(bookmarks[i], bookmark);
					break;
				}
			}
		}
		this.getViewletState().set("bookmarks", bookmarks);
		this._modifiedBookmark = bookmark;
		this.update();
	},
	
	hasZoomableContent: function() {
		return !this.isEditing();
	},
	
	destroy: function() {
		while(this.draggableContainers && this.draggableContainers.length > 0)
			this.draggableContainers.pop().destroy();
		if (this.editWidget)
			this.editWidget.destroy();
	}
});

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.EditBookmarkWidget",
	[dijit._Widget, dijit._Templated], {
	
	templateString: "<li dojoAttachPoint='_editArea' class='editArea'></li>",
	
	constructor: function(args) {
		if (args.li)
			this._li = args.li;
		if (args.callback)
			this._callback = args.callback;
		if (args.bookmark)
			this._bookmark = args.bookmark;
	},
	
	init: function() {
		var form = new dijit.form.Form({execute:dojo.hitch(this, this._save)});
		this._editArea.appendChild(form.domNode);
		
		var table = document.createElement("table");
		table.className = "edit-table";
		var tbody = document.createElement("tbody");
		tbody.appendChild(this._createTextField({name: messages.fieldTitle + "*", id: "title",
			value: this._bookmark && this._bookmark.title ? this._bookmark.title : ""}));
		tbody.appendChild(this._createTextField({name: messages.fieldURL + "*", id: "href",
			value: this._bookmark && this._bookmark.href ? this._bookmark.href : "http://"}));
		tbody.appendChild(this._createTextField({name: messages.fieldDesc, id: "description",
			value: this._bookmark && this._bookmark.description ? this._bookmark.description : ""}));
		tbody.appendChild(this._createTextField({name: messages.fieldCategory, id: "category",
			value: this._bookmark && this._bookmark.category ? this._bookmark.category : ""}));
		table.appendChild(tbody);

		var submitArea = document.createElement("div");
		submitArea.className="submit-area";
		
		var required = document.createElement("div");
		required.innerHTML = "* " + messages.required;
		required.className = "required-field";
		
		var submitButton = document.createElement("input");
		submitButton.type = "submit";
		submitButton.value = messages.ok;
		submitButton.className = "save-button";
		
		var cancelButton = document.createElement("input");
		cancelButton.type = "button";
		cancelButton.value = messages.cancel;
		cancelButton.className = "cancel-button";
		
		submitArea.appendChild(required);
		submitArea.appendChild(submitButton);
		submitArea.appendChild(cancelButton);
		form.domNode.appendChild(table);
		form.domNode.appendChild(submitArea);
		
		this.connect(cancelButton,"onmouseup",this.destroy);
		
		if (this._li) {
			this._li.parentNode.insertBefore(this.domNode, this._li);
			this._li.style.display = "none";
		}
	},
	
	_createTextField: function(/* object */ data) {
		var tr = document.createElement("tr");
		var label = document.createElement("td");
		label.className = "label";
		label.innerHTML = data.name;
		var field = document.createElement("td");
		field.className = "field";
		var textBox = new dijit.form.TextBox({name: data.id, value: data.value, trim: true});
		textBox.startup();
		field.appendChild(textBox.domNode);
		tr.appendChild(label);
		tr.appendChild(field);
		return tr;
	},
	
	_save: function(/* object */ newSettings) {
		if (newSettings.href == "" || newSettings.title == "") {
			var alertText = newSettings.title ? "" : "     - " + messages.fieldTitle + "\n"; 
			alertText += newSettings.href ? "" : "     - " + messages.fieldURL + "\n";
			alert(messages.errorRequired + "\n" + alertText + messages.pleaseComplete);
			return;
		}
		var href = newSettings.href;
		if (href && href.indexOf(":") == -1 && href.charAt(0) != "/") {
			newSettings.href = "http://"+href;
		}
		bookmark = this._bookmark ? this._bookmark : new Object();
		dojo.mixin(bookmark, newSettings);
		this._callback(bookmark);
		this.destroy();
	},
	
	destroy: function() {
		if (this._li)
			this._li.style.display = "block";
		var domNode = this.domNode;
		this.inherited(arguments);
		if (domNode && domNode.parentNode)
			domNode.parentNode.removeChild(domNode);
	}
});

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.EditBookmarkAction",
	[dijit._Widget, dijit._Templated], {
	
	templateString: "<div dojoAttachPoint='_content'></div>",
	
	constructor: function(args) {
		this._bookmark = args.bookmark;
		this._callback = args.callback;
		this._parent = args.parent;
		this._buttonClass = args.buttonClass;
	},
	postCreate: function() {
		this._content.className = this._buttonClass;
				
		this.connect(this._parent, "onmouseover", this._addClass);
		this.connect(this._parent, "onmouseout", this._removeClass);
		this.connect(this._content, "onmousedown", dojo.stopEvent);
		this.connect(this._content, "onclick", this._doCallback);
	},
	_addClass: function(){
		if (jazz.ui.dnd.draggableManager().isDragging) {
			return;
		}
		var query = dojo.query(".modified",this._parent.parentNode);
		for(var i = 0; i < query.length;i++) {
			dojo.removeClass(query[i],"modified");
		}
		dojo.addClass(this._content, this._buttonClass+"-over");
		dojo.addClass(this._parent,"modified");
	},
	_removeClass: function() {
		dojo.removeClass(this._content, this._buttonClass+"-over");
		dojo.removeClass(this._parent,"modified");
	},
	_doCallback: function(event) {
		dojo.stopEvent(event);
		this._callback(this._bookmark, event.target.parentNode.parentNode);
	}
});

})();
