/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.ui.internal.BuildPage");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("jazz.ui.SideNavigationLayout");

dojo.require("com.ibm.team.build.web.ui.internal.LeftNav");
dojo.require("com.ibm.team.build.web.ui.internal.DefinitionList");
dojo.require("com.ibm.team.build.web.ui.internal.DefinitionPage");
dojo.require("com.ibm.team.build.web.ui.internal.BuildResult");
dojo.require("com.ibm.team.build.web.ui.internal.BuildQueue");
dojo.require("com.ibm.team.build.web.ui.internal.BuildEngines");
dojo.require("com.ibm.team.build.web.ui.internal.BuildRequest");
dojo.require("com.ibm.team.build.web.ui.internal.SimplePage");
dojo.require("com.ibm.team.build.web.ui.internal.utils.ErrorUtil");

dojo.require("com.ibm.team.build.web.util.BuildUtil");
dojo.require("com.ibm.team.build.web.ui.BuildConstants");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.build.viewlets.web.ui.internal.BuildsViewlet");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

(function() {

var ErrorUtil = com.ibm.team.build.web.ui.internal.utils.ErrorUtil;
var BuildUtil = com.ibm.team.build.web.util.BuildUtil;
var BuildConstants = com.ibm.team.build.web.ui.BuildConstants;
var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var messages = dojo.i18n.getLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

dojo.declare("com.ibm.team.build.web.ui.internal.BuildPage",
	[dijit._Widget, dijit._Templated], {
	
	templateString: "<div class=\"com-ibm-team-build-web\"></div>",
	
	postCreate: function(){
		com.ibm.team.build.web.ui.internal._buildPage = this;
		this.layout = new jazz.ui.SideNavigationLayout({});
		this._scrollListener = dojo.connect(this.layout,"setMainContent",this,function() {
			window.scrollTo(0,0);
		});
		this.domNode.appendChild(this.layout.domNode);
		
		this._leftNav = new com.ibm.team.build.web.ui.internal.LeftNav({layout: this.layout, buildPage: this});
		
		// late bind to ActionRegistry to ensure that workbench is instantiated
		var ar = PlatformUI.getWorkbench().getActionRegistry();
		ar.registerAction(BuildConstants.ACTION_VIEW_DEFAULT, this, "viewDefault");
		ar.registerAction(BuildConstants.ACTION_VIEW_DEFINITION_LIST, this, "viewDefinitionList");
		ar.registerAction(BuildConstants.ACTION_VIEW_DEFINITION, this, "viewDefinition");
		ar.registerAction(BuildConstants.ACTION_VIEW_RESULT, this, "viewResult");
		ar.registerAction(BuildConstants.ACTION_VIEW_BUILD_QUEUE, this, "viewBuildQueue");
		ar.registerAction(BuildConstants.ACTION_VIEW_BUILD_ENGINES, this, "viewBuildEngines");
		ar.registerAction(BuildConstants.ACTION_REQUEST_BUILD, this, "requestBuild");
	},
	viewDefault:function(args) {
		if (!this._content) {
			PlatformUI.getWorkbench().setTitle(messages.builds);
			var content = document.createElement("div");
			content.style.marginLeft = "10px";
			content.style.marginTop = "10px";
			content.style.fontSize = "9pt";
			content.style.lineHeight = "1.2em";
			content.innerHTML = "<div style='font-size: 16pt; font-weight: normal;'>" + messages.welcome + "</div><p>" + messages.getStartInstruction + "</p>";
			this.layout.setMainContent(content);
		}
	},
	viewDefinitionList: function(args) {
		PlatformUI.getWorkbench().showMessage(messages.loadingDefs);
		if (this._content)
			this._content.destroy();
		this._content = new com.ibm.team.build.web.ui.internal.DefinitionList(args);
		if (this._content.domNode) {
			this.layout.setMainContent(this._content.domNode);
			this._leftNav.updateHighlighting({id: "buildDefinitions"});
		}
	},
	viewDefinition: function(args) {
		args.definitionId = args.id;
		args.id = null;
		PlatformUI.getWorkbench().showMessage(messages.loadingDef);
		if (this._content)
			this._content.destroy();
		this._content = new com.ibm.team.build.web.ui.internal.DefinitionPage(args);
		if (this._content.domNode) {
			this.layout.setMainContent(this._content.domNode);
			this._leftNav.recentSection.pageVisited(this._content);
			this._leftNav.updateHighlighting({id: args.definitionId});
		}
	},
	viewResult: function(args) {
		// if the result is already open, just update the tab
		if (this._content instanceof com.ibm.team.build.web.ui.internal.BuildResult && this._content.resultId == args.id) {
			this._content.tabContainer.selectTab(args.tab || BuildConstants.RESULT_TAB_OVERVIEW);
			PlatformUI.getWorkbench().setTitle(this._content.browserTitle);
			return;
		}
		PlatformUI.getWorkbench().showMessage(messages.loadingBuild);
		if (this._content)
			this._content.destroy();
		this._content = new com.ibm.team.build.web.ui.internal.BuildResult({resultId: args.id, tabId: args.tab, buildPage: this});
		if (this._content.domNode) {
			this.layout.setMainContent(this._content.domNode);
			this._leftNav.recentSection.pageVisited(this._content);
			this._leftNav.updateHighlighting({id: args.id});
		}
	},
	viewBuildQueue: function(args){
		PlatformUI.getWorkbench().showMessage(messages.loadingBuildQueue);
		if (this._content)
			this._content.destroy();
		this._content = new com.ibm.team.build.web.ui.internal.BuildQueue(args);
		if(this._content.domNode){
			this.layout.setMainContent(this._content.domNode);
			this._leftNav.updateHighlighting({id: "buildQueue"});
		}
	},
	viewBuildEngines: function(args){
		PlatformUI.getWorkbench().showMessage(messages.loadingBuildEngines);
		if (this._content)
			this._content.destroy();
		this._content = new com.ibm.team.build.web.ui.internal.BuildEngines(args);
		if(this._content.domNode){
			this.layout.setMainContent(this._content.domNode);
			this._leftNav.updateHighlighting({id: "buildEngines"});
		}
	},
	requestBuild: function(args) {
		if (args.definition) {
			args.definitionId = args.definition;
			args.definition = null;
		}
		if (args.result) {
			args.resultId = args.result;
			args.result = null;
		}
		PlatformUI.getWorkbench().showMessage(messages.loadingBuildRequest);
		if (this._content)
			this._content.destroy();
		this._content = new com.ibm.team.build.web.ui.internal.BuildRequest(args);
		if (this._content.domNode) {
			this.layout.setMainContent(this._content.domNode);
			this._leftNav._clearHighlighting();
		}
	},
	buildDeleted: function(itemId) {
		this._leftNav.itemDeleted(itemId);
	},
	getContentWidget: function() {
		return this._content;
	},
	showErrorPage: function(message) {
		// yield showing the error page to ensure the calling page has finished loading
		// so it will not be set as the content after the error
		setTimeout(dojo.hitch(this,function() {
			if (this._content)
				this._content.destroy();
			var error = new com.ibm.team.build.web.ui.internal.SimplePage({
				heading: messages.error,
				message: message
			});
			this.layout.setMainContent(error.domNode);
			PlatformUI.getWorkbench().hideMessageArea();
		}), 0);
	},
	destroy: function() {
		dojo.disconnect(this._scrollListener);
		this.inherited(arguments);
	}
	
});

})();
