/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.DashboardPage");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.Dashboard");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.AllDashboards");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.DashboardContainer");
dojo.require("com.ibm.team.dashboard.web.ui.internal.LeftNav");
dojo.require("com.ibm.team.dashboard.web.util.DashboardUtil");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("jazz.ui.SideNavigationLayout");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var AllDashboards = com.ibm.team.dashboard.web.ui.internal.AllDashboards;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var Dashboard = com.ibm.team.dashboard.web.ui.Dashboard;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardContainer = com.ibm.team.dashboard.web.ui.internal.DashboardContainer;
var DashboardUtil = com.ibm.team.dashboard.web.util.DashboardUtil;
var LeftNav = com.ibm.team.dashboard.web.ui.internal.LeftNav;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var SideNavigationLayout = jazz.ui.SideNavigationLayout;
var Util = com.ibm.team.dashboard.web.util.internal.Util;

var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

var messages;

/*
 * The Web UI page (vertical) containing the dashboard and side navigation bar.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.DashboardPage",
	[dijit._Widget, dijit._Templated], {
	
	templateString: "<div class=\"com-ibm-team-dashboard-web-ui-internal-DashboardPage\"></div>",
	
	constructor: function() {
		if (!messages) {
			messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		}
	},
	
	postCreate: function() {
		// create the overall layout
		this._layout = new SideNavigationLayout({});
		this._contentDiv = document.createElement("div");
		this._layout.setMainContent(this._contentDiv);
		this.domNode.appendChild(this._layout.domNode);
		
		// create the side nav content
		this._leftNav = new LeftNav({dashboardPage: this});
		
		// register actions
		var actionRegistry = PlatformUI.getWorkbench().getActionRegistry();
		actionRegistry.registerAction("com.ibm.team.dashboard.viewDashboard", this, "viewDashboard");
		actionRegistry.registerAction("com.ibm.team.dashboard.viewAllDashboards", this, "viewAllDashboards");
		actionRegistry.registerAction("com.ibm.team.dashboard.viewDefaultDashboard", this, "viewDefaultDashboard");
	},
	
	deleteDashboard: function(/* string */ itemId) {
		var serviceObj = {
			self: this,
			_itemId: itemId,
			postDelete: function() {
				// see if a dashboard is open
				if (this.self._contentWidget && this.self._contentWidget instanceof Dashboard) {
					var dashboard = this.self._contentWidget;
					if (dashboard._contentWidget && dashboard._contentWidget instanceof DashboardContainer) {
						var dashboardItem = dashboard._contentWidget.dashboardItem;
						var dashboardBundle = dashboard._contentWidget.dashboardBundle;
						// see if the deleted dashboard is open
						if (this._itemId == dashboardItem.itemId) {
							// send user to default dashboard
							this.self._dashboard.setCurrentDeleted();
							this.self._performDefaultRedirect();
						}
					}
				}
			},
			success: function(/* ResponseDTO */ responseDTO) {
				switch(responseDTO.status) {
					case "NOT_FOUND":
						alert(messages.deleteNoLongerExists);
						// fall through
					case "OK":
						this.self._leftNav._removeDashboardEntry(this._itemId);
						this.postDelete();
						break;
					case "ERROR":
						alert(responseDTO.msg);
						break;
				}
			},
			failure: function(errorObj) {
				console.error(Constants.LeftNav.REMOTE_ERROR + errorObj.message);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.deleteDashboard(itemId, srh);
	},
	
	destroy: function() {
		if (this._allDashboards) {
			this._allDashboards.destroy();
		}
		if (this._dashboard) {
			this._dashboard.destroy();
		}
		if (this._leftNav) {
			this._leftNav.destroy();
		}
		if (this._layout) {
			this._layout.destroy();
		}
		this.inherited(arguments);
	},
	
	navigateAwayEvent: function(event,workbenchNavigation) {
		if (this._dashboard) {
			return this._dashboard.navigateAwayEvent(event, workbenchNavigation);
		}
		return true;
	},
	
	viewAllDashboards: function(args) {
		if (this._dashboard) {
			this._dashboard._checkNavigateAwayHandle();
			this._dashboard.destroy();
			delete this._dashboard;
		}
		// only perform left nav highlighting if all dashboards is not already open
		if (this._contentWidget instanceof AllDashboards) {
			this._contentWidget.select(args.tab);
		}
		else {
			this._leftNav.updateHighlighting({viewAllDashboards: true});
			if (this._allDashboards) {
				this._allDashboards.destroy();
			}
			this._allDashboards = new AllDashboards({dashboardPage: this, tab: args.tab});
			this._allDashboards.init();
			this._setContent(this._allDashboards);
		}
		this._updateBrowserTitle();
		this._leftNav.allDashboardsVisited();
	},
	
	viewDashboard: function(args) {		
		this._leftNav.updateHighlighting(args);
		
		// prepare arguments
		var scope;
		var scopeItem;
		if (args.id) {
			scope = DashboardConstants.SCOPE_CONTRIBUTOR;
			scopeItem = {id: args.id};
		}
		else {
			if (args.team) {
				scope = DashboardConstants.SCOPE_TEAM_AREA;
				scopeItem = args.team.split("/");
			}
			else {
				scope = DashboardConstants.SCOPE_PROJECT_AREA;
			}
		}
			
		// check if already showing
		if (this._dashboard) {
			if (scope != this._dashboard.scope || dojo.toJson(scopeItem) != dojo.toJson(this._dashboard.scopeItem) || getCurrentProjectArea().name != this._dashboard.projectArea) {
				// different dashboard; destroy old one
				this._dashboard.destroy();
				delete this._dashboard;
			} else {
				// same dashboard - pass the current tab, if specified.
				this._dashboard.showTab(args.tab);
			}
		}
		
		// create new one if needed
		if (!this._dashboard) {
			this._dashboard = new Dashboard({
				scope: scope,
				scopeItem: scopeItem,
				projectArea: getCurrentProjectArea().name,
				tab: args.tab
			});
			this._dashboard.addListener(dojo.hitch(this, this._onLoad), "onload");
			this._dashboard.addListener(dojo.hitch(this, this._onDelete), "ondelete");
			this._dashboard.addListener(dojo.hitch(this, this._onTabChange), "tab");
			this._dashboard.addListener(dojo.hitch(this, this._onTitleChange), "title");
			this._dashboard.addListener(dojo.hitch(this, this._onSharedChange), "shared");
			this._setContent(this._dashboard);
		}

		if (this._title) {
			// "touch" the title so that it doesn't revert between navigations
			PlatformUI.getWorkbench().setTitle(this._title);
		}
	},
	
	viewDefaultDashboard: function(args) {
		if (this._dashboard || this._contentWidget instanceof AllDashboards) {
			// there's already a dashboard OR all dashboards is open; replace URL to match
			var serviceObj = {
				self: this,
				url: this._getCurrentHref(),
				callback: function() {
					this.self._replaceURL(this.url);
				}
			};
			// delay it a bit to give current URL a chance to fully process
			setTimeout(dojo.hitch(serviceObj, serviceObj.callback), 100);
		}
		else {
			// do a redirect to the default dashboard's actual URL
			this._performDefaultRedirect();
		}
	},
	
	/* void */ _performDefaultRedirect: function() {
		var serviceObj = {
			self: this,
			success: function(/* string */href){
				this.self._replaceURL(href);
			},
			failure: function(/* Error */error){
				// serious error - fall back to project dashboard
				this.self._replaceURL("#action=" + DashboardConstants.ACTION_VIEW_DASHBOARD);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.getDefaultDashboardHref(getCurrentProjectArea().name, srh);
	},
	
	/* string */ _getCurrentHref: function() {
		if (this._dashboard) {
			var scopeItem = this._dashboard.scopeItem;
			if (this._dashboard.scope == DashboardConstants.SCOPE_CONTRIBUTOR) {
				scopeItem = this._dashboard.scopeItem.id;
			}
			return DashboardUtil.getDashboardHref(this._dashboard.scope, scopeItem, this._dashboard.tab);
		} else if (this._contentWidget instanceof AllDashboards) {
			return Util.getAllDashboardsHref(this._contentWidget.tab);
		}
	},
	
	_onTitleChange: function() {
		this._leftNav.renameDashboard(this._dashboard.getDashboardItem());
		this._updateBrowserTitle();
	},
	
	_onSharedChange: function() {
		this._leftNav.updateSharedDashboard(this._dashboard.getDashboardItem());
	},
	
	_onLoad: function() {
		if (this._dashboard) {
			this._leftNav.dashboardOpened(this._dashboard._contentWidget);
			this._updateBrowserTitle();
			this._updateURL();
		}
	},
	
	_onDelete: function() {
		var dashboardItem = this._dashboard.getDashboardItem();
		this.deleteDashboard(dashboardItem.itemId);
	},
	
	_onTabChange: function(/* string */ propertyId, /* string */ tab) {
		this._updateBrowserTitle();
		this._updateURL();
	},
	
	_setContent: function(/* Widget */ widget) {
		if (this._contentDiv.firstChild != widget.domNode) {
			// swap in new widget
			if (this._contentDiv.firstChild) {
				this._contentDiv.replaceChild(widget.domNode, this._contentDiv.firstChild);
			}
			else {
				this._contentDiv.appendChild(widget.domNode);
			}
			this._contentWidget = widget;
		}
	},
	
	_updateBrowserTitle: function() {
		var title;
		var contentWidget = this._contentWidget;
		if (contentWidget && contentWidget instanceof Dashboard) {
			title = contentWidget.getTitle();
		}
		else if (contentWidget && contentWidget instanceof AllDashboards) {
			if (contentWidget.tab && contentWidget.tab == 1) {
				title = messages.allUsersTitle;
			}
			else {
				title = messages.allTeamAreasTitle;
			}
		}
		else {
			title = messages.dashboardsGenericTitle;
		}
		this._title = title;
		PlatformUI.getWorkbench().setTitle(title);
	},
	
	_updateURL: function() {
		if (this._dashboard && this._dashboard.getDashboardItem()) {
			location.href = this._getCurrentHref();
		}
	},
	_replaceURL: function(/* string */ url) {
		// IE uses an iframe to track history
		if (dojo.isIE) {
			// see net.jazz.ajax.core.internal.IEUriMonitor
			var uriMonitor = PlatformUI.getWorkbench()._uriMonitor;
			if (uriMonitor && uriMonitor.isTransitioning()) {
				setTimeout(dojo.hitch(this,this._replaceURL,url),100);
				return;
			}
			var iframe = window["net.jazz.ajax.iframe"];
			if (iframe) {
				var href = iframe.location.href;
				var index = href.indexOf('?');
				if (index != -1) {
					// main frame will detect and update itself
					iframe.location.replace(href.substring(0, index) + "?" + url.substring(1));
					return;
				}
			}
		}
		location.replace(url);
	}
});

})();
