/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportParamMultiSelect");

dojo.require("dojo.i18n");

dojo.require("jazz.ui.HoverPopup");

dojo.require("com.ibm.team.reports.web.ui.internal.ReportParam");
dojo.require("com.ibm.team.reports.web.ui.internal.utils.StringUtils");

dojo.require("jazz.ui.internal.SimpleToolbar");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() {
	
	var StringUtils = com.ibm.team.reports.web.ui.internal.utils.StringUtils;
	
    dojo.declare("com.ibm.team.reports.web.ui.internal.ReportParamMultiSelect", [com.ibm.team.reports.web.ui.internal.ReportParam], {
        templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ReportParamMultiSelect.html"),
		
		valuesMouseOver: null,
		valuesMouseOut: null,
		
		constructor: function() {
			this._numSelectedValues = 0;
			this._numValues = 0;
			this._addedReportParameterDTOs = [];
			this._liNodeLevels = [];
			this._dupCheckBoxes = [];
			
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
			
			this._localizedLabels = [];
			this._localizedLabels[this.statics.MY_TEAM_AREAS] = this.messages.myTeamAreas;
			this._localizedLabels[this.statics.CURRENT_ITERATION] = this.messages.currentIteration;
			this._localizedLabels[this.statics.CURRENT_CONTRIBUTOR] = this.messages.currentContributor;
		},
		
		postCreate: function() {
			this._toolbar = new jazz.ui.internal.SimpleToolbar({});
			this._toolbarDiv.appendChild(this._toolbar.domNode);

			this._toolbar.addButton(this, this._selectAll, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/selectall.gif"), label: this.messages.selectAll});
			this._toolbar.addButton(this, this._deselectAll, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/deselectall.gif"), label: this.messages.deSelectAll});
		},
		
		_selectAll: function() {
			this._mapCheckBoxes(dojo.hitch(this, function(checkBox) {
				if (!checkBox.checked && !checkBox.disabled && (!dojo.hasClass(checkBox._listItem, "display-none") || checkBox.isSpecial)) {
					checkBox.checked = true;
					this._onCheckBoxClick({target: checkBox, fireDirtyListener: true});
				}
			}));
			this._updateHeaderSelectedStats();
		},
		
		_deselectAll: function() {
			this._mapCheckBoxes(dojo.hitch(this, function(checkBox) {
				if (checkBox.checked && (!dojo.hasClass(checkBox._listItem, "display-none") || checkBox.isSpecial)) {
					checkBox.checked = false;
					this._onCheckBoxClick({target: checkBox, fireDirtyListener: true});
				}
			}));
			this._updateHeaderSelectedStats();
		},
		
		_handleSelectedMouseOver: function() {
			var tooltip = new jazz.ui.HoverPopup({
				around: this.spanSelected,
				createContents: dojo.hitch(this, this._populateTooltip)
			});
		},
		
		_populateTooltip: function(widget) {
			var strAry = [];
			this._mapSelectedCheckBoxes(function(checkBox) {
				strAry[checkBox.displayLabel] = 0;
			});
			var isFirst = true;
			var lblAry = [];
			for (lbl in strAry) {
				if (!isFirst) {
					lblAry.push("<br/>");
				} else {
					isFirst = false;
				}
				lblAry.push(lbl);
			}
			var div = document.createElement("div");
			div.innerHTML = lblAry.join("");
			widget.content = div;
		},
		
		/**
		 * Override
		 */
		init: function(reportParameterDTO) {
			// Remove the 'loading...' text.
			this._removeChildren(this.checkBoxList);
			this.inherited(arguments);
		},
		
		/**
		 * Override
		 */
		appendDefaultValues: function(reportParameterDTO) {
			if (!reportParameterDTO.labels) {
				return;
			}
			var labelsLen = reportParameterDTO.labels.length;
			for (var i = 0, j = this._defaultValues.length; i < labelsLen; i++) {
				var value = reportParameterDTO.values[i];
				var label = reportParameterDTO.labels[i];
				this._appendCheckBox(reportParameterDTO, value, label);
				this._defaultValues[j++] = [reportParameterDTO, value, label];
			}
			this._updateHeaderSelectedStats();
		},
		
		/**
		 * Override
		 */
		setDefaultValues: function() {
			var defaultValue = this.parameterDTO.defaultValue;
			if (defaultValue && defaultValue.length > 0) {
				var defaultCheckBoxId = this._uuid + "_" + defaultValue;
				var defaultCheckBox = dojo.byId(defaultCheckBoxId);
				if (defaultCheckBox) {
					if (!defaultCheckBox.checked) {
						defaultCheckBox.checked = true;
						this._onCheckBoxClick({
							target: defaultCheckBox,
							reportParameterDTO: this._reportParameterDTOs[defaultCheckBoxId],
							fireDirtyListener: false
						});
					}
					return true;
				}
			}
			return false;
		},
		
		setSelectedValues: function(values) {
			this._mapCheckBoxes(dojo.hitch(this, function(checkBox) {
				if (!checkBox.checked) {
					checkBox.checked = true;
				}
			}));
		},
		
		getSelectedValues: function() {
			var values = [];
			this._mapSelectedCheckBoxes(function(checkBox) {
				if (!checkBox.isDuplicate) {
					values.push(checkBox.paramValue);
				}
			});
			return values;
		},
		
		loadDefaults: function(reportParameterDTOs) {
			this.cascadeClear();
			if (!reportParameterDTOs) {
				var len = this.checkBoxList.childNodes.length;
				for (var i = 0; i < len; i++) {
					this._numValues++;
					var li = this.checkBoxList.childNodes[i];
					if (li.checkBoxChild) {
						dojo.removeClass(li, "display-none");
						this._displayGhostParents(li);
					}
				}
				dojo.removeClass(this.specialParamSpan, "display-none");
				this._updateHeaderSelectedStats();
				if (this.childReportParam) {
					this.childReportParam.loadDefaults();
				}
			} else {
				this.addValues(reportParameterDTOs);
			}
		},
		
		removeAllAndAdd: function(reportParameterDTOs) {
			this.cascadeClear();
			this.addValues(reportParameterDTOs);
		},
		
		cascadeClear: function() {
			this.clear();
			if (this.childReportParam) {
				this.childReportParam.cascadeClear();
			}
		},
		
		addValues: function(reportParameterDTOs) {
			for (var i = 0; i < reportParameterDTOs.length; i++) {
				var reportParameterDTO = reportParameterDTOs[i];
				if (reportParameterDTO) {
					this._addedReportParameterDTOs.push(reportParameterDTO);
					for (var j = 0; j < reportParameterDTO.values.length; ++j) {
						var value = reportParameterDTO.values[j];
						var cb = dojo.byId(this._uuid + "_" + value);
						if (cb) {
							if (cb.isSpecial) {
								if (dojo.hasClass(this.specialParamSpan, "display-none")) {
									dojo.removeClass(this.specialParamSpan, "display-none");
									this._numValues++;
								}
							} else if (dojo.hasClass(cb._listItem, "display-none")) {
								this._numValues++;
								cb._listItem.className = "";
								this._displayGhostParents(cb._listItem);
							}
						}
					}
					if (this.childReportParam) {
						this.childReportParam.addValues(reportParameterDTO.children);
					}
				}
			}
			this._updateHeaderSelectedStats();
		},
		
		_displayGhostParents: function(listItem) {
			var parentListItem = listItem.parentListItem;
			while (parentListItem) {
				if (dojo.hasClass(parentListItem, "display-none")) {
					dojo.removeClass(parentListItem, "display-none");
					if (!parentListItem.checkBoxChild.disabled) {
						this._numValues++;
					}
				}
				parentListItem = parentListItem.parentListItem;
			}
		},
		
		setValues: function(values) {
			values = StringUtils.splitValues(values);
			this._mapCheckBoxes(dojo.hitch(this, function(checkBox) {
				if (!checkBox.checked) {
					for (var i = 0; i < values.length; i++) {
						if (checkBox.paramValue == values[i]) {
							checkBox.checked = true;
							this._onCheckBoxClick({
								target: checkBox,
								fireDirtyListener: false
							});
							// Check duplicates
							this._mapCheckBoxDuplicates(checkBox, dojo.hitch(this, function(dupCheckBox) {
								dupCheckBox.checked = true;
								this._onCheckBoxClick({
									target: dupCheckBox,
									fireDirtyListener: false
								});
							}));				 
						}
					}
				}
			}));
		},
		
		getSelectedLabels: function() {
			var labels = [];
			this._mapSelectedCheckBoxes(dojo.hitch(this, function(checkBox) {
				labels.push(checkBox.displayLabel);
			}));
			return labels;
		},
		
		clear: function() {
			this._numSelectedValues = 0;
			this._numValues = 0;
			this._addedReportParameterDTOs = [];
			this._hideAll();
			dojo.addClass(this.specialParamSpan, "display-none");
		},
		
		_handleValuesMouseOver: function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			if (this.valuesMouseOver) {
				this.valuesMouseOver(this._valuesSection);
			}
		},
		
		_handleValuesMouseOut: function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			if (this.valuesMouseOut) {
				this.valuesMouseOut(this._valuesSection);
			}
		},
		
		_hideAll: function() {
			this._mapCheckBoxes(dojo.hitch(this, function(checkBox) {
				var li = checkBox._listItem;
				if (li) {
					li.className = "display-none";
				}
				checkBox.checked = false;
			}));
		},
		
		_updateHeaderSelectedStats: function () {
			this.spanSelected.innerHTML = dojo.string.substitute(this.messages.numSelected, [this._numSelectedValues, this._numValues]);
		},
		
		_mapCheckBoxes: function(func) {
			if (this._specialCheckBox) {
				func(this._specialCheckBox);
			}
			var listItem = this.checkBoxList.firstChild;
			while (listItem) {
				var checkBox = listItem.checkBoxChild;
				if (checkBox) {
					func(checkBox);
				}
				listItem = listItem.nextSibling;
			}
		},
		
		_mapCheckBoxDuplicates: function(checkBox, func) {
			if (checkBox.duplicateNum) {
				for (var i in this._dupCheckBoxes[checkBox.id]) {
					var dupCheckBox = this._dupCheckBoxes[checkBox.id][i];
					if (dupCheckBox) {
						func(dupCheckBox);
					}
				}
			}
		},
		
		_mapSelectedCheckBoxes: function(func) {
			var self = this;
			this._mapCheckBoxes(function(checkBox) {
				if (checkBox.checked) {
					func(checkBox);
					if (checkBox.duplicateNum) {
						self._mapCheckBoxDuplicates(checkBox, function(dupCheckBox) {
							func(dupCheckBox);
						});
					}
				}
			});
		},
		
		_onCheckBoxClick: function(event) {
			var checkBoxClicked = event ? event.target : window.event ? window.event.srcElement : null;
			var fireDirtyListener = event ? (event.fireDirtyListener == undefined ? true : event.fireDirtyListener) : true;
			var inc = 0;
			if (dojo.hasClass(checkBoxClicked._listItem, "selected")) {
				inc = -1;
				checkBoxClicked._listItem.className = "";
				checkBoxClicked.checked = false;
			} else {
				inc = 1;
				checkBoxClicked._listItem.className = "selected";
				checkBoxClicked.checked = true;
			}
			if (this.childReportParam) {
				var checkBoxesClicked = [];
				this._mapSelectedCheckBoxes(dojo.hitch(this, function(checkBox){
					var dto = this._reportParameterDTOs[checkBox.id];
					if (this.isRoot || this._addedReportParameterDTOs.length == 0) {
						checkBoxesClicked.push(checkBox);
					} else {
						if (event && event.reportParameterDTO != undefined) {
							checkBoxesClicked.push(checkBoxClicked);
						} else {
							for (var i in this._addedReportParameterDTOs) {
								if (dto == this._addedReportParameterDTOs[i]) {
									checkBoxesClicked.push(checkBox);
									break;
								}
							}
						}
					}
				}));
				var reportParameterDTOsToAdd = [];
				for (var i = 0; i < checkBoxesClicked.length; i++) {
					var currCheckBox = checkBoxesClicked[i];
					var selectedReportParameterDTO = this._reportParameterDTOs[currCheckBox.id];
					var values = selectedReportParameterDTO.values;
					for (var j = 0; j < values.length; ++j) {
						if (currCheckBox.paramValue == "'" + values[j] + "'") {
							reportParameterDTOsToAdd.push(selectedReportParameterDTO.children[j]);
						}
					}
				}
				
				this.childReportParam.cascadeClear();
				if (reportParameterDTOsToAdd.length == 0) {
					this._setAllOrNoneSelections();
				} else {
					this.childReportParam.addValues(reportParameterDTOsToAdd);
				}
			}
			if (checkBoxClicked.isDuplicate == undefined && 
				((inc == 1 && this._numSelectedValues < this._numValues) || 
				 (inc == -1 && this._numSelectedValues > 0))) {
				this._numSelectedValues += inc;
				this._updateHeaderSelectedStats();
			}
			if (fireDirtyListener == true) {
				this.onDirty();
			}
		},
		
		_appendCheckBox: function (reportParameterDTO, value, label) {
			var paramName = reportParameterDTO.parameterName;
			var checkBoxId = this._uuid + "_" + value;
			var isHidden = false;
			
			var isSpecial = false
			if (!this._specialCheckBox &&
				(value == this.statics.MY_TEAM_AREAS || 
				 value == this.statics.CURRENT_ITERATION ||
				 value == this.statics.CURRENT_CONTRIBUTOR)) {
				isSpecial = true;				
			}
			
			if (!isSpecial) {
				var sortBy = this.parameterDTO.sort;
				if ((reportParameterDTO.flags & this.statics.HIERARCHICAL) == this.statics.HIERARCHICAL) {
					if (!StringUtils.startsWith(label, "/")) {
						label = "/" + label;
					}
					var labelClean = label.substring(1);
					var parts = labelClean.split("/");
					var partsLen = parts.length;
					
					var parentListItem = null;
					var nodePath = "/"+parts[0];
					var i = 1;
					do {
						var listItem = dojo.byId(this._uuid + "_" + nodePath + "_Li");
						if (listItem == null) {
							listItem = this._createListItem(label == nodePath ? value : nodePath, parts[i-1], nodePath, parentListItem, i-1, (i-1)*15, nodePath == label);
							listItem.parentListItem = parentListItem;
							this._reportParameterDTOs[listItem.checkBoxChild.id] = reportParameterDTO;
							isHidden = listItem.checkBoxChild.isDuplicate;
							if (!isHidden) {
								if (sortBy == this.statics.NO_SORT) {
									this.checkBoxList.appendChild(listItem);
								}
								else {
									var listItems = !parentListItem ? this._rootListItems : parentListItem._childListItems;
									listItems = this._insertListItem(listItem, parentListItem, listItems, sortBy, true);
									if (!parentListItem) {
										this._rootListItems = listItems.slice();
									} else {
										parentListItem._childListItems = listItems.slice();
									}
								}
							}
						} else if (label == nodePath) {
							// We encountered a child for this parent in a previous call, must enable parent
							listItem.checkBoxChild.disabled = false;
							listItem.labelChild.style.color = "#000000";
							listItem.onclick = dojo.hitch(this, function() {
							this._onCheckBoxClick({target: listItem.checkBoxChild, fireDirtyListener: false});});
							listItem.onmouseover = function() {
								dojo.addClass(listItem, "highlight-multiselect");
							};
							listItem.onmouseout = function() {
								dojo.removeClass(listItem, "highlight-multiselect");
							};
						}
						if (i == partsLen) {
							break;
						}
						nodePath += "/" + parts[i];
						i++;
						parentListItem = listItem;
					} while (true);
				} else {
					var listItem = this._createListItem(value, label, null, null, 0, 0, true);
					this._reportParameterDTOs[listItem.checkBoxChild.id] = reportParameterDTO;
					isHidden = listItem.checkBoxChild.isDuplicate;
					if (!isHidden) {
						if (sortBy != 0) {
							this._insertListItem(listItem, null, this.checkBoxList.childNodes, sortBy, false);
						}
						else {
							this.checkBoxList.appendChild(listItem);
						}
					}
				}
			} else if (!this._specialCheckBox) {
				this._specialCheckBox = this._createSelectOption(value);
				this._specialCheckBox.displayLabel = this._localizedLabels[value];
				this._specialCheckBox.isSpecial = true;
				this._specialCheckBox.className = "";
				this.specialParamSpan.appendChild(this._specialCheckBox);	
				this.specialParamSpan.appendChild(document.createTextNode(this._specialCheckBox.displayLabel));
				this._reportParameterDTOs[this._specialCheckBox.id] = reportParameterDTO;
			}
			if (!isHidden) {
				this._numValues++;
			}
		},
		
		_createSelectOption: function(value, displayLabel) {
			var checkBox = null;
			if (displayLabel) {
				// Inner select option
				checkBox = document.createElement("div");
			} else {
				// Outer special select option - such as 'Current Iteration'
				checkBox = document.createElement("input");
				checkBox.onclick = dojo.hitch(this, this._onCheckBoxClick);
				checkBox._listItem = document.createElement("div"); // placeholder
			}
			checkBox.type = "checkbox";
			checkBox.paramValue = "'" + value + "'";
			checkBox.displayLabel = displayLabel;
			var checkBoxId = this._uuid + "_" + value;
			var currCheckBox = dojo.byId(checkBoxId);
			if (currCheckBox) {
				if (!this._dupCheckBoxes[checkBoxId]) {
					this._dupCheckBoxes[checkBoxId] = [];
				}
				this._dupCheckBoxes[checkBoxId].push(checkBox);
				if (!currCheckBox.duplicateNum) {
					currCheckBox.duplicateNum = 0;
				}
				currCheckBox.duplicateNum++;
				checkBoxId = checkBoxId + "_" + currCheckBox.duplicateNum;
				checkBox.isDuplicate = true;
			}
			checkBox.id = checkBoxId;
			return checkBox;	
		},
		
		_createListItem: function(value, displayLabel, nodePath, parentListItem, nodeLevel, paddingLeft, isEnabled) {
			var checkBox = this._createSelectOption(value, displayLabel);
			var li = document.createElement("li");
			checkBox._listItem = li;
			li.id = this._uuid + "_" + nodePath + "_Li";
			li.checkBoxChild = checkBox;
			li.paramValue = value;
			li.nodePath = nodePath;
			li.displayLabel = displayLabel;
			li.nodeLevel = nodeLevel;
			li.parentListItem = parentListItem;
			if (checkBox.isDuplicate) {
				li.className =  "display-none";
			}
			li.style.width = "100%";
			var labelElem = dojo.doc.createElement("label");
			li.labelChild = labelElem;
			li.appendChild(labelElem);
			labelElem.style.paddingLeft = paddingLeft + "px";
			if (!isEnabled) {
				checkBox.disabled = true;
				labelElem.style.color = "#999999";
			} else {
				li.onclick = dojo.hitch(this, function() {
					this._onCheckBoxClick({target: checkBox, fireDirtyListener: true});
				});
				li.onmouseover = function() {
					dojo.addClass(li, "highlight-multiselect");
				};
				li.onmouseout = function() {
					dojo.removeClass(li, "highlight-multiselect");
				};
			}
			labelElem.appendChild(checkBox);
			labelElem.appendChild(document.createTextNode(displayLabel));
			return li;
		},
		
		_insertListItem: function(newListItem, parentListItem, listItems, sortBy, isHierarchy) {
			if (!listItems || listItems.length == 0) {
				// Just insert into parent, none to compare against
				if (parentListItem != null) {
					parentListItem.parentNode.insertBefore(newListItem, parentListItem.nextSibling);
				} else {
					this.checkBoxList.appendChild(newListItem);
				}
				if (isHierarchy) {
					if (!listItems) {
						listItems = [];
					}
					listItems.push(newListItem);
				}
			} else {
				var lastSibling = null;
				var low = 0;
				var high = listItems.length;
				var mid = 0;
				while(low < high) {
					mid = Math.floor((low + high) / 2);
					var li = listItems[mid];
					switch (sortBy) {
						case this.statics.SORT_BY_VALUE_ASC:
							if (newListItem.paramValue.localeCompare(li.paramValue) < 0) {
								high = mid;
							} else {
								low = mid + 1;
							}
							break;
						case this.statics.SORT_BY_VALUE_DESC:
							if (newListItem.paramValue.localeCompare(li.paramValue) > 0) {
								high = mid;
							} else {
								low = mid + 1;
							}
							break;
						case this.statics.SORT_BY_LABEL_ASC:
							if (newListItem.displayLabel.localeCompare(li.displayLabel) < 0) {
								high = mid;
							} else {
								low = mid + 1;
							}
							break;
						case this.statics.SORT_BY_LABEL_DESC:
							if (newListItem.displayLabel.localeCompare(li.displayLabel) > 0) {
								high = mid;
							} else {
								low = mid + 1;
							}
							break;
					}
					lastSibling = li;
				}
				if (low == 0) {
					this.checkBoxList.insertBefore(newListItem, lastSibling);
				} else {
					var li = listItems[low - 1];
					if (li.nextSibling && li.nextSibling.nodeLevel == newListItem.nodeLevel) {
						this.checkBoxList.insertBefore(newListItem, li.nextSibling);
					} else {
						li = li.nextSibling;
						while (li && li.nodeLevel > newListItem.nodeLevel) {
							li = li.nextSibling;
						}
						this.checkBoxList.insertBefore(newListItem, li);
					}
				}
				if (isHierarchy) {
					listItems.splice(low,0, newListItem);
				}
			}
			return listItems;
		},
		
		_setAllOrNoneSelections: function() {
			if (this.childReportParam) {
				this.childReportParam.cascadeClear();
				var dtos = null;
				if (this.isRoot || this._addedReportParameterDTOs.length == 0) {
					dtos = this._reportParameterDTOs;
				}
				else {
					dtos = this._addedReportParameterDTOs;
				}
				var reportParameterDTOsToAdd = [];
				for (var k in dtos) {
					var dto = dtos[k];
					if (!dto.children) {
						continue;
					}
					var vals = dto.values;
					for (var j = 0; j < vals.length; ++j) {
						reportParameterDTOsToAdd.push(dto.children[j]);
					}
				}
				this.childReportParam.addValues(reportParameterDTOsToAdd);
			}
		},
		
		remoteError: function(error) {
			alert(error.message);
		}
		
	});
	
})();
