/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.viewlets.web.ui.internal.WorkItemsViewlet");

dojo.require("com.ibm.team.dashboard.web.ui.PagedTableViewlet");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.Observable");
dojo.require("com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryPreferenceProvider");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.viewlets.web.ui.internal.QueryClient2");
dojo.require("com.ibm.team.workitem.web.client.util");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.process.web.client.projectarea");

dojo.require("com.ibm.team.workitem.web.client.internal.Constants");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemAttributes");
dojo.require("com.ibm.team.workitem.viewlets.web.ui.internal.utils.UUIDUtility");

dojo.require("dojo.date");
dojo.require("dojo.date.locale");

dojo.require("jazz.app.ResourceLink");
dojo.require("jazz.app.proxy");
dojo.require("jazz.util.format");
dojo.require("jazz.util.html");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.requireLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "WorkItemsMessages");

(function() {
	
var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;

var PagedTableViewlet = com.ibm.team.dashboard.web.ui.PagedTableViewlet;
var WorkItemQueryPreferenceProvider = com.ibm.team.workitem.viewlets.web.ui.internal.utils.WorkItemQueryPreferenceProvider;

var Constants = com.ibm.team.workitem.web.client.internal.Constants;
var Observable = com.ibm.team.dashboard.web.ui.internal.util.Observable;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var QueryClient2 = com.ibm.team.workitem.viewlets.web.ui.internal.QueryClient2;
var WorkItemUtil = com.ibm.team.workitem.web.client.util;
var FormatUtil = jazz.util.format;

// unused for now due to null check bug, see work item 86674
//var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

var WorkItemAttributes = com.ibm.team.workitem.web.client.internal.WorkItemAttributes;
var UUIDUtility = com.ibm.team.workitem.viewlets.web.ui.internal.utils.UUIDUtility;

var IMG_PATH = dojo.moduleUrl("com.ibm.team.workitem.viewlets.web","graphics/work-items/images/");
var IMG_URL_CLOSED_OVERLAY = IMG_PATH + "wi_complete_ovr.gif";
var IMG_URL_IN_PROGRESS_OVERLAY = IMG_PATH + "wi_in_progress_ovr.gif";

var messages = dojo.i18n.getLocalization("com.ibm.team.workitem.viewlets.web.ui.internal", "WorkItemsMessages");

/*
 * (static) Observables to listen for viewlet log-in, to allow viewlets with C/ALM
 * queries to the same server to load as soon as one logs in.
 */
var _loginTrackersByURL;

dojo.declare("com.ibm.team.workitem.viewlets.web.ui.internal.WorkItemsViewlet",
	PagedTableViewlet, {
	
	contentClass: "com-ibm-team-workitem-viewlets-web-ui-internal-WorkItemsViewlet",
	
	init: function() {
		this.inherited(arguments);
		this._skipOAuth = true;
		// if the viewlet has a default query and it has not been changed, initialize the title and preference provider
		var defaultQuery = this.getPreference("defaultQuery");
		if (defaultQuery != -1 && this.getPreference("query") == -1) {
			var serviceObject = {
				self: this,
				_successCallback: function(/*array*/ queryDTO) {
					if (!this.self.isDestroyed() && queryDTO) {
						this.self._defaultQueryItemId = queryDTO.itemId;
						this.self.getSite().setTitle(queryDTO.name);
						this.self.getPreferenceProvider("query").initDefault(queryDTO);
					}
				},
				_errorCallback: function(error) {
					this.self.getSite().error(error);
				}
			};
			var responseHandler = new ServiceResponseHandler(serviceObject, "_successCallback", "_errorCallback");
			var args = {
				includeCount: false,
				id: defaultQuery
			};
			QueryClient2.getQuery2(responseHandler, args);
		}
		
		this._serverRoot = null;
		this._imageServerRoot = null;

		if (dojo.getObject("com.ibm.team.process.web.projectArea")) {
			this._local = true;
		} else {
			this._local = false;
		}		
		
	},
	
	_getIconURL: function(url) {		
		if (this._local) {
			// Local viewlet
			return url;
		}
		// Remote viewlet
		return this._getImageServerRoot() + url;
	},	
	
	_getServerRoot: function() {
		
		if (typeof(this._serverRoot) === "string") {
			return this._serverRoot;
		}
		
		// e.g. Data server root:
		// Local:   undefined
		// Remote:  https://127.0.0.1:9444/jazz
		var root = dojo.getObject("net.jazz.ajax._dataServerRoot");
		if (!root) {
			// e.g. Context root:
			// Local:   /jazz
			// Remote:  https://127.0.0.1:9444/jazz 
			root = dojo.getObject("net.jazz.ajax._contextRoot");
		}
		if (!root) {
			root = "";
		}
		this._serverRoot = root;
		
		return this._serverRoot;
	},

	_getImageServerRoot: function() {
		
		if (typeof(this._imageServerRoot) === "string") {
			return this._imageServerRoot;
		}		
		
		var serverRoot = this._getServerRoot();
		if (!serverRoot) {
			this._imageServerRoot = "";
			return this._imageServerRoot;
		}
		
		var root;
		var url = new dojo._Url(serverRoot);
		// Remove the path segments (e.g. /jazz)
		if (url.scheme && 
			url.host) {
			// e.g. URL
			// Remote: https://127.0.0.1:9444/jazz 			
			root = url.scheme + 
					"://" + 
					url.host;
			if (url.port) {
				root = root + ":" + url.port;
			}
			// Create a proxy url in order to embed images from another 
			// domain in the current domain.
			// Absolute links on their own work; however, security related
			// issues (e.g. firewall, invalid certificates, etc.) prevent the 
			// images from resolving in test environments.
			root = jazz.app.proxy.encodeProxiedURI(root);
		} else {
			// e.g. URL 
			// Local: /jazz
			root = "";
		}

		// e.g. Image server root
		// Local:   ""
		// Remote:  https://127.0.0.1:9444
		this._imageServerRoot = root;
		
		return this._imageServerRoot;
	},
	
	getResultCount: function() {
		if (this.getPreference("query") == -1 && this.getPreference("defaultQuery") == -1)
			return 0;
		return (this._totalResults || this._totalResults == 0) ? this._totalResults : -1;
	},
	
	getNoResultsMessage: function() {
		if (this._oauthURL)
			return this._getLoginMessage();
		if (this._deletedQuery)
			return this._getDeletedQueryMessage();
		if (this._unknownError)
			return this._getUnknownErrorMessage();
		if (this._customError)
			return this._getCustomErrorMessage(this._customError);
		if (this.getPreference("query") != -1 || this.getPreference("defaultQuery") != -1)
			return document.createTextNode(messages.noResults);
		return this._getNoQueryMessage();
	},
	
	settingsAreaToggled: function(opened) {
		this._isSettingsAreaOpen = opened;
		this.updateEditMessages();
	},
	
	setDashboardEditing: function(editing) {
		this.updateEditMessages();
	},
	
	updateEditMessages: function() {
		if (this._noQueryEditMessage)
			this._noQueryEditMessage.className = this.getSite().isDashboardEditing() && !this._isSettingsAreaOpen ? "" : "hidden";
		if (this._deletedQueryEditMessage)
			this._deletedQueryEditMessage.className = this.getSite().isDashboardEditing() && !this._isSettingsAreaOpen ? "" : "hidden";
	},
	
	_getUnknownErrorMessage: function() {
		if (!this._unknownErrorMessage) {
			this._unknownErrorMessage = document.createElement("span");
			this._unknownErrorMessage.innerHTML = messages.unknownError + "&nbsp; ";
			var refreshLink = this._createRefreshLink();
			this._unknownErrorMessage.appendChild(refreshLink);
		}
		return this._unknownErrorMessage;
	},
	
	_getCustomErrorMessage: function(error) {
		if (!this._customErrorMessage || this._lastCustomError !== error.message) {
			var span = document.createElement("span");
			span.innerHTML = jazz.util.html.escape(error.message) + "&nbsp; ";
			var refreshLink = this._createRefreshLink();
			span.appendChild(refreshLink);
			this._customErrorMessage = span;
			this._lastCustomError = error.message;
		}
		return this._customErrorMessage;
	},
	
	_createRefreshLink: function() {
		var refreshLink = document.createElement("a");
		refreshLink.href = "#";
		refreshLink.innerHTML = messages.refresh;
		refreshLink.title = messages.refresh;
		this.connect(refreshLink,"onclick",function(e) {
			e.preventDefault();
			this.getSite().refresh();
		});
		return refreshLink;
	},
	
	_getDeletedQueryMessage: function() {
		if (!this._deletedQueryMessage) {
			this._deletedQueryMessage = document.createElement("span");
			this._deletedQueryMessage.appendChild(document.createTextNode(messages.deletedQuery));
			this._deletedQueryEditMessage = this._createEditSettingsMessage(true);
			this.updateEditMessages();
			this._deletedQueryMessage.appendChild(this._deletedQueryEditMessage);
		}
		
		return this._deletedQueryMessage;
	},
	
	_getLoginMessage: function() {
		if (!this._loginMessage) {
			// show login required icon
			this.getSite().setIcon(dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/content/icons/login.gif"));
			this.getSite().setSecondaryTitle(messages.viewletNotLoggedIn);
			
			// listen for log-ins for this origin
			this._addLoginTracker();
			
			// show log-in section
			this._loginMessage = document.createElement("div");
			this._loginMessage.className = "login";
			this._loginMessage.style.margin = "1px"; // override default margins
			this._loginMessage.innerHTML = "<div class=\"message\">" + dojo.string.substitute(messages.viewletLogin, [this._oauthURL.host]) + "</div>";
			
			// add log-in link
			var linkDiv = document.createElement("div");
			linkDiv.className = "link";
			var a = document.createElement("a");
			a.href = "#";
			a.appendChild(document.createTextNode(messages.viewletLogin2));
			this.connect(a, "onclick", dojo.hitch(this, this._onLoginClick));
			linkDiv.appendChild(a);		
			this._loginMessage.appendChild(linkDiv);
		}
		return this._loginMessage;
	},
	
	_addLoginTracker: function() {
		if (!_loginTrackersByURL) {
			_loginTrackersByURL = new Object();
		}
		var url = this._oauthURL.scheme + "://" + this._oauthURL.host + (this._oauthURL.port ? ":" + this._oauthURL.port : "");
		this._loginTracker = _loginTrackersByURL[url];
		if (!this._loginTracker) {
			this._loginTracker = _loginTrackersByURL[url] = new Observable();
		}
		this._loginCallback = dojo.hitch(this, function() {
			// Workaround for 85382: Observable is not reentrant
			setTimeout(dojo.hitch(this, function() {
				if (!this.isDestroyed()) {
					// reset icon and title back to original
					this.getSite().setIcon(dojo.moduleUrl("com.ibm.team.workitem.viewlets.web", "graphics/work-items/icons/work-items.gif"));
					this.getSite().setSecondaryTitle("");
					this._removeLoginTracker();
					this.refresh();
				}
			}), 0);
		});
		this._loginTracker.addListener(this._loginCallback, "login");
	},
	
	_removeLoginTracker: function() {
		if (this._loginCallback && this._loginTracker) {
			this._loginTracker.removeListener(this._loginCallback, "login");
			delete this._loginCallback;
			delete this._loginTracker;
		}
	},
	
	_onLoginClick: function(/* Event */ event) {
		if (event) {
			dojo.stopEvent(event);
		}
		// reset icon and title back to original
		this.getSite().setIcon(dojo.moduleUrl("com.ibm.team.workitem.viewlets.web", "graphics/work-items/icons/work-items.gif"));
		this.getSite().setSecondaryTitle("");
		// fetch with oauth this time
		this._skipOAuth = false;
		this.refresh();
	},
	
	_getNoQueryMessage: function() {
		if (!this._noQueryMessage) {
			this._noQueryMessage = document.createElement("span");
			this._noQueryMessage.appendChild(document.createTextNode(messages.noQuery));
			this._noQueryEditMessage = this._createEditSettingsMessage(false);
			this.updateEditMessages();
			this._noQueryMessage.appendChild(this._noQueryEditMessage);
		}
		return this._noQueryMessage;
	},
	
	_createEditSettingsMessage: function(/* boolean */ isDeleted) {
		var result = document.createElement("span");
		var links = I18nUtil.addMessage(result, isDeleted ? messages.selectNewQuery : messages.selectQuery);
		this.connect(links[0], "onclick", this._doPreferences);
		return result;
	},
	
	_doPreferences: function(event) {
		dojo.stopEvent(event);
		
		this.getSite().toggleSettings();
	},
	
	getItemsPerPage: function() {
		return this.getPreference("items");
	},
	
	refresh: function() {
		if (this._detailsWidgets) {
			for (var i in this._detailsWidgets)
				this._detailsWidgets[i].destroy();
		}
		this._detailsWidgets = new Object();
		delete this._totalResults;
		delete this._token;
		delete this._deletedQuery;
		delete this._oauthURL;
		delete this._unknownError;
		delete this._customError
		this.inherited(arguments);
		// update the title on refresh in case the query was renamed
		this.getPreferenceProvider("query").getQueryName(this.getPreference("query"), dojo.hitch(this.getSite(),this.getSite().setTitle));
	},
	
	update: function() {
		if (dojo.isIE) {
			if (this._iframePollTimeouts) {
				this._clearTimeouts();
			}
			this._iframePollTimeouts = {};
		}
		this.inherited(arguments);
	},
	
	_totalResultsChanged: function() {
		this.updateTotalPageFooter();
		this.getSite().setSecondaryTitle("(" + this._totalResults + ")");
	},
	
	fetch: function(srh, start, count, entries, details) {
		var responseHandler;
		var serviceObject = {
			self: this,
			_onResponse: function() {
				// notify any login-waiting viewlets to load
				var tracker = this.self._loginTracker;
				if (tracker) {
					this.self._removeLoginTracker();
					tracker.propertyChanged("login");
				}
			},
			success: function(resultSet){
				if (!this.self.isDestroyed()) {
					this._onResponse();
					if (resultSet && resultSet.totalCount > 0) {
						this.self._resultHeaderIndex = {};
						this.self._resultHeaders = resultSet.headers;
						for (var i = 0; i < resultSet.headers.length; i++) 
							this.self._resultHeaderIndex[resultSet.headers[i].attributeId] = i;
						// only update the total result count if we didn't have a token or if the token is different (e.g. a new DB query was run)
						// this is to work around a bug in the service, see work items 66633 and 70982
						if ((!this.self.token || this.self._token != resultSet.token) && this.self._totalResults != resultSet.totalCount) {
							this.self._totalResults = resultSet.totalCount;
							this.self._totalResultsChanged();
						}
						this.self._token = resultSet.token;
						if (resultSet.rows) {
							var dtos = resultSet.rows;
							var result = new Object();
							if (entries) 
								result.entries = dtos;
							if (details)
								result.details = dtos;
							srh.doCallback(result);
						}
					}
					else {
						this.self._totalResults = 0;
						this.self._totalResultsChanged();
						srh.doCallback();
					}
				}
			},
			failure: function(error){
				// C/ALM cross-server queries require oauth login
				var oauthHeader;
				if (responseHandler.xhr) {
					oauthHeader = responseHandler.xhr.getResponseHeader("X-jazz-web-oauth-url");
				}
				if (oauthHeader) {
					this.self._oauthURL = new dojo._Url(oauthHeader);
					this.self._totalResults = 0;
					this.self.updateTotalPageFooter();
					this.self.getSite().setSecondaryTitle("");
					srh.doCallback();
				}
				else {
					this._onResponse();
					// 404 means query from settings area does not exist
					// 400 means some other issue with the query rest service
					if (error.status == 404 || error.status == 400 || error.status == 500) {
						if (error.status == 500 || (error.status == 400 && !error.message))
							this.self._unknownError = true;
						else if (error.status == 400)
							this.self._customError = error;
						else
							this.self._deletedQuery = true;
						this.self._totalResults = 0;
						this.self.updateTotalPageFooter();
						this.self.getSite().setSecondaryTitle("");
						srh.doCallback();
					}
					else {
						srh.doErrorCallback(error);
					}
				}
			}
		};
		responseHandler = new ServiceResponseHandler(serviceObject, "success", "failure");
		var items = this.getPreference("items");
		var args = {
			startIndex: start,
			maxResults: count,
			projectAreaItemId: com.ibm.team.process.web.projectArea ? com.ibm.team.process.web.projectArea.itemId : true,
			columnIdentifiers: [WorkItemAttributes.TYPE, WorkItemAttributes.SUMMARY, WorkItemAttributes.STATE,
				WorkItemAttributes.CREATOR, WorkItemAttributes.OWNER, WorkItemAttributes.SEVERITY, WorkItemAttributes.PRIORITY,
				WorkItemAttributes.CREATION_DATE, WorkItemAttributes.RESOLUTION, WorkItemAttributes.ESTIMATE,
				WorkItemAttributes.CATEGORY, WorkItemAttributes.TARGET, WorkItemAttributes.PROJECT_AREA, WorkItemAttributes.TAGS]
		};
		var queryItemId = this.getPreference("query");
		if (queryItemId == -1) {
			var queryStringId = this.getPreference("defaultQuery");
			if (queryStringId == -1) {
				srh.doCallback();
				return;
			}
			args.id = queryStringId;
		} else {
			args.itemId = queryItemId;
		}
		if (this._token)
			args.token = this._token;
		args.skipOAuth = this._skipOAuth;
		QueryClient2.getResultSet(null, args, responseHandler);
	},
	
	/* Node[] */ renderEntry: function(/* object */ resultDTO, /* object */ resultDetailsDTO, /* boolean */ zoomed) {
		var columns = new Array();
		
		var titleColSpan = 2;
		
		var iconCell = document.createElement("td");
		var iconContainer = document.createElement("div");
		iconContainer.className = "iconContainer";
		
		iconCell.className = "iconCell";
		var icon = document.createElement("img");
		icon.width = 16;
		icon.height = 16;
		icon.style.marginRight = "0.2em";
		icon.src = this._getIconURL(resultDTO.iconUrls[this._resultHeaderIndex[WorkItemAttributes.TYPE]]);
		iconContainer.appendChild(icon);
		
		// add overlay if available/needed
		if (resultDTO.stateGroup) {
			var src;
			switch (resultDTO.stateGroup) {
				case Constants.CLOSED_STATES:	
					src = IMG_URL_CLOSED_OVERLAY;
					dojo.addClass(icon, "closed");
					break;
				case Constants.IN_PROGRESS_STATES:	
					src = IMG_URL_IN_PROGRESS_OVERLAY;
					break;
			}
			if (src) {
				var overlayIcon = document.createElement("img");
				overlayIcon.className = "overlayIcon";
				overlayIcon.src = src;
				iconContainer.appendChild(overlayIcon);
			}
		}
		iconCell.appendChild(iconContainer);
		columns.push(iconCell);
		
		if (this.getPreference("secondIcon") != "none") {
			titleColSpan = 1;
			var secondIconCell = document.createElement("td");
			secondIconCell.className = "iconCell";
			var secondIcon = document.createElement("img");
			secondIcon.width = 16;
			secondIcon.height = 16;
			secondIcon.style.marginRight = "0.2em";
			var secondIconSrc;
			var secondIconName;
			if (this.getPreference("secondIcon") == "priority"){
				secondIconSrc = resultDTO.iconUrls[this._resultHeaderIndex[WorkItemAttributes.PRIORITY]];
				secondIconName = dojo.string.substitute(messages.priority,[resultDTO.labels[this._resultHeaderIndex[WorkItemAttributes.PRIORITY]]]);
			}
			else if (this.getPreference("secondIcon") == "severity") {
				secondIconSrc = resultDTO.iconUrls[this._resultHeaderIndex[WorkItemAttributes.SEVERITY]];
				secondIconName = dojo.string.substitute(messages.severity,[resultDTO.labels[this._resultHeaderIndex[WorkItemAttributes.SEVERITY]]]);
			}
			else if (this.getPreference("secondIcon") == "status") {
				secondIconSrc = resultDTO.iconUrls[this._resultHeaderIndex[WorkItemAttributes.STATE]];
				secondIconName = dojo.string.substitute(messages.status,[resultDTO.labels[this._resultHeaderIndex[WorkItemAttributes.STATE]]]);
			}
			else if (this.getPreference("secondIcon") == "resolution") {
				secondIconSrc = resultDTO.iconUrls[this._resultHeaderIndex[WorkItemAttributes.RESOLUTION]];
				secondIconName = dojo.string.substitute(messages.resolution,[resultDTO.labels[this._resultHeaderIndex[WorkItemAttributes.RESOLUTION]]]);
			}
			if (secondIconSrc) {
				secondIcon.src = this._getIconURL(secondIconSrc);
				secondIcon.title = secondIconName;
				secondIconCell.appendChild(secondIcon);
			}
			columns.push(secondIconCell);
		}
		
		var titleCell = document.createElement("td");
		titleCell.className = "titleCell";
		titleCell.colSpan = titleColSpan;
		var titleWrapper = document.createElement("div");
		titleWrapper.className = "titleWrapper";
		var titleLink = document.createElement("a");
		var presentationUri, resourceUri = this._getQualifiedUriRoot() + "/resource/itemName/com.ibm.team.workitem.WorkItem/" + resultDTO.id;
		// sniff the project area to see if we are local
		if (com.ibm.team.process.web.projectArea) {
			// use a presentation uri locally to avoid context switch for cross project queries
			presentationUri = WorkItemUtil.getViewWorkItemUri(resultDTO.id);
			titleLink.href = presentationUri;
		} else {
			titleLink.href = resourceUri;
		}
		titleLink.innerHTML = resultDTO.id + ": " + FormatUtil.insertWordBreaks(resultDTO.labels[this._resultHeaderIndex[WorkItemAttributes.SUMMARY]], 20, true);
		if (!zoomed) {
			var args = {retainLinkText: true, uri: resourceUri};
			if (presentationUri)
				args.presentationUri = presentationUri;
			new jazz.app.ResourceLink(args,titleLink);
		}
		titleWrapper.appendChild(titleLink);
		titleCell.appendChild(titleWrapper);
		columns.push(titleCell);
		
		return columns;
	},
	
	/* Node[] */ renderDetails: function(/* object */ resultDTO) {
		var columns = new Array();
		
		var detailsCell = document.createElement("td");
		detailsCell.className = "detailsCell";
		detailsCell.colSpan = "3";
		var div = document.createElement("div");
		div.className = "iframeWrapper";
		var iframe = document.createElement("iframe");
		var resizeContent = function() {
			if (iframe.contentWindow && iframe.contentWindow.innerHeight && iframe.contentWindow.scrollMaxY) {
				var win = iframe.contentWindow;
				var h = win.innerHeight + win.scrollMaxY;
			}
			else {
				var h = iframe.contentWindow.document.body.scrollHeight;
			}
			iframe.height = h + "px";
		};
		// onload will not fire in IE, workaround with a polling loop
		if (dojo.isIE) {
			var timeouts = this._iframePollTimeouts;
			var poll = function() {
				if (iframe.contentWindow && iframe.contentWindow.document.body && iframe.contentWindow.document.body.scrollHeight) {
					resizeContent();
				} else {
					timeouts[resultDTO.id] = setTimeout(poll,100);
				}
			};
			// yield to ensure the iframe is on dom
			timeouts[resultDTO.id] = setTimeout(poll,0);
		} else {
			dojo.attr(iframe,"onload",resizeContent);
		}
		iframe.src = this._getSmallPreviewUri(resultDTO);
		iframe.style.width = "100%";
		iframe.height = "0";
		iframe.frameBorder = "0";
		iframe.scrolling = "no";
		div.appendChild(iframe);
		detailsCell.appendChild(div);
		columns.push(detailsCell);
		
		return columns;
	},
	
	_getSmallPreviewUri: function(resultDTO) {
		var uriRoot = this._getQualifiedUriRoot();
		var uri = uriRoot + "/resource/itemOid/com.ibm.team.workitem.WorkItem/" + resultDTO.itemId + "?_mediaType=text%2Fhtml&_selector=smallHover&_context=web";
		return this._needsProxy() ? jazz.app.proxy.encodeProxiedURI(uri) : uri;
	},
	
	_getQualifiedUriRoot: function() {
		if (!this.uriRoot) {
			this.uriRoot = dojo.getObject("net.jazz.ajax._dataServerRoot");
			if (!this.uriRoot) {
				var winUrl = new dojo._Url(window.location.href);
				this.uriRoot = winUrl.scheme + "://" + winUrl.host + (winUrl.port ? ":" + winUrl.port : "" ) + net.jazz.ajax._contextRoot;
			}
		}
		return this.uriRoot;
	},
	
	_needsProxy: function() {
		if (typeof this.needsProxy === "undefined") {
			var viewletRoot = this._getQualifiedUriRoot();
			var winUrl = new dojo._Url(window.location.href);
			var localRoot = winUrl.scheme + "://" + winUrl.host + (winUrl.port ? ":" + winUrl.port : "" );
			this.needsProxy = viewletRoot.substring(0,localRoot.length) !== localRoot;
		}
		return this.needsProxy;
	},
	
	_getQueryHref: function(itemId) {
		// if there's a project area defined, stay in the same web ui
		// otherwise use the resource url
		if (com.ibm.team.process.web.projectArea)
			return WorkItemUtil.getRunSavedQueryUri(itemId);
		return this._getQualifiedUriRoot() + "/resource/itemOid/com.ibm.team.workitem.query.QueryDescriptor/" + itemId;
	},
	
	settingsChanged: function(/* object */ oldSettings, /* object */ newSettings) {
		if (oldSettings.preferences.query != newSettings.preferences.query) {
			this.reset(true);
		}
		else if (oldSettings.preferences.items != newSettings.preferences.items) {
			this.reset(false);
		}
		else if (oldSettings.preferences.secondIcon != newSettings.preferences.secondIcon) {
			this.update();
		}
		// getTextLabel is not CustomPreferenceProvider API
		// but is provided by the WorkItemQueryPreferenceProvider
		// always persist the query title (when it's available) on setting changes in case it has changed
		if (UUIDUtility.isValidUUID(newSettings.preferences.query) && this.getPreferenceProvider("query").getTextLabel(newSettings.preferences.query))
			this.getSite().setTitle(this.getPreferenceProvider("query").getTextLabel(newSettings.preferences.query), true);
	},
	
	getTitleHyperlinkURL: function(){
		var queryId = this.getPreference("query");
		if (queryId != "-1")
			return this._getQueryHref(queryId);
		if (this._defaultQueryItemId)
			return this._getQueryHref(this._defaultQueryItemId);
		return null;
	},
	
	getPreferenceProvider: function(/* string */ preferenceId) {
		if (!this._preferenceProvider)
			this._preferenceProvider = new WorkItemQueryPreferenceProvider({ prefId: preferenceId, queryCategoryPrefId: "queryCategory" });
		return this._preferenceProvider;
	},
	
	_clearTimeouts: function() {
		if (dojo.isIE && this._iframePollTimeouts) {
			for (var i in this._iframePollTimeouts) {
				clearTimeout(this._iframePollTimeouts[i]);
			}
		}
	},
	
	destroy: function() {
		for (var i in this._detailsWidgets)
			this._detailsWidgets[i].destroy();
		this._clearTimeouts();
		this.inherited(arguments);
	}
	
});

})();
