/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableTab");

dojo.require("com.ibm.team.dashboard.web.ui.internal.dnd.TabTarget");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("jazz.util.html");
dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var TabTarget = com.ibm.team.dashboard.web.ui.internal.dnd.TabTarget;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var HTML = jazz.util.html;
var messages;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.dnd.DraggableTab",
	[dijit._Widget, dijit._Templated], {
		
	tabTitle: null,
	editing: false,
	selected: false,
	draggableTabs: null,
	dashboardContainer: null,
	helper: null,
	_menuOpen: false,
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/DraggableTab.html"),
	
	constructor: function(args) {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
	},
	postCreate: function() {
		if (!this.selected) this._registerSelectEvent(true);
		this.setEditMode(this.editing);
	},
	setSelected: function(selected) {
		if (selected == this.selected) return;
		
		this.selected = selected;
		if (selected) {
			if (this.draggableTabs.selectedTab && !this.draggableTabs.selectedTab.isDestroyed)
				this.draggableTabs.selectedTab.setSelected(false);
			this._addTabDropDown();
			dojo.addClass(this.domNode, "selectedTab");
			this.draggableTabs.selectedTab = this;
			this._registerSelectEvent(false);
		} else {
			this._removeTabDropDown();
			this._registerSelectEvent(true);
			dojo.removeClass(this.domNode,"selectedTab");
		}
		this.setEditMode(this.editing);
	},
	setEditMode: function(editing) {
		this.editing = editing;
		this.setTitle();
		if (editing && !this.selected) {
			if (!this._tabTarget) {
				this._tabTarget = new TabTarget(this.domNode, {
					accept: ["viewlet"],
					index: this.index,
					dashboardContainer: this.dashboardContainer
				});
			}
		} else {
			if (this._tabTarget) {
				this._tabTarget.destroy();
				this._tabTarget = null;
			}
		}
	},
	setTitle: function(title) {
		if (!title) title = this.tabTitle;
		
		if (this.selected && this.editing) {
			if (!this._editableTitle) {
				this._tabTitle.innerHTML = "";
				
				var div = document.createElement("div");
				div.className = "editable-tab-title";
				div.innerHTML = HTML.escape(title);
		        this._editableTitle = new dijit.InlineEditBox({title:messages.clickToEdit, autoSave: true, width: "200px", noValueIndicator: "?", editorParams: {maxLength: "60"}, onChange: dojo.hitch(this, this._onTabTitleChange)}, div);
		        this._tabTitle.appendChild(div);				
				this._editableTitle.startup();
				
				this._tabTitleCloseMenuEvent = this.connect(this._editableTitle, "edit", this._closeTabDropDown);
				this._tabTitleKeypressStopper = this.connect(this._editableTitle.domNode.parentNode,"onkeypress", this._stopBubble);
				this._tabTitleClickStopper = this.connect(this._editableTitle.domNode.parentNode,"onmousedown", this._stopBubble);			
			}
		} else {
			if (this._editableTitle) {
				this.disconnect(this._tabTitleCloseMenuEvent);
				this.disconnect(this._tabTitleKeypressStopper);
				this.disconnect(this._tabTitleClickStopper);
				this._tabTitleKeypressStopper = null;
				this._tabTitleClickStopper = null;
				
				this._editableTitle.destroy();
				this._editableTitle = null;
			}
			this._tabTitle.innerHTML = title;
		}
		this.tabTitle = title;
	},
	_registerSelectEvent: function(doRegister) {
		if (doRegister) {
			if (!this._selectTabHandle) {
				this._selectTabHandle = this.connect(this.domNode, "onclick", this._selectTabHandler); 
			}
		} else {
			this.disconnect(this._selectTabHandle);
			this._selectTabHandle = null;
		}
	},
	_addTabDropDown: function() {
		if (this._menuItemClickHandler) {
			this._removeTabDropDown();
		}
		dojo.addClass(this.domNode, "tabWithMenu");
		var menuLink = document.createElement("a");
		menuLink.href = "#";
		menuLink.id = "tabDropDownIcon";
		var menuImg = document.createElement("img");
		menuImg.src = dojo.moduleUrl("com.ibm.team.dashboard.web", "graphics/metanav/images/tab-down-arrow.gif");
		menuImg.title = messages.menu;
		menuImg.alt = "v";  //$NON-NLS-1$
		menuLink.appendChild(menuImg);
		this._menuWrapper.appendChild(menuLink);
		this._menuWrapper.style.display = "block";
		this._menuItemClickHandler = this.connect(menuLink, 'onclick', this._openTabDropDown);
	},
	_removeTabDropDown: function() {
		if (this._menuItemClickHandler) {
			this.disconnect(this._menuItemClickHandler);
			this._menuItemClickHandler = null;
			dojo.removeClass(this.domNode, "tabWithMenu");
			this._menuWrapper.innerHTML = "";
			this._menuWrapper.style.display = "none";
		}
	},
	_openTabDropDown: function(event) {
		if (event) {
			dojo.stopEvent(event);
		}
		if (this._menuOpen) return;
		
		if (this.dashboardContainer.currentPage.killPopup){
			this.dashboardContainer.currentPage.killPopup();
		}
		
		//if the current tab is being edited, save it
		if (this._editableTitle && this._editableTitle.editing){
			this._editableTitle.save();
		}
		
		//Display menu
		this._createTabDropDown();
		dijit.popup.open({ popup: this._menu, parent: this, around: this._menuWrapper, orient: {"BL":"TL"}, onExecute: dojo.hitch(this,this._closeTabDropDown), onCancel: dojo.hitch(this,this._closeTabDropDown)});

		//register event handlers that cause menu to close
		this._tabMenuCloseHandle = this.connect(document.body,"onclick",this._closeTabDropDown);
		
		//style menu icon to indicate open state
		dojo.addClass(this._menuWrapper.firstChild,Constants.ViewletContainer.OPEN_MENU_CLASS);
		this.dashboardContainer.currentPage.killPopup = dojo.hitch(this, this._closeTabDropDown);
		this._menu.focus();
		this._menuOpen = true;
	},
	_closeTabDropDown: function(event) {
		if (this._menuOpen) {
			this._menuOpen = false;
			this.disconnect(this._tabMenuCloseHandle);
			dojo.removeClass(this._menuWrapper.firstChild, Constants.ViewletContainer.OPEN_MENU_CLASS);
			dijit.popup.close(this._menu);
		}
	},
	_createTabDropDown: function (){
		if (this._menu && this._menuIsEditing != this.editing) {
			this._menu.destroy();
			this._menu = null;
		}
		if (!this._menu) {
			// create items
			var menu = new dijit.Menu();
			dojo.addClass(menu.domNode, "com-ibm-team-dashboard-content-menu com-ibm-team-dashboard-tabs-menu");
			if (this.editing) {
				menu.addChild(new dijit.MenuItem({label: messages.addViewletText, iconClass: 'addIcon', onClick: dojo.hitch(this, this._expandChooser)}));
				menu.addChild(new dijit.MenuSeparator());
				menu.addChild(new dijit.MenuItem({label: messages.rename, iconClass: 'renameIcon',	onClick: dojo.hitch(this, this._renameTab)}));
			}
			menu.addChild(new dijit.MenuItem({label: messages.refresh, iconClass: 'refreshIcon', onClick: dojo.hitch(this.dashboardContainer, this.dashboardContainer.refresh)}));
			menu.addChild(new dijit.MenuSeparator());
			menu.addChild(new dijit.MenuItem({label: messages.copyTab, iconClass: 'copyTabIcon', onClick: dojo.hitch(this.dashboardContainer, this.dashboardContainer.copyTab)}));
			if (this.editing) {
				menu.addChild(new dijit.MenuItem({label: messages.duplicateTab, iconClass: 'duplicateTabIcon', onClick: dojo.hitch(this.dashboardContainer, this.dashboardContainer.duplicateTab)}));
				this._menuDeleteItem = new dijit.MenuItem({label: messages.deleteText, iconClass: 'deleteIcon', onClick: dojo.hitch(this, this._deleteTab)});
				menu.addChild(this._menuDeleteItem);
				menu.addChild(new dijit.MenuSeparator());
				var menu2 = new dijit.Menu();
				dojo.addClass(menu2.domNode, "com-ibm-team-dashboard-content-menu com-ibm-team-dashboard-tabs-menu");
				menu2.addChild(new dijit.MenuItem({label: messages.oneColumn, iconClass: 'oneColumnIcon', onClick: dojo.hitch(this,this._changeColumns,1)}));
				menu2.addChild(new dijit.MenuItem({label: messages.twoColumns, iconClass: 'twoColumnsIcon', onClick: dojo.hitch(this,this._changeColumns,2)}));
				menu2.addChild(new dijit.MenuItem({label: messages.threeColumns, iconClass: 'threeColumnsIcon', onClick: dojo.hitch(this,this._changeColumns, 3)}));
				var iconClass = this._getColumnIconClass();
				var popupMenu = new dijit.PopupMenuItem({label: messages.layout, iconClass: iconClass});
				popupMenu.popup = menu2;
				menu.addChild(popupMenu);
			}
			menu.startup();
			if (this.editing) {
				popupMenu.startup();
				menu2.startup();
				this._popupMenu = popupMenu;
			}
			this._menu = menu;
			this._menuIsEditing = this.editing;
		}
		// enable/disable items appropriately
		if (this.editing) {
			this._menuDeleteItem.domNode.style.display = (this.draggableTabs.widgets.length > 1) ? "" : "none";
		}
	},
	_onTabTitleChange: function(/* string */ newTitle) {
		if(!newTitle) return false;
		this.tabTitle = newTitle;
		this.dashboardContainer.dashboardItem.viewletPages[this.index].title = newTitle;
		this.draggableTabs.propertyChanged(Constants.PROPERTY_DASHBOARD);
		
		//workaround to innerHTML bug where extra <a> tag wrapper appears when it shouldn't.
		this._editableTitle.displayNode.innerHTML = "";
		this._editableTitle.displayNode.appendChild(document.createTextNode(newTitle));
	},
	destroy: function () {
		if (this._editableTitle) this._editableTitle.destroy();
		this.isDestroyed = true;
		this.inherited(arguments);
	},
	_selectTabHandler: function(event) {
		event.preventDefault();
		if (dojo.hasClass(event.currentTarget,"selectedTab")) return;
		
		//if the current tab is being edited, save it
		if (this._editableTitle && this._editableTitle.editing){
			this._editableTitle.save();
		}
		
		//kill any open popup menus
		if (this.draggableTabs.selectedTab) {
			try {
				var vp = this.dashboardContainer.viewletPages[this.draggableTabs.selectedTab.index];
				if (vp && vp.killPopup) vp.killPopup();
			} catch (e) {
				//eat the error
			}	
		}
		
		for (var i = 0; i < this.draggableTabs.widgets.length; i++) {
			if (event.currentTarget == this.draggableTabs.widgets[i].domNode) {
				this.draggableTabs._selectTabAction(this.draggableTabs.widgets[i]);
				break;
			}
		}
	},
	_deleteTab: function(event) {
		this.draggableTabs.remove(this.index);
	},
	_renameTab: function(event) {
		this._editableTitle._onClick(event);
	},
	_expandChooser: function() {
		this.dashboardContainer.openViewletChooser();
	},
	_changeColumns: function(toCols) {
		// redistribute all the viewlets
		var columns = this.dashboardContainer.dashboardItem.viewletPages[this.draggableTabs.selectedTab.index].columns;
		var diff = columns.length - toCols;
		if (diff > 0) {
			for (var i = 0; i < diff; i++) {
				var srcColumn = columns[columns.length-1];
				if (srcColumn.viewlets) {
					for (var j = srcColumn.viewlets.length - 1; j >= 0; j--) {
						var viewlet = srcColumn.viewlets[j];
						srcColumn.viewlets.splice(j,1);
						var destColumn = columns[j % toCols];
						if (destColumn.viewlets) {
							destColumn.viewlets.splice(0, 0, viewlet);
						}
						else {
							destColumn.viewlets = [viewlet];
						}
					}
				}
				columns.splice(columns.length-1,1);
			}
		}
		else {
			while(columns.length < toCols) {
				columns.push(new Object());
			}
		}
		this.dashboardContainer.refreshCurrentTab();
		
		// notify listeners
		this.draggableTabs.propertyChanged(Constants.PROPERTY_DASHBOARD);
		
		var iconClass = this._getColumnIconClass();
		this._popupMenu.domNode.firstChild.firstChild.className = "dijitMenuItemIcon " + iconClass;
	},
	_getColumnIconClass: function(){
		var cols = this.dashboardContainer.currentPage.viewletPageHelper.columns.length;
		
		var iconClass;
		if (cols == 1) {
			iconClass = 'oneColumnIcon';
		} else if (cols == 2){
			iconClass = 'twoColumnsIcon';
		} else {
			iconClass = 'threeColumnsIcon';
		}
		
		return iconClass;
	},
	_stopBubble: function(event){
		if (event) {
			if (this._editableTitle.editing) {
				event.cancelBubble = true;
			} else {
				return;
			}
		}
		return false;
	}
});

})();