﻿/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
class Grid extends ChartObject {
	
	public var gridData:GridData;
	
	public var xAxis:XAxis;

	public var yAxis:YAxis;
	
	public static var NORMALIZED_MULTIPLIERS:Array = [1, 2, 5];
	
	
	public function Grid() {
		super();
	}

	public function initializeGridData(width:Number, height:Number, max:Number, startX:Number, startY:Number, xAxisMaxLabelWidth:Number,
									   yAxisMaxLabelWidth:Number, graphAreaWidth:Number, graphAreaHeight:Number, 
									   yAxisSpacing:Number, yAxisIncrement:Number, yAxisSectionCount:Number, xAxisSpacing:Number, 
									   xAxisIncrement:Number, xAxisSectionCount:Number):Void {

		this.gridData = new GridData();
		
		this.gridData.width = width;
		this.gridData.height = height;

		this.gridData.max = max;
		this.gridData.startX = startX;
		this.gridData.startY = startY;

		this.gridData.yAxisSpacing = yAxisSpacing;
		this.gridData.yAxisIncrement = yAxisIncrement;
		this.gridData.yAxisSectionCount	= yAxisSectionCount;	

		this.gridData.xAxisSpacing = xAxisSpacing;
		this.gridData.xAxisIncrement = xAxisIncrement;
		this.gridData.xAxisSectionCount	= xAxisSectionCount;	
		
		this.gridData.xAxisMaxLabelWidth = xAxisMaxLabelWidth;
		this.gridData.yAxisMaxLabelWidth = yAxisMaxLabelWidth;
		
		this.gridData.graphAreaWidth = graphAreaWidth;
		this.gridData.graphAreaHeight = graphAreaHeight;
		
	}
	
	public function calculateIncrement(sectionCount:Number, maxValue:Number, area:Number, minSpacing:Number):Object {
		
		var increment:Number;
		var spacing:Number;
		
		// Calculate raw increment
		increment = Math.floor(maxValue / sectionCount);
		// Normalize increment to an acceptable value
		increment = this.normalizeIncrement(increment);
		sectionCount = Math.floor(maxValue / increment);
		spacing = area / sectionCount;
			
		// Adjust spacing, increment and section count as needed without violating constraints:
		// (1) Tick spacing must not go below the minimum threshold
		// (2) The biggest Y-axis value must be bigger than the biggest tally value
		while (((increment * sectionCount) < maxValue) || 
				(spacing < minSpacing)) {
			// Increase increment if tick spacing is below the minimum threshold
			while (spacing < minSpacing) {
				increment = this.increaseIncrement(increment);
				sectionCount = Math.floor(maxValue / increment);
				spacing = area / sectionCount;
			}
			// Increase the number of sections and max if the biggest tally value exceeds the biggest Y-axis value
			while ((increment * sectionCount) < maxValue) {
				sectionCount++;
				spacing = area / sectionCount;
				maxValue = increment * sectionCount;
			}			
		}
		
		var result:Object = new Object();
		result.increment = increment;
		result.sectionCount = sectionCount;
		result.spacing = spacing;
		result.maxValue = maxValue;
		
		return result;
	}
	
	public function increaseIncrement(increment:Number):Number {
		
		// e.g.
		// Increment:        2000
		// Exponent:         3
		// Base Value:       1000
		// Base Multiplier:  2
		// New Increment:    5000
		
		var exponent:Number = Math.floor(MathUtility.log10(increment));
		var baseValue:Number = Math.pow(MathUtility.BASE10, exponent);
		var baseMultiplier:Number = increment / baseValue;
		var limit:Number = Grid.NORMALIZED_MULTIPLIERS.length - 1;
		
		// Iterate over all multipliers except for the last one
		// Range: [0, 1, .. , N-2]
		for (var index:Number = 0; index < limit; index++) {
			if (baseMultiplier == Grid.NORMALIZED_MULTIPLIERS[index]) {
				// Multipler match found
				// Calculate the next highest increment using one multiplier above
				// the base multiplier
				var newIncrement:Number = baseValue * 
										  Grid.NORMALIZED_MULTIPLIERS[index + 1];
				// TEST
				//trace("INCREASE INCREMENT: " + increment + " -> " + newIncrement);
				return newIncrement;
			}
		}

		// TEST
		//trace("INCREASE INCREMENT: " + increment + " -> " + Math.pow(MathUtility.BASE10, exponent + 1));

		// Base multiplier is the last multiplier
		// Index: [N-1]
		// Calculate the next highest increment by increasing the exponent
		return Math.pow(MathUtility.BASE10, exponent + 1);
	}		
	
	public function normalizeIncrement(increment:Number):Number {
		
		if (increment < Grid.NORMALIZED_MULTIPLIERS[0]) {
			return Grid.NORMALIZED_MULTIPLIERS[0];
		}
		
		// Optimize the search for the normalized increment right away by
		// calculating the increment's base 10 exponent
		var lowerExponent:Number = Math.floor(MathUtility.log10(increment));
		var higherExponent:Number = lowerExponent;
		var lowerMultIndex:Number = -1;
		var higherMultIndex:Number = 0;
		
		// TEST
		//trace("NORMALIZE =================================");		
		
		while (true) {
			// Note: Loop iterations will not exceed the number of multipliers: 
			// iterations <= Grid.NORMALIZED_MULTIPLIERS.length
			
			higherMultIndex++;
			lowerMultIndex++;			
			
			// Note: Higher and lower indexes are staggered
			// Higher is always one increment ahead of lower
			// Increase in higher and lower indexes cannot happen at the same time
			if (higherMultIndex == Grid.NORMALIZED_MULTIPLIERS.length) {
				higherExponent++;
				higherMultIndex = 0;
			} else if (lowerMultIndex == Grid.NORMALIZED_MULTIPLIERS.length) {
				lowerExponent++;
				lowerMultIndex = 0;			
			}
			// Calculate the normalized increment candidate greater than the current increment
			var upper:Number = Math.pow(MathUtility.BASE10, higherExponent) * 
							   Grid.NORMALIZED_MULTIPLIERS[higherMultIndex];
			// TEST
			//trace("UPPER:     " +  upper);
			
			// The current increment is not in the range of the upper and lower normalized
			// increment candidates
			if (increment > upper) {
				continue;
			}
			// Range: upper >= increment >= lower
			// TEST
			//trace("INCREMENT: " +  increment);
			
			// Calculate the normalized increment candidate less than the current increment
			var lower:Number = Math.pow(MathUtility.BASE10, lowerExponent) * 
							   Grid.NORMALIZED_MULTIPLIERS[lowerMultIndex];
			// TEST
			//trace("LOWER:     " +  lower);
			
			// Return the normalized increment candidate that is closest to the
			// current increment value
			var lowerDelta:Number = increment - lower;
			var upperDelta:Number = upper - increment;
			
			if (upperDelta < lowerDelta) {
				return upper;
			} else {
				return lower;
			}

		}
		
		// Unreachable code
		return 1;
	}
	
	public function draw(parentMC:MovieClip):Void {
		
		this.mc = parentMC.createEmptyMovieClip("grid_mc", parentMC.getNextHighestDepth());
		
	}

	public function getYCoordinate(value:Number):Number {
		var releativeY:Number = this.gridData.max - value;
		var step:Number = this.gridData.yAxisSpacing / this.gridData.yAxisIncrement;
		
		return this.gridData.hTickOffsetY + (releativeY * step);
	}
	
	public function getXCoordinate(value:Number):Number {
		var step:Number = this.gridData.xAxisSpacing / this.gridData.xAxisIncrement;
		
		return this.gridData.xZeroPosition + (value * step);
	}	
	
	public function getGridData():GridData {
		return this.gridData;
	}
	
	public function destroy():Void {
		// Grid data
		if (this.gridData) {
			this.gridData.destroy();
			delete this.gridData;
		}
		// X axis
		if (this.xAxis) {
			this.xAxis.destroy();
			delete this.xAxis;
		}
		// Y axis
		if (this.yAxis) {
			this.yAxis.destroy();
			delete this.yAxis;
		}		
		
		super.destroy();
	}		
	
}