/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.page.PlanningPageView"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require("net.jazz.ajax.ui.SideNavigationLayout"); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require("jazz.ui.SearchBox"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.client.session.Session"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.QueryDescriptor"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.page.PlanningPageController"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.navigation.PredefinedQuery"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.navigation.RecentlyViewed"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.search.SearchResultsPage"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.IterationPlanPage"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.util"); //$NON-NLS-1$

dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var isGuest				= com.ibm.team.repository.web.client.session.isGuest;

var SearchResultsPage	= com.ibm.team.apt.web.ui.internal.view.search.SearchResultsPage;
var IterationPlanPage	= com.ibm.team.apt.web.ui.internal.view.plan.IterationPlanPage;
var PlanningUtil		= com.ibm.team.apt.web.client.util;
var SearchBox			= jazz.ui.SearchBox;
var RecentlyViewed		= com.ibm.team.apt.web.ui.internal.view.navigation.RecentlyViewed;
var PredefinedQuery		= com.ibm.team.apt.web.ui.internal.view.navigation.PredefinedQuery;
var QueryDescriptor		= com.ibm.team.apt.web.ui.internal.common.QueryDescriptor;

function getWorkbench() {
	return net.jazz.ajax.ui.PlatformUI.getWorkbench();
}

var EMPTY_PAGE_ID		= "__jazzIterationPlanEmptyPage"; //$NON-NLS-1$
var PLAN_VIEWER_ID		= "__jazzIterationPlanViewer"; //$NON-NLS-1$
var PLAN_RESULTS_ID		= "__jazzIterationPlanResults"; //$NON-NLS-1$

var DEFAULT_INSTANCE_ID	= "__jazzDefaultWidgetInstance"; //$NON-NLS-1$

var CACHE_SIZE			= 5;

var SCOPE_ALL = "all"; //$NON-NLS-1$
var SCOPE_CURRENT = "current"; //$NON-NLS-1$
 
var bind= dojo.string.substitute;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

/**
 * The main view for the iteration planning web UI. Responsible for displaying left-nav
 * content and for managing the main content area.
 */
dojo.declare("com.ibm.team.apt.web.ui.internal.page.PlanningPageView", //$NON-NLS-1$
			 [dijit._Widget, dijit._Templated], {

	constructor: function() {
		this._widgetCache= [];
		this._progressLabels= [];
	},
	
	// ---- API ------------------------------------------------------------------------------------------------------------	

	showCachedIterationPlan: function(iterationPlanId, viewOptions, currentSearchResults) {
		var cachedWidget= this._findCachedWidget("com.ibm.team.apt.web.ui.internal.view.plan.IterationPlanPage", iterationPlanId); //$NON-NLS-1$
		if (cachedWidget == null)
			return false;

		this._setMainContent(cachedWidget, iterationPlanId);
		cachedWidget.setViewOptions(viewOptions, currentSearchResults);
		cachedWidget.setEditMode(false);
		this._doHighlighting(this._touchPlan(cachedWidget));
		return true;
	},

	showIterationPlan: function(loadIterationPlanResultDTO, viewOptions, currentSearchResults) {
		var constructorArgs = IterationPlanPage.constructorArgs(loadIterationPlanResultDTO, this._controller);
		var iterationPlanWidget= new IterationPlanPage(constructorArgs);

		this._setMainContent(iterationPlanWidget, loadIterationPlanResultDTO.plan.itemId);
		iterationPlanWidget.setViewOptions(viewOptions, currentSearchResults);
		iterationPlanWidget.setEditMode(false);
		this._doHighlighting(this._touchPlan(iterationPlanWidget));
	},
	
	showCreatePlan: function(loadIterationPlanResultDTO){
		var constructorArgs = IterationPlanPage.constructorArgs(loadIterationPlanResultDTO, this._controller);
		var iterationPlanWidget= new IterationPlanPage(constructorArgs);
		iterationPlanWidget.setEditMode(true);
		this._setMainContent(iterationPlanWidget, loadIterationPlanResultDTO.plan.itemId);
	},
	
	showEditPlan: function(loadIterationPlanResultDTO) {
		var constructorArgs = IterationPlanPage.constructorArgs(loadIterationPlanResultDTO, this._controller);
		var iterationPlanWidget= new IterationPlanPage(constructorArgs);
		iterationPlanWidget.setEditMode(true);
		this._setMainContent(iterationPlanWidget, loadIterationPlanResultDTO.plan.itemId);
	},
	
	/**
	 * 
	 * @param {String} searchResultId
	 * @return {boolean} true if a previous search result with the given id is being shown
	 */
	showCachedSearchResult: function(searchResultId) {
		var cachedWidget= this._findCachedWidget("com.ibm.team.apt.web.ui.internal.view.search.SearchResultsPage", searchResultId); //$NON-NLS-1$
		if (cachedWidget == null)
			return false;
		if (this._predefinedSection && !cachedWidget._searchResults.getQueryDescriptor().getKeywords()) {
			var row = this._predefinedSection.getRow(cachedWidget._searchResults.getQueryDescriptor());
		}
		// if row is undefined, will clear highlighting
		this._doHighlighting(row);
		this._setMainContent(cachedWidget, searchResultId);
		return true;
	},
	
	/**
	 * Displays the resultpage for searchResult, and puts the page in the widget cache
	 * @param {SearchResult} searchResult
	 */
	showSearchResult: function(searchResult, startIndex) {
		if (this._predefinedSection && !searchResult.getQueryDescriptor().getKeywords()) {
			var row = this._predefinedSection.getRow(searchResult.getQueryDescriptor());
		}
		// if row is undefined, will clear highlighting
		this._doHighlighting(row);
		var constructorArgs = SearchResultsPage.constructorArgs(searchResult, startIndex, this._controller);
		var searchResultWidget= new SearchResultsPage(constructorArgs);
		this._setMainContent(searchResultWidget, searchResult.getQueryDescriptor().getIdentifier(startIndex));
	},
	
	// ---- widget lifecycle implementation --------------------------------------------------------------------------------
	
	_controller: null,

	templatePath: dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/page/templates/PlanningPageView.html"), //$NON-NLS-1$ //$NON-NLS-2$
	
	/*
	 * Called by Dojo during widget creation lifecycle.
	 */
	postCreate: function() {
	 	this._controller= new com.ibm.team.apt.web.ui.internal.page.PlanningPageController(this);

		this._leftNavigationLayout= new net.jazz.ajax.ui.SideNavigationLayout({horizontalScroll: true}, this._appContainer);
		
		this._controller.registerCommands();
		
		//Initialize the left-side navigation panel containing the lists of streams and workspaces.
		this._initializeNavigation();
		//Display the main page content.
		this._initializeContent();
	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------
	
	_recentlyViewedSection: null,
	_currentWidgetId: null,
	_currentInstanceId: null,
	_currentWidget: null,

	_initializeNavigation: function(navigationView) {
		// add keyword search section
		var keywordSearchWidget = new SearchBox({
			scopes: [
				{id: SCOPE_ALL, label: MSGS.QUERY_ALLPLANS, icon: dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/navigation/images/search-planIcon.gif")}, //$NON-NLS-1$ //$NON-NLS-2$
				{id: SCOPE_CURRENT, label: MSGS.QUERY_CURRENTPLANS, icon: dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/navigation/images/search-planIcon.gif")} //$NON-NLS-1$ //$NON-NLS-2$
			],
			onExecute: dojo.hitch(this, this._onExecute),
			tooltip: MSGS.PREDEFQUERY_DROPDOWN_TOOLTIP_LABEL_CURRENT
		});
	
		this._leftNavigationLayout._leftNav.appendChild(keywordSearchWidget.domNode);
		
		if(!isGuest()) {
			var newPlanCommand = this._leftNavigationLayout.addCommand({
				href: PlanningUtil.getPlanCreateURI(), 
				imageClass: "create-plan-command",  //$NON-NLS-1$
				buttonText: MSGS.SECTION_PLAN_CREATE, 
				boldFirstWord: true, 
				size: "small"}); //$NON-NLS-1$
			dojo.addClass(newPlanCommand.domNode, "create-plan-command"); //$NON-NLS-1$
		}

		// add recently viewed section
		this._recentlyViewedSection= new RecentlyViewed({controller: this._controller});
		var recentNavSection = this._leftNavigationLayout.addNavigationSection(this._recentlyViewedSection, MSGS.SECTION_RECENTLY_VIEWED, true);
		this._recentlyViewedSection.populate(recentNavSection);

		// add predefined queries section
		this._predefinedSection = new PredefinedQuery({controller: this._controller});
		var preNavSection = this._leftNavigationLayout.addNavigationSection(this._predefinedSection, MSGS.SECTION_PREDEFINED_QUERIES, true);
		this._predefinedSection.populate(preNavSection);

		if (!isGuest()) {
			this._predefinedSection.addQuery(new QueryDescriptor(null, PlanningUtil.QUERY_MYPLANS));
		}
		this._predefinedSection.addQuery(new QueryDescriptor(null, PlanningUtil.QUERY_CURRENTPLANS));
		this._predefinedSection.addQuery(new QueryDescriptor(null, PlanningUtil.QUERY_ALLPLANS));
	},
	
	_doHighlighting: function (node) {
		if (this._selectedNode)
			dojo.removeClass(this._selectedNode, "selected"); //$NON-NLS-1$
		if (node) {
			dojo.addClass(node, "selected"); //$NON-NLS-1$
			this._selectedNode = node;
		} else {
			delete this._selectedNode;
		}
	},
	
	_initializeContent: function() {
		this._emptyPage= new dijit._Widget({}, document.createElement("div")); //$NON-NLS-1$
	},
	
	/**
	 * Callback from the SearchBox when enter is selected or the search button is clicked.
	 */
	_onExecute: function(/* string */ query, /* string */ scope, /* function */ callback, /* function */ errorCallback) {
		// call search service
		
		var queryDescriptor= new QueryDescriptor(query, scope == SCOPE_CURRENT ? PlanningUtil.QUERY_CURRENTPLANS : null);
		window.location.href = PlanningUtil.getSearchURI(queryDescriptor);
		
	},

	_setMainContent: function(widget, instanceId) {
		if (this._currentWidget === widget) {
			if (dojo.isFunction(widget["onPageShow"])) { //$NON-NLS-1$
				widget.onPageShow();
			}
			return;
		}
		
		if (this._currentWidget != null && dojo.isFunction(this._currentWidget["onPageHide"])) { //$NON-NLS-1$
			this._currentWidget.onPageHide();
		}

		if (dojo.isFunction(widget["onPageShow"])) { //$NON-NLS-1$
			widget.onPageShow();
		}

		this._currentWidget= widget;
		this._currentInstanceId= instanceId;
		this._leftNavigationLayout.setMainContent(widget.domNode)
		
		var title= MSGS.DEFAULT_PAGE_TITLE;
		
		if (PlanningUtil.isDebugMode()) {
			title= "TODO: set title for widget " + widget.declaredClass + " (id: " + instanceId + ")"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}

		if (dojo.isFunction(widget["getTitle"])) { //$NON-NLS-1$
			title= widget.getTitle();
		}
		
		getWorkbench().setTitle(title);
		
		if(widget instanceof com.ibm.team.apt.web.ui.internal.view.plan.IterationPlanPage && widget.isEditMode())
			return; // don't cache iterationplanpage when in edit mode, see defect 79891
		
		this._registerWidget(widget, instanceId)
	},
	
	_touchPlan: function(iterationPlanPage) {
		return this._recentlyViewedSection.addRecentPlan(iterationPlanPage.getDecoratedPlanLabel(), iterationPlanPage.getPlanId(), iterationPlanPage.getPlanIcon());
	},
	
	_validateRecentPlans: function() {
		this._recentlyViewedSection.validateRecentPlans();
	},
	
	// ---- progress api for PlanningPageController ------------------------------------------------------------------------	

	_progressLabels: null,
	_messageIdCounter: 0,


	_showProgressMessage: function(label, hideCurrentContent) {
		var messageId= this._messageIdCounter++;

		var progressMessageCount= this._progressLabels.push({
			label: label,
			hideCurrentContent: hideCurrentContent,
			messageId: messageId
		});
		
		if (hideCurrentContent) {
			this._setMainContent(this._emptyPage, EMPTY_PAGE_ID);
		}
		
		this._updateProgressMessage();
		
		return messageId;
	},

	_hideProgressMessage: function(messageId) {
		for (var i= 0; i < this._progressLabels.length; i++) {
			if (this._progressLabels[i].messageId == messageId) {
				var listTail= this._progressLabels.slice(i + 1, this._progressLabels.length);
				this._progressLabels.length= i;
				this._progressLabels.push.apply(this._progressLabels, listTail);
				break;				
			}
		}

		this._updateProgressMessage();
	},

	_updateProgressMessage: function() {
		if (this._progressLabels.length > 0) {
			getWorkbench().showMessage(this._progressLabels[0].label);
		} else {
			getWorkbench().hideMessageArea();
		}
	},
	
	// ---- implementation: widget list ------------------------------------------------------------------------------------

	_widgetCache: null,

	// Find the cached version of the widget
	_findCachedWidget: function(widgetId, instanceId) {
		instanceId= instanceId || DEFAULT_INSTANCE_ID;

		for (var i= 0; i < this._widgetCache.length; i++) {
			if(this._widgetCache[i].widgetId == widgetId && this._widgetCache[i].instanceId == instanceId) {
				return this._widgetCache[i].widget;
			}
		}
		
		return null;
	},

	_registerWidget: function(widget, instanceId) {
		var widgetId= widget.declaredClass;
		
		for (var i= 0; i < this._widgetCache.length; i++) {
			if(this._widgetCache[i].widgetId == widgetId && this._widgetCache[i].instanceId == instanceId) {
				// we already know the widget -> throw it out of the cache
				this._removeWidget(i, this._widgetCache[i].widget != widget);
				break;
			}
		}

		var cachedObject= { widgetId: widgetId, instanceId: instanceId, widget: widget };

		while (this._widgetCache.length >= CACHE_SIZE) {
			// remove the first entry, 
			this._removeWidget(0, true);
		}

		this._widgetCache.push(cachedObject);
		return widget;		
	},
	
	_removeWidget: function(index, doDestroy) {
		if (doDestroy) {
			var widgetToRemove= this._widgetCache[index];			
			try {
				widgetToRemove.widget.destroy();
			} catch (e) {
				console.log("Error destroying widget id: " + widgetToRemove.instanceId); //$NON-NLS-1$
				console.log(e);
			}
		}

		var cacheTail= this._widgetCache.slice(index + 1, this._widgetCache.length);
		this._widgetCache.length= index;
		this._widgetCache.push.apply(this._widgetCache, cacheTail);
	},
	
	__sentinel: null // terminates this class definition
});

})();
