/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
/* declare the Dojo module */
dojo.provide("com.ibm.team.workitem.web.ui.internal.page.WorkItemMailConfigPageView");

/* declare dependencies on other Dojo modules */
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.team.workitem.web.ui.internal.page.WorkItemMailConfigPageController");
dojo.require("com.ibm.team.workitem.web.client.util");

dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.ProfileEditorPage");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemMailConfigMessages");

(function(){

    var _controller;
    var _config;
    var _workItemRelationships;
    var _workItemChanges;
    var _filterCheckboxes;
    var _modified;
    var _connectHandles;
    
    /* Aliases */
    var ProfileEditorPage = com.ibm.team.repository.web.ui.internal.admin.users.ProfileEditorPage;
    
    var MSGS = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemMailConfigMessages"); //$NON-NLS-1$ //$NON-NLS-2$
    /* declare and implement the widget "class" */
    dojo.declare("com.ibm.team.workitem.web.ui.internal.page.WorkItemMailConfigPageView", [ProfileEditorPage], {
    
        templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/page/templates/WorkItemMailConfigPageView.html"),
        //templateCssPath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/page/templates/WorkItemMailConfigPageView.css"),
        
        constructor: function(args){
        },
        
        /* externalized messages for the template */
        _messages: MSGS,
        
        postCreate: function(){
            //console.debug("postCreate()");
            this.inherited(arguments);
            
            this._controller = new com.ibm.team.workitem.web.ui.internal.page.WorkItemMailConfigPageController(this);
            this._modified = false;
            this._connectHandles = [];
            
            this._controller._loadAndShowMailConfig();
        },
        
        destroy: function(){
            this.inherited(arguments);
        },
        
        doSave: function(callback){
            this._onSave(callback);
            this.inherited(arguments);
        },
        
        _onSave: function(callback){
        
            /* Work Items (Global) */
            var mailWhenSubscribed = this._node(this._config, ["global", "workitems", "mailWhenSubscribed"]);
            var receiveMails = this._node(this._config, ["global", "workitems", "receiveMails"]);
            
            mailWhenSubscribed.value = this._mailConfigWorkItemSubscriberCheck.checked;
            receiveMails.value = this._mailConfigWorkItemEnablementCheck.checked;
            
            var filterNode = this._node(this._config, ["global", "workitems", "filter"]);
            filterNode.children = new Array();
            
            /* Work Items (Filter) */
            var relationShipCounter = 0;
            for (var i = 0; i < this._workItemRelationships.length; i++) {
                var workItemRelationship = this._workItemRelationships[i];
                
                var relationShipNode = {
                    id: workItemRelationship.id,
                    children: []
                };
                
                var addNode = false;
                var changeCounter = 0;
                for (var j = 0; j < this._workItemChanges.length; j++) {
                    var workItemChange = this._workItemChanges[j];
                    
                    var checkbox = this._filterCheckboxes[j * this._workItemRelationships.length + i];
                    if (checkbox.checked) {
                        relationShipNode.children[changeCounter++] = {
                            id: workItemChange.id
                        };
                        addNode = true;
                    }
                }
                
                if (addNode) 
                    filterNode.children[relationShipCounter++] = relationShipNode;
            }
            
            /* Message @Me */
            var messageAtMeNode = this._node(this._config, ["global", "messagesAtMe", "mailMessagesAtMe"]);
            messageAtMeNode.value = this._mailConfigMessageAtMeCheck.checked;
            
            /* Approvals */
            var receiveMails = this._node(this._config, ["global", "approvals", "receiveMails"]);
            var receiveOwnChanges = this._node(this._config, ["global", "approvals", "receiveOwnChanges"]);
            
            receiveMails.value = this._mailConfigApprovalsEnablementCheck.checked;
            receiveOwnChanges.value = this._mailConfigApprovalsOwnCheck.checked;
            
            /* Format */
            var htmlMailNode = this._node(this._config, ["global", "format", "htmlMail"]);
            htmlMailNode.value = this._mailConfigFormatMailsInHtmlCheck.checked;
            
            this._controller._performSave(this._config, callback);
        },
        
        refresh: function(){
            this._controller._loadAndShowMailConfig();
            this._onModified(false);
        },
        
        _onMailConfigWorkItemEnablementClick: function(e){
            this._setFiltersEnabled(this._mailConfigWorkItemEnablementCheck.checked);
            this._onInputClick(e);
        },
        
        _onMailConfigApprovalsEnablementClick: function(e){
            this._setOwnApprovalChangesEnabled(this._mailConfigApprovalsEnablementCheck.checked);
            this._onInputClick(e);
        },
        
        _setOwnApprovalChangesEnabled: function(enabled){
            this._mailConfigApprovalsOwnCheck.disabled = !enabled;
            this._mailConfigApprovalsOwnCheckLabel.className = enabled ? "checkbox" : "checkboxDisabled";
        },
        
        _onInputClick: function(e){
            this._onModified(true);
        },
        
        _onModified: function(modified){
            this._modified = modified;
            if (modified) 
                this.setDirty(true);
        },
        
        _setFiltersEnabled: function(enabled){
            var checkboxes = dojo.query("input", this._mailConfigWorkItemFilters);
            dojo.forEach(checkboxes, function(c){
                c.disabled = !enabled;
            });
            
            var tables = dojo.query("table", this._mailConfigWorkItemFilters);
            dojo.forEach(tables, function(t){
                t.className = enabled ? "mailConfigWorkItemsFilter" : "mailConfigWorkItemsFilterDisabled";
            });
            
            this._selectAllFiltersButton.disabled = !enabled;
            this._deselectAllFiltersButton.disabled = !enabled;
            this._restoreDefaultsButton.disabled = !enabled;
        },
        
        _setInput: function(input){
            //console.debug("_setInput()");
            
            this._config = input.configuration;
            this._workItemRelationships = input.workItemRelationships;
            this._workItemChanges = input.workItemChanges;
            
            this._showMailConfig(this._config);
        },
        
        _showMailConfig: function(configuration){
            //console.debug("_showMailConfiguration()");
            
            this._progressContainer.style.display = "none";
            
            this._workItemsSection.style.display = "block";
            this._messageAtMeSection.style.display = "block";
            this._approvalsSection.style.display = "block";
            this._formatSection.style.display = "block";
            
            this._showMailConfigWorkItems(configuration);
            this._showMailConfigMessageAtMe(configuration);
            this._showMailConfigApprovals(configuration);
            this._showMailConfigFormat(configuration);
        },
        
        _showMailConfigWorkItems: function(config){
            //console.debug("_showMailConfigWorkItems()");
            
            var mailWhenSubscribed = this._node(config, ["global", "workitems", "mailWhenSubscribed"]);
            var receiveMails = this._node(config, ["global", "workitems", "receiveMails"]);
            
            this._mailConfigWorkItemSubscriberCheck.checked = (mailWhenSubscribed.value == "true");
            this._mailConfigWorkItemEnablementCheck.checked = (receiveMails.value == "true");
            
            this._showMailConfigWorkItemsFilter(config);
            this._setFiltersEnabled(this._mailConfigWorkItemEnablementCheck.checked);
        },
        
        _showMailConfigWorkItemsFilter: function(config){
            //console.debug("_showMailConfigWorkItemsFilter()");
            
            /* For Each Change */
            var table = document.createElement("table");
            table.className = "mailConfigWorkItemsFilter";
            
            this._filterCheckboxes = new Array(this._workItemChanges.length * this._workItemRelationships.length);
            
            /* Table Body */
            var tbody = document.createElement("tbody");
            table.appendChild(tbody);
            
            /* First row is reserved for Relationship Labels */
            var tr = document.createElement("tr");
            tbody.appendChild(tr);
            
            for (var i = 0; i < this._workItemRelationships.length; i++) {
                var workItemRelationship = this._workItemRelationships[i];
                
                var th = document.createElement("th");
                th.className = "mailConfigWorkItemsFilter";
                tr.appendChild(th);
                
                var text = document.createTextNode(workItemRelationship.value);
                th.appendChild(text);
            }
            
            /* Empty Cell */
            var th = document.createElement("th");
            tr.appendChild(th);
            
            var counter = 0;
            for (var i = 0; i < this._workItemChanges.length; i++) {
                var workItemChange = this._workItemChanges[i];
                
                var tr = document.createElement("tr");
                tbody.appendChild(tr);
                
                /* For Each Relationship */
                for (var j = 0; j < this._workItemRelationships.length; j++) {
                    var workItemRelationship = this._workItemRelationships[j];
                    
                    var td = document.createElement("td");
                    td.className = "mailConfigWorkItemsFilterInput";
                    tr.appendChild(td);
                    
                    var input = document.createElement("input");
                    input.type = "checkbox";
                    this._filterCheckboxes[counter++] = input;
                    input.checked = this._hasNode(config, ["global", "workitems", "filter", workItemRelationship.id, workItemChange.id]);
                    
                    /* Track Modifications */
                    var self = this;
                    this._connectHandles.push(dojo.connect(input, "onclick", function(e){
                        self._onInputClick(e);
                    }));
                    
                    td.appendChild(input);
                }
                
                /* Append Changes */
                var td = document.createElement("td");
                tr.appendChild(td);
                td.className = "mailConfigWorkItemsFilterChangeLabel";
                
                var text = document.createTextNode(workItemChange.value);
                td.appendChild(text);
            }
            
            dojox.data.dom.replaceChildren(this._mailConfigWorkItemFilters, table);
            
            /* Need to set checked state after making checkboxes visible (IE Bug) */
            if (dojo.isIE) {
                for (var i = 0; i < this._workItemRelationships.length; i++) {
                    var workItemRelationship = this._workItemRelationships[i];
                    
                    for (var j = 0; j < this._workItemChanges.length; j++) {
                        var workItemChange = this._workItemChanges[j];
                        
                        var checkbox = this._filterCheckboxes[j * this._workItemRelationships.length + i];
                        checkbox.checked = this._hasNode(config, ["global", "workitems", "filter", workItemRelationship.id, workItemChange.id]);
                    }
                }
            }
        },
        
        _onSelectAllFilters: function(e){
            dojo.stopEvent(e);
            this._onModified(true);
            
            var checkboxes = dojo.query("input", this._mailConfigWorkItemFilters);
            dojo.forEach(checkboxes, function(c){
                c.checked = true;
            });
        },
        
        _onDeselectAllFilters: function(e){
            dojo.stopEvent(e);
            this._onModified(true);
            
            var checkboxes = dojo.query("input", this._mailConfigWorkItemFilters);
            dojo.forEach(checkboxes, function(c){
                c.checked = false;
            });
        },
        
        _onRestoreDefaults: function(e){
            dojo.stopEvent(e);
            this._onModified(true);
            
            for (var i = 0; i < this._workItemRelationships.length; i++) {
                var workItemRelationship = this._workItemRelationships[i];
                
                for (var j = 0; j < this._workItemChanges.length; j++) {
                    var workItemChange = this._workItemChanges[j];
                    
                    var checkbox = this._filterCheckboxes[j * this._workItemRelationships.length + i];
                    checkbox.checked = (i != 2 && j != 5 && j != 9); // Modifier / Subscriber modified, Owner started/stopped working
                }
            }
        },
        
        _showMailConfigApprovals: function(config){
            //console.debug("_showMailConfigApprovals()");
            
            var receiveMails = this._node(config, ["global", "approvals", "receiveMails"]);
            var receiveOwnChanges = this._node(config, ["global", "approvals", "receiveOwnChanges"]);
            
            this._mailConfigApprovalsEnablementCheck.checked = (receiveMails.value === "true");
            this._mailConfigApprovalsOwnCheck.checked = (receiveOwnChanges.value === "true");
            
            this._setOwnApprovalChangesEnabled(this._mailConfigApprovalsEnablementCheck.checked);
        },
        
        _showMailConfigFormat: function(config){
            //console.debug("_showMailConfigFormat()");
            
            var htmlMailNode = this._node(config, ["global", "format", "htmlMail"]);
            this._mailConfigFormatMailsInHtmlCheck.checked = (htmlMailNode.value === "true");
        },
        
        _showMailConfigMessageAtMe: function(config){
            //console.debug("_showMailConfigMessageAtMe()");
            
            var messageAtMeNode = this._node(this._config, ["global", "messagesAtMe", "mailMessagesAtMe"]);
            this._mailConfigMessageAtMeCheck.checked = (messageAtMeNode.value === "true");
        },
        
        _node: function(array, path){
            for (var i = 0; i < path.length; i++) {
                var nodeId = path[i];
                var found = false;
                
                if (array.children != null) {
                    var children = array.children;
                    for (var j = 0; j < children.length; j++) {
                        if (children[j].id != null && children[j].id == nodeId) {
                            array = children[j];
                            found = true;
                            break;
                        }
                    }
                }
                
                if (!found) 
                    return null;
            }
            
            return array;
        },
        
        _hasNode: function(array, path){
            return this._node(array, path) != null;
        },
        
        destroy: function(){
            //console.debug("destroy()");
            dojo.forEach(this._connectHandles, function(handle){
                dojo.disconnect(handle);
            });
        }
    });
})();
