/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.ui.internal.query.JUnitCellElementViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.build.web.ui.internal.Constants");
dojo.require("com.ibm.team.build.web.ui.internal.query.CellElementViewer");
dojo.require("com.ibm.team.build.web.ui.internal.utils.ResultUtil");
dojo.require("jazz.util.html");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

(function() {

/*
 * Summary: Inherit from the CellElementViewer cell widget. This class provides:
 * 			1) provides the specfic content to cell in tree-table's rows of build result's JUnit tab.
 */

var Constants = com.ibm.team.build.web.ui.internal.Constants;
var CellElementViewer = com.ibm.team.build.web.ui.internal.query.CellElementViewer;
var ResultUtil = com.ibm.team.build.web.ui.internal.utils.ResultUtil;
var messages = dojo.i18n.getLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

dojo.declare("com.ibm.team.build.web.ui.internal.query.JUnitCellElementViewer",
 [CellElementViewer],
{     	
	getCellData: function(cellType){
		var rowData = this.rowContainer.getRowData();
	    //Transfer the error/failure/test's bool value in test case to number: true<->1, false<->0
		var rowType = this.rowContainer.getRowType();  
		if(rowType == Constants.JUNIT_CASE_ROW){
			if(cellType == Constants.JUNIT_ERRORS_COLUMN){	
				dojo.addClass(this.domNode, "numbersColumn");		
				return rowData.error ? 1 :0;			
			}
			if(cellType == Constants.JUNIT_FAILURES_COLUMN){
				dojo.addClass(this.domNode, "numbersColumn");	
				return rowData.failure ? 1 :0;			
			}
			if(cellType == Constants.JUNIT_TESTS_COLUMN){
				dojo.addClass(this.domNode, "numbersColumn");	
				return 1;				
			}
		}
		if(cellType == Constants.JUNIT_ERRORS_COLUMN){		
			dojo.addClass(this.domNode, "numbersColumn");	
			return rowData.errorCount || 0;			
		}
		if(cellType == Constants.JUNIT_FAILURES_COLUMN){
			dojo.addClass(this.domNode, "numbersColumn");
			return rowData.failureCount || 0;			
		}
		if(cellType == Constants.JUNIT_TESTS_COLUMN){
			dojo.addClass(this.domNode, "numbersColumn");
			return rowData.testCount || 0;				
		}		
		if(cellType == Constants.JUNIT_COMPONENT_NAME_COLUMN){
			dojo.addClass(this.domNode, "nameColumn");
			return rowData.name ? jazz.util.html.escape(rowData.name) : "<span class='none-text'>" + messages.noComponent + "</span>";			
		}
		if(cellType == Constants.JUNIT_SUITE_NAME_COLUMN){
			var escapedName = rowData.testSuiteClassName ? jazz.util.html.escape(rowData.testSuiteClassName) : "";
			return escapedName ? ResultUtil.insertWordBreaks(escapedName, [".","_"]): "";
		}
		if(cellType == Constants.JUNIT_CLASS_NAME_COLUMN){
			var escapedName = rowData.className ? jazz.util.html.escape(rowData.className) : "";
			return escapedName ? ResultUtil.insertWordBreaks(escapedName, [".","_"]): "";
		}
		if(cellType == Constants.JUNIT_CASE_NAME_COLUMN){
			var escapedName = rowData.name ? jazz.util.html.escape(rowData.name) : "";
			return escapedName ? ResultUtil.insertWordBreaks(escapedName, [".","_"]): "";
		} 
	},
	
	getCellIconClass: function(cellType) {
		var rowData = this.rowContainer.getRowData();
		if (cellType == Constants.JUNIT_COMPONENT_NAME_COLUMN) {
			return rowData.errorCount || rowData.failureCount ? "component-error-icon" : "component-success-icon";
		}
		if (cellType == Constants.JUNIT_SUITE_NAME_COLUMN ||
				cellType == Constants.JUNIT_CLASS_NAME_COLUMN) {
			return rowData.errorCount || rowData.failureCount ? "junit-error-icon" : "junit-success-icon";
		}
		if (cellType == Constants.JUNIT_CASE_NAME_COLUMN) {
			return rowData.error || rowData.failure ? "junit-error-icon" : "junit-success-icon";
		}
		return "";
	}
	
});
	
})();
