/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.users.dialogs.AbsenceEditDialog"); //$NON-NLS-1$

dojo.require("jazz.ui.Dialog"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.form.DateTextBox"); //$NON-NLS-1$

dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {
	
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
var bind= dojo.string.substitute;

dojo.declare("com.ibm.team.apt.web.ui.internal.users.dialogs.AbsenceEditDialog", null, { //$NON-NLS-1$
	
	constructor: function(params){
		var widgetParams = { primaryTitle: MSGS.USER_EDITOR_ABSENCES_DIALOG_EDIT_ABSENCE };

		var onOk= dojo.hitch(this, function(attributes) {
			this.floatingPane.close();
			params.onOk({});
		});
		
		var onCancel= dojo.hitch(this, function(attributes) {
			this.floatingPane.close();
			params.onCancel({});
		}); 
		
		this.contentWidget= new com.ibm.team.apt.web.ui.internal.users.dialogs.AbsenceEditDialogContent({
			onOk: onOk,  
			onCancel: onCancel,
			absence: params.absence
		});
		widgetParams.contentNode = this.contentWidget.domNode;
		this.floatingPane = new jazz.ui.Dialog(widgetParams); 
		this.contentWidget.init();
	}
	
});

dojo.declare("com.ibm.team.apt.web.ui.internal.users.dialogs.AbsenceEditDialogContent", //$NON-NLS-1$
	[dijit._Widget, dijit._Templated], {
					
	templatePath: dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/users/dialogs/templates/AbsenceEditDialogContent.html"), //$NON-NLS-1$ //$NON-NLS-2$
					
	__okOk: null,
	__onCancel: null,
	__absence: null,
	
	__messages: MSGS,
	
	constructor: function(params){
		this.__onOk= params.onOk;
		this.__onCancel= params.onCancel;
		this.__absence= params.absence;
		
		this.__calendarIcon= dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/users/images/outofoffice_obj.gif"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	postCreate: function() {
		this.connect(this._okCmd, "onclick", dojo.hitch(this, function() { //$NON-NLS-1$
			this.__absence.description= this._input.value;
			this.__absence.startDate= this.__startDateWidget.attr("value"); //$NON-NLS-1$
			this.__absence.endDate= this.__endDateWidget.attr("value"); //$NON-NLS-1$
			
			this.__onOk();
		}));
		this.connect(this._cancelCmd, "onclick", this.__onCancel); //$NON-NLS-1$
		
		this._input.value= this.__absence.description;
		this.connect(this._input, "onkeypress", dojo.hitch(this, function(e) { //$NON-NLS-1$
			this.__updateState(this._input.value + e.keyChar);
		}));
		
		var constraints = {};
		if(dojo.i18n.normalizeLocale() == "en-us") { //$NON-NLS-1$
			constraints = { datePattern: "MMM d, yyyy"	};  //$NON-NLS-1$
		}
		this.__startDateWidget= new dijit.form.DateTextBox({
			isDate: true, 
			containerToggle: "fade",  //$NON-NLS-1$
			constraints: constraints 
		}, this._startDate);
		this.__startDateWidget.attr("value", this.__absence.startDate); //$NON-NLS-1$
		
		this.__startDateWidget.domNode.style.margin = "0"; //$NON-NLS-1$
		this.__startDateWidget.domNode.style.fontFamily = "inherit"; //$NON-NLS-1$
		
		this.connect(this.__startDateWidget, "validate", dojo.hitch(this, function(e) { //$NON-NLS-1$
			this.__updateState(this._input.value);
		}));
		
		this.connect(this.__setStartDate, "onclick", dojo.hitch(this, function(e) { //$NON-NLS-1$
			dojo.stopEvent(e);
			
			this.__startDateWidget.focus();
		}));
		
		this.__endDateWidget= new dijit.form.DateTextBox({
			isDate: true, 
			containerToggle: "fade",  //$NON-NLS-1$
			constraints: constraints 
		}, this._endDate);
		this.__endDateWidget.attr("value", this.__absence.endDate); //$NON-NLS-1$
		
		this.__endDateWidget.domNode.style.margin = "0"; //$NON-NLS-1$
		this.__endDateWidget.domNode.style.fontFamily = "inherit"; //$NON-NLS-1$
		
		this.connect(this.__endDateWidget, "validate", dojo.hitch(this, function(e) { //$NON-NLS-1$
			this.__updateState(this._input.value);
		}));
		
		this.connect(this.__setEndDate, "onclick", dojo.hitch(this, function(e) { //$NON-NLS-1$
			dojo.stopEvent(e);
			
			this.__endDateWidget.focus();
		}));
		
		this._okCmd.disabled= true;
	},
	
	init: function() {
	},
	
	__updateState: function(description) {
		this._okCmd.disabled= true;
		
		try {
			if (description == "") //$NON-NLS-1$
				return;
			
			var startDate= this.__startDateWidget.attr("value"); //$NON-NLS-1$
			if (!startDate)
				return;
			
			var endDate= this.__endDateWidget.attr("value"); //$NON-NLS-1$ 
			if (!endDate)
				return;
			
			if (endDate < startDate)
				return;
			
			this._okCmd.disabled= false;
		} catch (e) {
			return;
		}
	},
	
	__sentinel: null
});

})();
