/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemAttachments");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("dojo.io.iframe");

dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("jazz.ui.ImagePreview");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() { // module container

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;

var ImageClient				= com.ibm.team.workitem.web.client.internal.ImageClient;
var WorkItemClient			= com.ibm.team.workitem.web.client.internal.WorkItemClient;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;

var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;
var ImagePool 				= com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
var ImagePreview			= jazz.ui.ImagePreview;

var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var isGuest 				= com.ibm.team.repository.web.client.session.isGuest;

var UPLOAD_SERVICE_URI   = "com.ibm.team.workitem.service.internal.rest.IAttachmentRestService/";
var DOWNLOAD_SERVICE_URI = "com.ibm.team.workitem.service.internal.rest.IAttachmentRestService/repo/csid/Attachment/";

var ACTION_PROP          = "__jazzActionProp";
var FILE_UPLOAD_RESPONSE_PREFIX = "File Upload Response:"; // Must match response prefix in AttachmentRestService.java %NON-NLS-1% 
var EditorUtil	         = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var WorkItemUtil         = com.ibm.team.workitem.web.client.util;

var NEW_ATTACHMENT;
var UNKNOWN;
var _BYTES;

var SORT_DOWN = -1;
var SORT_DEFAULT = 0;
var SORT_UP = 1;

var _currentBindRequest;
var _currentUploadFrameNumber = 100;

var COLUMN_CONFIG;

function displayBlock(node, isDisplayed) {
	node.style.display= isDisplayed ? "block" : "none";
}


/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemAttachments",
	[dijit._Widget, dijit._Templated],
	{
		_isUploadInProgress: false,
		_wasUploadCancelled: false,
		_priorFullPath: null,
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/editor/templates/WorkItemAttachments.html"),
		// get styles from WorkItemEditor.css
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
			NEW_ATTACHMENT = this.messages.labelNow;
			UNKNOWN = this.messages.labelUnknown;
			_BYTES = this.messages.suffixBytes;
			this.labelAttachmentAddFile = this.messages.labelAttachmentAddFile;
			this.labelBusyUploading = this.messages.busyUploading;
			this.labelAttachmentCancelUpload = this.messages.labelAttachmentCancelUpload;
			this.titleAttachmentCancelUpload = this.messages.titleAttachmentCancelUpload;
			this.msgAttachmentNotLoggedIn = this.messages.msgAttachmentNotLoggedIn;
			this.labelNoAttachments = this.messages.labelNoAttachments;
		COLUMN_CONFIG = [
		 {
			label: this.messages.labelID,
			headerClass:"id-header-column",
			dataFunction: "_addIdCell",
			isRedWhenDeleted: true,
			align: "right",
			sortUp: function(a, b) {
						return b.linkDTO.target.id - a.linkDTO.target.id;
					},
			sortDown: function(a, b) {
						return a.linkDTO.target.id - b.linkDTO.target.id;
					}
		},
		 {
			label: this.messages.labelName,
			headerClass:"name-header-column",
			dataFunction: "_addFileNameCell",
			isRedWhenDeleted: true,
			sortUp: function(a, b) {
				         if(a.linkDTO.target.name < b.linkDTO.target.name) return 1;
						return -1;
					},
			sortDown: function(a, b) {
				         if(a.linkDTO.target.name < b.linkDTO.target.name) return -1;
						return 1;
					}
		},
		 {
			label: this.messages.labelSize,
			headerClass:"size-header-column",
			align: "right",
			dataFunction: "_addSizeCell",
			sortUp: function(a, b) {
						return b.linkDTO.target.contentLength - a.linkDTO.target.contentLength;
					},
			sortDown: function(a, b) {
						return a.linkDTO.target.contentLength - b.linkDTO.target.contentLength;
					}
		},
		{
			label: this.messages.labelCreatedBy,
			headerClass:"creator-header-column",
			dataFunction: "_addCreatorCell",
			sortUp: function(a, b) {
				         if(a.linkDTO.target.creator.name < b.linkDTO.target.creator.name) return 1;
						return -1;
					},
			sortDown: function(a, b) {
				         if(a.linkDTO.target.creator.name < b.linkDTO.target.creator.name) return -1;
						return 1;
					}
		},		
		{
			label: this.messages.labelDate,
			headerClass:"date-header-column",
			dataFunction: "_addDateCell",
			sortUp: function(a, b) {
						var aModified = 0;
						if(a.modified && a.modified.sortDate) {
							aModified= a.modified.sortDate.getTime();
						}
						
						var bModified = 0;
						if(b.modified && a.modified.sortDate) {
							bModified = b.modified.sortDate.getTime();
						}
						
						return bModified - aModified;
					},
			sortDown: function(a, b) {
						var aModified = 0;
						if(a.modified && a.modified.sortDate) {
							aModified= a.modified.sortDate.getTime();
						}
						
						var bModified = 0;
						if(b.modified && a.modified.sortDate) {
							bModified = b.modified.sortDate.getTime();
						}
						
						return aModified - bModified;
					}
	
		}
	] // end COLUMN_CONFIG
						
		},
		postCreate: function() {

			this._headerCells = [];
			this._initializeUploadInput();
			this._defaultSort = COLUMN_CONFIG[4].sortUp;
			this._sortFunction = this._defaultSort;
			this.connect(this._cancelUploadButton, "onclick",  "_handleCancelClick");
				
			this.connect(this._uploadForm, "onsubmit", "_performUploadSubmit");

			if(isGuest()) {
				this._attachmentHeaderArea.className = "attachmentHeaderArea-Guest";
			}
			
			this._initializeHeader();
			this.workingCopy.addListener({
				path: ["linkTypes",/* id */ "com.ibm.team.workitem.linktype.attachment"],
				event: "onchange",
				listener: this,
				functionName: "_handleWorkingCopyChange"
			});			
		},
		_handleWorkingCopyChange: function(parms) {
			this.refresh();
		},
		_getAttachmentLinkDTOs: function() {
			return this.workingCopy.getValue({
				path: ["linkTypes", /* id */ "com.ibm.team.workitem.linktype.attachment", "linkDTOs"],
				defaultValue: []
			});
		},
		_setWorkingCopyValue: function() {
			var linkDTOs = [];
			for(var i=0;i<this._attachmentArray.length; i++) {
				linkDTOs.push(this._attachmentArray[i].linkDTO)
			}
	
			this.workingCopy.setValue({
				caller: this,
				path: ["linkTypes", /* id */ "com.ibm.team.workitem.linktype.attachment", "linkDTOs"],
				value: linkDTOs,
				forceNotify: true
			});
		},
		destroyWidget: function() {
//			this.parentController.pageController.removeAllMouseListeners(this);
			if(this._imagePreview1) this._imagePreview1.destroy(); 
			if(this._imagePreview2) this._imagePreview2.destroy();
			if(this._uploadFileInput_onchange_handle) dojo.disconnect(this._uploadFileInput_onchange_handle);
			this.destroy();
		},
		showWidget: function() {
			this._showUploadFormOrMessage();
		},
		_initializeHeader: function() {
			var line= this._attachmentTableHead.insertRow(0);
			var row= this._attachmentTableHead.insertRow(1);
			
			for(var i=0;i< COLUMN_CONFIG.length;i++) {
				var lineNode = document.createElement("th");
				lineNode.className = "header-line";
				lineNode.innerHTML = "&nbsp;";
				line.appendChild(lineNode);
				this._addHeaderColumn(row,COLUMN_CONFIG[i]);
			}			
			var lineNode = document.createElement("th");
			lineNode.className = "header-line";
			lineNode.innerHTML = "&nbsp;";
			line.appendChild(lineNode);
			
			// Add a column for the delete/restore button
			var cell= document.createElement("TH");
			cell.appendChild(document.createTextNode(" ")); 
			row.appendChild(cell);
		},
		_addHeaderColumn: function(row, config) {
			var cell = document.createElement("TH"); 
			row.appendChild(cell);
			var nobr = document.createElement("nobr");
			cell.__jazzConfig = config;
			cell.__jazzConfig.sortState = SORT_DEFAULT;
			cell.className = config.headerClass;
			var img = document.createElement("IMG");
			img.src = ImagePool.PIXEL;
			img.className = "header-img";

			var span = document.createElement("SPAN");
			span.innerHTML = config.label;
			if(config.align == "right") {
				nobr.appendChild(img);
				nobr.appendChild(span);
			} else {
				nobr.appendChild(span);
				nobr.appendChild(img);				
			}
					

			cell.appendChild(nobr);
			cell.__img = img; 
			this._headerCells.push(cell);
			this.connect(cell, "onmouseover", dojo.hitch(this, this._handleHeaderRowMouseOver));
			this.connect(cell, "onmouseout", dojo.hitch(this, this._handleHeaderRowMouseOut));
			this.connect(cell, "onmousedown", dojo.hitch(this, this._handleHeaderRowClick));

		},
		_handleHeaderRowMouseOver: function(e)  {
			var cell = e.currentTarget;
			cell.style.backgroundColor = "#FBFBFB";
		},
		_handleHeaderRowMouseOut: function(e)  {
			var cell = e.currentTarget;
			cell.style.backgroundColor = "";			
		},
		_handleHeaderRowClick: function(e)  {
			var cell = e.currentTarget;
			var sortState = cell.__jazzConfig.sortState;
			for(var i =0;i<this._headerCells.length;i++) {
				this._headerCells[i].__img.src = ImagePool.PIXEL;
				this._headerCells[i].__jazzConfig.sortState = SORT_DEFAULT;
			}
			switch(sortState) {
				case SORT_UP: // up to default
					cell.__img.src = ImagePool.PIXEL;
					cell.__img.style.visibility = "hidden";
					cell.__jazzConfig.sortState = SORT_DEFAULT;
					this._sortData(cell.__jazzConfig, SORT_DEFAULT);
					break;
				case SORT_DOWN:
					cell.__img.src = ImagePool.SORT_UP;
					cell.__img.style.visibility = "visible";
					cell.__jazzConfig.sortState = SORT_UP;
					this._sortData(cell.__jazzConfig, SORT_UP);
					break;
				default: // default to down
					cell.__img.src = ImagePool.SORT_DOWN;
					cell.__img.style.visibility = "visible";
					cell.__jazzConfig.sortState = SORT_DOWN;
					this._sortData(cell.__jazzConfig, SORT_DOWN);

			}
		},
		_sortData: function(config, sortState) {
			switch(sortState) {
				case SORT_UP:
						this._sortFunction = config.sortUp;
						break;
				case SORT_DOWN:
						this._sortFunction = config.sortDown;
						break;
				default:
						this._sortFunction = this._defaultSort;
			}
			this._rebindAttachments();
		},
		reset: function() {
			
			this._attachmentArray= [];
			
			// remove all rows 
			dojox.data.dom.removeChildren(this._attachmentTableBody);
			
			// initialize basic user interface based on browser and logged-in state
			
			// show the upload form if the user is logged in
			this._showUploadFormOrMessage();

			this._initializeUploadInput(); // reset the form
		
			//TODO: Cancel the upload operation
			displayBlock(this._uploadInProgressMessageArea, /*isDisplayed*/ false);
			
			// show the log in required message if the user is a guest;
			// otherwise hide
			displayBlock(this._loginRequiredMessageArea, /* isGuest() */ false );
			
		},
		
		// IE does not support setting the value of an input type=file, so the only way to clear the form
		// is to destroy it and recreate it
		_initializeUploadInput: function() {

//		<input type="file" name="uploadFileInput" dojoAttachPoint="_uploadFileInput" class="UploadControl"/>
			var div= this._uploadInputDiv;
			dojox.data.dom.removeChildren(div);
			var input1= document.createElement("INPUT");
			input1.type= "file";
			input1.name= "uploadFileInput";
			input1.className="UploadControl";
			this._uploadFileInput = input1;
			this._uploadFileInput_onchange_handle = dojo.connect(this._uploadFileInput, "onchange", this, "_performUploadSubmit");
			div.appendChild(input1);
		},

		refresh: function() {
			
			var attachmentLinkDTOs = this._getAttachmentLinkDTOs();
			this._attachmentArray= [];
								
			// bail out early if there are no attachments
			if (attachmentLinkDTOs.length==0) {
				this._attachmentTable.style.display="none";
				this._noAttachmentsMessageArea.style.display="inline";
				return; }

			this._noAttachmentsMessageArea.style.display="none";
			this._attachmentTable.style.display="";
			// otherwise display the attachments in the table
			
			
			for (var i=0; i<attachmentLinkDTOs.length; i++) {
				var target = attachmentLinkDTOs[i].target;
				if(!target) continue;
				target.name = target.name ? target.name : UNKNOWN;
				var id = target.id ? target.id : "";

				
				var modified = "";
				if(target.creationDate) {
					modified = WorkItemUtil.getAllTimestamps(target.creationDate);
				}
				
				var attachment= {
					linkDTO: attachmentLinkDTOs[i],
					modified: modified,
					path: this.messages.labelStoredInRepository
				};
				this._attachmentArray.push(attachment);
			}
			this._rebindAttachments();
		},
		_rebindAttachments: function() {
			// remove all current rows
			dojox.data.dom.removeChildren(this._attachmentTableBody);
						
			this._attachmentArray.sort( this._sortFunction);
			for (var i = 0; i < this._attachmentArray.length; i++) {
				this._addAttachmentRow(this._attachmentArray[i]);
			}
		},
		_addAttachmentRow: function(attachment, isFirst) {
			this._noAttachmentsMessageArea.style.display="none";
			this._attachmentTable.style.display="";

			var tbody= this._attachmentTableBody;
			var row, cell, img;
//			if(isFirst) {
//				row= tbody.insertRow(0);
//			} else {
				row= tbody.insertRow(-1);
//			}
			
			for(var i=0;i< COLUMN_CONFIG.length;i++) {
				cell = row.insertCell(-1);
				cell.__jazzConfig = COLUMN_CONFIG[i];
				cell.__jazzAttachment = attachment;
				this[COLUMN_CONFIG[i].dataFunction](cell, attachment);
				if(attachment.linkDTO.isRemoved && COLUMN_CONFIG[i].isRedWhenDeleted) {
					dojo.addClass(cell,  "cellDeleted");
				}
			}
			cell = row.insertCell(-1);
			cell.className="action-column";
			if(attachment.linkDTO._isNew) {
				row.style.backgroundColor = "#F5F8FB";
			}
			var self = this;
			if(attachment.linkDTO.isRemoved) {
				row.style.backgroundColor = "#FFF9EF";
				img = document.createElement("IMG");
				img.src =  ImagePool.RESTORE;
				img.title = this.messages.titleAttachmentRestore
				// TODO: put back hovers - task 82207
//				img.onmouseover = function(e) {
//					img.src =  ImagePool.RESTORE_HOVER;
//				}
//				img.onmouseout = function(e) {
//					img.src =  ImagePool.RESTORE;
//				}
				cell.onmousedown = function(e) {
					self._performRestore(attachment);
				}
			} else {
				img = document.createElement("IMG");
				img.src =  ImagePool.REMOVE;
				img.title = this.messages.titleAttachmentDelete
	//			img.onmouseover = function(e) {
	//				img.src =  ImagePool.REMOVE_HOVER;
	//			}
	//			img.onmouseout = function(e) {
	//				img.src =  ImagePool.REMOVE;
	//			}
				cell.onmousedown = function(e) {
					self._performRemove(attachment);
				}
				if (!attachment.linkDTO._isNew) {
					img.style.visibility = "hidden";
				}
			}
			
			row.onmouseover = function(e){
				if (attachment.linkDTO.isRemoved) {
					row.style.backgroundColor = "#FFF4E1";
				}
				else {
					if (attachment.linkDTO._isNew) {
						row.style.backgroundColor = "#EEF4F8";
					}
					else {
						row.style.backgroundColor = "#F6F6F6";
						img.style.visibility = "visible";
					}
				}
			};
			row.onmouseout = function (e) {
				if (attachment.linkDTO.isRemoved) {
					row.style.backgroundColor = "#FFF9EF";
				}
				else {
					if (attachment.linkDTO._isNew) {
						row.style.backgroundColor = "#F5F8FB";
					}
					else {
						row.style.backgroundColor = "";
						img.style.visibility = "hidden";
					}
				}
			};				
			
			cell.appendChild(img);
			
		},
_addIdCell: function(cell, attachment) {
			cell.className="id-data-column"

			var id = attachment.linkDTO._isNew ? this.messages.labelNew : attachment.linkDTO.target.id;
			
			var anchor = this._getSaveAnchor(attachment, attachment.linkDTO.target.name, id);
			cell.appendChild(anchor);

			anchor.popupFileName = attachment.linkDTO.target.name;
			anchor.parentCell = cell;
			var index = attachment.linkDTO.target.name.lastIndexOf(".");
			var ext = index == -1 ? null : attachment.linkDTO.target.name.substring(index + 1);
			ext = ext ? ext.toLowerCase() : ext;
			if (ext == "gif" || ext == "png" || ext == "jpeg" || ext == "jpg") {
//				if(this._imagePreview1) this._imagePreview1.destroy();
				this._imagePreview1 = new ImagePreview({around:[anchor], src: anchor.href, filename: attachment.linkDTO.target.name}, document.createElement("div"));
			} else {
				anchor.title = attachment.linkDTO.target.name;				
			}
				
},
_addFileNameCell: function(cell, attachment) {

			var anchor = this._getSaveAnchor(attachment, attachment.linkDTO.target.name);
			cell.appendChild(anchor);
			var self=this;

			anchor.popupFileName = attachment.linkDTO.target.name;
			anchor.parentCell = cell;
			var index = attachment.linkDTO.target.name.lastIndexOf(".");
			var ext = index == -1 ? null : attachment.linkDTO.target.name.substring(index + 1);
			ext = ext ? ext.toLowerCase() : ext;
			if (ext == "gif" || ext == "png" || ext == "jpeg" || ext == "jpg") {
//				if(this._imagePreview2) this._imagePreview2.destroy();
				this._imagePreview2 = new ImagePreview({around:[anchor], src: anchor.href, filename: attachment.linkDTO.target.name}, document.createElement("div"));
			} else {
				anchor.title = attachment.linkDTO.target.name;				
			}
	
},
_addCreatorCell: function(cell, attachment) {
			if(!attachment.linkDTO || !attachment.linkDTO.target || !attachment.linkDTO.target.creator) {
				cell.appendChild(document.createTextNode(" "));
				return;
			}
			var creator = attachment.linkDTO.target.creator;
			if(creator.emailAddress) {
				var anchor = document.createElement("A");
				var span = document.createElement("SPAN"); // extra span needed for IE when @ appears in creator.name
				span.appendChild(document.createTextNode(creator.name || " "));
				anchor.appendChild(span);
				anchor.href=this.parentController._getMailto(creator.emailAddress);
				cell.appendChild(anchor);
			} else {
				cell.appendChild(document.createTextNode(creator.name || " "));
			}

	
},
_addSizeCell: function(cell, attachment) {

			cell.className="size-data-column"
			sizeSpan= document.createElement("span");

			var sizeText = "" + Math.ceil(attachment.linkDTO.target.contentLength / 1024);

			var rgx = /(\d+)(\d{3})/;
			while (rgx.test(sizeText)) {
				sizeText = sizeText.replace(rgx, '$1' + ',' + '$2');
			}
	
			var fullSizeText = "" + attachment.linkDTO.target.contentLength;
			var rgx = /(\d+)(\d{3})/;
			while (rgx.test(fullSizeText)) {
				fullSizeText = fullSizeText.replace(rgx, '$1' + ',' + '$2');
			}
			
			fullSizeText = dojo.string.substitute(this.messages.labelBytes, [fullSizeText]);		
			sizeText = dojo.string.substitute(this.messages.labelKBytes, [sizeText]);
			
			dojox.data.dom.replaceChildren(sizeSpan, document.createTextNode(sizeText));
			sizeSpan.title = fullSizeText;
			
			cell.appendChild(sizeSpan);
	
},
_addDateCell: function(cell, attachment) {
			cell.className = "attachment-date";
			var modifiedSpan= document.createElement("span");
			var modifiedText = "";
			if(attachment.modified) {
				modifiedText =attachment.modified.printTimestamp;
				modifiedSpan.title = attachment.modified.formattedTimestamp;				
			}
			
			dojox.data.dom.replaceChildren(modifiedSpan, document.createTextNode(modifiedText));
			cell.appendChild(modifiedSpan);	
},
		_getSaveAnchor: function(attachment, text, altText) {
				var saveAnchor= document.createElement("a");
//				saveAnchor.className= "actionLink";
				saveAnchor.href = TeamServerClient._getServicePrefix() + DOWNLOAD_SERVICE_URI +  text + "?itemId=" + attachment.linkDTO.itemId;
				saveAnchor.target = "_blank";
				if(altText) {
					saveAnchor.appendChild(document.createTextNode(altText));
				} else {
					saveAnchor.appendChild(document.createTextNode(text));
				}
				
				return saveAnchor;
		},
	
		_performRemove: function(attachment) {
			this.parentController.markDirty();
	 		for (var i=0; i<this._attachmentArray.length; i++) {
		 		if (this._attachmentArray[i].linkDTO.itemId == attachment.linkDTO.itemId) {
		 			if (this._attachmentArray[i].linkDTO._isNew) {
						this._attachmentArray.splice(i, 1);
					} else {
						this._attachmentArray[i].linkDTO.isRemoved = true;
					}
		 			break;
		 		}
	 		}
			this._setWorkingCopyValue();
			this._rebindAttachments();
		},
		
		_performRestore: function(attachment) {

	 		for (var i = 0; i< this._attachmentArray.length; i++) {
	 			if (this._attachmentArray[i].linkDTO.itemId == attachment.linkDTO.itemId) {
	 				this._attachmentArray[i].linkDTO.isRemoved = false;
	 				break;
	 			}
	 		}
			this._setWorkingCopyValue();
			this._rebindAttachments();
		},
		_showUploadFormOrMessage: function() {
			if(this._isUploadInProgress) {
				return;
			}
			displayBlock(this._uploadForm, !isGuest());
		},
			// hide progress message and show clearedupload form
		_showFormHideMessage: function() {
				this._isUploadInProgress = false;
				displayBlock(this._uploadInProgressMessageArea, /*isDisplayed*/ false);
				this._initializeUploadInput(); // reset the form
				this._showUploadFormOrMessage();
		},
		isUploadInProgress: function() {
			return this._isUploadInProgress;			
		},
		_handleCancelClick: function(e) {
			e.preventDefault();
			this.cancelUploadInProgress();
		},
		cancelUploadInProgress: function() {
			if(this._currentBindRequest) {
				this._currentBindRequest.cancel();
				if(dojo.isIE) {
					dojo.io.iframe.setSrc (dojo.io.iframe._frame, "", true);
					this._currentBindRequest.isCancelled = true;
				} 
				if(dojo.isMozilla){
					window.stop();
				}
			}
			
			this._showFormHideMessage();
		},
		_performUploadSubmit: function(e, isRestart){
			if (e) {
				e.preventDefault();
			}
			this._isUploadInProgress = true;
			// we always rebuild the input, so remove the change event
			dojo.disconnect(this._uploadFileInput_onchange_handle);
			
			// hide upload form and show progress message
			displayBlock(this._uploadForm, /*isDisplayed*/ false);
			displayBlock(this._uploadInProgressMessageArea, /*isDisplayed*/ true);
			
			var fullPath;
			// Check to see if the file is already loaded
			if (isRestart) {
				fullPath = this._priorFullPath;
			}
			else {
				fullPath = this._uploadFileInput.value;
			}
			if (fullPath.length === 0) {
				this._showFormHideMessage();
				return;
			}
			
			this._priorFullPath = fullPath;
			var parts = fullPath.split("\\");
			var fileName = parts[parts.length - 1];
			
			
			for (var i = 0; i < this._attachmentArray.length; i++) {
				if (this._attachmentArray[i].path == fullPath) {
					alert(dojo.string.substitute(this.messages.msgAttachmentAlreadyAdded, [fullPath]));
					this._showFormHideMessage();
					return;
				}
			}
			
			var self = this;
			if (this._currentBindRequest != null) {
				this._currentBindRequest.cancel();
				this._currentBindRequest = null;
			}
			
			/*
	 * Internet Explorer (with dojo) has a bug such that if you wait ten seconds after one file upload, the
	 * next one breaks. Resetting the iframe fixes the problem
	 * Unfortunately this code reaches into iframe private variables.
	 */
			if (dojo.io.iframe._frame) {
				document.body.removeChild(dojo.io.iframe._frame);
			}
			frameName = "dojoIoIframe" + _currentUploadFrameNumber++;
			dojo.io.iframe._iframeName = frameName;
			dojo.io.iframe._frame = null;
			var parms = "";
			
			var target = this.workingCopy.getValue({ path: ["attributes", "target", "id"] });			
			if (target) {
				parms += "&target=" + escape(target);
			}
			
			var category = this.workingCopy.getValue({ path: ["attributes", "category", "id"] });
			if (category) {
				parms += "&category=" + escape(category);
			}
			
			this._currentBindRequest = dojo.io.iframe.send({
				form: this._uploadForm,
				url: TeamServerClient._getServicePrefix() + UPLOAD_SERVICE_URI + "?projectId=" + this.workingCopy.getProjectAreaItemId() + parms,
				content: {
					mimetype: "text/html",
					fileFields: "uploadFileInput"
				},
				handleAs: "html",
				error: function(response, ioArgs) {
							if(response.dojoType && response.dojoType == "cancel") {
								return; // this is a cancelled request, ignore it
							}
							self._currentBindRequest  = null; // no need to cancel it any more
							alert(self.messages.msgAttachmentUploadError);
							self._showFormHideMessage();
							console.log(dojo.string.substitute(self.messages.errorAttachmentUpload, [response.message]));
							return;
				},
				handle: function(response, ioArgs){
					if (response instanceof Error) {
						if (response.dojoType && response.dojoType == "cancel") {
							return; // this is a cancelled request, ignore it
						}
						self._currentBindRequest = null; // no need to cancel it any more
						alert(self.messages.msgAttachmentUploadError);
						self._showFormHideMessage();
						console.log(dojo.string.substitute(self.messages.errorAttachmentUpload, [response.message]));
						return;
					}
					data = response;
					// If the user presses cancel on Firefox or Stop, then we get an extra call on the next upload. Skip it
					// The data URI of the bad call is for the page and not the service
					if (data && data.documentURI && data.documentURI.indexOf("IAttachmentRestService") == -1) {
						self._wasUploadCancelled = true;
						self._performUploadSubmit(null, true);
						return;
					}
					if (self._currentBindRequest && self._currentBindRequest.isCancelled) {
						self._currentBindRequest = null; // no need to cancel it any more
						return;
					}
					self._currentBindRequest = null; // no need to cancel it any more
					var uuidOrMessage = data.body.innerHTML; /* innerHTML OK */
					if (uuidOrMessage.indexOf(FILE_UPLOAD_RESPONSE_PREFIX) != 0) {
  						var message, messageDetails;
					    if(uuidOrMessage && uuidOrMessage.length > 0) {
					    	message = uuidOrMessage;
							messageDetails = uuidOrMessage;
							if(message.indexOf("&gt;") == 0) {
								message = dojo.string.substitute(self.messages.errorFileSizeExceedsMaximumAllowed, [uuidOrMessage.replace(/&gt;/gi, "")]);
							} else {
								message = self.messages.errorAttachmentServerNotResponding;
							}
					    } else {
					    	message = self.messages.errorAttachmentServerNotResponding;
					    	messageDetails = message;
					    }
						alert(message);
						console.log(dojo.string.substitute(self.messages.errorAttachmentUploadFailed, [messageDetails]));
						self._showFormHideMessage();
						return;
					}
					uuidOrMessage = uuidOrMessage.substring(FILE_UPLOAD_RESPONSE_PREFIX.length, uuidOrMessage.length);
					if (uuidOrMessage.substring(0, 1) != "_") {
						// If an upload was cancelled we get one of these anomolous calls
						if (self._wasUploadCancelled) {
							self._wasUploadCancelled = false;
							return;
						}
						alert(self.messages.errorAttachmentServerNotResponding);
						console.log(dojo.string.substitute(self.messages.errorAttachmentUploadFailed, [uuidOrMessage]));
						self._showFormHideMessage();
						return;
					}
					
					var sep = uuidOrMessage.indexOf(',');
					var size = 0;
					if (sep > 0) {
						size = uuidOrMessage.substring(sep + 1, uuidOrMessage.length);
						uuidOrMessage = uuidOrMessage.substring(0, sep);
					}
					
					if (size == "0") {
						if (!confirm(dojo.string.substitute(self.messages.promptAttachmentUploadSizeEmptyConfirm, [fileName]))) {
							self._showFormHideMessage();
							return;
						}
					}
					
					var attachment = {
						modified: {
							printTimestamp: NEW_ATTACHMENT,
							formattedTimestamp: NEW_ATTACHMENT,
							sortDate: new Date()
						},
						path: fullPath,
						linkDTO: {
							id: "com.ibm.team.workitem.linktype.attachment",
							_isNew: true,
							itemId: uuidOrMessage,
							target: {
								contentLength: size,
								id: "",
								creator: getAuthenticatedContributor(),
								name: fileName
							}
						}
					};
					self.parentController.markDirty();
					self._attachmentArray.push(attachment);
					self._setWorkingCopyValue();
					self._rebindAttachments();
					
					self._showFormHideMessage();
				}
			});
		}
	}
);

})();
