/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.cache.internal.WorkItemProxy");
dojo.provide("com.ibm.team.workitem.web.cache.internal.WorkItemProxyClient");
dojo.require("com.ibm.team.workitem.web.cache.internal.ItemProxy"); // required here for debugging
dojo.require("com.ibm.team.workitem.web.cache.internal.ItemListProxy"); // required here for debugging
dojo.require("com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.parts.ProcessAreaListPart");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.parts.WorkItemAnchorAttributePart");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.parts.AnchorAttributePart");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.parts.ProcessAreaAnchorAttributePart");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.parts.WorkItemListPart");

dojo.require("dojox.date.posix");

dojo.require("com.ibm.team.workitem.web.cache.internal.Cache");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.client.util");

// editor parts
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorHeader");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemOverview");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemTabContentLayout");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemClient			= com.ibm.team.workitem.web.client.internal.WorkItemClient;
var WorkItemUtil            = com.ibm.team.workitem.web.client.util;
var EditorUtil            = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var WorkItemProxy			= com.ibm.team.workitem.web.cache.internal.WorkItemProxy;
var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;

var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;

var getAuthenticatedContributor	= com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var isGuest 					= com.ibm.team.repository.web.client.session.isGuest;

var DEFAULT_TAB             = "overview"; // $NON-NLS-1$
var DATE_FORMAT_NO_TIME     = "%b %e, %Y";


/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditor",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/editor/templates/WorkItemEditor.html"),
		// templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/view/editor/templates/WorkItemEditor.css"),
		tabSpecs: {
				builtInOverviewLayout: {
//						title: "Overview",
						renderer: "_renderOverview",
						knownTab: "overview"
						},
				builtInLinksLayout: {
//						title: "Links",
						renderer: "_renderTabContent",
						knownTab: "links"
						},
				builtInApprovalsLayout: {
						renderer: "_renderTabContent",
						knownTab: "approvals"
						},
				builtInCustomAttributesLayout: {
//						title: "Custom",
						renderer: "_renderCustomAttributes"
						},
				builtInHLayout: {
						renderer: "_renderHLayout",
						slotMap: {
							left: "details",
							right: "description",
							top: "quickInfo",
							bottom: "discussion"
							}
					},
				builtInHistoryLayout: {
						renderer: "_renderTabContent",
						knownTab: "history"
						}
		},
		knownTabs: {
			links: false,
			approvals: false,
			overview: false
		},
		//
		// Widget initialization
		//
		
		/**
		 * Dojo widget lifecycle method; called once per web UI instance;
		 * responsible for creating header widget and subtab widgets.
		 */
		postCreate: function() {
			checkNotNull(this.parentController);
			
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages"); 
			
			this.isWorkItemEditor = true;
 			this.defaultTab = DEFAULT_TAB;

			// initialize editor parts map
			this._eventHandles = [];
			this._editorParts= {};
			this._linkManager = null;
			this.editorUtil= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
			this.pageController = this.parentController; // workItemPageController
			
			if (!this.isNew) {
				var createArgs = {
					parentController: this,
					editorUtil: this.editorUtil
				};
				
				// create the editor header
				this._editorParts["header"] = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorHeader(createArgs, this._headerPlaceholder);	
			}
			
			this._eventHandles.push(dojo.subscribe(EventTopics.WORKITEM_TYPE_CHANGED, this, this._workItemTypeChanged));
			this._eventHandles.push(dojo.subscribe(EventTopics.WORKITEM_EDITOR_CHANGED, this, this._workItemEditorChanged));
			this._eventHandles.push(dojo.subscribe(EventTopics.WORKITEM_ABOUT_TO_SAVE, this, this._workItemAboutToSave));
			this._eventHandles.push(dojo.subscribe(EventTopics.WORKITEM_PRESENTATION_CHANGED, this, this._workItemPresentationChanged));
			
		},
		_workItemPresentationChanged: function(parms) {
			// If the change is to our working copy...
			if(parms.workItemProxy === this.workingCopy) {
				this._refreshFromWorkingCopy();
			}
		},
		_workItemTypeChanged: function(parms) {
			// If the change is to our working copy...
			if(parms.workItemProxy === this.workingCopy) {
				this._refreshFromWorkingCopy();
			}
		},
		_workItemEditorChanged: function(workItemProxy, editor) {
			if(editor === this) return;
			if(workItemProxy) {
				if(workItemProxy === this.workingCopy) {
					if(workItemProxy.isDestroyed) {
						this.pageController.removeEditor(this.workingCopy.getId(), true);
					}
				}
			}
		},
		initializeWorkItemEditor: function(parms /* onSuccess, type, [tempId], [tab], [draftId]  */) {
			if(parms.isNew) {
				this.workingCopy = this._getThisWorkingCopy(parms.tempId, true);
			} else {
				this.workingCopy = this._getThisWorkingCopy(parms.id, true);
			}
			parms.controllerSuccessCallback = parms.onSuccess;
			parms.controllerErrorCallback = parms.onError;
			parms.fetchEditorPresentation = true;
			parms.fetchInlineEditorPresentation = false;
			parms.onSuccess = dojo.hitch(this, this._workingCopyInitialized);
			parms.onError = dojo.hitch(this, this._initializedNewWorkItemEditorError);
			if(this.workingCopy.isFetchedWithPresentation(parms) && !parms.isRefresh) {
				parms.workItem = this.workingCopy.object;
				this._workingCopyInitialized(parms);
			} else {
				this.workingCopy.initialize(parms); // Fetch work item and presentation
			}
		},
		_initializedNewWorkItemEditorError: function(parms) {
			if(parms.viewErrorCallback) {
				parms.viewErrorCallback(parms); // TODO: test this path
			}
		},
		/*
		 * The working copy now has a work item and a 
		 */
		_workingCopyInitialized: function(parms) {
			this.workingCopy.prepareToBeEdited();
			var createArgs= {
				parentController: this,
				editorUtil: this.editorUtil
			};			
			// create the editor header
			if(!this._editorParts["header"]) {
				this._editorParts["header"]= /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorHeader(
					createArgs, this._headerPlaceholder);				
			} else {
				this._editorParts["header"].endOperation();
			}

			// initialize dirty state
			this._isDirty= this.workingCopy.isChanged();
			if(this._isDirty) {
				this._editorParts.header.markDirty();
			}

			this._workItem = this.workingCopy.object;
			this._editorParts.header._workItem = this._workItem;
			this._editorParts.header.markBusy(this.messages.busyLoading);

			var isNew = this.workingCopy.isNewWorkItem(); 
			this._editorParts.header.focus(/* do resize */ false);
			
			this._editorParts.header.setHeaderToolbarState(isNew);
			this._editorParts.header.updateRefreshTimestamp(isNew);
									
			this._setEditableProperties(this.workingCopy.object, this.workingCopy.workItemSpec.editProps, this.workingCopy.workItemSpec.presentationProps, 
				parms.initialTab, parms.retainedState, parms.isTypeChange, parms.newTypeId);
			
			if(this._workItem && this._workItem.attributes && this._workItem.attributes.projectArea && this._workItem.attributes.projectArea.isArchived) {
					var msg = {
						message: this.messages.msgProjectAreaIsArchived, 
						action: "ALERT",
						isSticky: false,
						severity: 2 // info
					}
					this._editorParts.header.setHeaderMessage(msg);
			} else {
				this._checkProjectArea();
			}
			if(parms.controllerSuccessCallback) {
				parms.workItemEditor = this;
				parms.controllerSuccessCallback(parms);
			}

			if(this.workingCopy.infos) {
					this._editorParts.header._showInfosMessages({infos: this.workingCopy.infos, isMove:false});			
			} else {
				if(this.workingCopy.object && this.workingCopy.object.infos ) {
					this._editorParts.header._showInfosMessages({infos: this.workingCopy.object.infos, isMove:false});
				}
			}
			if(this.workingCopy.getId() < 0 || this.workingCopy.isChanged()) {
				this.markDirty();
			}
			this._enableMoveCopyCommand();
		},
		_enableMoveCopyCommand: function() {
			var srh= new com.ibm.team.repository.web.transport.ServiceResponseHandler(this, "_enableMoveCopyCommand2", "_handleGetProcessAreasError");
			WorkItemClient.getProcessAreaHierarchies(srh);
		},
		_enableMoveCopyCommand2: function(areas) {
			if(areas && areas.length > 0 && this._editorParts.header) {
				this._editorParts.header.displayMoveCopy(true);
			}
			},
		_handleGetProcessAreasError: function(e) {
			alert(e.message); // extremely rare
		}, 
		_workingCopyFetchError: function(errorObj) {
			this.parentController.handleError(errorObj, message);
		},
		_getThisWorkingCopy: function(id, createIfNeeded) {
			if (!this.workingCopy) {
				// First see if there is already a work item in the cache
				var workingCopy = com.ibm.team.workitem.web.cache.internal.Cache.getItem(id);
				if (workingCopy) {
					this.workingCopy = workingCopy;
				} else {
					if(createIfNeeded) {
						this.workingCopy = this._createThisWorkingCopy(id);
					} else {
						return null;
					}
				}
				if(this.workingCopy._isWorkItemCopyTarget) {
				   	this.workingCopy._isWorkItemCopyTarget = false; // now we're even
				} else {
            		this.workingCopy.incrementUseCount();					
				}
			}
			return this.workingCopy;
		},
		_createThisWorkingCopy: function(id) {
			return com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory.getWorkItemProxy({
				id: id,
				createIfNeeded: true
			});	
		},
		getDefaultTabName: function() {
			if(!this._editorParts || this._editorParts[this.defaultTab]) {
				return this.defaultTab;
			}
			for(var key in this._editorParts) {
				if(key != "header") {
					return key;
				}
			}
			return this.defaultTab ;
		},
		/* 
		 * DestroyWidget should be called whenever the page is destroyed and should
		 * propagate to all widgets.
		 */
		destroyWidget: function()  {
			for(var i=0;i<this._eventHandles.length;i++){
				dojo.unsubscribe(this._eventHandles[i]);
			}
			this._destroyChildWidgets();
			if(this.workingCopy) {
				this.workingCopy.decrementUseCount();
			}
			this.destroy();
		},
		_destroyChildWidgets: function(skipHeader) {
			for (var partName in this._editorParts) {
				if(skipHeader && partName == "header") continue;
				var editorPart= this._editorParts[partName];
				if(editorPart.destroyWidget) {
					editorPart.destroyWidget();	
				} else {
					if(editorPart.destroy) {
						editorPart.destroy();
					}
				}
				delete this._editorParts[partName];
			}
			this._linkManager = null;
			if(this.workingCopy) {
				this.workingCopy.removeAllListeners({listener: this, parentSelector: "parentWidget"});
				this.workingCopy.removeAllListeners({listener: this, parentSelector: "parentController"});
			}
		},

		/**
		 * Create a subtab for the work item editor.
		 * 
		 * @param {String} logicalName this name to identify the editor part via
		 * 	this._editorParts and also the name to specify which widget to show
		 *  in the viewWorkItem URL.
		 * @param {widget} widget is a widget to associate
		 *  with the reference from this._editorParts[logicalName].
		 * @param {Object} createArgs arguments with which to initialize the widget
		 * @param {String} tabLinkLabel the label of the hyperlink that allows users
		 *  to view the particular tab.
		 */
		_createTabPart: function(logicalName, widget,  tabLinkLabel) {
			if(logicalName) logicalName = escape(logicalName).replace(/\%/g, "_"); // NLS tab names
			this._editorParts[logicalName]= widget;
			// create hyperlink for each tab
			
			try {
			if (this._tabLinkContainer) {
				for (var i = 0; i < this._tabLinkContainer.childNodes.length; i++) {
					if (this._tabLinkContainer.childNodes[i] && this._tabLinkContainer.childNodes[i].__jazzTabName == logicalName) {
						return; // already exists
					}
				}
			}
			} catch(e) {console.debug(dojo.string.substitute(this.messages.errorCreatingTab, [ logicalName ])) };
			
			var link = document.createElement("a");
			dojo.addClass(link,"tab");
			link.href = "#";
			link.__jazzTabName= logicalName;
			link.title=tabLinkLabel;
			
			
			var innerWrapper = document.createElement("div");
			innerWrapper.className = "innerWrapper";
			
			var topWrapper = document.createElement("div");
			topWrapper.className = "topWrapper";
			
			var topBorderDiv = document.createElement("div");
			topBorderDiv.className = "topBorder";
			topWrapper.appendChild(topBorderDiv);
			
			var cornerDiv = document.createElement("div");
			cornerDiv.className = "corners";
			topWrapper.appendChild(cornerDiv);
									
			link.appendChild(topWrapper);

			var titleDiv = document.createElement("div");
			titleDiv.className = "title";
			titleDiv.innerHTML = tabLinkLabel;
			innerWrapper.appendChild(titleDiv);

			link.appendChild(innerWrapper);
						 
			this._tabLinkContainer.appendChild(link);		
		},
		getAttachmentInfo: function(number ) {
			if(!number) return;
			if(!isNaN(number)) {
				try{
					number = parseInt(number);
				} catch(e) { return };
			}
			if(!this._workItem || !this._workItem.linkTypes ||  this._workItem.linkTypes.length == 0) return;
			for(var i=0;i<this._workItem.linkTypes.length;i++) {
				if(this._workItem.linkTypes[i].id=="com.ibm.team.workitem.linktype.attachment") {
					if(this._workItem.linkTypes[i].linkDTOs) {
						for(j=0;j< this._workItem.linkTypes[i].linkDTOs.length;j++) {
							if(this._workItem.linkTypes[i].linkDTOs[j] 
								&& this._workItem.linkTypes[i].linkDTOs[j].target 
								&&  this._workItem.linkTypes[i].linkDTOs[j].target.id === number) {
								return this._workItem.linkTypes[i].linkDTOs[j].target;
							}
						}
					}					
				}
			}
		},
		_getWorkingCopy: function(id, workItem) {
			if (!this.workingCopy) {
				var workingCopy = com.ibm.team.workitem.web.cache.internal.Cache.getItem(id);
				if (workingCopy) {
					this.workingCopy = workingCopy;
				} else {
					if (workItem) {
						var clone = dojo.clone(workItem);
						this.workingCopy = com.ibm.team.workitem.web.cache.internal.Cache.getItem(id, new WorkItemProxy({
							object: clone,
							storedObject: workItem,
							isFetched: true
						}))	
					} else {
						return null;
					}
				}
            	this.workingCopy.incrementUseCount();
			}
			
			this.workingCopy.addListener({
				path: [ "root" ],
				event: "onrefresh",
				listener: this,
				functionName: "_handleWorkingCopyRefresh"
			});	
			return this.workingCopy;
		},
		_handleWorkingCopyRefresh: function() {
			this._refreshFromWorkingCopy();
		},
		focus: function () {
			if(this._editorParts && this._editorParts.header) this._editorParts.header.focus();
		},
		_refreshFromWorkingCopy: function(parms) {
			if(!this.workingCopy || !this.workingCopy.isInitialized) return; // on init, we pick changes up on the success path
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on cancel		
			if(!this.workingCopy || !this.workingCopy.workItemSpec) {
				this.refresh();
				return;
			}
			parms = parms || {};
			var workItem = this.workingCopy.object;
			var editProps = this.workingCopy.workItemSpec.editProps;
			this._setEditableProperties(workItem, editProps, this.workingCopy.workItemSpec.presentationProps, 
					this._currentTab, /* retained state */ {}, false, null);
								
			var isArchived= this.workingCopy.getValue({path:["attributes", "projectArea", "isArchived"]});
			if(isArchived) {
					var msg = {
						message: this.messages.msgProjectAreaIsArchived, 
						action: "ALERT",
						isSticky: false,
						severity: 2 // info
					}
					this._editorParts.header.setHeaderMessage(msg);
			} else {
				this._checkProjectArea();
			}
			if(this.workingCopy.isChanged()) this.markDirty();
		}, 
		// @deprecated
		_populateWorkItem: function(/* editProps, workItem, presentationProps, initialTab, 
						retainedState, isTypeChange, newTypeId, op, isRefresh */ params) {
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on cancel
			
//			if (!params.isTypeChange && params.isRefresh) {
//				this.workingCopy.refreshObject({
//					object: params.workItem,
//					caller: this
//				});
//			}
			
			this._setEditableProperties(params.workItem, params.editProps, params.presentationProps, 
				params.initialTab, params.retainedState, params.isTypeChange, params.newTypeId);

			if(params.isTypeChange || this.pollChildren("hasRetainedState", params.retainedState) || this._workItem._draftId) { 
				this.markDirty();
				if (params.op) { this.workingCopy.finishOperation(params.op); }
				this._editorParts.header.markFullyLoaded();
			}

			var hasRetainedState = false;
			for(var x in params.retainedState) {
				hasRetainedState = true;
				break;
			}
			if(hasRetainedState) this.markDirty();
			
			if(this._workItem && this._workItem.attributes && this._workItem.attributes.projectArea && this._workItem.attributes.projectArea.isArchived) {
					var msg = {
						message: this.messages.msgProjectAreaIsArchived, 
						action: "ALERT",
						isSticky: false,
						severity: 2
					}
					this._editorParts.header.setHeaderMessage(msg);
			} else {
				this._checkProjectArea();
			}
			
			// Handle the tab changing while loading
			var parms = WorkItemUtil.getHashRequestParams(this.getUrl());
			var tab;
			for(var j=0;j<parms.length;j++) {
				if(parms[j].name == "tab") {
					tab = parms[j].value;
					break;
				}
			}
			
			if(tab) {
				this.showTab(tab)
			} else {
				this.showTab(this.getDefaultTabName())
			}

		},
		/*
		 * Check that the current project area matches the one that is selected
		 */
		 _checkProjectArea: function() {
		 	var currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
		 	if(this._workItem && this._workItem.attributes && this._workItem.attributes.projectArea && this._workItem.attributes.projectArea.id != currentProjectArea.itemId) {
		 		    var text = dojo.string.substitute(this.messages.msgProjectAreaIsDifferent, [this._workItem.attributes.projectArea.label, currentProjectArea.name])
					var msg = {
						message: text, 
						details: this.messages.msgProjectAreaIsDifferentDetails,
						showOnlyMessage: true,
						action: "ALERT",
						isSticky: false,
						severity: 2
					}
					this._editorParts.header.setHeaderMessage(msg);
			}
		 },
		// Set the editable properties in each editor part 
		_setEditableProperties: function(workItem, editProps, presentationProps, initialTab,  retainedState, isTypeChange, newTypeId) {
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on cancel			

			dojox.data.dom.removeChildren(this._tabLinkContainer);
			dojox.data.dom.removeChildren(this._currentTabContainer);
			this._destroyChildWidgets(/* skipHeader */ true);
		
			// Restore the root listener removed in destroyChildWidgets
			if (this.workingCopy) {
				this.workingCopy.addListener({
					path: ["root"],
					event: "onrefresh",
					listener: this,
					functionName: "_handleWorkingCopyRefresh"
				});
			}	
			
			presentationProps = presentationProps || {}; 
			presentationProps.pages = presentationProps.pages || [];
			// get the default tab
			this.defaultTab = DEFAULT_TAB;
			var defaultPage = null;		
			for (var i=0;i< presentationProps.pages.length; i++) {
				var page = presentationProps.pages[i];
				if(page.layout == "internalHeader") {
					continue;
				}
				var spec = this.tabSpecs[page.layout];

				
				
				if (spec && spec.renderer && this[spec.renderer]) {
					var tabId = this[spec.renderer](editProps, page, retainedState, isTypeChange);
				if (!defaultPage && page && page.properties) {
					this.defaultTab = tabId;
					defaultPage = tabId;
				}
									
					if(spec.knownTab) {
						this.knownTabs[spec.knownTab] = tabId;
					}
				}
			}

			this._editorParts.header.reset(isTypeChange);
			var isCompatibleTypeChange = this._editorParts.header.bindWorkItem(this.workingCopy.object, retainedState, isTypeChange, newTypeId);

			this.showTab(initialTab || this.getDefaultTabName());
			if (!isTypeChange) {
				this._isDirty = this.workingCopy.isChanged();
			}

			this._editorParts.header.markFullyLoaded();
			this._editorParts.header.updateSubscribeSelf();
			
			// convert the array of attribute values to a map keyed by the property name
			var allValuesMap= {};
			for (var i=0; i<editProps.allValues.length; i++) {
				var cur= editProps.allValues[i];
				allValuesMap[cur.attributeName]= cur.uiItems;
			}
			
			this._editorParts.header.bindOptionalValues(allValuesMap, retainedState, isTypeChange, isCompatibleTypeChange);
			this._editorParts.header.bindRequiredProperties(editProps.requiredProperties || [], retainedState, isTypeChange);
			
			// update the tab URLs for the new work item
			var tabLinks= this._tabLinkContainer.getElementsByTagName("a");
			var self = this;
			var url = this.getUrl();
			dojo.forEach(tabLinks, function(link) {
				var tabName= link.__jazzTabName !== self.getDefaultTabName() ? link.__jazzTabName : null;
				link.href= self.editorUtil._getUriForTab(tabName, url);
			});

			this.notifyChildren("markFullyLoaded");			
		},

		// Hard coded Tab
		_renderCustomAttributes: function(editProps, pageProps, retainedState, isTypeChange) {
			// Convert the page props left and right to details and discussion
			var overviewSections = [];			
			if(pageProps && pageProps.sections) {
				var sections = pageProps.sections;
				for(var i=0;i<sections.length;i++){
					var isAutocollect = false;
					if(sections[i].properties) {
						for(var k=0;k<sections[i].properties.length;k++) {
							if(sections[i].properties[k] && sections[i].properties[k].key == "autocollect" && sections[i].properties[k].value == "true") {
								isAutocollect = true;
								break;
							}
						}
					}
					if (!isAutocollect) {
						if (sections[i].slot == "left") {
							sections[i].slot = "details";
						}
						else 
							if (sections[i].slot == "right") {
							   if(sections[i].presentations && sections[i].presentations[0] && sections[i].presentations[0].partId=="com.ibm.team.workitem.presentation.discussion") {
									sections[i].slot = "discussion";
							   } else {
									sections[i].slot = "description";
							   }
							}
						overviewSections.push(sections[i]);
					}
				}
			}
			if(overviewSections.length == 0) return; // skip tab			
			pageProps.sections = overviewSections;
			
			return this._renderOverview(editProps, pageProps, retainedState, isTypeChange, /* isCustomAttribute */ true);
		},
		_renderOverview: function(editProps, pageProps, retainedState, isTypeChange, isCustomAttributeLayout) {

			var createArgs= {
				workItem: this._workItem,
				parentController: this,
				editorUtil: this.editorUtil,
				editProps: editProps,
				pageProps: pageProps,
				retainedState: retainedState,
				isCustomAttributeLayout: isCustomAttributeLayout
				
			};
			
			var _overview = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemOverview( createArgs);

			var title = this.messages.defaultOverviewTabTitle; // "Overview"
			if (pageProps && pageProps.properties) {
				var props = pageProps.properties;
				for (var i = 0; i < props.length; i++) {
					if (props[i].key == "title") {
						title = props[i].value;
						break;
					}
				}
			}

			var tabName = title.toLowerCase().replace(/ /g,"");
			this._createTabPart(tabName, _overview, title);

			return tabName;
		},
		_renderHLayout: function(editProps, presentationProps, retainedState, isTypeChange) {
			if(!presentationProps) return;
		    var slotMap = this.tabSpecs.builtInHLayout.slotMap;
		    var sections = presentationProps.sections;
		    if(!sections) return;
		    for(var i=0;i<sections.length;i++) {
		        if(!sections[i].slot) continue;
		    	sections[i].slot = slotMap[sections[i].slot];
		    }
		    return this._renderOverview(editProps, presentationProps, retainedState, isTypeChange);
		},
		// Generic tab renderer	
		_renderTabContent: function(editProps, presentationProps, retainedState, isTypeChange) {

			var title = dojo.string.substitute(this.messages.defaultGenericTabTitlePrefix, [ "" + (this._tabLinkContainer.childNodes.length + 1)] );
			var props = presentationProps.properties;
			for(var i=0;i<props.length;i++){
				if(props[i].key == "title") {
					title = props[i].value;
					break;
				}
			}

			var tabId = title.toLowerCase().replace(/ /g,"_");
			tabId = escape(tabId).replace(/\%/g, "_"); // NLS tab names
			if(isTypeChange && this._editorParts.tabId) {
				return tabId;
			}
			
			var createArgs= {
				workItem: this._workItem,
				parentController: this,
				editProps: editProps,
				presentationProps: presentationProps,
				editorUtil: this.editorUtil,
				retainedState: retainedState,
				tabId: tabId
			};
			
			
			var _widget = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemTabContentLayout( createArgs);
			this._createTabPart(tabId, _widget, title);

			return tabId;			
		},	
		/**
		 * Returns <code>true</code> if the work item editor is displaying a new
		 * work item, <code>false</code> if the work item editor is displaying an
		 * existing work item. 
		 */
		isNewWorkItem: function() {
			if(!this.workingCopy) return false;
			return this.workingCopy.isNewWorkItem();
		},
		notifyChildren: function (functionName, parm1, parm2, parm3) {
			for (var partName in this._editorParts) {
				var editorPart= this._editorParts[partName];
				if (editorPart && dojo.isFunction(editorPart[functionName])) {
					editorPart[functionName](parm1, parm2, parm3);
				}
			}
		},
		/*
		 * If any editor part return true, return true
		 */
		pollChildren: function (functionName, parm1, parm2, parm3) {
			for (var partName in this._editorParts) {
				var editorPart= this._editorParts[partName];
				if (dojo.isFunction(editorPart[functionName])) {
					if(editorPart[functionName](parm1, parm2, parm3)) {
						return true;
					};
				}
			}
			return false;
		},
		disableSave: function(isDisabled) {
			this.notifyChildren("disableBottomSaveCommand", isDisabled);
		},
		
		/**
		 * Return the id of work item currently bound to the editor.
		 */
		getId: function() {
			return this.workingCopy.getId();
		},
		
		/*
		 * Get the full version of the label, e.g. "Defect 1235: Something broke".
		 */
		getSummaryLabel: function() {
			return this.editorUtil.getSummaryLabel(this._workItem);
		},
		/*
		 * Get the short version of the label, e.g. "Defect 1235".
		 */
		getShortLabel: function() {
			if(!this.workingCopy) return;
			return this.workingCopy.getLinkLabel();
		},
		getUrl: function(doReset) {
			return this.workingCopy.getUrl({ isReset: doReset == true });
		},
		getIconUrl: function() {
			var url = null; 
			try {
				if (this._editorParts && this._editorParts.header && this._editorParts.header._updatedTypeIconSrc) {
					url = this._editorParts.header._updatedTypeIconSrc;
				}
				else {
					if (this._workItem && this._workItem.attributes && this._workItem.attributes.workItemType) {
						url = this._workItem.attributes.workItemType.iconUrl;
					}
				}
			} 
			catch (e) {	}; // ignore error		
			return url;
		},
		
		/**
		 * Displays the tab specified by <code>tabName</code>.
		 * 
		 * @param {String} tabName the logical name of the tab to display.
		 */
		showTab: function(tabName) {

			// If the tab name is in the window url, override the method call.
			var urlTab = WorkItemUtil.getUrlParam(window.location.href, "tab");
			if(urlTab) {
				tabName = urlTab;
			} else {
				tabName = tabName || this.getDefaultTabName();
			}
			
			if(!this._editorParts[tabName]) {
				if(this.knownTabs[tabName] ){
					tabName = this.knownTabs[tabName];
				} else {
	 				tabName = this.getDefaultTabName();
				}

			}
			
			if(!this._editorParts[tabName]) return; // click during refresh
			if(this._editorParts[tabName] && this._editorParts[tabName].showWidget) {
				this._editorParts[tabName].showWidget();
			}
			
			// swap in contents
			if (this._currentTabContainer.firstChild) {
				this._currentTabContainer.replaceChild(this._editorParts[tabName].domNode, this._currentTabContainer.firstChild);
			}
			else {
				this._currentTabContainer.appendChild(this._editorParts[tabName].domNode);
			}
			
			// update tab selection
			var links= this._tabLinkContainer.childNodes;
			dojo.forEach(links, function(link) {
				if (link.__jazzTabName === tabName)
					dojo.addClass(link, "Selected");
				else {
					dojo.removeClass(link, "Selected");
				}
			});
    		this.workingCopy.setEditorTab(tabName);
			var url = this.getUrl(/* reset */ true);
			var ix = url.indexOf("&tab=");
			if(ix == -1){
				this.url = this.url + "&tab=" + tabName;
			} else {
				url = (url + "&").replace(/&&/g, "&");
				this.url = url.replace(/&tab=.+&/, "") + "&tab=" + tabName;
			}

			if(tabName != this._currentTab) {
				this._sendChangedEvent("Tab Changed");
			}
			
			// store a reference to the current tab
			this._currentTab= tabName;
		},
		/**
		 * Mark the editor dirty only if the editor is in the correct state.
		 * <p>
		 * Background: there's a tough edge case caused by supporting the Ctrl-S
		 * save method.  If you make changes in a text input or textarea and
		 * then hit Ctrl-S, the DOM 'onchange' event will fire as soon as you
		 * set focus anywhere else on the page.  This method attempts to detect
		 * this state, and if it detects it, skip the 'mark dirty' action and
		 * clear the flag so that future 'onchange' events result in correct
		 * form dirtying. 
		 * </p>
		 * 
		 * @param {Object} e a DOM event representing a potential change.
		 */
		conditionalMarkDirty: function(e) {
			var target= e.target || e.srcElement;
			if (target.tagName === "SELECT" || !this._ctrlSWasPressed) {
				target.__jazzChanged= this.markDirty();
			}
			this._ctrlSWasPressed= false;
		},
		
		/**
		 * Return <code>true</code> if the user wishes to abandon the current
		 * editor or <code>false</code> if the user doesn't wish to abandon the
		 * current editor. 
		 */
		confirmAbandon: function() {
			var hasUnsavedChanges = false;
			
			// Get any retained state from the parts
			hasUnsavedChanges = this.pollChildren("hasUnsavedChanges");
			
			var msg;
			if(this.isNewWorkItem()){
				var msg= this.messages.promptResetContinue;
				if(hasUnsavedChanges) {
					msg = this.messages.promptResetLostChangesContinue; ;
				}
			}else{
				var msg= this.messages.promptRefreshContinue;
				if(hasUnsavedChanges) {
					msg = this.messages.promptRefreshLostChangesContinue;
				}
			}
			return confirm(msg);
		},
		
		
		/**
		 * Respond to an error condition by forwarding the error object to the
		 * parent controller.
		 * 
		 * @param {Object} errorObj an error object created by a remote method.
		 * @param {Object} message an optional human-friendly error message
		 *  describing what the system was trying to do when the error occurred.
		 */
		handleLoadingError: function(errorObj, message) {
			this.parentController.handleError(errorObj, message);
		},
		checkLogin: function() {
			if (isGuest()) {
				if(this._isGuestRefreshing) return; // Prevent double questions
				this._isGuestRefreshing = true;
				this._askToLogIn();
				
				this.refresh();
				this._isGuestRefreshing = false;
				return false;
			}
			return true;
		},
		_askToLogIn: function() {
			if (isGuest()) {
				if (confirm(this.messages.promptLogInOrCancelWorkItemEdit)) {
					window.location.href = "https://" + window.location.hostname + ":" + com.ibm.team.repository.server.repourl.port.https + window.location.pathname + window.location.search + window.location.hash;
					 /* Will not return */
				}
				return false;
			}
			return true;
		},
		/**
		 * Mark the editor as dirty and update the internal dirty state
		 */
		markDirty: function() {
			if(!this.checkLogin()) return;
			// Always enable save
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on cancel
			this._editorParts.header.markDirty();
			if (!this.workingCopy.isChanged()) {
				this._isDirty= true;
				this.notifyChildren("markDirty");
				this.parentController.setIsDirty(this, true);
				this._sendChangedEvent("Save Enabled");
			}
			return true;
		},
		_sendChangedEvent: function(reason, isClosed) {
			// Notify listeners (Recently Viewed) that the state has changed
			// isDirty
			// Tab changed

			if(isClosed) { //TODO: switch to parms
				var parms = [this.workingCopy, this, true];
			} else {
				var parms = [this.workingCopy, this];				
			}
			dojo.publish(EventTopics.WORKITEM_EDITOR_CHANGED, parms);
		},
	    isDirty: function(id) {
			if(this.workingCopy) {
				return this.workingCopy.isChanged();
			} else {
				var workingCopy = this._getWorkingCopy(id);
				if(workingCopy) {
					return workingCopy.isChanged();
				}
			}
			return false;
		},
		/**
		 * Refresh the current work item by fetching and displaying its most
		 * recent state from the server.
		 * 
		 * @param {Object} skipConfirmation optional flag to suppress a
		 *  confirmation dialog.
		 */
		refresh: function(suppressConfirmation) {
			// do nothing if the editor is dirty and the user cancels
			if (!suppressConfirmation && this.workingCopy.isChanged() && !this.confirmAbandon()) {
				this._editorParts.header.endOperation();
				return;
			}
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on cancel
			this._editorParts.header.setHeaderToolbarState(this.workingCopy.isNewWorkItem());
			// If its a new work item editor, then reset it
			if(this.workingCopy.isNewWorkItem() < 0) {
				var type;
				if(this._workItem.attributes && this._workItem.attributes.workItemType) {
					type = this._workItem.attributes.workItemType.id;
				}			
				this.parentController.newWorkItem({ doReset: true, type: type, id: this.workingCopy.getId()});
				return;
			}
			this._editorParts.header.updateRefreshTimestamp(this.isNewWorkItem());
			
			// call viewWorkItem directly since the URL isn't going to change
			var obj= { id: this.workingCopy.getId() };
			if (this._getCurrentTab()) {
				obj.tab= this._getCurrentTab();
			}
			obj.doNotShowLoading = true;
			this.parentController.viewWorkItem(obj, /* forceReadWorkItem */ true);
		},
		/*
		 * Before the work item is saved, get the widgetFormElements and widgetCommands
		 * TODO: inherit from AbstractEditor
		 */
		_workItemAboutToSave: function(parms /* workItemProxy, widgetFormElements, widgetCommands */) {
			if(!(parms.workItemProxy === this.workingCopy)) return;			
            for (var widget in this._childWidgets) {
                if (this._childWidgets[widget].getFormElementValues) {
                    this._childWidgets[widget].getFormElementValues(parms.widgetFormElements);
                }
            }
			
            for (var partName in this._childWidgets) {
                if (this._childWidgets[partName].getSaveCommands) {
                    this._childWidgets[partName].getSaveCommands(parms.widgetCommands);
                }
            }
			
		},
		_workItemPrentationChanged: function(parms /* workItemProxy, widgetFormElements, widgetCommands */) {
			if(!(parms.workItemProxy === this.workingCopy)) return;			
    
		    this._refreshFromWorkingCopy();
			
		},
		/**
		 * Save the changes to work item currently displayed in the editor.
		 */
		save: function() {

			var attachmentPart = this.findObjectWithProperty({
					property: "isUploadInProgress"
				});
			
		    if(attachmentPart && attachmentPart.isUploadInProgress()) {
				if(confirm(this.messages.promptAttachmentUploadInProgress)) {
					attachmentPart.cancelUploadInProgress();
				} else {
					var msg = {
						message: this.messages.msgSaveCancelledDueToUpload,
						details: this.messages.msgSaveCancelledDueToUploadDetails, 
						action: "ALERT",
						isSticky: false,
						severity: 4 // warning
					}
					this._editorParts.header.setHeaderMessage(msg);
					return;
				}
			}

			var op = this.workingCopy.createOperation({
				message: this.messages.busySaving,
				id: "WorkItemEditor/Save",
				isModal: true,
				dialogTitle: this.messages.busyProgress
			});
			this.workingCopy.startOperation(op);	
			
	    	this._editorParts.header.clearValidationMessage(true);
	    	this._editorParts.header.markBusy(this.messages.busySaving);
			this._editorParts.header.disableSave(true);

			var beforeSaveId = this.workingCopy.getId();
			var wasNew = this.workingCopy.isNewWorkItem();
	    	var self= this;			
			this.workingCopy.storeWorkItem({
				instigator: this,
				skipErrorMessage: true, // shown here instead
				operationMsg: this.messages.busySaving,
				onSuccess: function(parms) {
					var workItem = parms.workItem;
					self._clearRetainedState(/* ignore Comment */true);
	    			self._saveCallback(workItem.attributes.id.id, op, wasNew, beforeSaveId);
					// Save is always followed by refresh so just let it spin
	    		},
				onError: function(error, parms) {
					var operationReport = parms.error;
	    			self._saveErrorCallback(operationReport, op);
					if(!self._editorParts || !self._editorParts.header) return; // timing issue on cancel
					self._editorParts.header.markFullyLoaded();
	    		}
			});

		},
		askToSave: function() {
			if(!this.workingCopy.isChanged()) return;
			if(confirm(dojo.string.substitute(this.messages.promptSave, [ this.getShortLabel()]))) {
				this.parentController._clearSaveParameter();
				this._editorParts.header._handleSaveClick();
			} else {
				this.parentController._clearSaveParameter();
			}
		},
		_isAttributeEqual:function(attr, other, key) {
			// attr comes from work item
			// other comes from form elements;
			if(attr == other) return true;
			if(attr == null && other == "") return true;
			if(other == null && attr == "") return true;

// If the attr is an iso string, then convert it to time
			try {
				if(attr != null && attr != "") {
					attrDate = WorkItemUtil.fromISOString(attr)
					if(attrDate) {
						var time = attrDate.getTime();
						if(time == other) return true;
					}
				}
			} catch( e) {} // ignore

			if(other && other.type == "date") {
				other = other.value;
				var attrDate, otherDate, attrStr, otherStr;
				// If the attr is an iso string, then convert it to time
				try {
					if(other != null && other != "") {
						otherDate = WorkItemUtil.fromISOString(other)
						if(otherDate) {
							var otherTime = otherDate.getTime();
							if(time == otherTime) return true;
						}
					}
				} catch( e) {} // ignore

				// test for null conditions. dates can be -1 to indicate unset.
				if(attr == null && other == -1) return true;
				if(attr == -1 && other == null) return true;
				
				// If the dates are the same, but the hours different, then the dates are equal
				// This time is always local, and this normalizes distant time zones
				try {
					if(other && !otherDate) {
						otherDate = new Date(other);
					}
					if (attrDate && otherDate) {
						attrStr = dojox.date.posix.strftime(attrDate, DATE_FORMAT_NO_TIME);
						otherStr = dojox.date.posix.strftime(otherDate, DATE_FORMAT_NO_TIME);
						if (attrStr == otherStr) 
							return true;
					}
				} catch(e) {} // ignore, not dates
			}

			
			if(other && other.toLowerCase) {
				other = other.toLowerCase();
			}
			if(attr && attr.toLowerCase) {
				attr = attr.toLowerCase();
			}
			
			if(attr == null && other == "unassigned") return true;
			if(other == null && attr == "unassigned") return true;
			if(attr == null && other == "undefined") return true;
			if(other == null && attr == "undefined") return true;

			return false;
		},
		_getMailto: function(address) {
			if(this.workingCopy.isNewWorkItem()) {
				var summaryLabel = this.getSummaryLabel();
				if(!summaryLabel ) summaryLabel = "";
				return "mailto:" + address + "?subject=" + summaryLabel;
			}
			return "mailto:" + address;
		},
		_clearRetainedState: function(ignoreComment) {
			this.workingCopy.clearRetainedState(ignoreComment);
			this.notifyChildren("clearRetainedState");
		},
		/*
		 * Get the Approvals data
		 */
		fetchSimpleInfo: function(fetchArgs) {
//			var fetchArgs = {
//				serviceSelector: "getApprovalTypes",
//				responseObject: this,
//				responseSelector: "refreshApprovalTypes",
//				progressText: "Loading Approval Types"
//			};
	        var op= this.workingCopy.createOperation({
				message: fetchArgs.progressText,
				id: "fetchSimpleInfo"
			});
	    	this.workingCopy.startOperation(op);

			var self=this;
			// fetch the values required to edit the work item
			var serviceObj= {
				success: function(simpleInfo) {
					if(fetchArgs.responseObject[fetchArgs.responseSelector]) {
						fetchArgs.responseObject[fetchArgs.responseSelector](simpleInfo);
					}					
					if (op) { self.workingCopy.finishOperation(op); }
				},
				failure: function(errorObj) {
					self.handleLoadingError(errorObj, dojo.string.substitute(self.messages.errorLoadingSimpleInfo, [ fetchArgs.serviceSelector ]));
					if (op) { self.workingCopy.finishOperation(op); }
				}
			};
			var srh= new ServiceResponseHandler(serviceObj, "success", "failure");

			var args = { projectAreaItemId: this.workingCopy.getProjectAreaItemId()};

			if(WorkItemClient[fetchArgs.serviceSelector]) {
				WorkItemClient[fetchArgs.serviceSelector](srh, args);
			} else {
				console.debug(dojo.string.substitute(this.messages.errorInvalidFetchSimpleInputSelector,[serviceSelector]));
			}
			
		},		
		handleSubscriptionChange: function() {
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on cancel
			this._editorParts.header.updateSubscribeSelf();
		},
		
		/**
		 * Update the label and icon of the work item type information displayed
		 * in the work item editor header.
		 * 
		 * @param {String} typeName the name of the type (e.g. 'Defect').
		 * @param {Object} imgSrc a URL for the icon for the type.
		 * @param {boolean} is this initialization or not (optional)
		 */
		updateType: function(value, typeName, imgSrc, isInitialization) {
			if(isInitialization) return; 
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on close
			
			var isTypeChange = false;
			this._editorParts.header.updateType(typeName, imgSrc, isInitialization);
		},
				
		/**
		 * Display the work item with the id specified by 'id'.
		 * 
		 * @param {Object} id the id (e.g. '1234') used to look up the work
		 *  item.
		 * @param {Object} (optional) initialTab the initial tab (e.g. 'Links')
		 *  to display when the editor displays the work item.
		 */		
		viewWorkItem: function(id, initialTab) {
			checkNotNull(id);
			this.parentController._internalViewWorkItem(id, initialTab);
		},

		/*
		 * Return the logical name of the current tab, or 'null' if the editor
		 * is displaying the default tab.
		 */
		_getCurrentTab: function() {
			return this._currentTab !== this.getDefaultTabName() ? this._currentTab : null;
		},
		/*
		 * Return an object containing the names and values of all of the form
		 * elements in the editor parts.
		 */
		_getFormElements: function(isReturningUnchangedAttributes) {
			var allFormElements= {};
			this.workingCopy.getFormElementValues(allFormElements);
			var allProperties =  { }; // Handle the case where a value is null
			for (var partName in this._editorParts) {
				var editorPartReqParms= this._editorParts[partName].getFormElements(isReturningUnchangedAttributes);
				for (var prop in editorPartReqParms) {
					allProperties[prop] = true;
					allFormElements[prop]= editorPartReqParms[prop];
				}
			}
						
			return allFormElements;
		},
		
		/*
		 * Callback for a successful work item save.
		 */
		_saveCallback: function(savedWorkItemId, op, wasNew, preSaveId) {
			this.parentController.setIsDirty(this, false);
			if (this._editorParts && this._editorParts.header) {
				this._editorParts.header.markFullyLoaded();
			}

			if (wasNew) {
				this.parentController.removeEditorTab(preSaveId);
				this.viewWorkItem(savedWorkItemId, this._getCurrentTab());
			} else {
				this.notifyChildren("handleSaveComplete")
				this.refresh(/*skipConfirmation*/true);
			}
			if (op) { this.workingCopy.finishOperation(op); }
			this._isDirty = this.workingCopy.isChanged();
		},

		/*
		 * Callback for an unsuccessful work item save.
		 */
		_saveErrorCallback: function(report, op) {
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on cancel
			this._editorParts.header.disableSave(false);

			if (op) { this.workingCopy.finishOperation(op); }
			
			if (report.infos && report.infos.length > 0) {
				var action, info= report.infos[0];
				if(info.description == "Specify duplicate") { // message must match value from server %NON-NLS-1%
					action = "ADD_DUPLICATE_OF_LINK_ACTION"
				} else {
					action = "ALERT"
				}
				var msg = {
						message: info.summary,
						details:  info.description,
						action: action,
						isSticky: false,
						severity: info.severity // 2=info, 4= warning, 8=error
					}
				this._editorParts.header.setHeaderMessage(msg);
			} else {
				var message = report.message;
				if(!message && this.workingCopy) {
					message = this.workingCopy.getErrorMessages(report).summaryMsg;
				}
				var msg = {
						message: message,
						action: "ALERT",
						isSticky: false,
						severity: 8 // error
					}
				this._editorParts.header.setHeaderMessage(msg);
			}
			this._editorParts.header.endOperation();
		},
		getManagerSelector: function(attributeId) {
			if(!attributeId) return null;
			 return "is" + attributeId + "Manager"; // $NON-NLS-2
		},
		getAttributeValue: function(attributeId) {
			if(!attributeId) return null;
			if(attributeId == "summary") {
				try {
					return this._editorParts.header.summary.value;
				} catch(e) {
					return ""; // not initialized
				}
				
			}
			try {
				var manager = this.findObjectWithProperty({
					property: this.getManagerSelector(attributeId)
				});
				if (manager && manager.getValue) {
					return manager.getValue();
				}
			} 
			catch (e) {
				console.debug(dojo.string.substitue(this.messages.errorGettingAttributeValue, [ attributeId ]));
			}
			return null;
		},
		getLinkManager: function() {
			if (this._linkManager == null) {
				this._linkManager = this.findObjectWithProperty({
					property: "isLinkManager"
				});
			}
			if(!this._linkManager == null) this._linkManager = false;
			return this._linkManager;
		},
		showLinksTab: function() {
			if(this.getLinkManager()) {
				return this._linkManager.showLinksTab();
			}	
		},
		hasDuplicateOfLink: function() {
			if(this.getLinkManager()) {
				return this._linkManager.hasDuplicateOfLink();
			}	
			return false;
		},
		notifyLinkManagerToAddDuplicateLink: function() {
			// find a tab that can handle it (probably links
			if(this.getLinkManager()) {
				this._linkManager.addDuplicateLink()
			}	
		},
		notifyLinkManagerToRemoveDuplicateLink: function() {
			// find a tab that can handle it (probably links
			if(this.getLinkManager()) {
				this._linkManager.removeDuplicateLink()
			}	
		},
		updateHeaderValidation: function() {
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on cancel
			this._editorParts.header.updateValidationMessage();
		},
		findObjectWithProperty: function(/* property, path */ parms) {
			if(this[parms.property]) return this;
			return EditorUtil.findObjectWithPropertyInObject({
				object: this._editorParts,
				path: parms.path,
				property: parms.property
			}, this);
		},
		/*
		 * Respond to any 'keydown' event triggered by a child form element. 
		 */
		_handleGenericKeydown: function(e) {
			this._handleGenericKeypress(e, "keydown");
		},
		
		_handleSaveClick: function(e) {
			if(!this._editorParts || !this._editorParts.header) return; // timing issue on cancel
			this._editorParts.header._handleSaveClick(e);
		},
		
		/*
		 * Respond to any 'keyup' event triggered by a child form element. 
		 */
		_handleGenericKeyup: function(e) {
			this._handleGenericKeypress(e, "keyup");
		},
		
		/*
		 * Logic for handling keydown and keyup events; critical logic for both
		 * marking the form dirty and for processing Ctrl-S keystrokes.
		 */
		_handleGenericKeypress: function(e, type) {
			
			// ensure that the keystroke is coming from either a textarea or
			// input control, and that the control hasn't explicitly marked
			// itself to be ignored
			var target= this.editorUtil._getOriginalTarget(e);
			var inputElement= target.tagName === "TEXTAREA" || target.tagName === "INPUT" ? target : null;
			
			// on Ctrl-S keydown, save work item if in a dirty state
			// 83 is the key code for 's'
			// metaKey is cmd for mac
			if (type === "keydown" && 
				((e.ctrlKey || e.metaKey) && e.keyCode === 83)
					 && !e.altKey) {
				
				if (this.workingCopy.isChanged()) {
					if (inputElement) {	this._ctrlSWasPressed= true; }
					
					// save work item
					this.save();
				}
				
				dojo.stopEvent(e);
				
				return false;
			}
			
			// if the target's not an input or the input has explicitly opted
			// out of being considered a change (e.g. the duplicate work item
			// search dialog box's text search field) then return
			if (!inputElement || inputElement.__jazzIgnoreKeystroke) { return; }
			
			// if the editor's already dirty and the element's already marked as
			// modified, then we don't need to test anything
			if (this._isDirty && inputElement.__jazzChanged) { return; }
			
			// In order to test whether or not a value has changed as the result
			// of any keystroke, we bind to both keydown and keyup.  On keydown,
			// the value of the control will not yet have changed, so we cache
			// it.  On keyup, the value may have changed, so we compare the
			// current value with what we cached onkeydown and mark the editor
			// dirty if they differ.
			//
			// This is basically the same logic as onchange but immediately
			// responsive to keystrokes rather than requiring the control to
			// lose focus.
			if (type === "keydown") {
				// on keydown, store the current (yet unmodified) value of the
				// control
				this._originalInputValue= inputElement.value;
				this._originalInputElement= inputElement;
			} else if (type === "keyup") {
				// on keyup, mark the editor dirty if the control's value has
				// changed
				if (inputElement === this._originalInputElement &&
						inputElement.value !== this._originalInputValue) {
					this.markDirty();
					inputElement.__jazzChanged= true;
				}
			} else {
				throw new Error(dojo.string.substitute(this.messages.errorUnknownKeypressType, [type]));
			}
		}
	}
);

})();
