/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.LinksPart");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.parts.RemoteItemPicker");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.SelectUriAndCommentDialog");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuDropDownActionWidget");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
/*
 * This is a wrapper widget that shows the Links Section
 */
(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var EditorUtil	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var SelectWorkItemDialog = com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog;
var ImagePool				= com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;

// TODO eventually get this stuff from the server so we don't have duplicated
// model logic on the client
//

var linkTypeEditors = {
	    "WorkItem": "_addWorkItemLink",
		"_calmLink": "_addCalmLink",
		"_defaultUri": "_addUriLink"
}

var EDITABLE_LINK_TYPES =  { // Coordinate changes with constructor
				related: {  // Default
					key: "related",
					id: "com.ibm.team.workitem.linktype.relatedworkitem",
					isSingleValued: false,
					isSource: false
				},
				duplicateOf: {  // Used to generate this kind of link type
					key: "duplicateOf",
					id: "com.ibm.team.workitem.linktype.duplicateworkitem",
					isSingleValued: true,
					isSource: true
				}
			};
var headingColor = "#666668";
var columnLayout = {
	name: {
		style: {
			fontWeight: "bold",
			height: "1.2em",
			margin: "0 0 0 8px",
			color: headingColor,
			width: "100%"
		}
	}
}

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.LinksPart",
	[dijit._Widget, dijit._Templated],
	{
		
		templateString: '<div class="LinksPart SectionPart">' +
						'<span dojoAttachPoint="_sectionMenuSPAN"></span>' +
						'<div dojoAttachPoint="_contentNode">' +
						'</div>' +
						'<BR></div>',
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
			
			EDITABLE_LINK_TYPES.related.displayName = this.messages.labelRelated;
			EDITABLE_LINK_TYPES.duplicateOf.displayName= this.messages.labelDuplicateOf;
			columnLayout.name.label=  this.messages.labelLink;
		},
		postCreate: function() {	
			this._childWidgets = [];
			this.isLinkManager = true; //marker
			this._isInitialized = false;
			this._workItem = this.parentController._workItem;
						
			this.workingCopy.addListener({
				path: ["linkTypes"],
				event: "onchange",
				listener: this,
				functionName: "_handleWorkingCopyChange"
			});			
			
			this._initializeEditableLinkTypes();
			
		},
		_initializeEditableLinkTypes: function() {
			
          	var responseHandler = new com.ibm.team.repository.web.transport.ServiceResponseHandler(this, "_initializeEditableLinkTypesSuccess", "_initializeEditableLinkTypesError");    
			var args = {
                projectAreaItemId: this.workingCopy.getProjectAreaItemId()
            }
            com.ibm.team.workitem.web.client.internal.WorkItemClient.getLinkTypes(responseHandler, args);
		},
		_initializeEditableLinkTypesError: function(errorObj) {
			alert(this.messages.errorFailedToLoadEditableLinkTypes + ". " + errorObj.message);
		},
		_initializeEditableLinkTypesSuccess: function(linkTypes) {
			this._editableLinkTypes = linkTypes || [];
			this._editableLinkTypes.sort(function(a, b) {
				try{
					return a.displayName.toLowerCase() > b.displayName.toLowerCase() ? -1 : 1;
				} catch(e) { 
					return -1 
				}
			});
			if(this._isRefreshPending) {
				this.refresh();
			}
		},
		_getWorkingCopyLinkTypes: function () {
			return this.workingCopy.getValue({
				path: ["linkTypes"],
				defaultValue: []
			});
		},
		_handleWorkingCopyChange: function(parms) {
			this.refresh();
		},
		_setWorkingCopyValue: function() {
			/* The linktypes are updated in place, so just notify senders */
			this.workingCopy.setValue({
				caller: this,
				path: ["linkTypes"],
				value: 	this._getWorkingCopyLinkTypes() // Current values
			});
		},
		destroyWidget: function() {
			this.resetWidget();
			this.destroy();
		},
		resetWidget: function() {
			for(var i = 0; i< this._childWidgets.length; i++) {
				if(this._childWidgets[i].destroyWidget)
					this._childWidgets[i].destroyWidget();
				this._childWidgets[i] = null;
			}

			this._childWidgets = [];
			dojox.data.dom.removeChildren(this._contentNode);
		},
		/*
		 * Load the tab when it is shown
		 */
		showWidget: function() {
			if(this._isInitialized) return;
			this._isInitialized = true;
			this.refresh();
		},
		
		
		_addCalmActions: function(actions, linkType) {
			// a CALM link type can be used by any number of service providers
			// add an action for each service provider
			var calmServiceDescriptors= linkType.calmServiceDescriptors;
			for (var k= 0; k < calmServiceDescriptors.length; k++) {
				var linkTypeCopy = dojo.mixin({}, linkType);
				linkTypeCopy.calmServiceDescriptor= calmServiceDescriptors[k];
				linkTypeCopy.itemType = "_calmLink";
				if (calmServiceDescriptors.length > 1) {
					linkTypeCopy.displayName= dojo.string.substitute(this.messages.actionLabelWithProject, [ linkTypeCopy.displayName, linkTypeCopy.calmServiceDescriptor.title]);
				}
				actions.push(linkTypeCopy);
			}
		},		
		
		
		/*
		 * Setup the page and read the states and types 
		 */
		refresh: function() {
			if(!this._editableLinkTypes) {
				this._isRefreshPending = true;
				return; // will get refreshed when the link types are loaded;
			}		
			this._isRefreshPending = false;
			var defaultAction;
			if (!this.workingCopy.isGuest()) {
				var actions = [];
				for (var i=0;i<this._editableLinkTypes.length;i++) {
				    var linkType= this._editableLinkTypes[i];
					if ((!linkType.itemType || linkTypeEditors[linkType.itemType] ) && linkType.isUserWriteable) {
						if (linkType.itemType == null) {
							var calmServiceDescriptors= linkType.calmServiceDescriptors;
		    				if (dojo.isArray(calmServiceDescriptors) && calmServiceDescriptors.length > 0) {
								this._addCalmActions(actions, linkType);
		    				} else {
								linkType.itemType = "_defaultUri";
								actions.push(linkType);
							}
						} else {
							if (linkType.endpointId == "related") {
								defaultAction = linkType;
							}
							actions.push(linkType);
						}
					}
				}
				actions = actions.reverse();
				var actionClass = com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuDropDownActionWidget;
				var menuClass = com.ibm.team.workitem.web.ui.internal.view.layout.SectionMenuWidget;
				this._sectionMenuWidget = WidgetUtil.createSectionMenu({
					domNode: this._sectionMenuSPAN,
					parentWidget: this,
					defaultAction: defaultAction || false,
					menuClass: menuClass,
					actionClass: actionClass,
					actionSelector: "_handleAddLinkButtonPressed",
					actionLabelSelector: "displayName",
					menuLabel: this.messages.buttonAdd,
					actions: actions
				});
				this._childWidgets.push(this._sectionMenuWidget);
			}
			this.resetWidget();			
//			this._markDirtyIfDifferent();
			var creationArgs = {
						parentWidget: this,
						parentController: this.parentController,
						isHeading: true,
						isFirst: true,
						noPadding: true
					};
			var listWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.SectionListWidget(creationArgs);
			var linkTypes = this._getWorkingCopyLinkTypes();			
			var rowItems = [];
			for (var i = 0; i < linkTypes.length; i++) {
				// skip attachments
				if(linkTypes[i].id == "com.ibm.team.workitem.linktype.attachment") continue;
				
				var isLast = i == linkTypes.length;
				if (linkTypes[i] && this._hasLinks(linkTypes[i], true)) {
					var item = {};
					item.columnWidgets = [this._getRow(listWidget, linkTypes[i], /* isFirst */ i == 0, isLast) ];
					item.value  = linkTypes[i];
					item.value.isNotHighlighted = true;
					item.showDelete = false;
					rowItems.push(item);
					this._addLinkItems( linkTypes[i],rowItems, item, listWidget);
				}
			}
			listWidget.setRows(
					{
						itemList: rowItems,
						deleteAction: {
								selector: "_handleDeleteItemPressed",
								title: this.messages.titleLinkDelete
						},
						restoreAction: {
								selector: "_handleRestoreItemPressed",
								title: this.messages.titleLinkRestore
						}
					}
					)
			this._contentNode.appendChild(listWidget.domNode);
			this._childWidgets.push(listWidget);	
			if( this.workingCopy.isGuest() && listWidget._tableWidget && listWidget._tableWidget._table) 
				listWidget._tableWidget._table.style.border="";
			this.parentController.updateHeaderValidation();

		},
		_hasLinks: function(linkType, showDeleted) {
			if(!linkType || !linkType.linkDTOs || linkType.linkDTOs.length == 0) return false;
			for (var i = 0; i < linkType.linkDTOs.length; i++) {
				if(showDeleted || !linkType.linkDTOs[i]._isDeleted) return true;
			}
			return false;
		},
		_addLinkItems: function(linkType, rowItems, item, listWidget) {
			var links = item.value.linkDTOs
			if(!links || links.length == 0) return;
			var target, targetType, id;
			for (var j = 0; j < links.length; j++) {
				var link = links[j];
				if(!link) continue;
				link.linkTypeItemType = linkType.itemType;
				link.linkTypeId = linkType.id;
				link.linkTypeIsUserDeleteable = linkType.isUserDeleteable;
				link.linkTypeIsSource = linkType.isSource;
//				if(link._isDeleted) continue;
				var node = document.createElement("SPAN");
				node.className = "SectionListChildRow SectionListLink"
				var img = document.createElement("IMG");
				img.src = ImagePool.LIST_ARROW;
				img.className = "linkArrow";
				target = link.target;
				id = null;
				stateGroup = -1;
  				targetType = target ? target._eQualifiedClassName : null;
				if (target && target.attributes) {
					for (var i = 0; i < target.attributes.length; i++) {
						if (target.attributes[i].key == "id") {
							id = target.attributes[i].value.id;
							break;
						}
					}
					for (var i = 0; i < target.attributes.length; i++) {
						if (target.attributes[i].key == "stateGroup") {
							stateGroup = target.attributes[i].value.id;
							break;
						}
					}
				}
				var anchor;
				
				// if no comment, make the link the comment
				if((!link.comment || link.comment.length == 0)) {
					if(link.url) {
						link.comment = link.url;
					} else {
						if(link.linkTypeItemType) {
							link.comment = link.linkTypeItemType;	
						} else {
							if(link.linkTypeId) {
								link.comment = link.linkTypeId
							} else {
								link.comment = this.messages.labelLink;
							}
						}
					}
				}
				
				link.comment = this.splitText(link.comment);
				
		  		anchor = document.createElement("a");
		  		anchor.__jazzValue = link;
		  		
		  		if(link.url) {
					var resourceLinkArgs= {
							retainLinkText: true,
							uri: link.url, // canonical url of the work item
							presentationUri: link.url, // may be massaged in WorkItemProxy 
							label: link.comment
						};
					this.workingCopy.applyResourceLinkToAnchor(resourceLinkArgs, anchor);			  			
		  		} else {
		  		  // This path occurs when a link is added from the Web UI, before save
		  		   anchor.appendChild(document.createTextNode(link.comment));
		  			if(link.linkTypeItemType == "WorkItem" && link.id ) {
		  				anchor.href = "#action=com.ibm.team.workitem.viewWorkItem&id=" + link.id;
		  			}
		  		}
		  						
				this.connect(anchor, "onfocus", function (e) {
					if (anchor.parentNode && anchor.parentNode.parentNode && anchor.parentNode.parentNode.parentNode) {
						listWidget.setRowBackground(anchor.parentNode.parentNode.parentNode, true);	
					}	
				});
				this.connect(anchor, "onblur", function () {
					if (anchor.parentNode && anchor.parentNode.parentNode && anchor.parentNode.parentNode.parentNode) {
						listWidget.setRowBackground(anchor.parentNode.parentNode.parentNode, false);	
					}	
				});
				
				node.appendChild(img);
				if(stateGroup == 2) {
					dojo.addClass(anchor,"Strikeout");	
				};
				node.appendChild(anchor);
				
				var fakeWidget = {
					domNode: node
				};
				
				var linkItem = {};

				linkItem.columnWidgets = [ fakeWidget ];
				linkItem.value  = link;
				
				var itemId = link.itemId;
				if(!itemId && target && target.attributes) {
	  				var id;
				  	for(var i=0;i<target.attributes.length;i++) {
				  		 if(target.attributes[i].key=="itemId") {
					 		itemId = target.attributes[i].value.itemId;
							break;
				 		}
			  		}
				}
				
				var typeIdKey = link.linkTypeId.replace(/\./g, "_").toLowerCase();
				var canBeDeleted = link.linkTypeIsUserDeleteable != false; 
				
				linkItem.itemId = itemId;
				linkItem.showDelete =canBeDeleted; 
				rowItems.push(linkItem);
			}
			
		},
		splitText: function(text) {
			if(!text) return text;
			
			// Max unbroken chars = 55. 
			// 55 ensures no clipping when the page is minimal width
			// Add a space if there are 55 unbroken chars without one
			return dojo.trim( text.replace(/[^ ]{55}/g, '$& ') );
		},
		addDuplicateLink: function() {
			this._openAddWorkItemDialog({
				label: this.messages.labelDuplicateOf,
				value: EDITABLE_LINK_TYPES.duplicateOf
			});
		}, 
		removeDuplicateLink: function() {
				var linkTypes = this._getWorkingCopyLinkTypes();
				if(!linkTypes) return;
				for(var i = 0;i<linkTypes.length;i++ ){
					if(linkTypes[i].id == "com.ibm.team.workitem.linktype.duplicateworkitem" && linkTypes[i].isSource ) {
						for(var j=0;j<linkTypes[i].linkDTOs.length; j++) {
							if(!linkTypes[i].linkDTOs[j]._isDeleted)
								linkTypes[i].linkDTOs[j]._isDeleted = true;
						} 
					}
				}
				this.refresh();
		},
		_findInArray: function(item, array) {
			for(var i=0;i<array.length;i++){
				if(item.itemId == array[i].itemId) return true;
			}
			return false;
		},
//		_markDirtyIfDifferent: function() {
//			if(this.workingCopy.isChanged()){
//				this.parentWidget.parentController.markDirty();
//			}
//		},
		_getRow: function(listWidget, linkType, isFirst, isLast) {
			var layout = WidgetUtil.mergeObjects({}, columnLayout.name);
			layout.label = linkType.displayName;
			layout.isFirst = isFirst;
			layout.isLast = isLast;
			return listWidget.getLabelWidget(layout);
		},
		_findLinkType: function(searchLinkType) {
			var linkType;
			var found = false;	
			var linkTypes = this._getWorkingCopyLinkTypes();		
			for(var i=0;i<linkTypes.length;i++){
				if(linkTypes[i].id == searchLinkType.id && linkTypes[i].isSource == searchLinkType.isSource) {
					found = true;
					linkType =linkTypes[i]; 
					break;
				}
			}

			if (!found) {
				linkTypes = this._editableLinkTypes;
				for (var i = 0; i < linkTypes.length; i++) {
					if (linkTypes[i].id == searchLinkType.id && linkTypes[i].isSource == searchLinkType.isSource) {
						found = true;
						linkType = linkTypes[i];
						this._getWorkingCopyLinkTypes().push(linkType);
						break;
					}
				}
			}

			if(!found) { // Should never happen
				linkType =  EDITABLE_LINK_TYPES.related;
				linkType.itemType = "WorkItem";
				linkType._isNew = true;
				this._getWorkingCopyLinkTypes().push(linkType);
			}
			
			if(!linkType.linkDTOs) {
				linkType.linkDTOs = [];
			}
			return linkType;
		},
		_addItemToLinks: function (linkType, item) {
			var links = linkType.linkDTOs
			var itemId = item.workItemItemId ? item.workItemItemId : item.itemId;
			var found = false;
			for(var i=0;i<links.length;i++){
				if(links[i].itemId == itemId) {
					found = true;
					links[i]._isDeleted = false;
					break;
				}
			}
			
			if (!found) {
				var link = {
					_isNew: true,
					id: item.id,
					itemId: itemId,
					comment: item._linkComment
				}
				var idAttr = {
					id: item.id
				}
				var target = {
					_eQualifiedClassName: "com.ibm.team.workitem.rest.dto:WorkItemDTO",
					attributes: [
							{ key: "id",
							  value: idAttr } ]
				}
				link.target = target;
				
				if (linkType.isSingleValued) {
					// mark all the other links deleted
					var newLinks = [];
					for (var i = 0; i < links.length; i++) {
						if (links[i].itemId != itemId) {
							if(!links[i]._isNew) {
								newLinks.push(links[i]);
							}
							links[i]._isDeleted = true;
						}
					}
					links = newLinks;
					linkType.linkDTOs = links; 
				}
				
				links.push(link);
			}	
		},
		hasDuplicateOfLink: function() {
				// If the work item has a duplicate link
				// and it hasn't been deleted
				// or it has been added
				// then it is ok, otherwise, show a message
				
				// Does the work item have duplicate links
				var currentDuplicates = [];
				var linkTypes = this._getWorkingCopyLinkTypes();
				if(!linkTypes) return false;
				for(var i = 0;i<linkTypes.length;i++ ){
					if(linkTypes[i].id == "com.ibm.team.workitem.linktype.duplicateworkitem" && linkTypes[i].isSource ) {
						for(var j=0;j<linkTypes[i].linkDTOs.length; j++) {
							if(!linkTypes[i].linkDTOs[j]._isDeleted)
								currentDuplicates.push(linkTypes[i].linkDTOs[j].itemId);
						} 
					}
				}
				
//				var foundNewDuplicate = false;
//				var saveCommands = this.parentController._getSaveCommands();
//				if(saveCommands.updateLinks) {
//					for(var i=0;i<saveCommands.updateLinks.length;i++) {
//						var cmd = saveCommands.updateLinks[i];
//						if(cmd.cmd == "addLink" && cmd.end == "source" && cmd.type=="com.ibm.team.workitem.linktype.duplicateworkitem") {
//							foundNewDuplicate = true;
//						}
//						if(cmd.cmd == "removeLink" && cmd.end == "source" && cmd.type=="com.ibm.team.workitem.linktype.duplicateworkitem") {
//							for(var j=0;j<currentDuplicates.length;j++) {
//								if(currentDuplicates[j] == cmd.itemId) {
//									currentDuplicates.splice(j);
//									break;
//								}
//							}
//						}
//					}
//				}
//				
//			 return currentDuplicates.length > 0 || foundNewDuplicate;

			return currentDuplicates.length > 0
		},	
		_handleDeleteItemPressed: function(e) {
			if(e && e.target && e.target.parentWidget && e.target.parentWidget._deleteLink) {
				e.target.parentWidget._deleteLink(e.target);
			}
		},
		_handleRestoreItemPressed: function(e) {
			if (e && e.target && e.target.parentWidget && e.target.parentWidget._restoreLink) {
				e.target.parentWidget._restoreLink(e.target);
			}
		},
		_getLinkTypeSpec: function(actionValue) {
		    if(actionValue.isSource) return actionValue; // already a link
			// Find or add the link type
			var linkType = {
			    	itemType: actionValue.linkTypeItemType,
					id: actionValue.linkTypeId,
					isUserDeleteable: actionValue.linkTypeIsUserDeleteable,
					isSource: actionValue.linkTypeIsSource
			}
			return linkType
		},
		_deleteLink: function(action) {
			var itemToDelete = action.value;
			var linkTypeSpec = this._getLinkTypeSpec(action.value);
			var linkType = this._findLinkType(linkTypeSpec);
			var links = linkType.linkDTOs; 
			var wasDeleted = false;
			if(itemToDelete.itemId) {
				for(var i=0;i<links.length;i++){
					if(links[i].itemId == itemToDelete.itemId) {
						links[i]._isDeleted = true;
						if(links[i]._isNew) {
							links.splice(i,1);
						}
						wasDeleted = true;
						break;
					}
				}
			} else {
				if(itemToDelete.url && itemToDelete.comment) {
					for(var i=0;i<links.length;i++){
					if(!links[i]._isDeleted && links[i].url && links[i].comment  && links[i].url == itemToDelete.url && links[i].comment == itemToDelete.comment) {
						wasDeleted = true;
						links[i]._isDeleted = true;
						if(links[i]._isNew) {
							links.splice(i,1);
						}
						break;
					}
				}	
				}		
			}
			
			if(wasDeleted) {
				this._parentMarkDirty();
			}
			this._setWorkingCopyValue();
			this.refresh();
		},
		_parentMarkDirty: function() {
			if(this.parentWidget && this.parentWidget.markDirty) {
					this.parentWidget.markDirty();
				} 

			if(this.parentWidget && this.parentWidget.parentController && this.parentWidget.parentController.markDirty) {
						this.parentWidget.parentController.markDirty();
				}
		},
		_restoreLink: function(action) {
			var itemToDelete = action.value;
			
			// Find or add the link type
			var linkTypeSpec = this._getLinkTypeSpec(action.value);
			var linkType = this._findLinkType(linkTypeSpec);
			var links = linkType.linkDTOs; 

			if(itemToDelete.itemId) {
				for(var i=0;i<links.length;i++){
					if(links[i].itemId == itemToDelete.itemId) {
						links[i]._isDeleted = false;
						break;
					}
				}
			} else {
				if(itemToDelete.url && itemToDelete.comment) {
					for(var i=0;i<links.length;i++){
					if(!links[i]._isDeleted && links[i].url && links[i].comment  && links[i].url == itemToDelete.url && links[i].comment == itemToDelete.comment) {
						links[i]._isDeleted = false;
						break;
					}
				}	
				}		
			}
			this._setWorkingCopyValue();
			this.refresh();
		},
		_handleAddLinkButtonPressed: function(action) {
			if(!action || !action.value || !action.value.itemType) return;
			var selector = linkTypeEditors[action.value.itemType];
			if(selector) {
				this[selector](action);
			}
		},
		showLinksTab: function() {
			if(this.parentWidget.tabId) {
				var url = window.location.href;
				url = (url + "&").replace(/&&/g, "&");
				url = url.replace(/&tab=.+&/, "") + "&tab=" + this.parentWidget.tabId;
				window.location.href = url;
			}
		},
		_addCalmLink: function(action) {
			var self= this;
			
			// acquire remote picker description
			var serviceObject = {
				_successCallback: function(pickerDescription) {
					var params= {
						dialogTitle: pickerDescription.title,
						secondaryTitle: "",
						width: pickerDescription.widthHint,
						height: pickerDescription.heightHint,
						namespace: pickerDescription.namespace,
						parentElement: self.domNode,
						onOk: function(results) {
							self._addCalmLinksOK(action, results);
						},
						onClose: function() {
							self.parentController.focus();
						},
						pickerURL: pickerDescription.url
					};
					new com.ibm.team.workitem.web.ui.internal.view.editor.parts.RemoteItemPicker(params);
				},
				_errorCallback: function(errorObj) {
					alert(dojo.string.substitute(self.messages.errorFailedToLoadServiceDocument, [errorObj.message]));
				}
			};
			
			var serviceDocument= action.value.calmServiceDescriptor.url;
			var responseHandler = new com.ibm.team.repository.web.transport.ServiceResponseHandler(serviceObject, "_successCallback", "_errorCallback");
			var args = {
				serviceDocumentLocation: serviceDocument,
				workItemLinkTypeId: action.value.id
			}
			com.ibm.team.workitem.web.client.internal.WorkItemClient.getPickerDescription(responseHandler, args);
		},
		_addUriLink: function(action) {
		    var self= this;
			var title = dojo.string.substitute(this.messages.buttonAddLink, [ action.label ] );
			var params= {
				dialogTitle: title,
				parentController: this.parentWidget.parentController || this.parentWidget,
				parentElement: this.domNode,
				onOk: function(parms) {
					self._addUriLinkOK(parms);
				},
				onClose: function() {
					self.parentController.focus();
				},
				action: action
			};
			new com.ibm.team.workitem.web.ui.internal.view.editor.SelectUriAndCommentDialog(params);

		},
		_addCalmLinksOK: function(action, links) {
			if(!action) return;
			var linkType = this._findLinkType(action.value);
			for (var i= 0; i < links.length; i++) {
				var item= links[i];
				var dto = {
					url: item.url,
					comment: item.label,
					_isNew: true
				}
				linkType.linkDTOs.push(dto);
			}
			this._parentMarkDirty();
			this._setWorkingCopyValue();
			this.refresh();
		},
		_addUriLinkOK: function(parms) {
			if(!parms) return;
			var linkType = this._findLinkType(parms.action.value);
			var dto = {
				url: parms.uri,
				comment: parms.comment,
				_isNew: true
			}
			linkType.linkDTOs.push(dto);
			this._parentMarkDirty();
			this._setWorkingCopyValue();
			this.refresh();
		},
		_addWorkItemLink: function(action) {
			this._openAddWorkItemDialog(action);
		},
		_openAddWorkItemDialog: function(action) {
			if(!EditorUtil.checkLogin()) return;
			var self= this;
			var title = dojo.string.substitute(this.messages.buttonAddLink, [ action.label ] );
			var params= {
				dialogTitle: title,
				allowsMultiple: !action.value.isSingleValued,
				parentController: this.parentWidget.parentController || this.parentWidget,
				parentElement: this.domNode,
				exclusionList: this.getCurrentLinkItemIds(),
				onOk: function(selectedWorkItems, action) {
					self.addNewLink(selectedWorkItems, action);
				},
				onClose: function() {
					self.parentController.focus();
				},
				action: action
			};
			new SelectWorkItemDialog(params);
		},
		addNewLink: function(workItems, action) {
			workItems.sort(
				function(a, b) {
					aId= parseInt(a.id, 10);
					bId= parseInt(b.id, 10);
					return aId - bId;
				});
				
			// These are either work items or wrappers from the dialog
			for (var i=0; i<workItems.length; i++) {
				if(workItems[i].attributes && workItems[i].attributes.summary) {
					workItems[i]._linkComment = EditorUtil.getSummaryLabel(workItems[i]);
				} else {
					workItems[i]._linkComment = workItems[i].id + ": " + EditorUtil.restoreTags(workItems[i].summary);
				}
			}
			
			// Find or add the link type
			var linkType = this._findLinkType(action.value); 
			
			// Add the work items to the type's linkDTOs
			var workItem, wasUpdated=false;
			for (var i=0; i<workItems.length; i++) {
				if(this._workItem.itemId == workItems[i].workItemItemId) {
					alert(this.messages.msgLinkCannotReferenceItself);	
				}	else {
					wasUpdated = true;
					this._addItemToLinks(linkType, workItems[i]);
				}
			};
			if(!wasUpdated) return;
			this._parentMarkDirty();
			this._setWorkingCopyValue();
			this.refresh();
		},
		/**
		 * Returns an array of the itemIds of the current linkTypes.
		 */
		getCurrentLinkItemIds: function() {
			var result= [];
			
			if(this.workingCopy && this.workingCopy.object && this.workingCopy.object.itemId) {
				result.push(this.workingCopy.object.itemId);				
			}
			var linkTypes = this._getWorkingCopyLinkTypes();
			for (var i=0; i<linkTypes.length; i++) {
				if(!linkTypes[i]._isDeleted && linkTypes[i].itemType == "WorkItem") result.push(linkTypes[i].itemId);
			}
			
			return result;
		}
	}
);

})();
