/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.StatusAttributePart");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.EnumerationWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.parts.AbstractAttributePart");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "AttributePresentationMessages");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;

dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.StatusAttributePart",
	[com.ibm.team.workitem.web.ui.internal.view.layout.EnumerationWidget, com.ibm.team.workitem.web.ui.internal.view.editor.parts.AbstractAttributePart],
	{
        constructor: function() {
			this.attrMessages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "AttributePresentationMessages");
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
		},
		postCreate: function()  {
//			if(this.isNew()) this.setReadOnly(true);
			if(!this.attributeId) this.attributeId = "internalState";
			if(!this.attributeValue) this.attributeValue = this.workingCopy.getValue({
				path: "attributes.internalState"
			});

			this._getPossibleValues();
			
			// a state change can be an action
			this.workingCopy.addListener({
				path: ["attributes", "workflowAction", "id"],
				event: "onchange",
				listener: this,
				functionName: "_handleAttributeChange"
			});
			// handle the editProps changing			
			this._workItemChangedHandle = dojo.subscribe(com.ibm.team.workitem.web.ui.internal.page.EventTopics.WORKITEM_CHANGED, dojo.hitch(this, this._handleWorkItemChanged));
			this.inherited(arguments);
		},
		destroyWidget: function() {
			dojo.unsubscribe(this._workItemChangedHandle);
			this.inherited(arguments);
		},
		getDefaultLabel: function() {
			return this.messages.labelState;
		},
		_handleWorkItemChanged: function(workingCopy) {
			if(workingCopy !== this.workingCopy) return
			this._handleAttributeChange();
		},
		_getEmptyValueLabel: function() {
			return this.messages.optionSaveToEnable;
		},
		_handleAttributeChange: function() {	
			this._getPossibleValues();
			if(this._possibleValues && this._possibleValues.length > 0) {
				this._bindValuesToSelect(this._possibleValues, this._select, this._image);
				var value = this.workingCopy.getValue({path: "attributes.workflowAction.id"});
				if(!value) {
					value = this.workingCopy.getValue({path:"attributes.internalState.id"});	
				}
				this.setValue(value, /* nonotify */ true);
			}			
		},
		_getPossibleValues: function() {
			this._possibleValues = [ ];			
			var state = this.workingCopy.getValue({path: "attributes.internalState"});
			if(state) this._possibleValues.push(state); 
			if(this.workingCopy && this.workingCopy.workItemSpec && this.workingCopy.workItemSpec.editProps) {
				var allValues = this.getAllValueAttributeMap(this.workingCopy.workItemSpec.editProps);
				if(allValues && allValues.actionValues) {
					for(var i=0;i<allValues.actionValues.length; i++) {
						this._possibleValues.push(allValues.actionValues[i]);
					}
				}
			}
			
			if (this._possibleValues.length == 0) {
				var fakeItem = {
					iconUrl: false,
					id: "1",
					itemType: "smallString",
					label: " "
				}
				this._possibleValues.push(fakeItem);
				this._isAlwaysDisabled = true;
				this._select.disabled = true;
			}
			else {
				this._isAlwaysDisabled = false;
				this._select.disabled = false;
			}
		},
		/*
		 * If there's a workflowAction use that
		 * else the internalState
		 */ 
		_getOverrideAttributeId: function() {
			if(this.workingCopy.getValue({path: "attributes.workflowAction"})) {
				return "workflowAction";
			}
			return this.attributeId;
		},
		_handleSelectChange: function(e) {
			// a state change is actually an action 
			this.workingCopy.setValue({
				path: ["attributes", "workflowAction", "id"],
				attributeId: "workflowAction",
				isInitialization: false,
				value: this.getValue()
			});
			if(this.parentWidget && this.parentWidget._handleDirtyingEvent) {
				this.parentWidget._handleDirtyingEvent();
			}
		}
	}
);

})();
