/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.TagWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.string.BidiComplex");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.TagSelector");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.TagWidget",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/layout/templates/TagWidget.html"),
		postCreate: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
			var value = this._getAttributeValue();
			value = value && value.content ? value.content : "";
			
			this._setTagsValue(value);				
			this._setReadOnlyTagValues(this.tags.value);
			this._previousTagsValue = this.tags.value;
			
			var self = this;
			this._tagSelector= /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.TagSelector( {	
					_tagNode: this.tags,
					parentWidget: this,
					_parentNode: this.domNode,
					_callback: function(str) {
						self._updateTagText(str);
					}
				});
			if (this.workingCopy.isGuest()) {
				dojo.addClass(this.tags, "GuestField");
			}
			this._isChanged = false;
			this.setReadOnly(false);

			this.workingCopy.addListener({
				path: ["attributes", this.attributeId],
				event: "onchange",
				listener: this,
				functionName: "_handleAttributeChange"
			});
			
			this._tagSelector.setAllTags([]);
		},
		_handleAttributeChange: function(parms) {
			var value = parms.attributeValue; /* content */
			this._setTagsValue(value);
			this._setReadOnlyTagValues(this.tags.value);
			this._previousTagsValue = this.tags.value;
		},
		_getAttributeValue: function() {
			return this.workingCopy.getValue({ path: ["attributes", this.attributeId] });			
		},
		_setAttributeValue: function(value, isInit){
			value = dojox.string.BidiComplex.stripSpecialCharacters(value);
			this.workingCopy.setValue({
				caller: this,
				path: ["attributes", this.attributeId, "content"],
				savedAttributePath: ["attributes", this.attributeId, "content"],
				attributeId: this.attributeId,
				isInitialization: isInit,
				value: value
			});
		},
		destroy: function(){
			try {
				if (this._tagSelector) 
					this._tagSelector.destroy();
				this.inherited(arguments);
			} 
			catch (e) {
			}
		},
		setReadOnly: function(isReadOnly) {
			if (isReadOnly) { // was mouse out
				if (this._isChanged) {
					this.setReadOnly(false);
					return;
				}
				this.tags.style.display = "none";
				this._readOnlyTagValues.style.display = "inline";
			}
			else {
				this.tags.style.display = "inline";
				this._readOnlyTagValues.style.display = "none";
			}
		},
		_setReadOnlyTagValues: function(value){
			value = dojox.string.BidiComplex.createDisplayString(value || "","EMAIL");
			dojox.data.dom.removeChildren(this._readOnlyTagValues)
			this._readOnlyTagValues.appendChild(document.createTextNode(value));
		},
		_markDirty: function() {
			this._isChanged=true;
			//					this.parentController.markDirty();
		},
		_updateTagText: function(str) {
			this._setTagsValue(this._cleanUpTags(this.tags.value));
			var allTagsArr= this.tags.value.split(",");
			allTagsArr[this._tagReplaceIndex]= this._tagReplaceIndex > 0 ? " " + str : str;
			this.tags.value= allTagsArr.join(",");
			this.tags.focus();
			this._previousTagsValue= this.tags.value;
			this._setAttributeValue(this.tags.value);
		},
 		_handleTagKeydown: function(e)  {
			
			if (!this._tagSelector.isShowingNow) { return; }
			
			switch(e.keyCode) {
			// tab
			case 9:
			// enter
			case 13:
				this._tagSelector.selectCurrentTag();
				e.stopPropagation();
				break;
			// down key
			case 40:
				this._tagSelector.highlightNextTag("down");
				break;
			// up key
			case 38:
				this._tagSelector.highlightNextTag("up");
				break;
			}
			
			var value = this.tags.value;
			value = value.replace(/\n/g, " ");
			value = value.replace(/\r/g, " ");
			value = value.replace(/  /g, " ");
			this._setTagsValue(value);
			this._setAttributeValue(this.tags.value);
		},
 		_handleTagKeystroke: function(e)  {
			
			var orig= this._previousTagsValue;

			var newVal= this._cleanUpTags(this.tags.value);
			var orig = this._cleanUpTags(this._previousTagsValue);	

			var maxLen= Math.max(orig.length, newVal.length);

//			console.debug("newval:>" + newVal + '<')
			var tagIndex= 0;
			for (var i=0; i<maxLen; i++) {
				if (orig.charAt(i) !== newVal.charAt(i)) {
					// if we reached this point, the text of the field has changed
					this._markDirty();
					if (newVal.charAt(i) === "," && newVal.length > orig.length) {
						// if the user just typed a comma, close the popup
						this._tagSelector.close();
						break;
					}
					// tokenize the entire tag field by commas
					var allTagsArr= newVal.split(",");
					// we've been keeping track of how many commas we hit before
					// we discovered a change between the current value and the
					// previous value, so the search term is simply the token
					// at the current index
					var searchTerm= dojo.trim(allTagsArr[tagIndex], /*trimFromStart*/1);
//					console.debug("searchTerm:>" + searchTerm + '<')
					if (searchTerm) {
						// if we have a valid search term (i.e. a non-empty
						// string), we'll show the tag picker
						
						// the set of terms to exclude are all of current tags
						// other than the search term
						allTagsArr.splice(tagIndex, 1);
						// remember which of the tokens should be replaced if
						// the user clicks on one of the matching tags
						this._tagReplaceIndex= tagIndex;
						// display the matching 
						this._showTagSelector(searchTerm, allTagsArr);
					} else {
						// if we don't have a valid search term, hide the tag
						// picker
						this._tagSelector.close();
					}
					break;
				}
				if (newVal.charAt(i) === ",") {
					// if we're here, the tags are still the same and we've hit
					// a comma; increment the tagIndex to denote that we're now
					// within a new term
					tagIndex++;
				}
			}
			if (dojo.trim(this.tags.value) === "") {
				this._tagSelector.close();
			}
			// now that we're done processing the key event, set the current
			// value of the tags value as the old value for next time
			this._previousTagsValue= this.tags.value;
			this._setAttributeValue(this.tags.value);
		},
		_showTagSelector: function(searchTerm, allTagsArray) {
			    if(this._areTagsLoaded) {
					this._tagSelector.showMatchingTags(searchTerm, allTagsArray);
					return			    	
			    }
			this._lastSearchTerms =  {searchTerm: searchTerm, allTags: allTagsArray}; // use the last value requested

			if(this._isLoadingTags) return; // prevent multiple loading
			this._isLoadingTags = true;
            var self = this;
            var serviceObj = {
                success: function(tags){
					self._tagSelector.setAllTags(tags[0].uiItems);
					self._areTagsLoaded = true;
					self._tagSelector.showMatchingTags(self._lastSearchTerms.searchTerm, self._lastSearchTerms.allTagsArray);
                },
                failure: function(error){
					console.log(this.messages.errorLoadingProperties + " - " + this.attributeId);
                }
            };
			var srh= new com.ibm.team.repository.web.transport.ServiceResponseHandler(serviceObj, "success", "failure");
			
			var args= { newWorkItem: true, ids: [ this.attributeId ] };
			com.ibm.team.workitem.web.client.internal.WorkItemClient.getAllValues(srh, args);							
		},
		_handleDirtyingEvent: function(e)  {

		},
		_handleTagBlur: function(e) {
			
			if(!this._tagSelector.isMouseOver) {
				this._tagSelector.close();				
			}

			this._setTagsValue(this._cleanUpTags(this.tags.value));
			this._setReadOnlyTagValues(this.tags.value);
		},
		_setTagsValue: function(value) {
			value = dojox.string.BidiComplex.createDisplayString(value || "","EMAIL");
			this.tags.value = value;	
		},
		_cleanUpTags: function(value) {
			value = dojox.string.BidiComplex.stripSpecialCharacters(value);
			
			var newVal= value.toLowerCase();
			newVal = newVal.replace(/\n/g, " "); // enter to space
			newVal = newVal.replace(/\r/g, " "); // enter to space
			// Replace spaces with commas, double commas and empty spaces
			// There's probably a better way to do do this...
			while(newVal.indexOf("  ") != -1) {
				newVal = newVal.replace(/  /g,' ');  // double space to spaces				
			}
			newVal = newVal.replace(/ /g,','); // spaces to commas
			while (newVal.indexOf(",,") != -1) {
				newVal = newVal.replace(/,,/g, ','); // double commas to commas
			}
			newVal = newVal.replace(/,/g,', '); // commas to space+comma
			while (newVal.indexOf(", , ") != -1) {
				newVal = newVal.replace(/, , /g, ', '); // remove extra space+commas
			}
			
			return newVal
		}		
	}
);

})();
