/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.PlanItemViewerItemColumn"); //$NON-NLS-1$

dojo.require("dijit.Menu"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.internal.ImageClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.WorkItemSaveManager"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.dialogs.SaveErrorDialog"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.dialogs.IntegerSelectionDialog"); //$NON-NLS-1$

dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var ImageClient				= com.ibm.team.apt.web.client.internal.ImageClient;
var WorkItemSaveManager		= com.ibm.team.apt.web.client.internal.WorkItemSaveManager;
var SaveErrorDialog			= com.ibm.team.apt.web.ui.internal.view.plan.dialogs.SaveErrorDialog;
var IntegerSelectionDialog	= com.ibm.team.apt.web.ui.internal.view.plan.dialogs.IntegerSelectionDialog;

var doc= document;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
var bind= dojo.string.substitute;

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.PlanItemViewerItemColumn", null, {//$NON-NLS-1$

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------

	_planItemViewerItem: null,
	_planItem: null,
	_attribute: null,
	
	_value: null,
	_node: null,
	
	__editLinkListener: null,
	
	constructor: function(attributeId, planItemViewerItem) {
		this._planItemViewerItem= planItemViewerItem;
		this._planItem= planItemViewerItem.getPlanItem();
		this._attribute= this._planItem.getPlanModel().findAttribute(attributeId);

		this._value= this._planItem.getAttributeValue(this._attribute);
	},

	// ---- PlanItemViewerItem api -----------------------------------------------------------------------------------------

	_createContainer: function() {
		this.__createContent();
		return this._node;
	},
	
	_dispose: function() {
		if (this.__editLinkListener != null) {
			dojo.disconnect(this.__editLinkListener);
		}
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	__createContent: function() {
		var localDoc= doc;
		var node= localDoc.createElement("td"); //$NON-NLS-1$
		
		dojo.addClass(node, "attribute"); //$NON-NLS-1$
		
		var container= localDoc.createElement("div"); //$NON-NLS-1$
		dojo.addClass(container, "attributeDropDownIndicator"); //$NON-NLS-1$
		
		var icon= this._getIcon();
		if (icon != null) {
			var iconElement= localDoc.createElement("img"); //$NON-NLS-1$
			dojo.addClass(iconElement, "icon"); //$NON-NLS-1$
			iconElement.src= ImageClient.getImageUrl(icon);
			container.appendChild(iconElement);
		}

		var labelElement= localDoc.createElement("span"); //$NON-NLS-1$
		dojo.addClass(labelElement, "label"); //$NON-NLS-1$
		
		var label=this._getLabel();
		if (label === null)
			label= ""; //$NON-NLS-1$
		
		if (label.length > 75)
			label= label.substring(0, 75) + "..."; //$NON-NLS-1$
		
		labelElement.appendChild(localDoc.createTextNode(label));
		container.appendChild(labelElement);
		 
		if (this._isEditable()) {
			dojo.addClass(node, "editableColumn"); //$NON-NLS-1$
			this.__editLinkListener= dojo.connect(container, "onclick", this, this._onDropDownClick); //$NON-NLS-1$
		}
		
		node.appendChild(container);
		this._node= node;
	},
	

	// ---- implementation -------------------------------------------------------------------------------------------------
	
	_getIcon: function() {
		var value= this._value;
		if (value) {
			return value.iconURL;
		}
		
		return null;
	},
	
	_getLabel: function() {
		var value= this._value;
		if (value === undefined || value === null)
			return null;
		
		return String(value.label !== undefined ? value.label : value);
	},
	
	_isEditable: function() {
		if (this._attribute.attributeType == com.ibm.team.apt.client.PlanningAttributeType.INTEGER) {
			return (this._value === undefined) ?  false: !this._attribute.readOnly; //$NON-NLS-1$
		};
		
		if (this._attribute.attributeType == com.ibm.team.apt.client.PlanningAttributeType.ENUMERATION)
			return this._getValues() != null && !this._attribute.readOnly;
		
		if (this._attribute.attributeType == com.ibm.team.apt.client.PlanningAttributeType.ITEM)
			return this._getValues() != null && !this._attribute.readOnly;
		
		return false;
	},
	
	_getValues: function() {
		if (!this._value === undefined)
			return null;
		
		var valueSet= this._attribute.getValueSet(); 
		if (!valueSet)
			return null;
		
		if (!valueSet.values)
			return null;
		
		if (valueSet.values.length == 0)
			return null;
		
		return valueSet.values;
	},
	
	_setValue: function(newValue) {
		WorkItemSaveManager.compoundSave(function() {
			this._planItem.setAttributeValue(this._attribute, newValue);
		}, this._planItemViewerItem.getViewModel(), {
			showErrorMessage: function(message, description) {
				if (message != null) {
					new SaveErrorDialog({message: message, description: description});
				}
			}
		}, this);
	},
	
	_onDropDownClick: function(e) {
		if (e) {
			dojo.stopEvent(e);
		}
		if (this._menu) {
			this._closeMenu();
			return;
		}
		
		if (this._attribute.attributeType == com.ibm.team.apt.client.PlanningAttributeType.INTEGER) {
			this._editIntegerItem();
		} else {
			this._editValueSetItem();
		}
	},
	
	_editIntegerItem: function() {		
		var dialog= new IntegerSelectionDialog({
			attributeName: this._attribute.getLabel(),
			attributeValue: this._value,
			_self: this,
			onOk: function(args) {this._self._setValue(args.value);}
		});
	},
	
	_createMenuEntries: function() {
		var result= new Array();
		
		var values=this._getValues();
		for ( var i = 0; i < values.length; i++) {
			var value= values[i];
			result.push(this._createMenuEntry(
					value.label ? value.label : value, 
					value.iconURL ? ImageClient.getImageUrl(value.iconURL) : null,
					{newValue: value},
					dojo.hitch(this, function(context) {
						this._setValue(context.newValue);
					})
			));
		}
		
		return result;
	},
	
	_createMenuEntry: function(label, iconUrl, context, callback) {
		return new com.ibm.team.apt.web.ui.internal.view.plan.planneditems.column.IconUrlMenuItem({
			label: label,
			iconUrl: iconUrl,
			context: context,
			onClick: function() {callback(context);}
		});
	},
	
	_editValueSetItem: function() {
		var menu = new dijit.Menu();
		dojo.addClass(menu.domNode, "com-ibm-team-apt-attribute-value-selection-drop-down-menu"); //$NON-NLS-1$
		
		var entries= this._createMenuEntries();
		for ( var i = 0; i < entries.length; i++) {
			menu.addChild(entries[i]);
		}
	
		var pos = dojo.coords(this._node, true);
		
		this._node.setAttribute("open", "true"); //$NON-NLS-1$ //$NON-NLS-2$
		dijit.popup.open({
			popup: menu,
			parent: this._planItemViewerItem.getViewer(),
			x: pos.x - 2,
			y: pos.y + pos.h - 2,
			onExecute: dojo.hitch(this, this._closeMenu),
			orient: 'BL' //$NON-NLS-1$
		});
		
		var documentBody= dojo.body();
		this._menuCloseOnBody= dojo.connect(window.document, "onclick", this, this.__menuCloseListener); //$NON-NLS-1$
		
		this._menu = menu;
		
		menu.focus();
	},
	
	_closeMenu: function(e) {
		if (e) {
			dojo.stopEvent(e);
		}
		dojo.disconnect(this._menuCloseOnBody);
		dijit.popup.close(this._menu);
		this._menu = null;
		this._node.focus();
		this._node.setAttribute("open", "false"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	__menuCloseListener: function(evt){ //$NON-NLS-1$
		try {
			var documentBody= dojo.body();
			var node= evt.target || evt.srcElement;
			var found= false;
			while (node) {
				if (node === documentBody) {
					break;
				}

				if (node === this._menu.domNode || node === this._node) {
					found= true;
					break;
				}
				node= node.parentNode;
			}
			if (!found) {
				this._closeMenu();
			}
		} catch (e) {
		}
	},
	
	__sentinel: null // terminates this class definition
});

dojo.declare("com.ibm.team.apt.web.ui.internal.view.plan.planneditems.column.IconUrlMenuItem", //$NON-NLS-1$
		[dijit.MenuItem],
		{
		// summary: A line item in a Menu Widget

		// Make 3 columns
		//   icon, label, and expand arrow (BiDi-dependent) indicating sub-menu
		templateString:
			 '<tr class="dijitReset dijitMenuItem" dojoAttachPoint="focusNode" waiRole="menuitem" tabIndex="-1"' //$NON-NLS-1$
			+'dojoAttachEvent="onmouseenter:_onHover,onmouseleave:_onUnhover,ondijitclick:_onClick">' //$NON-NLS-1$
			+'<td class="dijitReset" waiRole="presentation"><div class="dijitMenuItemIcon" style="background-image: url(${iconUrl})" dojoAttachPoint="iconNode"></div></td>' //$NON-NLS-1$
			+'<td class="dijitReset dijitMenuItemLabel" dojoAttachPoint="containerNode"></td>' //$NON-NLS-1$
			+'<td class="dijitReset dijitMenuArrowCell" waiRole="presentation">' //$NON-NLS-1$
				+'<div dojoAttachPoint="arrowWrapper" style="display: none">' //$NON-NLS-1$
					+'<div class="dijitMenuExpand"></div>' //$NON-NLS-1$
					+'<span class="dijitMenuExpandA11y">+</span>' //$NON-NLS-1$
				+'</div>' //$NON-NLS-1$
			+'</td>' //$NON-NLS-1$
			+'</tr>', //$NON-NLS-1$

		__sentinel: null // terminates this class definition
});

})();
