/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.OperationSectionContainer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.queryeditor.PopupButton");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
(function() {
	
var PopupButton= com.ibm.team.workitem.web.ui.internal.view.queryeditor.PopupButton;
var Messages= dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "QueryMessages");
	
dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.OperationSectionContainer",
	[dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/templates/OperationSectionContainer.html"),
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args) {
		this._labelProperty= "name";
		this.sectionWidget= args.sectionWidget;
		this._labelClose= Messages.labelClose;
		this._labelRun= Messages.labelRun;
		this._labelClear= Messages.labelClear;
		this._labelStop= Messages.buttonStop;
	},
	
	postCreate: function() {
		this._operationsContainer.appendChild(this.sectionWidget.domNode);
		this.setStopButtonEnabled(false);
		if (this.sectionWidget.setSectionSite)
			this.sectionWidget.setSectionSite(this);	
		this._changeHandle = dojo.subscribe(com.ibm.team.workitem.web.ui.internal.page.EventTopics.QUERY_EDITOR_CHANGED, dojo.hitch(this,this._handleQueryChange));
	},
	_handleQueryChange: function() {
		this.sectionWidget.setRunButtonEnabled();
	},
	destroy: function() {
		if(this._menuWidget && this._menuWidget.destroy) this._menuWidget.destroy();
		if (this.sectionWidget)
			this.sectionWidget.destroy();
		dojo.unsubscribe(this._changeHandle);
		this.inherited(arguments);
	},
	
	setTitle: function(title) {
		dojox.data.dom.replaceChildren(this._headerText, document.createTextNode(title));
	},
	
	setupMenu: function(property, callback) {
		this._labelProperty= property;
		this._callback= callback;
	},
	
	setMenuItems: function(menuItems, callback) {
		var menu= this._getMenu();
		menu.setValues(menuItems);
	},
	
	setSelectedMenuItem: function(item) {
		var menu= this._getMenu();
		menu.setSelection(item);
	},
	setStopButtonEnabled: function(isEnabled) {
		if(isEnabled) {
			dojo.removeClass(this._stopAction, "stop-disabled");
			dojo.addClass(this._stopAction, "stop");
		} else {
			dojo.removeClass(this._stopAction, "stop");
			dojo.addClass(this._stopAction, "stop-disabled");
		}
		
	},
	setRunButtonEnabled: function(isEnabled) {
		if(isEnabled) {
			dojo.removeClass(this._runAction, "run-disabled");
			dojo.addClass(this._runAction, "run");
		} else {
			dojo.removeClass(this._runAction, "run");
			dojo.addClass(this._runAction, "run-disabled");
		}
		
	},
	///////////////////////////////////////////////////////////////////////////////////
	// Internals
	
	_getMenu: function() {
		if (!this._menuWidget) {
			var menu= new PopupButton({ labelProperty: this._labelProperty, callback: this._callback });
			this._headerMenu.appendChild(menu.domNode);
			this._menuWidget= menu;
		}
		return this._menuWidget;
	},
	onClear: function() {
			this.sectionWidget.clearOperations();
	},
	onRun: function() {
		if(dojo.hasClass(this._runAction, "run-disabled")) return;
		var rowCount = this.sectionWidget.getSelectedRows().length;
		if(rowCount == 0) {
			console.debug("Error... no items selected for operation run.") // $NON-NLS-1 should never happen
			return;
		}
		var message = dojo.string.substitute(Messages.confirmRunOperation, [rowCount + ""]);
		if(!confirm(message)) return;
		this.setRunButtonEnabled(false);
		this.sectionWidget.runOperation();
		},
	onStop: function() {
		this.sectionWidget.stopOperation();
		},
	onClose: function() {
		var destroy= true;
		var sectionWidget= this.sectionWidget;
		if (sectionWidget && sectionWidget.onClose) {
			destroy= sectionWidget.onClose();
		}
		if (destroy) {
			this.destroy();
		}
	},
	checkEnterForRun: function(e) {
		var characterCode;
		characterCode = e.keyCode;
		if(characterCode == 13) { //the key presed was 'enter'
			this.onRun();
		}		
	},
	checkEnterForStop: function(e) {
		var characterCode;
		characterCode = e.keyCode;
		if(characterCode == 13) { //the key presed was 'enter'
			this.onStop();
		}		
	},
	checkEnterForClear: function(e) {
		var characterCode;
		characterCode = e.keyCode;
		if(characterCode == 13) { //the key presed was 'enter'
			this.onClear();
		}		
	},
	checkEnterForClose: function(e) {
		var characterCode;
		characterCode = e.keyCode;
		if(characterCode == 13) { //the key presed was 'enter'
			this.onClose();
		}		
	}
});

})();
