/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.story.StoryProgressBar");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojox.data.dom");

dojo.require("jazz.ui.HoverPopup");

dojo.require("com.ibm.team.workitem.web.client.internal.PlanClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.story.StoryProgressInformation");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.story.StoryProgressLabelProvider");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "StoryMessages");

(function() {

var StoryProgressInformation	= com.ibm.team.workitem.web.ui.internal.view.story.StoryProgressInformation;
var StoryProgressLabelProvider	= com.ibm.team.workitem.web.ui.internal.view.story.StoryProgressLabelProvider;

var PlanClient					= com.ibm.team.workitem.web.client.internal.PlanClient;
var ServiceResponseHandler		= com.ibm.team.repository.web.transport.ServiceResponseHandler;

var replaceChildren				= dojox.data.dom.replaceChildren;

/**
 * Widget to display the progress of a story work item.
 */
dojo.declare("com.ibm.team.workitem.web.ui.internal.view.story.StoryProgressBar", 
			 [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/story/templates/StoryProgressBar.html"),
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "StoryMessages");
		this.textNoWorkEstimated = this.messages.labelNoWorkEstimated;
	},
	isTall: true, // Layout as a "tall" presentation
	postCreate: function() {
		com.ibm.team.workitem.web.ui.internal.view.story.StoryProgressBar.superclass.postCreate.apply(this)
		
		replaceChildren(this._noWorkEstimatedLabel, document.createTextNode(this.messages.busyLoading));
		this._showText(true);

		var self= this;
		var responseHandler= new ServiceResponseHandler( { 
				_success: function(result) {
					var progressInfo= new StoryProgressInformation(result);
					var labelProvider= new StoryProgressLabelProvider(progressInfo);

					var message= labelProvider.getMessage();

					if (message != null) {
						self._showText(true);
						replaceChildren(self._noWorkEstimatedLabel, document.createTextNode(message));
					} else {
						self._showText(false);
						
						self._applyStyle(".barContainer .notEstimated", "height", 1 - (progressInfo.getQualityOfPlanning() / 100.0));
						self._applyStyle(".barContainer .estimated", "height", (progressInfo.getQualityOfPlanning() / 100.0));
			
						var workHourProgress= progressInfo.getWorkHourProgress();
			
						var completed= 0;
						if (progressInfo.isAllWorkDone()) {
							completed= 1;
						} else if (progressInfo.isRealTimeSpecified()) {
							var realTimeProgress= progressInfo.getRealTimeProgress();
							completed= Math.min(realTimeProgress, workHourProgress);
						} else {
							// no end date on the iteration
							completed= workHourProgress;
						}
			
						self._applyStyle(".barContainer .completed", "width", completed);
						self._applyStyle(".barContainer .open", "width", 1 - completed);
					}
					
					replaceChildren(self._progressHours, labelProvider.getHoursTextNode());
					replaceChildren(self._progressQuality, labelProvider.getEstimateTextNode());
					
					new jazz.ui.HoverPopup({
						around: [self._progressBar], html: labelProvider.getTooltipContent()
					});
					
				},
				_error: function(error) {
					self._remoteError(error);
				}
			},
			"_success",
			"_error");

		var id= this.workingCopy.getItemId();
		if (id)
			PlanClient.getWorkItemProgress(responseHandler, { itemId: id });
	},

	getLabel: function() {
		return this.messages.labelProgress;
	},
	getDefaultLabel: function() {
		return this.getLabel();
	},
	_showText: function(showText) {
		dojo.style(this._barContainer, "display", showText ? "none" : "block");
		dojo.style(this._noWorkEstimatedLabel, "display", showText ? "block" : "none");
	},
	
	_applyStyle: function(query, style, value) {
		dojo.query(query, this._progressBar).style(style, Math.round(value * 100) + "%");
	},
	
	_remoteError: function(error) {
		dojox.data.dom.replaceChildren(this._noWorkEstimatedLabel, document.createTextNode(this.messages.errorFetchingProgress));
		this._showText(true);
	}
});

})();