/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

//
// Note to readers: this file is intended to demonstrate and explain the common
// features of the Jazz Ajax framework 'Page' mechanism. Because of this, we've
// documented many common idioms which we would not comment.

// The following line defines and executes an anonymous function which is used
// to create a 'module closure' that allows us to define variables that seem
// like global variables within the state of the function, but do not clutter
// the real global namespace.
//
(function(){

dojo.provide("com.ibm.rational.cm.web.ui.page.CqPageView");

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.layout.ContentPane");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.util.ResizeNotifier");
dojo.require("jazz.ui.SideNavigationLayout");
dojo.require("jazz.ui.SearchBox");

dojo.require("com.ibm.rational.cm.web._base");
dojo.require("com.ibm.rational.cm.web.ui.page.CqPageController");
dojo.require("com.ibm.rational.cm.web.ui.page.CqLoginController");
dojo.require("com.ibm.rational.cm.web.ui.page.CqWelcome");
dojo.require("com.ibm.rational.cm.web.ui.view.navigation.FavoritesView");
dojo.require("com.ibm.rational.cm.web.ui.view.navigation.RecentView");
dojo.require("com.ibm.rational.cm.web.ui.view.navigation.NavigatorModel");
dojo.require("com.ibm.rational.cm.web.ui.view.navigation.NavigatorView");

dojo.requireLocalization("com.ibm.rational.cm.web.ui", "CqPageView");

//
// The following is what we call a 'utility function'. Utility functions must
// be stateless (i.e. they must not refer to 'this') because they are declared
// outside the scope of the widget definitions. Usually we use utility functions
// when we want to factor out some code that appears multiple times within the
// widget definition but doesn't have anything to do with the widget object.
//
// In this case it's just a helper to get a handle to the running web workbench.
//
function getWorkbench() {
	return net.jazz.ajax.ui.PlatformUI.getWorkbench();
}

var messages;
var cq = com.ibm.rational.cm.web;

var loginController = com.ibm.rational.cm.cqLoginController;

//
// Define the 'Page' widget, just as you would define any other Dojo widget.
//
// Note that we implement this widget as a TemplatedWidget even though the
// template is dead-simple and therefore unnecessary; the need to use a
// templated widget will go away when we implement Enhancement #31378.
//
dojo.declare(
	"com.ibm.rational.cm.web.ui.page.CqPageView",
	[dijit._Widget, dijit._Templated],
{
	// Includes a namespaced class to allow the page to declare other styles
	// without fear of a style name conflict.
	//
	// Note that the CSS for this page is defined in
	// resources/ui/templates/Page1.css. We don't have to make an explicit
	// reference to it because the UI framework at runtime appends all styles
	// declared within web plugin CSS files into a single monolithic style
	// sheet.
	templatePath: dojo.moduleUrl("com.ibm.rational.cm.web", "ui/page/templates/CqPageView.html"),
	
	constructor: function(){
		messages = dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "CqPageView");
		this._views = {};
	},
	
	// Dojo calls this method after it's created the widget; this is the place
	// to perform any one-time widget setup needs. In the case of a Jazz 'Page',
	// this is where you bind your action IDs (declared in plugin.xml) to
	// actual code.
	postCreate: function(){
		this.inherited(arguments);

	 	this.controller = new com.ibm.rational.cm.web.ui.page.CqPageController(this);
	 	
		// Create Left-nav layout and append it to the page widget; note that
		// Dojo defines the property 'this.domNode' and points it at the root
		// DOM node of the current widget (in this case the page widget itself).
		this._leftNavigationLayout = new jazz.ui.SideNavigationLayout();
		this.domNode.appendChild(this._leftNavigationLayout.domNode);
		this._initializeNavigation();
		
		this.resizeMonitor = jazz.util.ResizeNotifier.register({
			context: this,
			callback: this._resizeMainContent
		});
	},
	
	destroy: function(/*Boolean*/ preserveDom){
		if(this.resizeMonitor) { jazz.util.ResizeNotifier.unregister(this.resizeMonitor); }
		if(this.controller){ this.controller.destroyController(); }
		if(this._subscriptions) { dojo.forEach(this._subscriptions, dojo.unsubscribe); }
		this.inherited(arguments);
	},
	
	// Helper method for creating the left-nav sections. Note that this cannot
	// be defined as a utility function because it references the widget
	// instance 'this'.
	_initializeNavigation: function(){
		var navLayout = this._leftNavigationLayout;
	
		// setup search box 
		var searchInputWidget = null;

		// This is a temporary 'fix' till when we address work item: 79568		
		// if (com.ibm.rational.cm.cqLoginController.getLoginInfo("isFullTextSearchEnabled") == true) {
		if (true) {
			searchInputWidget = new jazz.ui.SearchBox( 
						{ onExecute: dojo.hitch(this.controller, this.controller._handleQuickSearch),
						  scopes: [
							{ id: "byId", label: messages.ById, icon: dojo.moduleUrl("com.ibm.rational.cm.web", "ui/images/search.gif") },
							{ id: "fullText", label: messages.FullText, icon: dojo.moduleUrl("com.ibm.rational.cm.web", "ui/images/search.gif") }
							],
							tooltip: messages.SearchClearQuestRecords
						 });
		}
		else {
			searchInputWidget = new jazz.ui.SearchBox( 
						{ onExecute: dojo.hitch(this.controller, this.controller._handleQuickSearch),
						  scopes: [
							{ id: "byId", label: messages.ById, icon: dojo.moduleUrl("com.ibm.rational.cm.web", "ui/images/search.gif") }
							],
							tooltip: messages.SearchClearQuestRecords
						 });
		}
		//container div used for visual alignment
		var searchContainer = dojo.doc.createElement("DIV");
		dojo.addClass(searchContainer, "searchcontainer");
		searchContainer.appendChild(searchInputWidget.domNode);
		navLayout._leftNav.appendChild(searchContainer);

		var _basicPrefix = "#action=com.ibm.rational.cm.";
		
		// setup create record dropdown button
//		if(!com.ibm.team.repository.web.client.session.isGuest()) {
			var btnTxt = dojo.string.substitute(messages.CreateRecord, ['<span id="defaultRecType">'+messages.DefaultRecordTypeLabal+'</span>']);
			var newWorkItemCommand = navLayout.addCommand({
				href: _basicPrefix+"newRecord",
				imageClass: "create-workitem-command",
				buttonText: btnTxt,
//				boldFirstWord: true, // don't use this option since it will break-up 'btnTxt' with the first space
				size: "large"
			});
			this.controller.setCreateRecordButton(newWorkItemCommand);
//		}
			
		// setup create query button
//		this._newQueryCommand = navLayout.addCommand({
//			href: _basicPrefix+"newQuery",
//			imageClass: "create-query-command",
//			buttonText: messages.CreateQuery,
//			boldFirstWord: true,
//			size: "small"
//		});
		
		// recent view
		var recentView = this._views.RECENT = new com.ibm.rational.cm.web.ui.view.navigation.RecentView({
			maxNum: 5
		});
		var recentSection = navLayout.addNavigationSection(recentView, messages.RecentView, true);
		recentView.populate(recentSection);
		var recentStyle = recentSection.domNode.style;
		recentStyle.display = "none";
		this.connect(recentView, "onItemChanged", function(items){
			if(recentStyle.display == "none" && items.length > 0){
				recentStyle.display = "";
			}else if(recentStyle.display != "none" && items.length === 0){
				recentStyle.display = "none";
			}
		});

		// favorites view
		var favView = this._views.FAVORITES = new com.ibm.rational.cm.web.ui.view.navigation.FavoritesView();
		var favNavSection = navLayout.addNavigationSection(favView, messages.Favorites, true);
		favView.populate(favNavSection);
		var favStyle = favNavSection.domNode.style;
		favStyle.display = "none";
		this.connect(favView, "onItemChanged", function(items){
			if(favStyle.display == "none" && items.length > 0){
				favStyle.display = "";
			}else if(favStyle.display != "none" && items.length === 0){
				favStyle.display = "none";
			}
		});
		
		// CQ Navigator
		var props = com.ibm.rational.cm.getNavigatorProperties();
		var navigatorView = this._views.NAVIGATOR = new com.ibm.rational.cm.web.ui.view.navigation.NavigatorView(props);
		navLayout.addNavigationSection(navigatorView, messages.Navigator, true);
	},
	
	getView: function(name){
		return this._views[name];
	},
	
	/*
	 * Show widget to main content area
	 * props is a JavaScript object which have the form:
	 * {
	 *	title : String
	 * 	widget : widget
	 * }
	 */
	setMainContent: function(props){
		var content = props.content;
		if(!content){ return; }
		
		var node = (content instanceof dijit._Widget) ?  content.domNode : content;
		
		this._leftNavigationLayout.setMainContent(node);
		this._mainContent = content;
		
		getWorkbench().setTitle(props.title);
		
		this._resizeMainContent();
	},
	
	// hacking code for defect 80452
	_findJazzUIContent: function(){
		try {
			var jazzUIContentNode = (this.domNode.parentNode) ? this.domNode.parentNode.parentNode : null;
			if(jazzUIContentNode && jazzUIContentNode.id == "content-cell"){ // JazzUI.html
				return jazzUIContentNode;
			}
		} finally {
			// not found
		}
		return null;
	},
	
	_resizeMainContent: function(){
		var content = this._mainContent;
		if(!content || !content.resize){ return; }
		var wtarget = this._leftNavigationLayout._contentArea;
		var htarget = this._findJazzUIContent() || wtarget; 
		content.resize({
			l: 0,
			t: 0,
			w: dojo.contentBox(wtarget).w,
			h: dojo.contentBox(htarget).h 
		});
	},
	
	/*
	 * Opens welcome page
	 */
	showWelcomePage: function(){
		var welcomePage = this._welcomePage;
		if(!welcomePage){
			welcomePage = this._welcomePage = new com.ibm.rational.cm.web.ui.page.CqWelcome();
			
			var self = this;
			welcomePage.connect(welcomePage, "_handleClick", function(){
				self.controller.initializeViews();
			});
			
			// 83670: There is a disabled Log In link in CQ Records welcome page after log in CQ web from dashboard
			// if already authenticated in other places (e.g. Dashboard),
			// we need to hide login link on creation of welcome widget
			if(loginController.isAuthenticated()){
				this.updateWelcomeMessage({ authenticated: true });
			}
		}
		this.setMainContent({
			title: messages.Welcome,
			content: welcomePage
		});
	},
	
	/*
	 * Update description in welcome page when login succeeded or error occurred
	 */
	updateWelcomeMessage: function(message){
		if(this._welcomePage){
			this._welcomePage.updateMessage(message);
		}
	}
});

})();