/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2007, 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */


/********************************************************************
com.ibm.rational.cm.web.view.filter.CqFilterValuePane, is an extention of dojo.ContentPane to 
input values for a filter of a query.


********************************************************************/
dojo.provide("com.ibm.rational.cm.web.view.filter.CqFilterValuePane");

dojo.require("dijit.layout.ContentPane");
dojo.require("dijit._Templated");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.FilteringSelect");
dojo.require("dijit.form.CheckBox");

dojo.require("com.ibm.rational.cm.web.ui.view.filter.CqFilterOPListBox");
dojo.require("com.ibm.rational.cm.web.ui.view.filter.CqFilterInput");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rational.cm.web.ui","cqQueryFilters");

(function(){
var cq = com.ibm.rational.cm.web;

dojo.declare(
	"com.ibm.rational.cm.web.view.filter.CqFilterValuePane",
	[dijit.layout.ContentPane, dijit._Templated],
{
	templatePath: dojo.moduleUrl("com.ibm.rational.cm.web", "ui/view/filter/templates/CqFilterValuePane.html"),
	
	widgetsInTemplate: true,
	/*
		could be used if we use Dojo Markup to classify this widget
	*/
	widgetType: "CqFilterValuePane",
	
	/*
		filterData stores the JSON object structure for the filters
	*/
	filterData: null,
	
	filterInfo: null, //filter info
	
	opwdgt: null,
	
	inputWdgt: null,
	
	updateDisplayFilterValue: function(str){
		this.dispFilterValue.innerHTML = str;
	},
	
	// a stub function that can be connected
	onFilterInfoChanged: function(){},

	setFilterInfo: function(filterInfo){
		this.filterInfo = filterInfo;
		this._filterChanged();
	},
	
	updateUnderlay: function(){
		//FIXME right now we have to update the underlay
		//as user selects different filters
		var isChecked = this.filterInfo.isDynamic[0];
		this.opwdgt.setDisabled(isChecked);
		this.inputWdgt.setDisabled(isChecked);
		this.promptTxt.setDisabled(!isChecked);
	},
	
	_filterChanged: function(){
		// create opwdgt and inputWdgt
		if(!this.opwdgt){
			this.opwdgt = this._createOPWidget();
		}
		if(!this.inputWdgt){
			this.inputWdgt = this._createValueWidget();
		}
		this.opwdgt.connectInputWidget(this.inputWdgt);
		this.inputWdgt.connectOpWidget(this.opwdgt);
		
		//update prompt section
		var isChecked = false;
		if(this.filterInfo.isDynamic && this.filterInfo.isDynamic[0]){
			isChecked =  this.filterInfo.isDynamic[0];
		}
		
		this.promptCheck.setChecked(isChecked);
		this._updatePrompt(isChecked);
	},
	
	_createOPWidget: function(){
		var validOPs = this.filterInfo.compops;
		var opwdgt = new com.ibm.rational.cm.web.view.filter.CqFilterOPListBox({});
		this.operatoranchor.appendChild(opwdgt.domNode);
		if(opwdgt.listwdgt){
			this.opLabel.setAttribute("for", opwdgt.listwdgt.id);
		}
		opwdgt.setStore(validOPs);
		this.connect(opwdgt, "onOpChanged", dojo.hitch(this, function(op){
			this.filterInfo.op = op;
			this.onFilterInfoChanged();
		}));
		// RATLC01358852 : Creating a Query using Filter should displayed the default operator
		if((!this.filterInfo.op || this.filterInfo.op == 0) && validOPs.length > 0){
			this.filterInfo.op = cq.ui.filter.getDefaultOperator(this.filterInfo.fieldType, validOPs);
		}
		opwdgt.setValue(this.filterInfo.op);
		return opwdgt;
	},
	
	_createValueWidget: function(){
		var filterInfo = this.filterInfo;
		var isSingle = (dojo.indexOf(filterInfo.compops, 13) == -1);
		var choiceListInfo = null;
		// RATLC01235425 : Choice-list data should not be cached by referencedRecordType
		// this.filterInfo is an dojo.data item, and value is stored in array
		var hasDynamicChoiceList = filterInfo.hasDynamicChoiceList;
		if(hasDynamicChoiceList && dojo.isArray(hasDynamicChoiceList) && hasDynamicChoiceList.length > 0){
			hasDynamicChoiceList = hasDynamicChoiceList[0];
		}
		if(filterInfo.referencedRecordType || hasDynamicChoiceList == true){
			choiceListInfo = {
				paramName: "fieldId",
				resourceId: filterInfo.stableLocation
			};
		}
		var inputWdgt = new com.ibm.rational.cm.web.view.filter.CqFilterInput({
			op: filterInfo.op,
			valueList: filterInfo.valueList,
			fieldType: filterInfo.fieldType,
			choiceListInfo: choiceListInfo,
			isSingleSelect: isSingle,
			hasOpenChoiceList: filterInfo.hasOpenChoiceList
		}, this.valuewidget);
		if(inputWdgt.wdgt){
			this.valLabel.setAttribute("for", inputWdgt.wdgt.id);
		}
		inputWdgt.setData(this.filterInfo.values);
		this.connect(inputWdgt, "onValueChanged", dojo.hitch(this, function(values){
			this.filterInfo.values = values;
			this.onFilterInfoChanged();
		}));
		return inputWdgt;
	},

	updatePrompt: function(e){
		this.filterInfo.prompt = this.promptTxt.getValue();
		this.onFilterInfoChanged();
	},
	
	// RATLC01236262 : [IE only] Filter operator/value fields are not enabled even if "dynamic filters" is unchecked
	_updatePrompt: function(isChecked){
		this.opwdgt.setDisabled(isChecked);
		this.inputWdgt.setDisabled(isChecked);
		this.promptTxt.setDisabled(!isChecked);
		this.filterInfo.isDynamic = [isChecked];
		if(isChecked){
			var tempVal = this.promptTxt.getValue();
			this.filterInfo.prompt = (tempVal) ? tempVal : "";
			this.promptTxt.focus();
		}
		this.onFilterInfoChanged();
	},
	
	postMixInProperties: function(){
		this.i18nStrings = dojo.i18n.getLocalization("cq", "cqQueryFilters", this.lang);
		this.inherited("postMixInProperties", arguments);
	},

	/*
		overwrite the post create method to insert our filter Tab container with data
	*/	
	postCreate: function(args, frag){
		// Just use default prompt message in English for all locales, since CQ Database may not support the client browser locale
		// RATLC01259439 : Falcon - Dynamic Filter message is vague
//		this.promptTxt.setValue(dojo.i18n.getLocalization("cq", "cqQueryFilters", "en").msgPrompt);
		var promptMsg = this.filterInfo.prompt;
		if(!promptMsg || promptMsg.length === 0){
			promptMsg = "Enter: "+this.filterInfo.fieldPath; // use hard-coded string since above line doesn't work when build version of dojo is used
		}
		this.promptTxt.setValue(promptMsg);
		this._filterChanged();
	},
		
	destroy: function(/*Boolean*/ finalize){
		if(this.opwdgt){ this.opwdgt.destroy(); }
		if(this.inputWdgt){ this.inputWdgt.destroy(); }
		this.inherited("destroy", arguments);
	}
});


})();