/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2007, 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.ui.view.resultset.ResultSetGrid");

dojo.require("com.ibm.rational.cm.web.ui.view.common.Grid");
dojo.require("com.ibm.rational.cm.web.ui.data.GridModel");
dojo.require("dojo.i18n");
dojo.require("com.ibm.rational.cm.web._base");
dojo.requireLocalization("com.ibm.rational.cm.web.ui", "ResultSetMessages");

(function(){

var messages;
var cq = com.ibm.rational.cm.web;
	
dojo.declare(
	"com.ibm.rational.cm.web.ui.view.resultset.ResultSetGrid",
	com.ibm.rational.cm.web.ui.view.common.Grid,
{	
	baseUrl: "cqqueryresults.cq",

	// query parameters
	action: "ExecuteQuery", // takes 3 values ExecuteQuery|ExecuteFullTextSearch|ExecuteUniqueKeyFilterQuery
	resourceId: null,
	startIndex: 1,
	count: 20, // avoid using name "rowCount" since it conflicts with dojox.VirtualGrid#rowCount and unintentinally gets modified by superclass
	data: null, // prompt query
	newItemsOnly: false, // for startup query
	startDate: null, // for startup query
	searchRecordTypes: null, // for full text search
	usrSoftLimitValue: null, // for resultset limit
	persistForSession: null, // for resultset limit
	sortColumn: null, // for server-sort
	sortOrder: null, // for server-sort
	
	_colInfo: null, // an array storing column info: { field: "field1", width: 15, index: 3
	isServerSort: false,

	hasRowNumber: true,
	maxChars: 50,
	
	postMixInProperties: function(){
		messages = dojo.i18n.getLocalization("com.ibm.rational.cm.web.ui", "ResultSetMessages", this.lang);
		
		// needed when parameters are passed as query strings for AJAX module
		var startIndex = this.startIndex;
		if(startIndex && dojo.isString(startIndex)){
			this.startIndex = parseInt(startIndex);
		}
		var count = this.count;
		if(count && dojo.isString(count)){
			this.count = parseInt(count);
		}
		this.inherited(arguments);
	},

	postCreate: function(){
		dojo.addClass(this.domNode, "com-ibm-rcm-ResultSetGrid");

		this.inherited(arguments);
		
		var model = new com.ibm.rational.cm.web.ui.data.GridModel();
		this.setModel(model);
		this._setStore();
		var self = this;
		this.connect(model, "onComplete", function(items, requestObject){
			var store = requestObject.store;
			if(!store){ return; }
			self._updateStructure(store.dataObject);
		});

		// override so that it treats like Ctrl click for checkbox column
		this.selection.clickSelectEvent = function(e){
			var ctrlKey = (e.cellIndex == 0) ? true : e.ctrlKey;
			self.selection.clickSelect(e.rowIndex, ctrlKey, e.shiftKey);
			self.handleRowClick(e.rowIndex, e.cellIndex);
		}
		
		this.connect(this, "dokeydown", "_handleKeyDown");	// somehow, dojox.grid.publicEvents#onKeyDown cannot capture key all the time
		this.sendRequest();
	},

	// if bForce is true, store is updated even if startIndex is unchanged
	// if bForce=true store is re-created
	// params : additional parameters to be passed as a content of XHR
	sendRequest: function(queryParams, /* boolean */ refresh){
		if(this.model.isExecuting()){ return; } // RATLC01361532 : double-click "Refresh"/"Next" in result-set has problems: "TypeError arr has no properties"

		if(!this.model.store || refresh || !queryParams ||
			this.startIndex != queryParams.startIndex || this.count != queryParams.count){
			this._setStore();
		}		
		this.setQueryParams(queryParams);
		var content = this.getXhrContent();
		if(refresh){ // resultset won't be refreshed unless store is re-created
			content.refresh = true;
		}

		this.model.requestRows(0, this.count, content);
	},

	setQueryParams: function(queryParams){
		for(var key in queryParams){
			this[key] = queryParams[key];
		}
	},
	
	getXhrContent: function(/* boolean */ isNoPagination){
		// required parameters
		var content = {
			action: this.action,
			resourceId: this.resourceId,
			format: "JSON"
		};

		// for server-sort
		if(this.isServerSort){
			content.sortColumn = this.sortColumn;
			content.sortOrder = this.sortOrder;
		}
		
		// for export query/printable version, don't send startIndex/rowCount
		if(!isNoPagination){
			content.startIndex = this.startIndex;
			content.rowCount = this.count;
		}
		
		// add optional parameters
		
		// prompt query
		if(this.data){
			content.data = this.data;
		}

		// startup query
		if(this.newItemsOnly){
			content.newItemsOnly = true;
			content.startDate = this.startDate;
		}
		
		// record chooser
		if(this.searchDisplayFields){
			content.searchDisplayFields = this.searchDisplayFields;
		}
		
		// full text search
		if(this.searchRecordTypes){
			content.searchRecordTypes = this.searchRecordTypes;
		}
		
		// resultset limit
		if(this.usrSoftLimitValue){
			content.usrSoftLimitValue = this.usrSoftLimitValue;
		}
		if(this.persistForSession){
			content.persistForSession = this.persistForSession;
		}
		
		return content;
	},
		
	_setStore: function(){
		dojo["require"]("com.ibm.rational.cm.web.ui.data.Store");
		var store = new com.ibm.rational.cm.web.ui.data.ReadStore({
			url: this.baseUrl,
			requestMethod: "POST" // RATLC01066099: Public query ClearTeam\7.1\Found in 7.1\All Open Found in 7.1 errors out
		});
		
		// override cq.data.CqItemFileReadStore
		// for resultset-specific error handling & save properties to store
		var self = this;
		store.format = function(dataObject){
			// keep dataObject in order to get various data from it (totalNumRows etc.)
			store.dataObject = dataObject; 
			
			if(dataObject){
				var resultSetData = dataObject.resultSetData;
				if(resultSetData && resultSetData.rowData){
					store.colData = resultSetData.colData; // GridModel#_setupFields assumes this value
					return {
						//identifier: "recordId",	// Defect 79256, remove the identifier to avoid conflict in some query.
						items: resultSetData.rowData
					}
				}
			}
			
			// other cases return null so that it won't be treated as error
			return null;
		}

		this.model.setData(store);
	},
	
	// Returns cell data for a given row index and a property name
	getCellData: function(inIndex, propName){
		if(!propName || propName.length === 0){ return null; }
		var row = this.model.getRow(inIndex);
		if(!row){ return; }
		return this.model.store.getValue(row.__dojo_data_item, propName);
	},
	
	_handleKeyDown: function(e){
		var selection = this.selection;
		var focus = this.focus;
		var idx = selection.getFirstSelected();
		var prevCellIndex;
		var keyCode = e.keyCode;
		var keys = dojo.keys;
		if(keyCode == keys.UP_ARROW && idx-1>=0){
			selection.select(idx-1);
			focus.setFocusIndex(idx-1, focus.cell.index); // doesn't scroll properly... dojo's bug?
		}else if(keyCode == keys.DOWN_ARROW){
			//RATLC01050495 : Selection disappears by hitting down key when last record of the resultset page is selected
			if(idx < this.model.getRowCount() - 1){
				selection.select(idx+1);
				focus.setFocusIndex(idx+1, focus.cell.index);
			}else if(idx == this.model.getRowCount() - 1){
				selection.select(idx);
				
				// workaround for focus being lost
				prevCellIndex = focus.cell.index;
				focus.previous();
				focus.setFocusIndex(idx, prevCellIndex);
			}
		}else if(keyCode == keys.ENTER){
			this.handleRowClick(idx, focus.cell.index);
		}
	},
	
	// update checkboxes according to selection of grid rows
	handleRowClick: function(inIndex, cellIndex){
		if(!this._prevSelection){
			this._prevSelection = [];
		}
		// clear checkboxes for previous selected rows
		var editor = this.getCell(0).editor;
		if(editor){
			dojo.forEach(this._prevSelection, function(selIndex){
				editor.getNode(selIndex).checked = false;
			});
		}
		// check newly selected rows
		var selection = this.selection.getSelected();
		if(editor){
			dojo.forEach(selection, function(selIndex){
				editor.getNode(selIndex).checked = true;
			});
		}
		this._prevSelection = selection;
	},
	
	// clear all checkboxes (work item 88072)
	clearCheckboxes: function(){
		if(this._prevSelection){
			this._prevSelection = [];			
		}
		if (this.selection.selected) {
			this.selection.selected = [];
		}
		this.update();
	},
		
	_updateStructure: function(dataObject){
		if(!dataObject){ return; }
		
		var resultSetData = dataObject.resultSetData;
		if(!resultSetData){ return; }
		
		// clear existing checkbox selections (work item 88072)
		this.clearCheckboxes();
		
		var colData = resultSetData.colData;

		// RATLC01235918 : Added column is not shown in resultset grid when refresh query
		// if colData has changed, need to update structure
		if(this._colInfo){ // check only for the first run (this._colInfo doesn't exist)
			var len = colData.length;
			if(len == this._colInfo.length){
				var needUpdate = false;
				for(var i = 0; i < len; i++){
					if(colData[i].field != this._colInfo[i].field){
						needUpdate = true;
						break;
					}
				}
				// Don't update structure for pagination and refresh with no column change so that column size information won't be lost
				if(!needUpdate){ return; }
			}
		}

		// RATLC01063995 : Falcon RATLC IHD: Query result columns wider than they need to be
		// get cell data that has maximum characters
		var maxChars = this.maxChars;
		var maxStrMap = {};
		var rowData = resultSetData.rowData;
		dojo.forEach(colData, function(item){
			var name = item.field;
			maxStrMap[name] = name;
			for(var i = 0, len = rowData.length; i < len; i++){
				if(rowData[i][name]){
					var cellData = rowData[i][name] + "";
					if(maxStrMap[name].length < cellData.length){
						if(cellData.length > maxChars){
							maxStrMap[name] = cellData.substring(0, maxChars);
							break;
						}
						maxStrMap[name] = cellData;
					}
				}
			}
		});
		
		var self = this;
		
		// for handling mastership column
		var model = this.model;
		var altTxt = messages.remotelyMasteredRecord;
		var remoteUrl = dojo.moduleUrl("com.ibm.rational.cm.web", "ui/view/record/images/remot_record.gif");
		var convertMastership = function(inRowIndex){
			var index = this.fieldIndex;
			return (index != -1 && model.getDatum(inRowIndex, index) === false) ?
				'<img src="'+remoteUrl+'" style="width: 16px; height: 16px;" alt="'+altTxt+'" title="'+altTxt+'" />' : "";
		};

		// RATLC01050012 : When editing a query without display fields, it can't be run
		this._colInfo = [];
		if(colData.length > 0){
			dojo.forEach(colData, function(item, idx){
				var field = item.field;
				var prop = {
					field: field,
					name: cq.sanitizeString(item.name || field), // RATLC01258902 : Title disappears if inputting '<' as the first character in display fields
					width: (self._getMeasuredWidth(maxStrMap[field])+10)+"px",
					index: idx // custom property
				};

				// for handling mastership column
				if(item.isHiddenMastership){
					prop.name = "#";
					prop.get = convertMastership;
					prop.width = 3;
					prop.styles = "text-align:center;"; 
				}else if(item.dataType == "DateTime"){ // RATLC01362177 : Submit_Date does't display locale time format in query on chinese locale
					prop.dataType = "DateTime";
				}

				self._colInfo.push(prop);
			});
		}else{ // when there are no display columns, show just row count
			this._colInfo.push({
				name: "index",
				get: function(inRowIndex){
					return (self.startIndex+inRowIndex);
				},
				width: "auto"
			});
		}
		if (this._colInfo.length > 0) {
			this._colInfo[this._colInfo.length-1].width = "auto";
		}

		var structure = [{ cells: [ this.createCheckboxColumn().concat(this._colInfo) ] }];
		if(this.hasRowNumber){
			structure = this.addRowNumberColumn(structure);
		}
		this.setStructure(structure);
	},
	
	/*
	 * Calculates width for index column using total rows
	 */
	setIndexColumnWidth: function(totalNumOfRows){
		this.indexColWidth = this._getMeasuredWidth(totalNumOfRows);
	},
	
	// calculate width using test node
	_getMeasuredWidth: function(str){
		var testNode = this.testNode;
		if(!testNode){
			testNode = this.testNode = dojo.doc.createElement("SPAN");
		}
		testNode.innerHTML = str;
		// RATLC01260773 : Query result column sizing messed up on shell refresh
		// test node should not be added this.domNode since it may be dispay=none and returns incorrect width
		var par = dojo.body();
		par.appendChild(testNode);
		var w = dojo.marginBox(testNode).w;
		par.removeChild(testNode);
		return w;
	},

	// RATLC01063658 : No numbering of rows of resultsets
	addRowNumberColumn: function(structure){
		var w = this.indexColWidth;
		if(w){
			w += 5; // add magic number...
		}else{
			w = 20; // default width
		}
		structure.unshift({ type: 'com.ibm.rational.cm.web.ui.view.resultset.ResultSetRowView', width: w+'px', grid: this });
		return structure;
	},
	
	// creates checkbox column for multi-selection
	createCheckboxColumn: function(){
		var cells = [];
		if(this.selection.multiSelect){
			// add checkbox column for multi-selection
			cells.push({
				field: "checkbox",
				name: " ",
				width: 2,
				styles: 'text-align: center;',
				value: false,
				editor: dojox.grid.editors.Bool
			});
		}
		return cells;
	},
	
	setServerSort: function(/* boolean */ isServerSort){
		this.isServerSort = isServerSort;
		if(!isServerSort){
			this.sortColumn = null;
			this.sortOrder = null;
		}
	},
	
	// override dojox.Gridsort functions
	// RATLC01050485 : Clicking on column heading when server-sort is being executed will cause an error
	canSort: function(inSortInfo){
		if(this.isServerSort && this.model.isExecuting()){ 
			return false;
		}
		return this.inherited(arguments);
	},
	
	// override dojox.Grid in order to support server-sort
	sort: function(){
		if(this.isServerSort){
			var fIndex = this.getSortField();
			var sortOrder = (fIndex >= 0) ? "ASC" : "DSC";
			this._setStore();
			this.sendRequest({
				startIndex: 1,	// should start from 1 when serverSort
				
				// this will send display name for 7.1GA, 2009A, 2009B
				// this will send display name for 2009C or later
				// 2009C needs to include RATLC01290389 for server sort to work
				sortColumn: this.model.fields.get(Math.abs(fIndex)-1).name,
				sortOrder: sortOrder
			}, true);
		}else{
			this.inherited(arguments);
		}
	},

	// input is an array of field names
	customizeColumns: function(fieldNameArray){
		var hasAuto = false;
		var arr = [];
		var self = this;

		var len = fieldNameArray.length;
		var STR_PIXELS = "px";
		var STR_AUTO = "auto";
		var defaultWidth = 15;
		
		var w;
		var totalWidth = 0;
		// RATLC01050254 : Cannot resize some column in resultset after customizing column order
		dojo.forEach(this._colInfo, function(col){
			var index = dojo.indexOf(fieldNameArray, col.field);
			w = self.getColumnWidth(col.index);
			if(index != -1){
				if(col.index == -1){ // set cached width for columns that were hidden previously
					if(col.__cachedWidth){
						col.width = col.__cachedWidth + STR_PIXELS;
					}else{
						col.width = defaultWidth;
					}
				}else{ // set measured width for columns that were visible previously
					if(col.width == STR_AUTO && index == len-1){
						// if previously "auto" and is the last column now, do nothing. Other than that, set measured width
					}else{
						// RATLC01060768 : result set: column size is reset on "customize columns"
						if(w != -1){
							col.width = w + STR_PIXELS;
						}else{
							col.width = defaultWidth;
						}
					}
				}
				if(w != -1){
					totalWidth += w;
				}
				arr[index] = col;
			}
			if(w != -1){
				col.__cachedWidth = w;
			}
			col.index = index;
		});
		
		var gridViewIndex = 1; // 0 is row numbering column
		if(this.views.views.length > gridViewIndex){
			if(totalWidth <= parseInt(this.views.views[gridViewIndex].getContentWidth())){ // if the whole grid is displayed without horizontal scrollbar, set the last column to "auto"
				arr[arr.length - 1].width = "auto";
			}
		}
		
		var structure = [{ cells: [ this.createCheckboxColumn().concat(arr) ] }];
		if(this.hasRowNumber){
			structure = this.addRowNumberColumn(structure);
		}
		this.setStructure(structure);
		
		// clear existing checkbox selections (work item 88072)
		this.clearCheckboxes();
		
		// this.update();
	}
});

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.resultset.ResultSetRowView",
	dojox.GridRowView,
{
	buildRowContent: function(inRowIndex, inRowNode){
		var structure = this.structure;
		var index = (structure && structure.grid && structure.grid.startIndex) ? (structure.grid.startIndex + inRowIndex) : (inRowIndex + 1);
		inRowNode.innerHTML = '<table style="width:' + (this.contentNode.offsetWidth - this.padBorderWidth) + 'px;" role="wairole:presentation"><tr><td class="dojoxGrid-rowbar-inner">'+index+'</td></tr></table>';
	}
});

})();