/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2007, 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */
 
dojo.provide("com.ibm.rational.cm.web.ui.view.resultset.ResultSetView");

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit._Templated");
dojo.require("dijit.Toolbar");
dojo.require("dijit.form.Button");
dojo.require("dijit.Menu"); // needed for export resultset dropdown menu
dojo.require("com.ibm.rational.cm.web._base");
dojo.require("com.ibm.rational.cm.web.ui.view.resultset.Pagination");
dojo.require("com.ibm.rational.cm.web.ui.view.resultset.ResultSetGrid");

// 77417: CSS modularity fails to load css files for dynamically loaded scripts
// should lazy load
dojo.require("com.ibm.rational.cm.web.ui.view.common.DoubleListBox");

dojo.requireLocalization("com.ibm.rational.cm.web.ui", "common");
dojo.requireLocalization("com.ibm.rational.cm.web.ui", "ResultSetMessages");
dojo.requireLocalization("com.ibm.rational.cm.web.ui", "cqRecordStrings");

(function(){

var cq = com.ibm.rational.cm.web;
var cqNS = "com.ibm.rational.cm.web.ui";

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.resultset.ResultSetView",
	[ dijit.layout.BorderContainer, dijit._Templated ],
{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("com.ibm.rational.cm.web", "ui/view/resultset/templates/ResultSetView.html"),

	title: "",
	
	// query parameters
	action: "ExecuteQuery", // default runs normal query. for Full text search, action=ExecuteFullTextSearch
	resourceId: null,
	startIndex: 1,
	count: 20,
	data: null, // for prompt query
	newItemsOnly: false, // for startup query
	startDate: null, // for startup query
	searchRecordTypes: null, // for full text search
	
	isServerSort: false,
	
	gutters: false,
	
	postMixInProperties: function(){
		var getLoc = dojo.i18n.getLocalization;
		var lang = this.lang;
		this.commonStr = getLoc(cqNS, "common", lang);
		this.messages = getLoc(cqNS, "ResultSetMessages", lang);
		this._cqRecordStrings = getLoc(cqNS, "cqRecordStrings");
		this.inherited(arguments);
	},
	
	postCreate: function(){
		this.inherited(arguments);
		
		this.setTitleCell(this.title);
		
		// pagination
		this.connect(this.pagination, "onChange", function(obj){
			self.executeQuery(obj);
		});
		
		// setup grid
		var self = this;
		var grid = this.rsGrid = new com.ibm.rational.cm.web.ui.view.resultset.ResultSetGrid({
			action: this.action,
			resourceId: this.resourceId,
			startIndex: this.startIndex,
			count: this.count,
			data: this.data, // for prompt query
			newItemsOnly: this.newItemsOnly, // for startup query
			startDate: this.startDate, // for startup query
			searchRecordTypes: this.searchRecordTypes,
			isServerSort: this.isServerSort,
			elasticView: 0,
			autoHeight: true,   // added for 79742
			height: 0,
			singleClickEdit: true
		}, this.gridNode);
		var model = grid.model;
		this.connect(model, "onComplete", "_onGridDataLoaded");
		this.connect(model, "onBeforeFetch", function(){
			self.setLoading(true, dojo.string.substitute(self.messages.ExecutingQuery, [self.title]));
		});
		this.connect(model, "onReturn", function(){
			self.setLoading(false);
		});
		
		this.connect(grid, "handleRowClick", function(inIndex, cellIndex){
			if(cellIndex > 0){
				self.openRecordByRowIndex(inIndex);
			}
		});

		// setup toolbar
		this.connect(model, "_setupFields", function(colData){
			self.customizeColBtn.attr("disabled", !(colData.length > 0));
			
			// store mastership column name
			for(var i = 0, len = colData.length; i < len; i++){
				if(colData[i].isHiddenMastership){
					self._mastershipColumnName = colData[i].field;
					break;
				}
			}
		});
		
		// custom property : need a bettwer way...
		this.exportFileMenuItem.fileFormat = "FILE";
	//	this.exportPdfMenuItem.fileFormat = "PDF";
		this.exportExcelMenuItem.fileFormat = "EXCEL";
		//For Favorites
		this.favoriteBookmarkFileMenuItem.fileFormat = "BOOKMARK";
		this.favoriteEmailFileMenuItem.fileFormat = "EMAIL";	
	},
	
	// a stub function that can be connected
	setLoading: function(/* boolean */ loading, /* String */ message){
	},
	
	executeQuery: function(/* Object */ queryParams, /* boolean */ refresh){
		this.rsGrid.sendRequest(queryParams, refresh);
	},
		
	openRecordByRowIndex: function(inIndex, openNewWindow){
		var grid = this.rsGrid;
		var recordId = grid.getCellData(inIndex, "recordId");
		if(!recordId || recordId.length === 0){ return; }

		if(!openNewWindow){
			openNewWindow = 0;
		}
		
		// RATLC01261482 : Performance: GetAction should be called in parallel with GetRecordDetail
		var hasMastership = grid.getCellData(inIndex, this._mastershipColumnName);
		if(hasMastership !== true && hasMastership !== false){
			hasMastership = true;	// if multi-site is not enabled, there is no mastership column. So set to locally mastered
		}
		
		dojo.publish('CQ_RECORD_SELECTED', [{
			selectedArtifactIds: [{
				recordId: recordId+"",
				hasMastership: hasMastership
			}],
			openNewWindow: openNewWindow
		}]);
	},
	
	startup: function(){
		this.inherited(arguments);
		this.rsGrid.resize();
	},

	// RATLC01252246 : The width of query result grid couldn't extend automatically after clicking on the split bar to hide the left pane.
	resize: function(args){
		var prevBox = this._contentBox;
		this.inherited(arguments);

		// RATLC01066344 : result set should remember the position of horizontal scroll bar when I go to next record
		// only resize grid only if contentBox changed
		if(!prevBox || this._contentBox.t != prevBox.t || this._contentBox.l != prevBox.l ||
			this._contentBox.w != prevBox.w || this._contentBox.h != prevBox.h){
			this.rsGrid.resize();
		}
	},

	/*
	 * Updates resultset toolbar when the data has been loaded
	 */
	_onGridDataLoaded: function(items, requestObject){
		// set record type (needed for "Edit" query)
		// RATLC01060775 : edit query: never finished loading "fields" field
		// recordType must be set even if no records are returned
		var store = requestObject.store;
		var totalNumOfRows = 0;
		var grid = this.rsGrid;
		if(store){
			var dataObject = store.dataObject;
			if(dataObject){
				var resultSetData = dataObject.resultSetData;
				if(resultSetData){
					this.recordType = resultSetData.recordType;
				
					// RATLC01060310 : Full-Text Search result-set page is not showing the term being searched on
					var searchString = resultSetData.searchString;
					if(this.action == "ExecuteFullTextSearch" && searchString && searchString.length > 0){
						this.setTitleCell(dojo.string.substitute(this._cqRecordStrings.searchResult, [searchString]));
					}
				
					totalNumOfRows = resultSetData.totalNumOfRows;
					
					// show resultset llimit dialog
					if(resultSetData.isOverLimit == true &&
						(!this._showLimit || (requestObject && requestObject.content && requestObject.content.refresh))){
						dojo["require"]("com.ibm.rational.cm.web.ui.view.resultset.ResultSetLimit");
						var dlg = com.ibm.rational.cm.web.ui.view.resultset.limitDialog;
						dlg.setLimits(grid, resultSetData.totalNumOfRows, resultSetData.hardLimit);
						dlg.show();
						this._showLimit = true;
					}
				}
			}
		}
		
		// update pagination
		var len = items.length;
		this.pagination.update(grid.startIndex, len, totalNumOfRows);

		// update grid area
		// workaround for a propblem of grid not being layout properly
		// when loading is finished and this tab is not selected
		var par = this.getParent();
		if(par && par.domNode){
			this._isUpdated = false;
		}
		var noRecord = (len == 0);
		this.rsMessage.style.display = noRecord ? "" : "none";
		var wasGridHidden = (grid.domNode.style.display == "none");
		grid.domNode.style.display = noRecord ? "none" : "";
		// RATLC01066731 : Display issues when refreshing queries that had a few hits, then no hits, then a few hits again
		if(wasGridHidden){
			grid.update();
		}
		grid.setIndexColumnWidth(totalNumOfRows); // calculate index column width

		// update toolbar
		this.customizeColBtn.attr("disabled", noRecord);
		
		if (noRecord) {
			alert(this.messages.NoRecordsFound + searchString);
		}
	},

	// RATLC01062034 : Long text in Full-Text Search will obscure the header of the result column
	setTitleCell: function(title){
		this.titleCell.innerHTML = cq.sanitizeString(title);
		this.titleCell.title = title;
	},
	
	refreshGrid: function(e){
		// RATLC01068112 : refresh button on a query resultset tab reruns the query with the previous input, rather than reprompt the user
		if(this.data){
			dojo.publish("CQ_QUERY_EXECUTE", [{
				id: this.resourceId,
				title: this.title
			}]);
		}else{
			this.executeQuery({}, true);
		}
	},

	// RCM
	addToFavorites: function(){
		dojo["require"]("com.ibm.rational.cm.web.ui.page.FavoritesController");
		com.ibm.rational.cm.favoritesController.addFavorite({
			type: "LIST",
			title: this.title, 
			stableLocation: this.resourceId,
			masterReplica: "<local>" // TODO : hard-coded
		});
	},

	/*
	 * Bookmarks the result set in the Browser. ex: "ClearQuest:All Defects"
	 */
	bookMark: function(){
		dojo["require"]("com.ibm.rational.cm.web.ui.utils.Bookmark");
		com.ibm.rational.cm.web.ui.utils.addToBookmark({
			type: "LIST",
			title: this.title,
			stableLocation: this.resourceId
		});
	},

	/*
	 * Opens up your Email Client, creates an email with Subject as "ClearQuest:All Defects"
	 * and the link in the body of the email.
	 */		
	emailLink: function(){
		dojo["require"]("com.ibm.rational.cm.web.ui.utils.Bookmark");
		com.ibm.rational.cm.web.ui.utils.addEmailLink({
			type: "LIST",
			title: this.title,
			stableLocation: this.resourceId 
		});
	},
	
	/*
	 * Exports resultset data to CSV text/Excel file
	 * For performance, delay loading of scripts until they are actually required 
	 */
	exportGrid: function(menuItem){
		dojo["require"]("com.ibm.rational.cm.web.ui.view.resultset.actions.exportGrid");
		com.ibm.rational.cm.web.ui.view.resultset.actions.exportGrid(this, menuItem);
	},
	
	/*
	 * Shows printable version of resultset in simple table format
	 * For performance, delay loading of scripts until they are actually required 
	 */
	showPrintableVersion: function(e){
		dojo["require"]("com.ibm.rational.cm.web.ui.view.resultset.actions.printableVersion");
		com.ibm.rational.cm.web.ui.view.resultset.actions.showPrintableVersion(this);
	},

	/*
	 * Shows a dialog that allows to customize columns of a resultset
	 * For performance, delay loading of scripts until they are actually required 
	 */
	showCustomizeDialog: function(){
		dojo["require"]("com.ibm.rational.cm.web.ui.view.resultset.actions.customizeColumns");
		com.ibm.rational.cm.web.ui.view.resultset.actions.showCustomizeDialog(this);
	},
	
	/*
	 * Loads Modify actions for multi-record update
	 * For performance, delay loading of scripts until they are actually required 
	 */
	_onBatchMenuOpen: function(){
		dojo["require"]("com.ibm.rational.cm.web.ui.view.resultset.actions.multiRecordUpdate");
		com.ibm.rational.cm.web.ui.view.resultset.actions.loadActions(this);
	},
	
	handleRowsPerPageChanged: function(e){
		this.pagination.setCount(parseInt(this.rowsPerPage.value));
	},
	
	destroy: function(/*Boolean*/ finalize){
		if(this._customizeDialog){ this._customizeDialog.destroy(); }
		var exportMenu = this.exportButton.dropDown;
		if(exportMenu && exportMenu.domNode){
			exportMenu.destroyRecursive();
		}
		var batchMenu = this.batchButton.dropDown;
		if(batchMenu && batchMenu.domNode){
			batchMenu.destroyRecursive();
		}
		var FavoritesMenu = this.tool_favoriteMenu.dropDown;
		if(FavoritesMenu && FavoritesMenu.domNode){
			FavoritesMenu.destroyRecursive();
		}
		this.inherited(arguments);
	}
});

com.ibm.rational.cm.web.ui.view.resultset.getQueryParameters = function(message){
	var params = {
		resourceId: message.id
	};
	params.action = (message.type == "FULL_TEXT") ? "ExecuteFullTextSearch" : "ExecuteQuery";

	// prompt query
	if(message.filtervalues){
		params.data = dojo.toJson(message.filtervalues.filters);
	}
	// startup query
	if(message.newItemsOnly){
		params.newItemsOnly = true;
		params.startDate = message.startDate;
	}
	// full text search
	if(message.type == "FULL_TEXT"){
		params.searchRecordTypes = dojo.toJson(message.recordTypes);
	}
	return params;
}

})();