/*******************************************************************************
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * Licensed Materials - Property of IBM
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.client.PlanItem"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.client.Duration"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.PlanDuration"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.PlanElement"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.SequenceValue"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.PlanningAttributeIdentifier"); //$NON-NLS-1$

dojo.require("com.ibm.team.workitem.web.cache.internal.Cache"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.util"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.PlanningClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.ProgressMonitoredServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.WorkItemLinkChangeRunnable"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.WorkItemSaveManager"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$

(function() {

var PlanningClient					= com.ibm.team.apt.web.client.internal.PlanningClient;
var ResponseHandler					= com.ibm.team.apt.web.client.internal.ProgressMonitoredServiceResponseHandler;

var PlanningAttributeIdentifier		= com.ibm.team.apt.client.PlanningAttributeIdentifier;
var Duration						= com.ibm.team.apt.client.Duration;
var PlanDuration					= com.ibm.team.apt.client.PlanDuration;
var PlanElement						= com.ibm.team.apt.client.PlanElement;
var SequenceValue					= com.ibm.team.apt.client.SequenceValue;
var WorkingCopyRegistry				= com.ibm.team.workitem.web.cache.internal.Cache;
var WorkItemLinkChangeRunnable		= com.ibm.team.apt.web.client.internal.WorkItemLinkChangeRunnable;
var WorkItemSaveManager				= com.ibm.team.apt.web.client.internal.WorkItemSaveManager;

var Assert							= com.ibm.team.apt.web.ui.internal.common.Assert;

var EstimateConfig					= com.ibm.team.apt.web.client.util.EstimateConfig;

var PlanItem= dojo.declare("com.ibm.team.apt.client.PlanItem", PlanElement, { //$NON-NLS-1$

	__draftUuid: null,
	
	__scheduleItemAttriuteDelegates: null,
	__workItem: null,
	__workItemWorkingCopy: null,
	__plan: null,
	__parent: null,
	
	__dirty: false,
	__auxiliary: false,
	__primary: true,
	__draftItem: false,
	
	__duration: null,
	__cachedSequenceValue: null,
	__isResolved: null,

	constructor: function(plan, parent, workItemDTO, scheduleItemAttriuteDelegates, workingCopy) {
		this.__plan= plan;
		this.__parent= parent;
		this.__scheduleItemAttriuteDelegates= scheduleItemAttriuteDelegates;
		if (!workItemDTO) {
			this.__workItemWorkingCopy= workingCopy;
			this.__workItem= new Object();
			this.__workItem.attributeValues= [];
			this.__draftItem= true;
			this.__primary= true;
			this.__draftUuid= this.__workItemWorkingCopy.getValue({path: ["itemId"]}); //$NON-NLS-1$
		} else {
			this.__workItem= workItemDTO;
			this.__primary= plan._matchesPlanQuery(this);
		}
		this.__auxiliary= !this.__primary;
		this.__duration= this.__createDuration();
	},

	equals: function(other) {
		if (this === other)
			return true;
		
		if (other == null)
			return false;

		if (this.declaredClass !== other.declaredClass)
			return false;

		return this.getUuid() === other.getUuid();
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	setWorkItem: function(workItem) {
		this.__draftItem= false;
		this.__workItem= workItem;
		this.__duration= this.__createDuration();
	},

	getUuid: function() {
		if (this.__draftItem)
			return this.__draftUuid;
		
		return this.__workItem.itemId;
	},
	
	getElementKey: function() {
		return this.getUuid();
	},

	getId: function() {
		return this.getAttributeValue(PlanItem.ID);
	},

	getCheckReport: function() {
		var currentReport= this.__plan.getCheckReport();
		return currentReport != null ? currentReport.getReport(this) : null;
	},

	getAuxiliaryPlanItemReasons: function() {
		return this.__plan._getAuxiliaryPlanItemReasons(this);
	},
	
	schedule: function() {
		var sequenceManager= this._getSequenceManager();
		if (sequenceManager != null) {
			if (arguments.length == 1) {
				var when= arguments[0];
				sequenceManager.schedule(this, when);
			} else if (arguments.length == 2) {
				var predecessor= arguments[0];
				var successor= arguments[1];
				
				if (predecessor == null && successor == null) {
					sequenceManager.unschedule(this, true);
				} else {
					sequenceManager.schedule(this, predecessor, successor);
				}
				
			}
		}
	},

	// ---- deprecated API. will be removed --------------------------------------------------------------------------------
	// TODO remove these method and the respective references to it. The values should be accessed via the attributes only

	getTarget: function() {
		return this.getAttributeValue(PlanItem.TARGET);
	},

	getItemType: function() {
		return this.getAttributeValue(PlanItem.ITEM_TYPE);
	},
	
	getWorkflowAction: function() {
		return this.getAttributeValue(PlanItem.WORKFLOW_ACTION);
	},
	
	setWorkflowAction: function(action) {
		this.setAttributeValue(PlanItem.WORKFLOW_ACTION, action);
	},
	
	getState: function() {
		return this.getAttributeValue(PlanItem.STATE);
	},
	
	setState: function(state) {
		this.setAttributeValue(PlanItem.STATE, state);
	},

	getPriority: function() {
		return this.getAttributeValue(PlanItem.PRIORITY);
	},

	getResolutionDate: function() {
		return this.getAttributeValue(PlanItem.RESOLUTION_DATE);
	},
	
	isProgressPlanItem: function() {
		return this.__workItem.progressItem;
	},
	
	isTopLevelPlanItem: function() {
		return this.getAttributeValue(PlanItem.ITEM_TYPE).topLevel;
	},

	// ---- accessor methods for reflective attributes ---------------------------------------------------------------------

	getSummary: function() {
		return this.__scheduleItemAttriuteDelegates[PlanItem.SUMMARY.getId()].getValue(this);
	},

	setSummary: function(summary) {
		this.__scheduleItemAttriuteDelegates[PlanItem.SUMMARY.getId()].setValue(this, summary);
	},
	
	getOwner: function() {
		return this.__scheduleItemAttriuteDelegates[PlanItem.OWNER.getId()].getValue(this);
	},

	setOwner: function(owner) {
		this.__scheduleItemAttriuteDelegates[PlanItem.OWNER.getId()].setValue(this, owner);
	},

	getTimeline: function() {
		return null;
	},

	getSequenceValue: function() {
		return this.__scheduleItemAttriuteDelegates[PlanItem.SEQUENCE_VALUE.getId()].getValue(this);
	},

	setSequenceValue: function(sequenceValue, doSave) {
		this.__scheduleItemAttriuteDelegates[PlanItem.SEQUENCE_VALUE.getId()].setValue(this, sequenceValue, doSave);
	},
	
	isResolved: function() {
		if (this.__isResolved === null) {
			if (this.__draftItem) {
				this.__isResolved= false;
			} else {
				var state= this.getState(); 
				var workflowInfo= this.getWorkflowInfo();
				if (!state || !workflowInfo) {
					this.__isResolved= false;
				} else {
					this.__isResolved= workflowInfo.getStateGroup(state.getId()) == com.ibm.team.apt.web.client.internal.mixins.WorkflowInfo.DONE;
				}
			}
		}
		return this.__isResolved;
	},

	getPrimaryTag: function() {
		return this.getAttributeValue(PlanItem.TAGS)[0];
	},
	
	setPrimaryTag: function(tag) {
		throw new Error("write access not yet implemented"); //$NON-NLS-1$
	},

	getWorkflowInfo: function() {
		var itemType= this.getItemType();
		return itemType.getWorkflow();
	},

	getEstimate: function() {
		return this.getDuration().getCurrentEstimate();
	},
	
	setEstimate: function(value) {
		if (this.getAttributeValue(PlanItem.CORRECTED_ESTIMATE).specified) {
			this.setAttributeValue(PlanItem.CORRECTED_ESTIMATE, value);
			if (!value.specified) {
				this.setAttributeValue(PlanItem.ORIGINAL_ESTIMATE, value);
			}
		} else {
			this.setAttributeValue(PlanItem.ORIGINAL_ESTIMATE, value);
		}
		
		this.__duration= this.__createDuration();
	},
	
	getEffort: function() {
		if ("TimeSpent" == EstimateConfig.estimateMode) { //$NON-NLS-1$
			return this.getDuration().getTimeSpent();
		} else {
			return this.getDuration().getTimeRemaining();
		}
	},
	
	setEffort: function(value) {
		this.getPlanModel().compoundChange(function() {
			var currentDuration= this.getDuration();
			if ("TimeSpent" == EstimateConfig.estimateMode) { //$NON-NLS-1$
				this.setAttributeValue(PlanItem.TIMESPENT, value);
			} else {
				if(!value.specified) {
					this.setAttributeValue(PlanItem.TIMESPENT, value);
				} else if(!currentDuration.getCurrentEstimate().specified) {
					this.setEstimate(value);
				} else if(currentDuration.getCurrentEstimate() < value) {
					value= PlanDuration.encodeInvalidTimeRemaining(value);
				} else {
					value= new Duration(currentDuration.getCurrentEstimate().getMilliseconds() - value.getMilliseconds());
				}
				this.setAttributeValue(PlanItem.TIMESPENT, value);
			}
		}, this); 
	},
	
	getDuration: function() {
		return this.__duration;
	},
	
	setDuration: function(value) {
		if (this.isResolved()) {
			this.setAttributeValue(PlanItem.TIMESPENT, value);
		} else {
			this.setEstimate(value);
		}
		
		this.__duration= this.__createDuration();
	},

	getScheduledTime: function() {
		var sequenceManager= this._getSequenceManager();
		return sequenceManager != null ? sequenceManager._getScheduledTime(this) : null;
	},

	isDraftItem: function() {
		return this.__draftItem;
	},
	
	isAuxiliaryPlanItem: function() {
		return this.__auxiliary; 
	},
	
	isPrimaryPlanItem: function() {
		return this.__primary;
	},
	
	isMarkedRead: function() {
		return true;
	},
	
	isNewItem: function() {
		return false; // TODO
	},

	// ---- PlanElement implementation -------------------------------------------------------------------------------------
	
	getParent: function() {
		return this.__parent;
	},

	moveTo: function(newParent) {
		var oldParent= this.getParent();
		
		if (oldParent.getElementKey() != newParent.getElementKey()) {
			this._parentChanged(oldParent, newParent);

			var linkType= "com.ibm.team.workitem.linktype.parentworkitem"; //$NON-NLS-1$
			var linkAction= null;
			var linkTarget= null;

			if (newParent instanceof com.ibm.team.apt.client.PlanModel) {
				linkAction= "removeLink"; //$NON-NLS-1$
				linkTarget= oldParent.getUuid();
			} else {
				linkAction= "addLink"; //$NON-NLS-1$
				linkTarget= newParent.getUuid();
			}
			
			var undoRunnable= dojo.hitch(this, function() {
				this._parentChanged(newParent, oldParent);
			});
			
			if (!this.__draftItem) {
				WorkItemSaveManager.save(new WorkItemLinkChangeRunnable(WorkItemLinkChangeRunnable.constructorArgs(this, linkType, linkAction, linkTarget, undoRunnable)));
			} else if (this.__workItemWorkingCopy) {
				this.__workItemWorkingCopy.setValue({path: ["linkTypes", linkType], value: [{itemId: linkTarget}]}); //$NON-NLS-1$
			}
		}
	},

	isDirty: function() {
		return this.__dirty;
	},

	getPlanModel: function() {
		return this.__plan;
	},

	/**
	 * synchronize this plan item with its corresponding work item working copy, 
	 * if any exists. Does nothing otherwise.
	 */
	synchronizeWithWorkItemWorkingCopy: function() {
		this.__workItemWorkingCopy= WorkingCopyRegistry.getItem(this.getId());
		if (this.__workItemWorkingCopy) {

			// 1) update our attribute values from the working copy
			this.__plan.compoundChange(function(builder) {
				var attributeMap= this.__plan._getWorkItemAttributes();
				for (var workItemAttributeId in attributeMap) {
					var planningAttribute= attributeMap[workItemAttributeId];
					planningAttribute.updateFromWorkingCopy(this, builder);
				}

				var wasAuxiliary= this.__auxiliary;
				this.__auxiliary= !this.getPlanModel()._matchesPlanQuery(this);
				if (wasAuxiliary !== this.__auxiliary) {
					builder.changed(this, PlanItem.AUXILIARY_ITEM, wasAuxiliary, this.__auxiliary);

					if (!this.__primary && !this.__auxiliary) {
						this.__primary= true;
						builder.changed(this, PlanItem.PRIMARY_ITEM, false, true);
					}
				}
				
			}, this);

			// 2) we must patch our stateId with the one from the workingCopy to avoid save conflicts
			this.__workItem.stateId= this.__workItemWorkingCopy.getValue({path: ["stateId"]}); //$NON-NLS-1$
			
			// 3) look out for changed parent links and move this item if necessary
			var linkTypes= this.__workItemWorkingCopy.getValue({path: ["linkTypes"]}); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			if (linkTypes) {
				var parentLink= null;
				for (var i= 0; i < linkTypes.length; i++) {
					if (linkTypes[i].id == "com.ibm.team.workitem.linktype.parentworkitem" && !linkTypes[i].isSource) { //$NON-NLS-1$
						if (linkTypes[i].linkDTOs.length == 1) {
							parentLink= linkTypes[i].linkDTOs[0];
						}
						break;
					}
				}

				var oldParent= this.__parent;
				var newParent= parentLink != null ? this.__plan.getItem(parentLink.itemId) : this.__plan;					

				if (newParent && oldParent.getElementKey() != newParent.getElementKey()) {					
					// TODO we ignore unresolved parents here
					if (!this._introducesParentCycle(newParent)) {			
						this._parentChanged(oldParent, newParent);
					}
				}
			}
		}
	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------
	
	__createDuration: function() {
		var correctedEstimate= this.getAttributeValue(PlanItem.CORRECTED_ESTIMATE);
		var originalEstimate= this.getAttributeValue(PlanItem.ORIGINAL_ESTIMATE);
		var timeSpent= this.getAttributeValue(PlanItem.TIMESPENT);
		var isResolved= this.isResolved();
		return PlanDuration.create(originalEstimate, correctedEstimate, timeSpent, isResolved);
	},
	
	_introducesParentCycle: function(newParent) {
		var cycleDetectionSubject= newParent;
		while (cycleDetectionSubject != null) {
			if (cycleDetectionSubject == this) {
				return true;
			}
			cycleDetectionSubject= cycleDetectionSubject.getParent();
		}
	},

	_childAdded: function(parent, child) {
		this.getPlanModel()._childAdded(parent, child);
	},
	
	_childRemoved: function(parent, child) {
		this.getPlanModel()._childRemoved(parent, child);
	},
	
  	_parentChanged: function(oldParent, newParent) {
		this.getPlanModel().compoundChange(function(builder) {
			builder.aboutToMove(oldParent, newParent);
			oldParent.removeChild(this);
			this.__workItem.parent= newParent.itemId;
			Assert.isLegal(!this._introducesParentCycle(newParent), "Prevented introducing parent cycle"); //$NON-NLS-1$
			this.__parent= newParent;
			newParent.addChild(this);
			builder.moved();
			builder.changed(this, com.ibm.team.apt.client.PlanItem.PARENT, undefined, undefined);
		}, this);
  	},

  	_workItemAttributeChanged: function(attribute, newValue) {
		if (PlanItem.CORRECTED_ESTIMATE.equals(attribute) || PlanItem.ORIGINAL_ESTIMATE.equals(attribute) || PlanItem.TIMESPENT.equals(attribute) || PlanItem.STATE.equals(attribute)) {
			var oldDuration= this.__duration;
			this.__duration= this.__createDuration();
			
			this.getPlanModel().compoundChange(function(builder) {
				if (!oldDuration.equals(this.__duration)) {
					builder.changed(this, com.ibm.team.apt.client.PlanItem.DURATION, oldDuration, this.__duration);
				}
				if (!oldDuration.getCurrentEstimate().equals(this.__duration.getCurrentEstimate())) {
					builder.changed(this, com.ibm.team.apt.client.PlanItem.ESTIMATE, oldDuration.getCurrentEstimate(), this.__duration.getCurrentEstimate());
				}
			}, this);
		}

		if (PlanItem.ITEM_TYPE.equals(attribute) || PlanItem.STATE.equals(attribute)) {
			this.__isResolved= null;
		}
  	},
	
	_getSequenceManager: function() {
		return this.__plan.getItemSequenceManager(this.getOwner());
	},

	__sentinel: null // terminates this class definition
});

com.ibm.team.apt.client.PlanItem.SUMMARY= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.summary"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.OWNER= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.owner"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.SCHEDULED_TIME= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.scheduledTime"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.SEQUENCE_VALUE= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.sequenceValue"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.RESOLVED= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.resolved"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.ID= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.id"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.ITEM_TYPE= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.itemType"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.DESCRIPTION= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.description"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.CREATOR= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.creator"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.CATEGORY= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.filedAgainst"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.TARGET= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.plannedFor"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.TIMELINE= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.timeline"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.PRIMARY_TAG= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.primaryTag"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.TAGS= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.tags"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.STATE= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.state"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.WORKFLOW_ACTION= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.workflowAction"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.PRIORITY= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.priority"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.SEVERITY= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.severity"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.COMPLEXITY= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.complexity"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.ESTIMATE= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.estimate"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.EFFORT= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.effort"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.MINIMAL_ESTIMATE= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.minimalEstimate"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.MAXIMAL_ESTIMATE= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.maximalEstimate"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.ORIGINAL_ESTIMATE= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.originalEstimate"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.CORRECTED_ESTIMATE= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.correctedEstimate"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.TIMESPENT= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.timeSpent"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.DURATION= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.duration"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.DUE_DATE= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.dueDate"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.RESOLUTION_DATE= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.resolutionDate"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.DRAFT_ITEM= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.draftItem"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.AUXILIARY_ITEM= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.auxiliaryItem"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.PRIMARY_ITEM= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.primaryItem"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.PARENT= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.parent"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.MARKED_READ= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.markedRead"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.NEW_ITEM= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.newItem"); //$NON-NLS-1$
com.ibm.team.apt.client.PlanItem.TEAM_MEMBER_AREA= new PlanningAttributeIdentifier("com.ibm.team.apt.attribute.planitem.teamMemberArea"); //$NON-NLS-1$

})();
