/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.client.internal.attributes.WorkflowActionAttribute"); //$NON-NLS-1$

dojo.require("com.ibm.team.foundation.Assert"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.PlanningAttribute"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory"); //$NON-NLS-1$
// XXX Layer Breaker
dojo.require("com.ibm.team.apt.web.client.internal.WorkItemStateChangeRunnable"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.WorkItemSaveManager"); //$NON-NLS-1$

(function() {

var Assert							= com.ibm.team.foundation.Assert;
var PlanningAttribute				= com.ibm.team.apt.client.PlanningAttribute;
var WorkItemStateChangeRunnable		= com.ibm.team.apt.web.client.internal.WorkItemStateChangeRunnable;
var WorkItemSaveManager				= com.ibm.team.apt.web.client.internal.WorkItemSaveManager;
var WorkItemProxyFactory			= com.ibm.team.workitem.web.cache.internal.WorkItemProxyFactory;

var WORKITEM_WORKING_COPY_PATH		= ["attributes", "workflowAction", "id"]; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

dojo.declare("com.ibm.team.apt.client.internal.attributes.WorkflowActionAttribute",  PlanningAttribute, { //$NON-NLS-1$
	
	constructor: function(id, descriptor) {
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	getValue: function(receiver) {
		if (!(receiver instanceof com.ibm.team.apt.client.PlanItem))
			return undefined;

		return this._convertToPlanItem(receiver.__workItem.workflowAction, receiver);
	},
	
	setValue: function(receiver, newValue) {
		if (receiver instanceof com.ibm.team.apt.client.PlanItem) {
			this.__doSetValue(receiver, newValue, true);
		}
	},
	
	updateFromWorkingCopy: function(receiver, deltaBuilder) {
		Assert.isNotNull(receiver.__workItemWorkingCopy, "WorkItem WorkingCopy is missing"); //$NON-NLS-1$
		
 		var newRawValue= receiver.__workItemWorkingCopy.getValue({ path: WORKITEM_WORKING_COPY_PATH });
 		if (newRawValue) {
 			var newValue= this._convertToPlanItem(newRawValue, receiver);
 			var oldValue= this.getValue(receiver);
 			
 			if (!this.__equalValues(oldValue, newValue)) {
 				deltaBuilder.changed(receiver, this, oldValue, newValue);
 				receiver.__workItem.workflowAction= newRawValue;
 			}
 		}
	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------
  	
	__doSetValue: function(receiver, newValue, triggerWorkItemSave) {
		var oldValue= this.getValue(receiver);
		if (!this.__equalValues(oldValue, newValue)) {
			var newRawValue= this._convertToWorkItem(newValue, receiver);
			receiver.getPlanModel().compoundChange(function(builder) {
 				builder.changed(receiver, this, oldValue, newValue);
 				receiver.__workItem.workflowAction= newRawValue;

				if (triggerWorkItemSave && newValue != null) {
					var undoRunnable= dojo.hitch(this, function() {
						this.__doSetValue(receiver, oldValue, false);
					});

					WorkItemSaveManager.save(new WorkItemStateChangeRunnable(WorkItemStateChangeRunnable.constructorArgs(receiver, newRawValue, newValue.resolutionId, newValue.endState, undoRunnable)));
				}

			}, this);
			
			var workingCopy= WorkItemProxyFactory.getWorkItemProxy({id: receiver.getId(), createIfNeeded: false});
			if (workingCopy && newRawValue) {
				workingCopy.setValue({path: WORKITEM_WORKING_COPY_PATH, attributeId: "workflowAction", value: newRawValue}); //$NON-NLS-1$
			}
		}
	},
	
	_convertToPlanItem: function(value, receiver) {
		return value != null ? receiver.getWorkflowInfo().getWorkflowAction(value) : value;
	},

	_convertToWorkItem: function(value, receiver) {
		return value != null ? value.actionId : null;
	},

	__equalValues: function(v1, v2) {
		if (v1 === undefined)
			return v2 === undefined;

		if (v1 === null)
			return v2 === null;

		return v2 && v1.actionId == v2.actionId;
	},

	__sentinel: null // terminates this class definition
});

})();
