/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.client.internal.schedule.UnionIterator"); //$NON-NLS-1$

dojo.require("com.ibm.team.foundation.Assert"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.client.internal.Timespan"); //$NON-NLS-1$

(function() {

var Timespan						= com.ibm.team.apt.client.internal.Timespan;
var Assert							= com.ibm.team.foundation.Assert;

dojo.declare("com.ibm.team.apt.client.internal.schedule.UnionIterator", null, { //$NON-NLS-1$
	
	__iterators: null,
	__current: null,
	__forward: null,
	
	constructor: function(iterators) {
		Assert.isLegal(dojo.isArray(iterators) && iterators.length > 0);
		this.__iterators= iterators;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	reset: function(earliestStart, forward) {
		this.__forward= forward;
		for (var i= 0; i < this.__iterators.length; i++) {
			this.__iterators[i].reset(earliestStart, forward);
		}
		this.__current= this.__validate(earliestStart);
	},
	
	current: function() {
		return this.__current;
	},

	next: function() {
		if (this.__current != null) {
			this.__current= this.__validate(this.__forward ? this.__current.end : this.__current.start);
		}
		return this.__current;
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	__validate: function(earliestStart) {
		var i, currentPeriod;
		var begin, end;
		do {
			begin= earliestStart;
			end= undefined;
			// 1) find latest start
			for (i= 0; i < this.__iterators.length; i++) {
				currentPeriod= this.__iterators[i].current();
				if (currentPeriod == null)
					return null;

				if (begin == undefined || (this.__forward ? currentPeriod.start.after(begin) : currentPeriod.end.before(begin))) {
					begin= this.__forward ? currentPeriod.start : currentPeriod.end;
				}
			}

			// 2) check if latest start is contained in all timespans. advance the iterator if not
			var candidateInstant= begin;
			for (i= 0; i < this.__iterators.length; i++) {
				currentPeriod= this.__iterators[i].current();
				if (!this.__contains(currentPeriod, candidateInstant)) {
					
					while(this.__forward ? currentPeriod.end.beforeEq(candidateInstant) : currentPeriod.start.afterEq(candidateInstant)) {
						currentPeriod= this.__iterators[i].next();
						if (currentPeriod == null)
							return null;
					}
					begin= undefined;
					end= undefined;
				} 

				if (end == undefined || (this.__forward ? currentPeriod.end.before(end) : currentPeriod.start.after(end))) {
					end= this.__forward ? currentPeriod.end : currentPeriod.start;
				}
			}
		} while (begin == undefined || end == undefined);

		if (this.__forward) {
			return new Timespan(earliestStart.after(begin) ? earliestStart : begin, end);
		} else {
			return new Timespan(end, earliestStart.before(begin) ? earliestStart : begin);
		}
	},

	__contains: function(timespan, instant) {
		if (this.__forward) {
			return timespan.start.beforeEq(instant) && timespan.end.after(instant);
		} else {
			return timespan.end.afterEq(instant) && timespan.start.before(instant);
		}
	},
	
	__sentinel: null // terminates this class definition
});

})();
